/* Generated by Cython 0.29.21 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "ultron.sentry.Math.Accumulators.StatefulAccumulators",
        "sources": [
            "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx"
        ]
    },
    "module_name": "ultron.sentry.Math.Accumulators.StatefulAccumulators"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_21"
#define CYTHON_HEX_VERSION 0x001D15F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ultron__sentry__Math__Accumulators__StatefulAccumulators
#define __PYX_HAVE_API__ultron__sentry__Math__Accumulators__StatefulAccumulators
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#include <math.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx",
  "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd",
  "stringsource",
  "__init__.pxd",
  "type.pxd",
  "ultron/sentry/Math/Accumulators/impl.pxd",
  "ultron/sentry/Math/Distributions/NormalDistribution.pxd",
  "ultron/sentry/Math/Accumulators/IAccumulators.pxd",
  "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd",
};

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":689
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":690
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":691
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":696
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":697
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":698
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":703
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":704
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":713
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":714
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":715
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":717
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":718
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":719
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":721
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":722
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":724
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":725
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":726
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque;
struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":728
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":729
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":730
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":732
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump;

/* "ultron/sentry/Math/Accumulators/impl.pxd":12
 *     cdef public size_t count
 * 
 *     cdef double dump(self, double value, double default=*)             # <<<<<<<<<<<<<<
 *     cpdef void dumps(self, values)
 *     cdef inline size_t size(self)
 */
struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump {
  int __pyx_n;
  double __pyx_default;
};
struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_11Accumulator_transform;

/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":19
 *     cpdef push(self, dict data)
 *     cpdef double result(self)
 *     cpdef transform(self, data, str name=*, bint to_sort=*)             # <<<<<<<<<<<<<<
 * 
 * cdef bint isanumber(a)
 */
struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_11Accumulator_transform {
  int __pyx_n;
  PyObject *name;
  int to_sort;
};
struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert;
struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyEnsureRaise;
struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinWarning;
struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_isClose;

/* "ultron/sentry/Utilities/Asserts.pxd":3
 * # -*- coding: utf-8 -*-
 * 
 * cpdef int pyFinAssert(condition, exception, str msg=*) except -1             # <<<<<<<<<<<<<<
 * 
 * cpdef int pyEnsureRaise(exception, str msg=*) except -1
 */
struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert {
  int __pyx_n;
  PyObject *msg;
};

/* "ultron/sentry/Utilities/Asserts.pxd":5
 * cpdef int pyFinAssert(condition, exception, str msg=*) except -1
 * 
 * cpdef int pyEnsureRaise(exception, str msg=*) except -1             # <<<<<<<<<<<<<<
 * 
 * cpdef int pyFinWarning(condition, warn_type, str msg=*)
 */
struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyEnsureRaise {
  int __pyx_n;
  PyObject *msg;
};

/* "ultron/sentry/Utilities/Asserts.pxd":7
 * cpdef int pyEnsureRaise(exception, str msg=*) except -1
 * 
 * cpdef int pyFinWarning(condition, warn_type, str msg=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef bint isClose(double a, double b=*, double rel_tol=*, double abs_tol=*) nogil
 */
struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinWarning {
  int __pyx_n;
  PyObject *msg;
};

/* "ultron/sentry/Utilities/Asserts.pxd":9
 * cpdef int pyFinWarning(condition, warn_type, str msg=*)
 * 
 * cpdef bint isClose(double a, double b=*, double rel_tol=*, double abs_tol=*) nogil             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_isClose {
  int __pyx_n;
  double b;
  double rel_tol;
  double abs_tol;
};

/* "ultron/sentry/Math/Accumulators/impl.pxd":4
 * 
 * 
 * cdef class Deque:             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t window
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *__pyx_vtab;
  size_t window;
  int is_full;
  double *con;
  size_t start;
  size_t count;
};


/* "ultron/sentry/Math/Distributions/NormalDistribution.pxd":3
 * # -*- coding: utf-8 -*-
 * 
 * cdef class InverseCumulativeNormal(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _average
 */
struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *__pyx_vtab;
  double _average;
  double _sigma;
  int _fullAcc;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":6
 * 
 * 
 * cdef class IAccumulator(object):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator {
  PyObject_HEAD
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":10
 * 
 * 
 * cdef class Accumulator(IAccumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public bint _isFull
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator __pyx_base;
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *__pyx_vtab;
  int _isFull;
  PyObject *_dependency;
  size_t _window;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":25
 * 
 * 
 * cdef class Negative(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _inner
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":33
 * 
 * 
 * cdef class CombinedValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _left
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_left;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_right;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":41
 * 
 * 
 * cdef class AddedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":46
 * 
 * 
 * cdef class MinusedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":51
 * 
 * 
 * cdef class MultipliedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":56
 * 
 * 
 * cdef class DividedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":61
 * 
 * 
 * cdef class LtOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":66
 * 
 * 
 * cdef class LeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":71
 * 
 * 
 * cdef class GtOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":76
 * 
 * 
 * cdef class GeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":81
 * 
 * 
 * cdef class EqOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":86
 * 
 * 
 * cdef class NeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":91
 * 
 * 
 * cdef class Identity(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _value
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _value;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":99
 * 
 * 
 * cdef class Current(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _current
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _current;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":107
 * 
 * 
 * cdef class Latest(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _latest
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _latest;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":115
 * 
 * 
 * cdef class CompoundedValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _left
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_left;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_right;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":124
 * 
 * 
 * cdef class IIF(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _cond
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_cond;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_left;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_right;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":134
 * 
 * 
 * cdef class BasicFunction(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _origValue
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _origValue;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":142
 * 
 * 
 * cdef class Exp(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":147
 * 
 * 
 * cdef class Log(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":152
 * 
 * 
 * cdef class Sqrt(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":157
 * 
 * 
 * cdef class Pow(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _n
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
  double _n;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":164
 * 
 * 
 * cdef class Abs(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":169
 * 
 * 
 * cdef class Sign(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":174
 * 
 * 
 * cdef class Acos(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":179
 * 
 * 
 * cdef class Acosh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":184
 * 
 * 
 * cdef class Asin(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":189
 * 
 * 
 * cdef class Asinh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":194
 * 
 * 
 * cdef class NormInv(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cdef InvNormImpl _inv
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *_inv;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":201
 * 
 * 
 * cdef class Ceil(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":206
 * 
 * 
 * cdef class Floor(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":211
 * 
 * 
 * cdef class Round(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":215
 *     cpdef double result(self)
 * 
 * cdef class Sigmoid(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":219
 *     cpdef double result(self)
 * 
 * cdef class Tanh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":7
 * 
 * 
 * cdef class Diff(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _curr
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _curr;
  double _previous;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":17
 * 
 * 
 * cdef class SimpleReturn(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _diff
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _diff;
  double _curr;
  double _previous;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":28
 * 
 * 
 * cdef class LogReturn(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _diff
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _diff;
  double _curr;
  double _previous;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":39
 * 
 * 
 * cdef class PositivePart(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _pos
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _pos;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":48
 * 
 * 
 * cdef class NegativePart(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _neg
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _neg;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":57
 * 
 * 
 * cdef class Max(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentMax
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _currentMax;
  int _first;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":67
 * 
 * 
 * cdef class Maximum(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentMax
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _currentMax;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_y;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":77
 * 
 * 
 * cdef class Min(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentMin
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _currentMin;
  int _first;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":87
 * 
 * 
 * cdef class Minimum(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentMin
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _currentMin;
  int _first;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_y;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":98
 * 
 * 
 * cdef class Sum(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _currentSum;
  int _first;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":108
 * 
 * 
 * cdef class Average(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _currentSum;
  int _currentCount;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":118
 * 
 * 
 * cdef class XAverage(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _average
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _average;
  double _exp;
  int _count;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":129
 * 
 * 
 * cdef class Variance(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _currentSum;
  double _currentSumSquare;
  int _currentCount;
  int _isPop;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":141
 * 
 * 
 * cdef class Product(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _product
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _product;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":8
 * 
 * 
 * cdef class StatefulValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public Deque _deque
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *_deque;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":16
 * 
 * 
 * cdef class Shift(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _popout
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  double _popout;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":26
 * 
 * 
 * cdef class Delta(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _popout
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  double _popout;
  double _current;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":37
 * 
 * 
 * cdef class SingleValuedValueHolder(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public Accumulator _x
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":42
 * 
 * 
 * cdef class SortedValueHolder(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef list _sortedArray
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  PyObject *_sortedArray;
  double _cur_pos;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":48
 *     cpdef push(self, dict data)
 * 
 * cdef class MovingMax(SortedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":53
 * 
 * 
 * cdef class MovingArgMax(SortedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":58
 * 
 * 
 * cdef class MovingMin(SortedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":63
 * 
 * 
 * cdef class MovingArgMin(SortedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":68
 * 
 * 
 * cdef class MovingRank(SortedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":73
 * 
 * 
 * cdef class MovingQuantile(SortedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":78
 * 
 * 
 * cdef class MovingAllTrue(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public size_t _countedTrue
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  size_t _countedTrue;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":86
 * 
 * 
 * cdef class MovingAnyTrue(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public size_t _countedTrue
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  size_t _countedTrue;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":94
 * 
 * 
 * cdef class MovingSum(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningSum;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":102
 * 
 * 
 * cdef class MovingAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningSum;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":110
 * 
 * 
 * cdef class MovingDecay(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 *     cdef public double _runningSum
 *     cdef double _runningWeightedSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningSum;
  double _runningWeightedSum;
  double _newestValue;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":119
 * 
 * 
 * cdef class MovingPositiveAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningPositiveSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningPositiveSum;
  int _runningPositiveCount;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":128
 * 
 * 
 * cdef class MovingPositiveDifferenceAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public MovingAverage _runningAverage
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *_runningAverage;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":136
 * 
 * 
 * cdef class MovingNegativeDifferenceAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public MovingAverage _runningAverage
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *_runningAverage;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":144
 * 
 * 
 * cdef class MovingRSI(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public MovingPositiveDifferenceAverage _posDiffAvg
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *_posDiffAvg;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *_negDiffAvg;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":153
 * 
 * 
 * cdef class MovingNegativeAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningNegativeSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningNegativeSum;
  int _runningNegativeCount;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":162
 * 
 * 
 * cdef class MovingVariance(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningSum;
  double _runningSumSquare;
  int _isPop;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":172
 * 
 * 
 * cdef class MovingStandardDeviation(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningSum;
  double _runningSumSquare;
  int _isPop;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":182
 * 
 * 
 * cdef class MovingNegativeVariance(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningNegativeSum
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningNegativeSum;
  double _runningNegativeSumSquare;
  int _runningNegativeCount;
  int _isPop;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":193
 * 
 * 
 * cdef class MovingCountedPositive(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public int _counts
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  int _counts;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":201
 * 
 * 
 * cdef class MovingCountedNegative(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public int _counts
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  int _counts;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":209
 * 
 * 
 * cdef class MovingCorrelation(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningSumLeft
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  double _runningSumLeft;
  double _runningSumRight;
  double _runningSumSquareLeft;
  double _runningSumSquareRight;
  double _runningSumCrossSquare;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_y;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *_deque_y;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":224
 * 
 * 
 * cdef class MovingProduct(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningProduct
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningProduct;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":232
 * 
 * 
 * cdef class MACD(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public Accumulator _short_average
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_short_average;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_long_average;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":241
 * 
 * 
 * cdef class MovingLogReturn(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _runningReturn
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
  double _runningReturn;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":249
 * 
 * 
 * cdef class MovingSharp(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public MovingAverage _mean
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *_mean;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *_var;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_y;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *_deque_y;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":261
 * 
 * 
 * cdef class MovingSortino(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public MovingAverage _mean
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *_mean;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *_negativeVar;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_y;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *_deque_y;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":273
 * 
 * 
 * cdef class MovingDrawdown(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef MovingMax _maxer
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *_maxer;
  double _runningCum;
  double _currentMax;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":284
 * 
 * 
 * cdef class MovingMaxDrawdown(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef MovingDrawdown _drawdownCalculator
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *_drawdownCalculator;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *_minimer;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
};


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":294
 * 
 * 
 * cdef class MovingResidue(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _cross
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  double _cross;
  double _xsquare;
  double _lastx;
  double _lasty;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_x;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_y;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *_deque_y;
};



/* "ultron/sentry/Math/Accumulators/impl.pxd":4
 * 
 * 
 * cdef class Deque:             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t window
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque {
  double (*dump)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *, double, struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump *__pyx_optional_args);
  void (*dumps)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *, PyObject *, int __pyx_skip_dispatch);
  size_t (*size)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *);
  int (*isFull)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *);
  size_t (*idx)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *, double, int __pyx_skip_dispatch);
  double (*sum)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *, int __pyx_skip_dispatch);
  void (*set_data)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *, PyObject *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_4impl_Deque;


/* "ultron/sentry/Math/Distributions/NormalDistribution.pxd":3
 * # -*- coding: utf-8 -*-
 * 
 * cdef class InverseCumulativeNormal(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _average
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal {
  double (*_standard_value)(struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *, double);
  double (*inv)(struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *, double);
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *__pyx_vtabptr_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":10
 * 
 * 
 * cdef class Accumulator(IAccumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public bint _isFull
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator {
  int (*isFull)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch);
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch);
  double (*result)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch);
  PyObject *(*transform)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_11Accumulator_transform *__pyx_optional_args);
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":25
 * 
 * 
 * cdef class Negative(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _inner
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":33
 * 
 * 
 * cdef class CombinedValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _left
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":41
 * 
 * 
 * cdef class AddedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":46
 * 
 * 
 * cdef class MinusedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":51
 * 
 * 
 * cdef class MultipliedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":56
 * 
 * 
 * cdef class DividedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":61
 * 
 * 
 * cdef class LtOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":66
 * 
 * 
 * cdef class LeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":71
 * 
 * 
 * cdef class GtOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":76
 * 
 * 
 * cdef class GeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":81
 * 
 * 
 * cdef class EqOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":86
 * 
 * 
 * cdef class NeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":91
 * 
 * 
 * cdef class Identity(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _value
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":99
 * 
 * 
 * cdef class Current(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _current
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":107
 * 
 * 
 * cdef class Latest(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _latest
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":115
 * 
 * 
 * cdef class CompoundedValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _left
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":124
 * 
 * 
 * cdef class IIF(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _cond
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":134
 * 
 * 
 * cdef class BasicFunction(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _origValue
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":142
 * 
 * 
 * cdef class Exp(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":147
 * 
 * 
 * cdef class Log(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":152
 * 
 * 
 * cdef class Sqrt(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":157
 * 
 * 
 * cdef class Pow(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _n
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":164
 * 
 * 
 * cdef class Abs(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":169
 * 
 * 
 * cdef class Sign(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":174
 * 
 * 
 * cdef class Acos(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":179
 * 
 * 
 * cdef class Acosh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":184
 * 
 * 
 * cdef class Asin(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":189
 * 
 * 
 * cdef class Asinh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":194
 * 
 * 
 * cdef class NormInv(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cdef InvNormImpl _inv
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":201
 * 
 * 
 * cdef class Ceil(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":206
 * 
 * 
 * cdef class Floor(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":211
 * 
 * 
 * cdef class Round(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":215
 *     cpdef double result(self)
 * 
 * cdef class Sigmoid(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":219
 *     cpdef double result(self)
 * 
 * cdef class Tanh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":7
 * 
 * 
 * cdef class Diff(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _curr
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":17
 * 
 * 
 * cdef class SimpleReturn(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _diff
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":28
 * 
 * 
 * cdef class LogReturn(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _diff
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":39
 * 
 * 
 * cdef class PositivePart(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _pos
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":48
 * 
 * 
 * cdef class NegativePart(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _neg
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":57
 * 
 * 
 * cdef class Max(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentMax
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":67
 * 
 * 
 * cdef class Maximum(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentMax
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":77
 * 
 * 
 * cdef class Min(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentMin
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":87
 * 
 * 
 * cdef class Minimum(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentMin
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":98
 * 
 * 
 * cdef class Sum(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentSum
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":108
 * 
 * 
 * cdef class Average(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentSum
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":118
 * 
 * 
 * cdef class XAverage(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _average
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":129
 * 
 * 
 * cdef class Variance(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _currentSum
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance;


/* "ultron/sentry/Math/Accumulators/StatelessAccumulators.pxd":141
 * 
 * 
 * cdef class Product(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _product
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":28
 * 
 * 
 * cdef class StatefulValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  size_t (*size)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":46
 * 
 * 
 * cdef class Shift(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  int (*lag)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":71
 * 
 * 
 * cdef class Delta(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
  int (*lag)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":101
 * 
 * 
 * cdef class SingleValuedValueHolder(StatefulValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, window, x):
 *         super(SingleValuedValueHolder, self).__init__(window)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":109
 * 
 * 
 * cdef class SortedValueHolder(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":137
 * 
 * 
 * cdef class MovingMax(SortedValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, window, x):
 *         super(MovingMax, self).__init__(window, x)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":151
 * 
 * 
 * cdef class MovingArgMax(SortedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":170
 * 
 * 
 * cdef class MovingMin(SortedValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, window, x):
 *         super(MovingMin, self).__init__(window, x)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":184
 * 
 * 
 * cdef class MovingArgMin(SortedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":203
 * 
 * 
 * cdef class MovingRank(SortedValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, window, x):
 *         super(MovingRank, self).__init__(window, x)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":214
 * 
 * 
 * cdef class MovingQuantile(SortedValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, window, x):
 *         super(MovingQuantile, self).__init__(window, x)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":230
 * 
 * 
 * cdef class MovingAllTrue(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":262
 * 
 * 
 * cdef class MovingAnyTrue(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":294
 * 
 * 
 * cdef class MovingSum(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":318
 * 
 * 
 * cdef class MovingAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":348
 * 
 * 
 * cdef class MovingDecay(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":386
 * 
 * 
 * cdef class MovingPositiveAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":422
 * 
 * 
 * cdef class MovingPositiveDifferenceAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":440
 * 
 * 
 * cdef class MovingNegativeDifferenceAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":458
 * 
 * 
 * cdef class MovingRSI(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":483
 * 
 * 
 * cdef class MovingNegativeAverage(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":519
 * 
 * 
 * cdef class MovingVariance(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x, isPopulation=False):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":563
 * 
 * 
 * cdef class MovingStandardDeviation(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x, isPopulation=False):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":608
 * 
 * 
 * cdef class MovingNegativeVariance(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x, isPopulation=0):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":663
 * 
 * 
 * cdef class MovingCountedPositive(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":691
 * 
 * 
 * cdef class MovingCountedNegative(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":720
 * 
 * # Calculator for one pair of series
 * cdef class MovingCorrelation(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x, y):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":779
 * 
 * 
 * cdef class MovingProduct(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":805
 * 
 * 
 * cdef class MACD(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, short_win, long_win, x, method=XAverage):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":828
 * '''
 * 
 * cdef class MovingLogReturn(SingleValuedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":850
 * 
 * 
 * cdef class MovingSharp(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x, y):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":892
 * 
 * 
 * cdef class MovingSortino(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x, y):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":933
 * 
 * 
 * cdef class MovingDrawdown(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":959
 * 
 * 
 * cdef class MovingMaxDrawdown(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown;


/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":984
 * 
 * 
 * cdef class MovingResidue(StatefulValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, window, x, y):
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* DelItemInt.proto */
#define __Pyx_DelItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_DelItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_DelItem_Generic(o, to_py_func(i))))
static int __Pyx_DelItem_Generic(PyObject *o, PyObject *j);
static CYTHON_INLINE int __Pyx_DelItemInt_Fast(PyObject *o, Py_ssize_t i,
                                               int is_list, int wraparound);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static size_t __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_size(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_isFull(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_lag(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_lag(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_isFull(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;

/* Module declarations from 'ultron.sentry.Math.Accumulators.impl' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque = 0;

/* Module declarations from 'ultron.sentry.Math.Distributions.NormalDistribution' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal = 0;

/* Module declarations from 'ultron.sentry.Math.Accumulators.IAccumulators' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh = 0;
static PyObject *(*__pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder)(PyObject *, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'libc.math' */

/* Module declarations from 'ultron.sentry.Math.Accumulators.StatelessAccumulators' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product = 0;

/* Module declarations from 'ultron.sentry.Utilities.Asserts' */
static int (*__pyx_f_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert)(PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert *__pyx_optional_args); /*proto*/
static int (*__pyx_f_6ultron_6sentry_9Utilities_7Asserts_isClose)(double, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_isClose *__pyx_optional_args); /*proto*/

/* Module declarations from 'ultron.sentry.Math.udfs' */
static double (*__pyx_f_6ultron_6sentry_4Math_4udfs_consecutive_int_sum)(double, double); /*proto*/

/* Module declarations from 'ultron.sentry.Math.MathConstants' */
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_DBL_MIN = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_DBL_MIN (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_DBL_MIN)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_QL_EPSILON = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_QL_EPSILON (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_QL_EPSILON)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT2 = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_SQRT2 (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT2)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT_2 = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_SQRT_2 (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT_2)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRTPI = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_SQRTPI (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRTPI)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_1_SQRTPI = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_1_SQRTPI (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_1_SQRTPI)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_PI = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_PI (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_PI)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_NAN = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_NAN)

/* Module declarations from 'ultron.sentry.Math.Accumulators.StatefulAccumulators' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue = 0;
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_StatefulValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_Shift__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_Delta__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_SingleValuedValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_SortedValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMax__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingArgMax__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMin__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingArgMin__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingRank__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingQuantile__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAllTrue__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAnyTrue__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSum__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingDecay__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingPositiveAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingPositiveDifferenceAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeDifferenceAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingRSI__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingVariance__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingStandardDeviation__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeVariance__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCountedPositive__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCountedNegative__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCorrelation__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingProduct__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MACD__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingLogReturn__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSharp__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSortino__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingDrawdown__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMaxDrawdown__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingResidue__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "ultron.sentry.Math.Accumulators.StatefulAccumulators"
extern int __pyx_module_is_main_ultron__sentry__Math__Accumulators__StatefulAccumulators;
int __pyx_module_is_main_ultron__sentry__Math__Accumulators__StatefulAccumulators = 0;

/* Implementation of 'ultron.sentry.Math.Accumulators.StatefulAccumulators' */
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_lag[] = "lag";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_six[] = "six";
static const char __pyx_k_MACD[] = "MACD";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_push[] = "push";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_Delta[] = "Delta";
static const char __pyx_k_Shift[] = "Shift";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_bisect[] = "bisect";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_isFull[] = "isFull";
static const char __pyx_k_method[] = "method";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_result[] = "result";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_window[] = "window";
static const char __pyx_k_deepcopy[] = "deepcopy";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_long_win[] = "long_win";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_MovingMax[] = "MovingMax";
static const char __pyx_k_MovingMin[] = "MovingMin";
static const char __pyx_k_MovingRSI[] = "MovingRSI";
static const char __pyx_k_MovingSum[] = "MovingSum";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_short_win[] = "short_win";
static const char __pyx_k_MovingRank[] = "MovingRank";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_dependency[] = "dependency";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_MovingDecay[] = "MovingDecay";
static const char __pyx_k_MovingSharp[] = "MovingSharp";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_bisect_left[] = "bisect_left";
static const char __pyx_k_MovingArgMax[] = "MovingArgMax";
static const char __pyx_k_MovingArgMin[] = "MovingArgMin";
static const char __pyx_k_isPopulation[] = "isPopulation";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_MovingAllTrue[] = "MovingAllTrue";
static const char __pyx_k_MovingAnyTrue[] = "MovingAnyTrue";
static const char __pyx_k_MovingAverage[] = "MovingAverage";
static const char __pyx_k_MovingProduct[] = "MovingProduct";
static const char __pyx_k_MovingResidue[] = "MovingResidue";
static const char __pyx_k_MovingSortino[] = "MovingSortino";
static const char __pyx_k_mathrm_MA_0_1[] = "\\mathrm{{MA}}({0}, {1})";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_MovingDrawdown[] = "MovingDrawdown";
static const char __pyx_k_MovingQuantile[] = "MovingQuantile";
static const char __pyx_k_MovingVariance[] = "MovingVariance";
static const char __pyx_k_MovingLogReturn[] = "MovingLogReturn";
static const char __pyx_k_mathrm_MMax_0_1[] = "\\mathrm{{MMax}}({0}, {1})";
static const char __pyx_k_mathrm_MMin_0_1[] = "\\mathrm{{MMin}}({0}, {1})";
static const char __pyx_k_mathrm_MSum_0_1[] = "\\mathrm{{MSum}}({0}, {1})";
static const char __pyx_k_mathrm_mrsi_0_1[] = "\\mathrm{{mrsi}}({0}, {1})";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_mathrm_Delta_0_1[] = "\\mathrm{{Delta}}({0}, {1})";
static const char __pyx_k_mathrm_MRank_0_1[] = "\\mathrm{{MRank}}({0}, {1})";
static const char __pyx_k_mathrm_Res_0_1_2[] = "\\mathrm{{Res}}({0}, {1}, {2})";
static const char __pyx_k_mathrm_Shift_0_1[] = "\\mathrm{{Shift}}({0}, {1})";
static const char __pyx_k_mathrm_mprod_0_1[] = "\\mathrm{{mprod}}({0}, {1})";
static const char __pyx_k_MovingCorrelation[] = "MovingCorrelation";
static const char __pyx_k_MovingMaxDrawdown[] = "MovingMaxDrawdown";
static const char __pyx_k_SortedValueHolder[] = "SortedValueHolder";
static const char __pyx_k_mathrm_mstd_0_1_2[] = "\\mathrm{{mstd}}({0}, {1}, {2})";
static const char __pyx_k_mathrm_mvar_0_1_2[] = "\\mathrm{{mvar}}({0}, {1}, {2})";
static const char __pyx_k_pyx_unpickle_MACD[] = "__pyx_unpickle_MACD";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_mathrm_MADecay_0_1[] = "\\mathrm{{MADecay}}({0}, {1})";
static const char __pyx_k_mathrm_MArgMax_0_1[] = "\\mathrm{{MArgMax}}({0}, {1})";
static const char __pyx_k_mathrm_MArgMin_0_1[] = "\\mathrm{{MArgMin}}({0}, {1})";
static const char __pyx_k_mathrm_mcorr_0_1_2[] = "\\mathrm{{mcorr}}({0}, {1}, {2})";
static const char __pyx_k_mathrm_mnegavg_0_1[] = "\\mathrm{{mnegavg}}({0}, {1})";
static const char __pyx_k_mathrm_mposavg_0_1[] = "\\mathrm{{mposavg}}({0}, {1})";
static const char __pyx_k_pyx_unpickle_Delta[] = "__pyx_unpickle_Delta";
static const char __pyx_k_pyx_unpickle_Shift[] = "__pyx_unpickle_Shift";
static const char __pyx_k_StatefulValueHolder[] = "StatefulValueHolder";
static const char __pyx_k_mathrm_MAllTrue_0_1[] = "\\mathrm{{MAllTrue}}({0}, {1})";
static const char __pyx_k_mathrm_MAnyTrue_0_1[] = "\\mathrm{{MAnyTrue}}({0}, {1})";
static const char __pyx_k_mathrm_MQuantile_0_1[] = "\\mathrm{{MQuantile}}({0}, {1})";
static const char __pyx_k_mathrm_mnegcount_0_1[] = "\\mathrm{{mnegcount}}({0}, {1})";
static const char __pyx_k_mathrm_mnegvar_0_1_2[] = "\\mathrm{{mnegvar}}({0}, {1}, {2})";
static const char __pyx_k_mathrm_mposcount_0_1[] = "\\mathrm{{mposcount}}({0}, {1})";
static const char __pyx_k_MovingCountedNegative[] = "MovingCountedNegative";
static const char __pyx_k_MovingCountedPositive[] = "MovingCountedPositive";
static const char __pyx_k_MovingNegativeAverage[] = "MovingNegativeAverage";
static const char __pyx_k_MovingPositiveAverage[] = "MovingPositiveAverage";
static const char __pyx_k_MovingNegativeVariance[] = "MovingNegativeVariance";
static const char __pyx_k_mathrm_mnegdiffavg_0_1[] = "\\mathrm{{mnegdiffavg}}({0}, {1})";
static const char __pyx_k_mathrm_mposdiffavg_0_1[] = "\\mathrm{{mposdiffavg}}({0}, {1})";
static const char __pyx_k_pyx_unpickle_MovingMax[] = "__pyx_unpickle_MovingMax";
static const char __pyx_k_pyx_unpickle_MovingMin[] = "__pyx_unpickle_MovingMin";
static const char __pyx_k_pyx_unpickle_MovingRSI[] = "__pyx_unpickle_MovingRSI";
static const char __pyx_k_pyx_unpickle_MovingSum[] = "__pyx_unpickle_MovingSum";
static const char __pyx_k_MovingStandardDeviation[] = "MovingStandardDeviation";
static const char __pyx_k_SingleValuedValueHolder[] = "SingleValuedValueHolder";
static const char __pyx_k_pyx_unpickle_MovingRank[] = "__pyx_unpickle_MovingRank";
static const char __pyx_k_pyx_unpickle_MovingDecay[] = "__pyx_unpickle_MovingDecay";
static const char __pyx_k_pyx_unpickle_MovingSharp[] = "__pyx_unpickle_MovingSharp";
static const char __pyx_k_pyx_unpickle_MovingArgMax[] = "__pyx_unpickle_MovingArgMax";
static const char __pyx_k_pyx_unpickle_MovingArgMin[] = "__pyx_unpickle_MovingArgMin";
static const char __pyx_k_pyx_unpickle_MovingAllTrue[] = "__pyx_unpickle_MovingAllTrue";
static const char __pyx_k_pyx_unpickle_MovingAnyTrue[] = "__pyx_unpickle_MovingAnyTrue";
static const char __pyx_k_pyx_unpickle_MovingAverage[] = "__pyx_unpickle_MovingAverage";
static const char __pyx_k_pyx_unpickle_MovingProduct[] = "__pyx_unpickle_MovingProduct";
static const char __pyx_k_pyx_unpickle_MovingResidue[] = "__pyx_unpickle_MovingResidue";
static const char __pyx_k_pyx_unpickle_MovingSortino[] = "__pyx_unpickle_MovingSortino";
static const char __pyx_k_pyx_unpickle_MovingDrawdown[] = "__pyx_unpickle_MovingDrawdown";
static const char __pyx_k_pyx_unpickle_MovingQuantile[] = "__pyx_unpickle_MovingQuantile";
static const char __pyx_k_pyx_unpickle_MovingVariance[] = "__pyx_unpickle_MovingVariance";
static const char __pyx_k_pyx_unpickle_MovingLogReturn[] = "__pyx_unpickle_MovingLogReturn";
static const char __pyx_k_pyx_unpickle_MovingCorrelation[] = "__pyx_unpickle_MovingCorrelation";
static const char __pyx_k_pyx_unpickle_MovingCountedNega[] = "__pyx_unpickle_MovingCountedNegative";
static const char __pyx_k_pyx_unpickle_MovingCountedPosi[] = "__pyx_unpickle_MovingCountedPositive";
static const char __pyx_k_pyx_unpickle_MovingMaxDrawdown[] = "__pyx_unpickle_MovingMaxDrawdown";
static const char __pyx_k_pyx_unpickle_MovingNegativeAve[] = "__pyx_unpickle_MovingNegativeAverage";
static const char __pyx_k_pyx_unpickle_MovingNegativeDif[] = "__pyx_unpickle_MovingNegativeDifferenceAverage";
static const char __pyx_k_pyx_unpickle_MovingNegativeVar[] = "__pyx_unpickle_MovingNegativeVariance";
static const char __pyx_k_pyx_unpickle_MovingPositiveAve[] = "__pyx_unpickle_MovingPositiveAverage";
static const char __pyx_k_pyx_unpickle_MovingPositiveDif[] = "__pyx_unpickle_MovingPositiveDifferenceAverage";
static const char __pyx_k_pyx_unpickle_MovingStandardDev[] = "__pyx_unpickle_MovingStandardDeviation";
static const char __pyx_k_pyx_unpickle_SingleValuedValue[] = "__pyx_unpickle_SingleValuedValueHolder";
static const char __pyx_k_pyx_unpickle_SortedValueHolder[] = "__pyx_unpickle_SortedValueHolder";
static const char __pyx_k_pyx_unpickle_StatefulValueHold[] = "__pyx_unpickle_StatefulValueHolder";
static const char __pyx_k_MovingNegativeDifferenceAverage[] = "MovingNegativeDifferenceAverage";
static const char __pyx_k_MovingPositiveDifferenceAverage[] = "MovingPositiveDifferenceAverage";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_sampling_standard_deviation_can[] = "sampling standard deviation can't be calculated with window size < 2";
static const char __pyx_k_ultron_sentry_Math_Accumulators[] = "ultron.sentry.Math.Accumulators.StatefulAccumulators";
static const char __pyx_k_window_length_should_be_greater[] = "window length should be greater than 0";
static const char __pyx_k_Delta_window_value_should_not_be[] = "Delta window value should not be less than 1";
static const char __pyx_k_Incompatible_checksums_s_vs_0x17[] = "Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x1d[] = "Incompatible checksums (%s vs 0x1dd363e = (_currentMax, _dependency, _deque, _isFull, _maxer, _runningCum, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x21[] = "Incompatible checksums (%s vs 0x21341b2 = (_dependency, _deque, _isFull, _runningProduct, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x2e[] = "Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x34[] = "Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x40[] = "Incompatible checksums (%s vs 0x40162d2 = (_dependency, _deque, _deque_y, _isFull, _mean, _negativeVar, _window, _x, _y))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x54[] = "Incompatible checksums (%s vs 0x547d786 = (_dependency, _isFull, _long_average, _short_average, _window))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x62[] = "Incompatible checksums (%s vs 0x62254c8 = (_dependency, _deque, _isFull, _popout, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x78[] = "Incompatible checksums (%s vs 0x7871d50 = (_dependency, _deque, _isFull, _isPop, _runningNegativeCount, _runningNegativeSum, _runningNegativeSumSquare, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x7a[] = "Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x7c[] = "Incompatible checksums (%s vs 0x7c4e29b = (_dependency, _deque, _isFull, _runningPositiveCount, _runningPositiveSum, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x7d[] = "Incompatible checksums (%s vs 0x7d09828 = (_dependency, _deque, _isFull, _runningReturn, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x7e[] = "Incompatible checksums (%s vs 0x7ea5ed3 = (_dependency, _deque, _isFull, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x93[] = "Incompatible checksums (%s vs 0x93f2e8c = (_dependency, _deque, _isFull, _window))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x9b[] = "Incompatible checksums (%s vs 0x9b495dd = (_dependency, _deque, _isFull, _runningNegativeCount, _runningNegativeSum, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xa4[] = "Incompatible checksums (%s vs 0xa4dfc89 = (_dependency, _deque, _isFull, _negDiffAvg, _posDiffAvg, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xad[] = "Incompatible checksums (%s vs 0xadfc35c = (_cross, _dependency, _deque, _deque_y, _isFull, _lastx, _lasty, _window, _x, _xsquare, _y))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xb3[] = "Incompatible checksums (%s vs 0xb35140c = (_dependency, _deque, _isFull, _newestValue, _runningSum, _runningWeightedSum, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xb5[] = "Incompatible checksums (%s vs 0xb597db2 = (_current, _dependency, _deque, _isFull, _popout, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xba[] = "Incompatible checksums (%s vs 0xbacbe0f = (_dependency, _deque, _deque_y, _isFull, _runningSumCrossSquare, _runningSumLeft, _runningSumRight, _runningSumSquareLeft, _runningSumSquareRight, _window, _x, _y))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xd6[] = "Incompatible checksums (%s vs 0xd61dd72 = (_dependency, _deque, _deque_y, _isFull, _mean, _var, _window, _x, _y))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xef[] = "Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xfe[] = "Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_sampling_variance_can_t_be_calcu[] = "sampling variance can't be calculated with window size < 2";
static const char __pyx_k_shift_value_should_not_be_less_t[] = "shift value should not be less than 1";
static const char __pyx_k_window_parameter_should_be_a_pos[] = "window parameter should be a positive int however {0} received";
static const char __pyx_k_Incompatible_checksums_s_vs_0x7c_2[] = "Incompatible checksums (%s vs 0x7c30e67 = (_dependency, _deque, _drawdownCalculator, _isFull, _minimer, _window, _x))";
static PyObject *__pyx_n_s_Delta;
static PyObject *__pyx_kp_s_Delta_window_value_should_not_be;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x17;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x1d;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x21;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x2e;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x34;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x40;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x54;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x62;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x78;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x7a;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x7c;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x7c_2;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x7d;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x7e;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x93;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x9b;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xa4;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xad;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xb3;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xb5;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xba;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xd6;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xef;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xfe;
static PyObject *__pyx_n_s_MACD;
static PyObject *__pyx_n_s_MovingAllTrue;
static PyObject *__pyx_n_s_MovingAnyTrue;
static PyObject *__pyx_n_s_MovingArgMax;
static PyObject *__pyx_n_s_MovingArgMin;
static PyObject *__pyx_n_s_MovingAverage;
static PyObject *__pyx_n_s_MovingCorrelation;
static PyObject *__pyx_n_s_MovingCountedNegative;
static PyObject *__pyx_n_s_MovingCountedPositive;
static PyObject *__pyx_n_s_MovingDecay;
static PyObject *__pyx_n_s_MovingDrawdown;
static PyObject *__pyx_n_s_MovingLogReturn;
static PyObject *__pyx_n_s_MovingMax;
static PyObject *__pyx_n_s_MovingMaxDrawdown;
static PyObject *__pyx_n_s_MovingMin;
static PyObject *__pyx_n_s_MovingNegativeAverage;
static PyObject *__pyx_n_s_MovingNegativeDifferenceAverage;
static PyObject *__pyx_n_s_MovingNegativeVariance;
static PyObject *__pyx_n_s_MovingPositiveAverage;
static PyObject *__pyx_n_s_MovingPositiveDifferenceAverage;
static PyObject *__pyx_n_s_MovingProduct;
static PyObject *__pyx_n_s_MovingQuantile;
static PyObject *__pyx_n_s_MovingRSI;
static PyObject *__pyx_n_s_MovingRank;
static PyObject *__pyx_n_s_MovingResidue;
static PyObject *__pyx_n_s_MovingSharp;
static PyObject *__pyx_n_s_MovingSortino;
static PyObject *__pyx_n_s_MovingStandardDeviation;
static PyObject *__pyx_n_s_MovingSum;
static PyObject *__pyx_n_s_MovingVariance;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_Shift;
static PyObject *__pyx_n_s_SingleValuedValueHolder;
static PyObject *__pyx_n_s_SortedValueHolder;
static PyObject *__pyx_n_s_StatefulValueHolder;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_bisect;
static PyObject *__pyx_n_s_bisect_left;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_deepcopy;
static PyObject *__pyx_n_s_dependency;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_isFull;
static PyObject *__pyx_n_s_isPopulation;
static PyObject *__pyx_n_s_lag;
static PyObject *__pyx_n_s_long_win;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_kp_s_mathrm_Delta_0_1;
static PyObject *__pyx_kp_s_mathrm_MADecay_0_1;
static PyObject *__pyx_kp_s_mathrm_MA_0_1;
static PyObject *__pyx_kp_s_mathrm_MAllTrue_0_1;
static PyObject *__pyx_kp_s_mathrm_MAnyTrue_0_1;
static PyObject *__pyx_kp_s_mathrm_MArgMax_0_1;
static PyObject *__pyx_kp_s_mathrm_MArgMin_0_1;
static PyObject *__pyx_kp_s_mathrm_MMax_0_1;
static PyObject *__pyx_kp_s_mathrm_MMin_0_1;
static PyObject *__pyx_kp_s_mathrm_MQuantile_0_1;
static PyObject *__pyx_kp_s_mathrm_MRank_0_1;
static PyObject *__pyx_kp_s_mathrm_MSum_0_1;
static PyObject *__pyx_kp_s_mathrm_Res_0_1_2;
static PyObject *__pyx_kp_s_mathrm_Shift_0_1;
static PyObject *__pyx_kp_s_mathrm_mcorr_0_1_2;
static PyObject *__pyx_kp_s_mathrm_mnegavg_0_1;
static PyObject *__pyx_kp_s_mathrm_mnegcount_0_1;
static PyObject *__pyx_kp_s_mathrm_mnegdiffavg_0_1;
static PyObject *__pyx_kp_s_mathrm_mnegvar_0_1_2;
static PyObject *__pyx_kp_s_mathrm_mposavg_0_1;
static PyObject *__pyx_kp_s_mathrm_mposcount_0_1;
static PyObject *__pyx_kp_s_mathrm_mposdiffavg_0_1;
static PyObject *__pyx_kp_s_mathrm_mprod_0_1;
static PyObject *__pyx_kp_s_mathrm_mrsi_0_1;
static PyObject *__pyx_kp_s_mathrm_mstd_0_1_2;
static PyObject *__pyx_kp_s_mathrm_mvar_0_1_2;
static PyObject *__pyx_n_s_method;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_s_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_s_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_push;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_Delta;
static PyObject *__pyx_n_s_pyx_unpickle_MACD;
static PyObject *__pyx_n_s_pyx_unpickle_MovingAllTrue;
static PyObject *__pyx_n_s_pyx_unpickle_MovingAnyTrue;
static PyObject *__pyx_n_s_pyx_unpickle_MovingArgMax;
static PyObject *__pyx_n_s_pyx_unpickle_MovingArgMin;
static PyObject *__pyx_n_s_pyx_unpickle_MovingAverage;
static PyObject *__pyx_n_s_pyx_unpickle_MovingCorrelation;
static PyObject *__pyx_n_s_pyx_unpickle_MovingCountedNega;
static PyObject *__pyx_n_s_pyx_unpickle_MovingCountedPosi;
static PyObject *__pyx_n_s_pyx_unpickle_MovingDecay;
static PyObject *__pyx_n_s_pyx_unpickle_MovingDrawdown;
static PyObject *__pyx_n_s_pyx_unpickle_MovingLogReturn;
static PyObject *__pyx_n_s_pyx_unpickle_MovingMax;
static PyObject *__pyx_n_s_pyx_unpickle_MovingMaxDrawdown;
static PyObject *__pyx_n_s_pyx_unpickle_MovingMin;
static PyObject *__pyx_n_s_pyx_unpickle_MovingNegativeAve;
static PyObject *__pyx_n_s_pyx_unpickle_MovingNegativeDif;
static PyObject *__pyx_n_s_pyx_unpickle_MovingNegativeVar;
static PyObject *__pyx_n_s_pyx_unpickle_MovingPositiveAve;
static PyObject *__pyx_n_s_pyx_unpickle_MovingPositiveDif;
static PyObject *__pyx_n_s_pyx_unpickle_MovingProduct;
static PyObject *__pyx_n_s_pyx_unpickle_MovingQuantile;
static PyObject *__pyx_n_s_pyx_unpickle_MovingRSI;
static PyObject *__pyx_n_s_pyx_unpickle_MovingRank;
static PyObject *__pyx_n_s_pyx_unpickle_MovingResidue;
static PyObject *__pyx_n_s_pyx_unpickle_MovingSharp;
static PyObject *__pyx_n_s_pyx_unpickle_MovingSortino;
static PyObject *__pyx_n_s_pyx_unpickle_MovingStandardDev;
static PyObject *__pyx_n_s_pyx_unpickle_MovingSum;
static PyObject *__pyx_n_s_pyx_unpickle_MovingVariance;
static PyObject *__pyx_n_s_pyx_unpickle_Shift;
static PyObject *__pyx_n_s_pyx_unpickle_SingleValuedValue;
static PyObject *__pyx_n_s_pyx_unpickle_SortedValueHolder;
static PyObject *__pyx_n_s_pyx_unpickle_StatefulValueHold;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_result;
static PyObject *__pyx_kp_s_sampling_standard_deviation_can;
static PyObject *__pyx_kp_s_sampling_variance_can_t_be_calcu;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_kp_s_shift_value_should_not_be_less_t;
static PyObject *__pyx_n_s_short_win;
static PyObject *__pyx_n_s_six;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_ultron_sentry_Math_Accumulators;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_window;
static PyObject *__pyx_kp_s_window_length_should_be_greater;
static PyObject *__pyx_kp_s_window_parameter_should_be_a_pos;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_y;
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, PyObject *__pyx_v_window); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_2size(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_4isFull(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_6lag(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_8__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_6lag(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_8__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_4__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_4__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_6__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_isPopulation); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_isPopulation); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_isPopulation); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_short_win, PyObject *__pyx_v_long_win, PyObject *__pyx_v_x, PyObject *__pyx_v_method); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_4isFull(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_StatefulValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_2__pyx_unpickle_Shift(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4__pyx_unpickle_Delta(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_6__pyx_unpickle_SingleValuedValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_8__pyx_unpickle_SortedValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10__pyx_unpickle_MovingMax(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12__pyx_unpickle_MovingArgMax(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14__pyx_unpickle_MovingMin(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_16__pyx_unpickle_MovingArgMin(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_18__pyx_unpickle_MovingRank(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_20__pyx_unpickle_MovingQuantile(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22__pyx_unpickle_MovingAllTrue(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_24__pyx_unpickle_MovingAnyTrue(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_26__pyx_unpickle_MovingSum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_28__pyx_unpickle_MovingAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_30__pyx_unpickle_MovingDecay(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_32__pyx_unpickle_MovingPositiveAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_34__pyx_unpickle_MovingPositiveDifferenceAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_36__pyx_unpickle_MovingNegativeDifferenceAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_38__pyx_unpickle_MovingRSI(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_40__pyx_unpickle_MovingNegativeAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_42__pyx_unpickle_MovingVariance(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_44__pyx_unpickle_MovingStandardDeviation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_46__pyx_unpickle_MovingNegativeVariance(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_48__pyx_unpickle_MovingCountedPositive(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_50__pyx_unpickle_MovingCountedNegative(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_52__pyx_unpickle_MovingCorrelation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_54__pyx_unpickle_MovingProduct(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_56__pyx_unpickle_MACD(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_58__pyx_unpickle_MovingLogReturn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_60__pyx_unpickle_MovingSharp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_62__pyx_unpickle_MovingSortino(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_64__pyx_unpickle_MovingDrawdown(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_66__pyx_unpickle_MovingMaxDrawdown(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_68__pyx_unpickle_MovingResidue(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_0_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_24234567;
static PyObject *__pyx_int_31274558;
static PyObject *__pyx_int_34816434;
static PyObject *__pyx_int_48560527;
static PyObject *__pyx_int_55178044;
static PyObject *__pyx_int_67199698;
static PyObject *__pyx_int_88594310;
static PyObject *__pyx_int_102913224;
static PyObject *__pyx_int_126295376;
static PyObject *__pyx_int_128252030;
static PyObject *__pyx_int_130223719;
static PyObject *__pyx_int_130343579;
static PyObject *__pyx_int_131110952;
static PyObject *__pyx_int_132800211;
static PyObject *__pyx_int_155135628;
static PyObject *__pyx_int_162829789;
static PyObject *__pyx_int_172883081;
static PyObject *__pyx_int_182436700;
static PyObject *__pyx_int_188027916;
static PyObject *__pyx_int_190414258;
static PyObject *__pyx_int_195870223;
static PyObject *__pyx_int_224517490;
static PyObject *__pyx_int_251250905;
static PyObject *__pyx_int_266717091;
static PyObject *__pyx_k_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_codeobj__5;
static PyObject *__pyx_codeobj__7;
static PyObject *__pyx_codeobj__9;
static PyObject *__pyx_codeobj__11;
static PyObject *__pyx_codeobj__13;
static PyObject *__pyx_codeobj__15;
static PyObject *__pyx_codeobj__17;
static PyObject *__pyx_codeobj__19;
static PyObject *__pyx_codeobj__21;
static PyObject *__pyx_codeobj__23;
static PyObject *__pyx_codeobj__25;
static PyObject *__pyx_codeobj__27;
static PyObject *__pyx_codeobj__29;
static PyObject *__pyx_codeobj__31;
static PyObject *__pyx_codeobj__33;
static PyObject *__pyx_codeobj__35;
static PyObject *__pyx_codeobj__37;
static PyObject *__pyx_codeobj__39;
static PyObject *__pyx_codeobj__41;
static PyObject *__pyx_codeobj__43;
static PyObject *__pyx_codeobj__45;
static PyObject *__pyx_codeobj__47;
static PyObject *__pyx_codeobj__49;
static PyObject *__pyx_codeobj__51;
static PyObject *__pyx_codeobj__53;
static PyObject *__pyx_codeobj__55;
static PyObject *__pyx_codeobj__57;
static PyObject *__pyx_codeobj__59;
static PyObject *__pyx_codeobj__61;
static PyObject *__pyx_codeobj__63;
static PyObject *__pyx_codeobj__65;
static PyObject *__pyx_codeobj__67;
static PyObject *__pyx_codeobj__69;
static PyObject *__pyx_codeobj__71;
static PyObject *__pyx_codeobj__73;
/* Late includes */

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":30
 * cdef class StatefulValueHolder(Accumulator):
 * 
 *     def __init__(self, window):             # <<<<<<<<<<<<<<
 *         super(StatefulValueHolder, self).__init__()
 *         if not isinstance(window, int):
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 30, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_window = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 30, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self), __pyx_v_window);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, PyObject *__pyx_v_window) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":31
 * 
 *     def __init__(self, window):
 *         super(StatefulValueHolder, self).__init__()             # <<<<<<<<<<<<<<
 *         if not isinstance(window, int):
 *             raise ValueError("window parameter should be a positive int however {0} received"
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":32
 *     def __init__(self, window):
 *         super(StatefulValueHolder, self).__init__()
 *         if not isinstance(window, int):             # <<<<<<<<<<<<<<
 *             raise ValueError("window parameter should be a positive int however {0} received"
 *                              .format(window))
 */
  __pyx_t_4 = PyInt_Check(__pyx_v_window); 
  __pyx_t_5 = ((!(__pyx_t_4 != 0)) != 0);
  if (unlikely(__pyx_t_5)) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":34
 *         if not isinstance(window, int):
 *             raise ValueError("window parameter should be a positive int however {0} received"
 *                              .format(window))             # <<<<<<<<<<<<<<
 *         pyFinAssert(window > 0, ValueError, "window length should be greater than 0")
 *         self._deque = Deque(window)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_window_parameter_should_be_a_pos, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":33
 *         super(StatefulValueHolder, self).__init__()
 *         if not isinstance(window, int):
 *             raise ValueError("window parameter should be a positive int however {0} received"             # <<<<<<<<<<<<<<
 *                              .format(window))
 *         pyFinAssert(window > 0, ValueError, "window length should be greater than 0")
 */
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 33, __pyx_L1_error)

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":32
 *     def __init__(self, window):
 *         super(StatefulValueHolder, self).__init__()
 *         if not isinstance(window, int):             # <<<<<<<<<<<<<<
 *             raise ValueError("window parameter should be a positive int however {0} received"
 *                              .format(window))
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":35
 *             raise ValueError("window parameter should be a positive int however {0} received"
 *                              .format(window))
 *         pyFinAssert(window > 0, ValueError, "window length should be greater than 0")             # <<<<<<<<<<<<<<
 *         self._deque = Deque(window)
 *         self._isFull = False
 */
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_window, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 35, __pyx_L1_error)
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.msg = __pyx_kp_s_window_length_should_be_greater;
  __pyx_t_6 = __pyx_f_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert(__pyx_t_2, __pyx_builtin_ValueError, 0, &__pyx_t_7); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":36
 *                              .format(window))
 *         pyFinAssert(window > 0, ValueError, "window length should be greater than 0")
 *         self._deque = Deque(window)             # <<<<<<<<<<<<<<
 *         self._isFull = False
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque), __pyx_v_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_deque);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_deque));
  __pyx_v_self->_deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":37
 *         pyFinAssert(window > 0, ValueError, "window length should be greater than 0")
 *         self._deque = Deque(window)
 *         self._isFull = False             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t size(self):
 */
  __pyx_v_self->__pyx_base._isFull = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":30
 * cdef class StatefulValueHolder(Accumulator):
 * 
 *     def __init__(self, window):             # <<<<<<<<<<<<<<
 *         super(StatefulValueHolder, self).__init__()
 *         if not isinstance(window, int):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":39
 *         self._isFull = False
 * 
 *     cpdef size_t size(self):             # <<<<<<<<<<<<<<
 *         return self._deque.size()
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_3size(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static size_t __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_size(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("size", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_3size)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 39, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":40
 * 
 *     cpdef size_t size(self):
 *         return self._deque.size()             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint isFull(self):
 */
  __pyx_r = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->_deque->__pyx_vtab)->size(__pyx_v_self->_deque);
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":39
 *         self._isFull = False
 * 
 *     cpdef size_t size(self):             # <<<<<<<<<<<<<<
 *         return self._deque.size()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder.size", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_3size(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_2size[] = "StatefulValueHolder.size(self) -> size_t";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_3size(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("size (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_2size(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_2size(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("size", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_size(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder.size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":42
 *         return self._deque.size()
 * 
 *     cpdef bint isFull(self):             # <<<<<<<<<<<<<<
 *         return self._isFull
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_5isFull(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_isFull(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isFull", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_5isFull)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 42, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 42, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":43
 * 
 *     cpdef bint isFull(self):
 *         return self._isFull             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_self->__pyx_base._isFull;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":42
 *         return self._deque.size()
 * 
 *     cpdef bint isFull(self):             # <<<<<<<<<<<<<<
 *         return self._isFull
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder.isFull", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_5isFull(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_4isFull[] = "StatefulValueHolder.isFull(self) -> bool";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_5isFull(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isFull (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_4isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_4isFull(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isFull", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_isFull(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder.isFull", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":10
 * cdef class StatefulValueHolder(Accumulator):
 * 
 *     cdef public Deque _deque             # <<<<<<<<<<<<<<
 * 
 *     cpdef size_t size(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_deque));
  __pyx_r = ((PyObject *)__pyx_v_self->_deque);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_deque);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_deque));
  __pyx_v_self->_deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder._deque.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_deque);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_deque));
  __pyx_v_self->_deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6__reduce_cython__[] = "StatefulValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._window)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_deque));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->_deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._window)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._window)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._window)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_deque) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None
 *     if use_setstate:
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_StatefulValueHold); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_155135628);
    __Pyx_GIVEREF(__pyx_int_155135628);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_155135628);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, None), state
 *     else:
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StatefulValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_StatefulValueHold); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_155135628);
    __Pyx_GIVEREF(__pyx_int_155135628);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_155135628);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StatefulValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_8__setstate_cython__[] = "StatefulValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StatefulValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_StatefulValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StatefulValueHolder, (type(self), 0x93f2e8c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StatefulValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":48
 * cdef class Shift(StatefulValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(Shift, self).__init__(window)
 *         pyFinAssert(window >= 1, ValueError, "shift value should not be less than 1")
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 48, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 48, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 48, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert __pyx_t_5;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":49
 * 
 *     def __init__(self, window, x):
 *         super(Shift, self).__init__(window)             # <<<<<<<<<<<<<<
 *         pyFinAssert(window >= 1, ValueError, "shift value should not be less than 1")
 *         self._x = build_holder(x)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":50
 *     def __init__(self, window, x):
 *         super(Shift, self).__init__(window)
 *         pyFinAssert(window >= 1, ValueError, "shift value should not be less than 1")             # <<<<<<<<<<<<<<
 *         self._x = build_holder(x)
 *         self._window = self._x.window + window
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_window, __pyx_int_1, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 50, __pyx_L1_error)
  __pyx_t_5.__pyx_n = 1;
  __pyx_t_5.msg = __pyx_kp_s_shift_value_should_not_be_less_t;
  __pyx_t_4 = __pyx_f_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert(__pyx_t_1, __pyx_builtin_ValueError, 0, &__pyx_t_5); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":51
 *         super(Shift, self).__init__(window)
 *         pyFinAssert(window >= 1, ValueError, "shift value should not be less than 1")
 *         self._x = build_holder(x)             # <<<<<<<<<<<<<<
 *         self._window = self._x.window + window
 *         self._dependency = deepcopy(self._x.dependency)
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":52
 *         pyFinAssert(window >= 1, ValueError, "shift value should not be less than 1")
 *         self._x = build_holder(x)
 *         self._window = self._x.window + window             # <<<<<<<<<<<<<<
 *         self._dependency = deepcopy(self._x.dependency)
 *         self._popout = NAN
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->__pyx_base.__pyx_base._window = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":53
 *         self._x = build_holder(x)
 *         self._window = self._x.window + window
 *         self._dependency = deepcopy(self._x.dependency)             # <<<<<<<<<<<<<<
 *         self._popout = NAN
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_dependency); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_7, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __pyx_v_self->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":54
 *         self._window = self._x.window + window
 *         self._dependency = deepcopy(self._x.dependency)
 *         self._popout = NAN             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_popout = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":48
 * cdef class Shift(StatefulValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(Shift, self).__init__(window)
 *         pyFinAssert(window >= 1, ValueError, "shift value should not be less than 1")
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":56
 *         self._popout = NAN
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._x.push(data)
 *         self._popout = self._deque.dump(self._x.result())
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":57
 * 
 *     cpdef push(self, dict data):
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         self._popout = self._deque.dump(self._x.result())
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->push(__pyx_v_self->_x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":58
 *     cpdef push(self, dict data):
 *         self._x.push(data)
 *         self._popout = self._deque.dump(self._x.result())             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  __pyx_v_self->_popout = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base._deque, ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->result(__pyx_v_self->_x, 0), NULL);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":59
 *         self._x.push(data)
 *         self._popout = self._deque.dump(self._x.result())
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":56
 *         self._popout = NAN
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._x.push(data)
 *         self._popout = self._deque.dump(self._x.result())
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2push[] = "Shift.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 56, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":61
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._popout
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 61, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 61, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":62
 * 
 *     cpdef double result(self):
 *         return self._popout             # <<<<<<<<<<<<<<
 * 
 *     cpdef int lag(self):
 */
  __pyx_r = __pyx_v_self->_popout;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":61
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._popout
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_4result[] = "Shift.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":64
 *         return self._popout
 * 
 *     cpdef int lag(self):             # <<<<<<<<<<<<<<
 *         return self._window - self._x.window
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7lag(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_lag(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lag", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_lag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7lag)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 64, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 64, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":65
 * 
 *     cpdef int lag(self):
 *         return self._window - self._x.window             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":64
 *         return self._popout
 * 
 *     cpdef int lag(self):             # <<<<<<<<<<<<<<
 *         return self._window - self._x.window
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.lag", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7lag(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_6lag[] = "Shift.lag(self) -> int";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7lag(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lag (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_6lag(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_6lag(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lag", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_lag(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.lag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":67
 *         return self._window - self._x.window
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '\mathrm{{Shift}}({0}, {1})'.format(str(self._x), self._window - self._x.window)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_8__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_8__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":68
 * 
 *     def __str__(self):
 *         return '\mathrm{{Shift}}({0}, {1})'.format(str(self._x), self._window - self._x.window)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_Shift_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->_x)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyNumber_Subtract(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 68, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_3 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":67
 *         return self._window - self._x.window
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '\mathrm{{Shift}}({0}, {1})'.format(str(self._x), self._window - self._x.window)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":18
 * cdef class Shift(StatefulValueHolder):
 * 
 *     cdef public double _popout             # <<<<<<<<<<<<<<
 *     cdef public Accumulator _x
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_popout); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift._popout.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 18, __pyx_L1_error)
  __pyx_v_self->_popout = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift._popout.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":19
 * 
 *     cdef public double _popout
 *     cdef public Accumulator _x             # <<<<<<<<<<<<<<
 * 
 *     cpdef int lag(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __pyx_r = ((PyObject *)__pyx_v_self->_x);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 19, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift._x.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_10__reduce_cython__[] = "Shift.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._popout, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_popout); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_x));
  PyTuple_SET_ITEM(__pyx_t_4, 5, ((PyObject *)__pyx_v_self->_x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._popout, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._popout, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._popout, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->_x) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_Shift); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_102913224);
    __Pyx_GIVEREF(__pyx_int_102913224);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_102913224);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, None), state
 *     else:
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Shift__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_Shift); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_102913224);
    __Pyx_GIVEREF(__pyx_int_102913224);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_102913224);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Shift__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_12__setstate_cython__[] = "Shift.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Shift__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_Shift__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Shift, (type(self), 0x62254c8, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Shift__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":73
 * cdef class Delta(StatefulValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(Delta, self).__init__(window)
 *         pyFinAssert(window >= 1, ValueError, "Delta window value should not be less than 1")
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 73, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 73, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 73, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert __pyx_t_5;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":74
 * 
 *     def __init__(self, window, x):
 *         super(Delta, self).__init__(window)             # <<<<<<<<<<<<<<
 *         pyFinAssert(window >= 1, ValueError, "Delta window value should not be less than 1")
 *         self._x = build_holder(x)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":75
 *     def __init__(self, window, x):
 *         super(Delta, self).__init__(window)
 *         pyFinAssert(window >= 1, ValueError, "Delta window value should not be less than 1")             # <<<<<<<<<<<<<<
 *         self._x = build_holder(x)
 *         self._window = self._x.window + window
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_window, __pyx_int_1, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
  __pyx_t_5.__pyx_n = 1;
  __pyx_t_5.msg = __pyx_kp_s_Delta_window_value_should_not_be;
  __pyx_t_4 = __pyx_f_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert(__pyx_t_1, __pyx_builtin_ValueError, 0, &__pyx_t_5); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":76
 *         super(Delta, self).__init__(window)
 *         pyFinAssert(window >= 1, ValueError, "Delta window value should not be less than 1")
 *         self._x = build_holder(x)             # <<<<<<<<<<<<<<
 *         self._window = self._x.window + window
 *         self._dependency = deepcopy(self._x.dependency)
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":77
 *         pyFinAssert(window >= 1, ValueError, "Delta window value should not be less than 1")
 *         self._x = build_holder(x)
 *         self._window = self._x.window + window             # <<<<<<<<<<<<<<
 *         self._dependency = deepcopy(self._x.dependency)
 *         self._popout = NAN
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->__pyx_base.__pyx_base._window = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":78
 *         self._x = build_holder(x)
 *         self._window = self._x.window + window
 *         self._dependency = deepcopy(self._x.dependency)             # <<<<<<<<<<<<<<
 *         self._popout = NAN
 *         self._current = NAN
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_dependency); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_7, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __pyx_v_self->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":79
 *         self._window = self._x.window + window
 *         self._dependency = deepcopy(self._x.dependency)
 *         self._popout = NAN             # <<<<<<<<<<<<<<
 *         self._current = NAN
 * 
 */
  __pyx_v_self->_popout = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":80
 *         self._dependency = deepcopy(self._x.dependency)
 *         self._popout = NAN
 *         self._current = NAN             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_current = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":73
 * cdef class Delta(StatefulValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(Delta, self).__init__(window)
 *         pyFinAssert(window >= 1, ValueError, "Delta window value should not be less than 1")
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":82
 *         self._current = NAN
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._x.push(data)
 *         self._current = self._x.result()
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 82, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 82, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":83
 * 
 *     cpdef push(self, dict data):
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         self._current = self._x.result()
 *         self._popout = self._deque.dump(self._current)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->push(__pyx_v_self->_x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":84
 *     cpdef push(self, dict data):
 *         self._x.push(data)
 *         self._current = self._x.result()             # <<<<<<<<<<<<<<
 *         self._popout = self._deque.dump(self._current)
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_v_self->_current = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->result(__pyx_v_self->_x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":85
 *         self._x.push(data)
 *         self._current = self._x.result()
 *         self._popout = self._deque.dump(self._current)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  __pyx_v_self->_popout = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base._deque, __pyx_v_self->_current, NULL);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":86
 *         self._current = self._x.result()
 *         self._popout = self._deque.dump(self._current)
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":82
 *         self._current = NAN
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._x.push(data)
 *         self._current = self._x.result()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2push[] = "Delta.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 82, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":88
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._isFull:
 *             return self._current - self._popout
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 88, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 88, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":89
 * 
 *     cpdef double result(self):
 *         if self._isFull:             # <<<<<<<<<<<<<<
 *             return self._current - self._popout
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._isFull != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":90
 *     cpdef double result(self):
 *         if self._isFull:
 *             return self._current - self._popout             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    __pyx_r = (__pyx_v_self->_current - __pyx_v_self->_popout);
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":89
 * 
 *     cpdef double result(self):
 *         if self._isFull:             # <<<<<<<<<<<<<<
 *             return self._current - self._popout
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":92
 *             return self._current - self._popout
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *     cpdef int lag(self):
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":88
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._isFull:
 *             return self._current - self._popout
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_4result[] = "Delta.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":94
 *             return NAN
 * 
 *     cpdef int lag(self):             # <<<<<<<<<<<<<<
 *         return self._window - self._x.window
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7lag(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_lag(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lag", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_lag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7lag)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 94, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":95
 * 
 *     cpdef int lag(self):
 *         return self._window - self._x.window             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":94
 *             return NAN
 * 
 *     cpdef int lag(self):             # <<<<<<<<<<<<<<
 *         return self._window - self._x.window
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.lag", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7lag(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_6lag[] = "Delta.lag(self) -> int";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7lag(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lag (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_6lag(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_6lag(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lag", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_lag(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.lag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":97
 *         return self._window - self._x.window
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '\mathrm{{Delta}}({0}, {1})'.format(str(self._x), self._window - self._x.window)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_8__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_8__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":98
 * 
 *     def __str__(self):
 *         return '\mathrm{{Delta}}({0}, {1})'.format(str(self._x), self._window - self._x.window)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_Delta_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->_x)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyNumber_Subtract(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 98, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_3 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":97
 *         return self._window - self._x.window
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '\mathrm{{Delta}}({0}, {1})'.format(str(self._x), self._window - self._x.window)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":28
 * cdef class Delta(StatefulValueHolder):
 * 
 *     cdef public double _popout             # <<<<<<<<<<<<<<
 *     cdef double _current
 *     cdef public Accumulator _x
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_popout); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta._popout.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 28, __pyx_L1_error)
  __pyx_v_self->_popout = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta._popout.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":30
 *     cdef public double _popout
 *     cdef double _current
 *     cdef public Accumulator _x             # <<<<<<<<<<<<<<
 * 
 *     cpdef int lag(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __pyx_r = ((PyObject *)__pyx_v_self->_x);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 30, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta._x.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_10__reduce_cython__[] = "Delta.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._current, self._dependency, self._deque, self._isFull, self._popout, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_current); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_popout); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(7); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_5, 2, ((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_x));
  PyTuple_SET_ITEM(__pyx_t_5, 6, ((PyObject *)__pyx_v_self->_x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._current, self._dependency, self._deque, self._isFull, self._popout, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_5 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v__dict = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "(tree fragment)":7
 *     state = (self._current, self._dependency, self._deque, self._isFull, self._popout, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._current, self._dependency, self._deque, self._isFull, self._popout, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, None), state
 */
  /*else*/ {
    __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (((PyObject *)__pyx_v_self->__pyx_base._deque) != Py_None);
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_x) != Py_None);
    __pyx_t_8 = (__pyx_t_6 != 0);
    __pyx_t_7 = __pyx_t_8;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_7;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, None), state
 *     else:
 */
  __pyx_t_7 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_Delta); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_190414258);
    __Pyx_GIVEREF(__pyx_int_190414258);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_190414258);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, None), state
 *     else:
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Delta__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_Delta); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_190414258);
    __Pyx_GIVEREF(__pyx_int_190414258);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_190414258);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Delta__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_12__setstate_cython__[] = "Delta.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Delta__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_Delta__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Delta, (type(self), 0xb597db2, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Delta__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":102
 * 
 * cdef class SingleValuedValueHolder(StatefulValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(SingleValuedValueHolder, self).__init__(window)
 *         self._x = build_holder(x)
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 102, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 102, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 102, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SingleValuedValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":103
 * cdef class SingleValuedValueHolder(StatefulValueHolder):
 *     def __init__(self, window, x):
 *         super(SingleValuedValueHolder, self).__init__(window)             # <<<<<<<<<<<<<<
 *         self._x = build_holder(x)
 *         self._window = self._x.window + window
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":104
 *     def __init__(self, window, x):
 *         super(SingleValuedValueHolder, self).__init__(window)
 *         self._x = build_holder(x)             # <<<<<<<<<<<<<<
 *         self._window = self._x.window + window
 *         self._dependency = deepcopy(self._x.dependency)
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":105
 *         super(SingleValuedValueHolder, self).__init__(window)
 *         self._x = build_holder(x)
 *         self._window = self._x.window + window             # <<<<<<<<<<<<<<
 *         self._dependency = deepcopy(self._x.dependency)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->__pyx_base.__pyx_base._window = __pyx_t_4;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":106
 *         self._x = build_holder(x)
 *         self._window = self._x.window + window
 *         self._dependency = deepcopy(self._x.dependency)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_dependency); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __pyx_v_self->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":102
 * 
 * cdef class SingleValuedValueHolder(StatefulValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(SingleValuedValueHolder, self).__init__(window)
 *         self._x = build_holder(x)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SingleValuedValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":39
 * cdef class SingleValuedValueHolder(StatefulValueHolder):
 * 
 *     cdef public Accumulator _x             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __pyx_r = ((PyObject *)__pyx_v_self->_x);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 39, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SingleValuedValueHolder._x.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2__reduce_cython__[] = "SingleValuedValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(5); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_x));
  PyTuple_SET_ITEM(__pyx_t_3, 4, ((PyObject *)__pyx_v_self->_x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._deque) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_x) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    __pyx_t_5 = __pyx_t_6;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_SingleValuedValue); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_132800211);
    __Pyx_GIVEREF(__pyx_int_132800211);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_132800211);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, None), state
 *     else:
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SingleValuedValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_SingleValuedValue); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_132800211);
    __Pyx_GIVEREF(__pyx_int_132800211);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_132800211);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SingleValuedValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SingleValuedValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_4__setstate_cython__[] = "SingleValuedValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_4__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_4__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SingleValuedValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_SingleValuedValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SingleValuedValueHolder, (type(self), 0x7ea5ed3, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SingleValuedValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SingleValuedValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":111
 * cdef class SortedValueHolder(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(SortedValueHolder, self).__init__(window, x)
 *         self._sortedArray = []
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 111, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 111, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 111, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SortedValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":112
 * 
 *     def __init__(self, window, x):
 *         super(SortedValueHolder, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._sortedArray = []
 *         self._cur_pos = NAN
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":113
 *     def __init__(self, window, x):
 *         super(SortedValueHolder, self).__init__(window, x)
 *         self._sortedArray = []             # <<<<<<<<<<<<<<
 *         self._cur_pos = NAN
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_sortedArray);
  __Pyx_DECREF(__pyx_v_self->_sortedArray);
  __pyx_v_self->_sortedArray = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":114
 *         super(SortedValueHolder, self).__init__(window, x)
 *         self._sortedArray = []
 *         self._cur_pos = NAN             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_cur_pos = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":111
 * cdef class SortedValueHolder(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(SortedValueHolder, self).__init__(window, x)
 *         self._sortedArray = []
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SortedValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":116
 *         self._cur_pos = NAN
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         cdef int delPos
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  int __pyx_v_delPos;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  double __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":120
 *         cdef int delPos
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 *         if isnan(value):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":121
 * 
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 *         if isnan(value):
 *             return NAN
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":122
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         if self._deque.isFull():
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":123
 *         cdef double value = self._x.result()
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         if self._deque.isFull():
 *             popout = self._deque.dump(value)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":122
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         if self._deque.isFull():
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":124
 *         if isnan(value):
 *             return NAN
 *         if self._deque.isFull():             # <<<<<<<<<<<<<<
 *             popout = self._deque.dump(value)
 *             delPos = bisect.bisect_left(self._sortedArray, popout)
 */
  __pyx_t_5 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":125
 *             return NAN
 *         if self._deque.isFull():
 *             popout = self._deque.dump(value)             # <<<<<<<<<<<<<<
 *             delPos = bisect.bisect_left(self._sortedArray, popout)
 *             del self._sortedArray[delPos]
 */
    __pyx_v_popout = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, NULL);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":126
 *         if self._deque.isFull():
 *             popout = self._deque.dump(value)
 *             delPos = bisect.bisect_left(self._sortedArray, popout)             # <<<<<<<<<<<<<<
 *             del self._sortedArray[delPos]
 *             self._cur_pos = bisect.bisect_left(self._sortedArray, value)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_bisect); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_bisect_left); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_popout); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_self->_sortedArray, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_self->_sortedArray, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->_sortedArray);
      __Pyx_GIVEREF(__pyx_v_self->_sortedArray);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_self->_sortedArray);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_delPos = __pyx_t_6;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":127
 *             popout = self._deque.dump(value)
 *             delPos = bisect.bisect_left(self._sortedArray, popout)
 *             del self._sortedArray[delPos]             # <<<<<<<<<<<<<<
 *             self._cur_pos = bisect.bisect_left(self._sortedArray, value)
 *             self._sortedArray.insert(int(self._cur_pos), value)
 */
    if (unlikely(__pyx_v_self->_sortedArray == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 127, __pyx_L1_error)
    }
    if (unlikely(__Pyx_DelItemInt(__pyx_v_self->_sortedArray, __pyx_v_delPos, int, 1, __Pyx_PyInt_From_int, 1, 1, 1) < 0)) __PYX_ERR(0, 127, __pyx_L1_error)

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":128
 *             delPos = bisect.bisect_left(self._sortedArray, popout)
 *             del self._sortedArray[delPos]
 *             self._cur_pos = bisect.bisect_left(self._sortedArray, value)             # <<<<<<<<<<<<<<
 *             self._sortedArray.insert(int(self._cur_pos), value)
 *         else:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_bisect); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_bisect_left); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_7)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_self->_sortedArray, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 128, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_self->_sortedArray, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 128, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 128, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->_sortedArray);
      __Pyx_GIVEREF(__pyx_v_self->_sortedArray);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_self->_sortedArray);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 128, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_self->_cur_pos = __pyx_t_8;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":129
 *             del self._sortedArray[delPos]
 *             self._cur_pos = bisect.bisect_left(self._sortedArray, value)
 *             self._sortedArray.insert(int(self._cur_pos), value)             # <<<<<<<<<<<<<<
 *         else:
 *             self._deque.dump(value)
 */
    if (unlikely(__pyx_v_self->_sortedArray == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "insert");
      __PYX_ERR(0, 129, __pyx_L1_error)
    }
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = PyList_Insert(__pyx_v_self->_sortedArray, ((Py_ssize_t)__pyx_v_self->_cur_pos), __pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 129, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":124
 *         if isnan(value):
 *             return NAN
 *         if self._deque.isFull():             # <<<<<<<<<<<<<<
 *             popout = self._deque.dump(value)
 *             delPos = bisect.bisect_left(self._sortedArray, popout)
 */
    goto __pyx_L4;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":131
 *             self._sortedArray.insert(int(self._cur_pos), value)
 *         else:
 *             self._deque.dump(value)             # <<<<<<<<<<<<<<
 *             self._cur_pos = bisect.bisect_left(self._sortedArray, value)
 *             self._sortedArray.insert(int(self._cur_pos), value)
 */
  /*else*/ {
    (void)(((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, NULL));

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":132
 *         else:
 *             self._deque.dump(value)
 *             self._cur_pos = bisect.bisect_left(self._sortedArray, value)             # <<<<<<<<<<<<<<
 *             self._sortedArray.insert(int(self._cur_pos), value)
 *         self._isFull = self._isFull or self._deque.isFull()
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_bisect); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_bisect_left); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self->_sortedArray, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self->_sortedArray, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->_sortedArray);
      __Pyx_GIVEREF(__pyx_v_self->_sortedArray);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_6, __pyx_v_self->_sortedArray);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_6, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_self->_cur_pos = __pyx_t_8;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":133
 *             self._deque.dump(value)
 *             self._cur_pos = bisect.bisect_left(self._sortedArray, value)
 *             self._sortedArray.insert(int(self._cur_pos), value)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
    if (unlikely(__pyx_v_self->_sortedArray == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "insert");
      __PYX_ERR(0, 133, __pyx_L1_error)
    }
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = PyList_Insert(__pyx_v_self->_sortedArray, ((Py_ssize_t)__pyx_v_self->_cur_pos), __pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L4:;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":134
 *             self._cur_pos = bisect.bisect_left(self._sortedArray, value)
 *             self._sortedArray.insert(int(self._cur_pos), value)
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_10 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_10) {
  } else {
    __pyx_t_5 = __pyx_t_10;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_10 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_10;
  __pyx_L5_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":116
 *         self._cur_pos = NAN
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         cdef int delPos
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SortedValueHolder.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_2push[] = "SortedValueHolder.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 116, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SortedValueHolder.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_4__reduce_cython__[] = "SortedValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_4__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_4__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_cur_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(7); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->_sortedArray);
  __Pyx_GIVEREF(__pyx_v_self->_sortedArray);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->_sortedArray);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 6, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->_sortedArray != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_SortedValueHolder); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, None), state
 *     else:
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SortedValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_SortedValueHolder); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SortedValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SortedValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_6__setstate_cython__[] = "SortedValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_6__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_6__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_SortedValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_SortedValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_SortedValueHolder, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_SortedValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.SortedValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":138
 * 
 * cdef class MovingMax(SortedValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingMax, self).__init__(window, x)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 138, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 138, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 138, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMax.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":139
 * cdef class MovingMax(SortedValueHolder):
 *     def __init__(self, window, x):
 *         super(MovingMax, self).__init__(window, x)             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":138
 * 
 * cdef class MovingMax(SortedValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingMax, self).__init__(window, x)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMax.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":141
 *         super(MovingMax, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._sortedArray:
 *             return self._sortedArray[-1]
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_3result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 141, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 141, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":142
 * 
 *     cpdef double result(self):
 *         if self._sortedArray:             # <<<<<<<<<<<<<<
 *             return self._sortedArray[-1]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base._sortedArray != Py_None)&&(PyList_GET_SIZE(__pyx_v_self->__pyx_base._sortedArray) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":143
 *     cpdef double result(self):
 *         if self._sortedArray:
 *             return self._sortedArray[-1]             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    if (unlikely(__pyx_v_self->__pyx_base._sortedArray == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 143, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->__pyx_base._sortedArray, -1L, long, 1, __Pyx_PyInt_From_long, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":142
 * 
 *     cpdef double result(self):
 *         if self._sortedArray:             # <<<<<<<<<<<<<<
 *             return self._sortedArray[-1]
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":145
 *             return self._sortedArray[-1]
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":141
 *         super(MovingMax, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._sortedArray:
 *             return self._sortedArray[-1]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMax.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_2result[] = "MovingMax.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_2result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMax.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":147
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MMax}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_4__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":148
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MMax}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MMax_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":147
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MMax}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMax.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_6__reduce_cython__[] = "MovingMax.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base._cur_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(7); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._sortedArray);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base._sortedArray != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingMax); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, None), state
 *     else:
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingMax__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingMax); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMax.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingMax__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_8__setstate_cython__[] = "MovingMax.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingMax__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMax__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingMax, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingMax__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMax.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":153
 * cdef class MovingArgMax(SortedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingArgMax, self).__init__(window, x)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 153, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 153, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 153, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMax.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":154
 * 
 *     def __init__(self, window, x):
 *         super(MovingArgMax, self).__init__(window, x)             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":153
 * cdef class MovingArgMax(SortedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingArgMax, self).__init__(window, x)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMax.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":156
 *         super(MovingArgMax, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double currMax
 *         cdef double idx
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_v_currMax;
  double __pyx_v_idx;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_3result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 156, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 156, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":159
 *         cdef double currMax
 *         cdef double idx
 *         if self._sortedArray:             # <<<<<<<<<<<<<<
 *             currMax = self._sortedArray[-1]
 *             idx = self._deque.idx(currMax)
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base._sortedArray != Py_None)&&(PyList_GET_SIZE(__pyx_v_self->__pyx_base._sortedArray) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":160
 *         cdef double idx
 *         if self._sortedArray:
 *             currMax = self._sortedArray[-1]             # <<<<<<<<<<<<<<
 *             idx = self._deque.idx(currMax)
 *             return self.size() - idx - 1
 */
    if (unlikely(__pyx_v_self->__pyx_base._sortedArray == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 160, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->__pyx_base._sortedArray, -1L, long, 1, __Pyx_PyInt_From_long, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_currMax = __pyx_t_5;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":161
 *         if self._sortedArray:
 *             currMax = self._sortedArray[-1]
 *             idx = self._deque.idx(currMax)             # <<<<<<<<<<<<<<
 *             return self.size() - idx - 1
 *         else:
 */
    __pyx_v_idx = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque->__pyx_vtab)->idx(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque, __pyx_v_currMax, 0);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":162
 *             currMax = self._sortedArray[-1]
 *             idx = self._deque.idx(currMax)
 *             return self.size() - idx - 1             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    __pyx_r = ((((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.size(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self), 0) - __pyx_v_idx) - 1.0);
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":159
 *         cdef double currMax
 *         cdef double idx
 *         if self._sortedArray:             # <<<<<<<<<<<<<<
 *             currMax = self._sortedArray[-1]
 *             idx = self._deque.idx(currMax)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":164
 *             return self.size() - idx - 1
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":156
 *         super(MovingArgMax, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double currMax
 *         cdef double idx
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMax.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_2result[] = "MovingArgMax.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_2result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMax.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":166
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MArgMax}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_4__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":167
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MArgMax}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MArgMax_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":166
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MArgMax}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMax.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_6__reduce_cython__[] = "MovingArgMax.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base._cur_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(7); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._sortedArray);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base._sortedArray != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingArgMax); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, None), state
 *     else:
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingArgMax__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingArgMax); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMax.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingArgMax__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_8__setstate_cython__[] = "MovingArgMax.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingArgMax__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingArgMax__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingArgMax, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingArgMax__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMax.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":171
 * 
 * cdef class MovingMin(SortedValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingMin, self).__init__(window, x)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 171, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 171, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 171, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMin.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":172
 * cdef class MovingMin(SortedValueHolder):
 *     def __init__(self, window, x):
 *         super(MovingMin, self).__init__(window, x)             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":171
 * 
 * cdef class MovingMin(SortedValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingMin, self).__init__(window, x)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMin.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":174
 *         super(MovingMin, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._sortedArray:
 *             return self._sortedArray[0]
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 174, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_3result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 174, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 174, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":175
 * 
 *     cpdef double result(self):
 *         if self._sortedArray:             # <<<<<<<<<<<<<<
 *             return self._sortedArray[0]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base._sortedArray != Py_None)&&(PyList_GET_SIZE(__pyx_v_self->__pyx_base._sortedArray) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":176
 *     cpdef double result(self):
 *         if self._sortedArray:
 *             return self._sortedArray[0]             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    if (unlikely(__pyx_v_self->__pyx_base._sortedArray == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 176, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->__pyx_base._sortedArray, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":175
 * 
 *     cpdef double result(self):
 *         if self._sortedArray:             # <<<<<<<<<<<<<<
 *             return self._sortedArray[0]
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":178
 *             return self._sortedArray[0]
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":174
 *         super(MovingMin, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._sortedArray:
 *             return self._sortedArray[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMin.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_2result[] = "MovingMin.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_2result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMin.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":180
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MMin}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_4__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":181
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MMin}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MMin_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":180
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MMin}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMin.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_6__reduce_cython__[] = "MovingMin.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base._cur_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(7); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._sortedArray);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base._sortedArray != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingMin); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, None), state
 *     else:
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingMin__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingMin); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMin.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingMin__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_8__setstate_cython__[] = "MovingMin.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingMin__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMin__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingMin, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingMin__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMin.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":186
 * cdef class MovingArgMin(SortedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingArgMin, self).__init__(window, x)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 186, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 186, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 186, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMin.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":187
 * 
 *     def __init__(self, window, x):
 *         super(MovingArgMin, self).__init__(window, x)             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":186
 * cdef class MovingArgMin(SortedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingArgMin, self).__init__(window, x)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMin.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":189
 *         super(MovingArgMin, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double currMax
 *         cdef double idx
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_v_idx;
  PyObject *__pyx_v_currMin = NULL;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_3result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 189, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":192
 *         cdef double currMax
 *         cdef double idx
 *         if self._sortedArray:             # <<<<<<<<<<<<<<
 *             currMin = self._sortedArray[0]
 *             idx = self._deque.idx(currMin)
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base._sortedArray != Py_None)&&(PyList_GET_SIZE(__pyx_v_self->__pyx_base._sortedArray) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":193
 *         cdef double idx
 *         if self._sortedArray:
 *             currMin = self._sortedArray[0]             # <<<<<<<<<<<<<<
 *             idx = self._deque.idx(currMin)
 *             return self.size() - idx - 1
 */
    if (unlikely(__pyx_v_self->__pyx_base._sortedArray == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 193, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->__pyx_base._sortedArray, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_currMin = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":194
 *         if self._sortedArray:
 *             currMin = self._sortedArray[0]
 *             idx = self._deque.idx(currMin)             # <<<<<<<<<<<<<<
 *             return self.size() - idx - 1
 *         else:
 */
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_v_currMin); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 194, __pyx_L1_error)
    __pyx_v_idx = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque->__pyx_vtab)->idx(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque, __pyx_t_5, 0);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":195
 *             currMin = self._sortedArray[0]
 *             idx = self._deque.idx(currMin)
 *             return self.size() - idx - 1             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    __pyx_r = ((((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.size(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self), 0) - __pyx_v_idx) - 1.0);
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":192
 *         cdef double currMax
 *         cdef double idx
 *         if self._sortedArray:             # <<<<<<<<<<<<<<
 *             currMin = self._sortedArray[0]
 *             idx = self._deque.idx(currMin)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":197
 *             return self.size() - idx - 1
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":189
 *         super(MovingArgMin, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double currMax
 *         cdef double idx
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMin.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_currMin);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_2result[] = "MovingArgMin.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_2result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMin.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":199
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MArgMin}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_4__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":200
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MArgMin}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MArgMin_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":199
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MArgMin}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMin.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_6__reduce_cython__[] = "MovingArgMin.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base._cur_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(7); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._sortedArray);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base._sortedArray != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingArgMin); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, None), state
 *     else:
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingArgMin__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingArgMin); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMin.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingArgMin__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_8__setstate_cython__[] = "MovingArgMin.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingArgMin__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingArgMin__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingArgMin, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingArgMin__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMin.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":204
 * 
 * cdef class MovingRank(SortedValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingRank, self).__init__(window, x)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 204, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 204, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 204, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRank.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":205
 * cdef class MovingRank(SortedValueHolder):
 *     def __init__(self, window, x):
 *         super(MovingRank, self).__init__(window, x)             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":204
 * 
 * cdef class MovingRank(SortedValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingRank, self).__init__(window, x)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRank.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":207
 *         super(MovingRank, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._cur_pos
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_3result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 207, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":208
 * 
 *     cpdef double result(self):
 *         return self._cur_pos             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = __pyx_v_self->__pyx_base._cur_pos;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":207
 *         super(MovingRank, self).__init__(window, x)
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._cur_pos
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRank.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_2result[] = "MovingRank.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_2result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRank.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":210
 *         return self._cur_pos
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MRank}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_4__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":211
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MRank}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MRank_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":210
 *         return self._cur_pos
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MRank}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRank.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_6__reduce_cython__[] = "MovingRank.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base._cur_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(7); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._sortedArray);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base._sortedArray != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingRank); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, None), state
 *     else:
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingRank__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingRank); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRank.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingRank__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_8__setstate_cython__[] = "MovingRank.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingRank__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingRank__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingRank, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingRank__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRank.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":215
 * 
 * cdef class MovingQuantile(SortedValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingQuantile, self).__init__(window, x)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 215, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 215, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 215, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingQuantile.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":216
 * cdef class MovingQuantile(SortedValueHolder):
 *     def __init__(self, window, x):
 *         super(MovingQuantile, self).__init__(window, x)             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":215
 * 
 * cdef class MovingQuantile(SortedValueHolder):
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingQuantile, self).__init__(window, x)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingQuantile.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":219
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t n = len(self._sortedArray)
 *         if n > 1:
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_v_n;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_3result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":220
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         cdef size_t n = len(self._sortedArray)             # <<<<<<<<<<<<<<
 *         if n > 1:
 *             return self._cur_pos / (n - 1)
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base._sortedArray;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 220, __pyx_L1_error)
  }
  __pyx_t_6 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_n = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":221
 *     cpdef double result(self):
 *         cdef size_t n = len(self._sortedArray)
 *         if n > 1:             # <<<<<<<<<<<<<<
 *             return self._cur_pos / (n - 1)
 *         else:
 */
  __pyx_t_7 = ((__pyx_v_n > 1) != 0);
  if (__pyx_t_7) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":222
 *         cdef size_t n = len(self._sortedArray)
 *         if n > 1:
 *             return self._cur_pos / (n - 1)             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    __pyx_r = (__pyx_v_self->__pyx_base._cur_pos / (__pyx_v_n - 1));
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":221
 *     cpdef double result(self):
 *         cdef size_t n = len(self._sortedArray)
 *         if n > 1:             # <<<<<<<<<<<<<<
 *             return self._cur_pos / (n - 1)
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":224
 *             return self._cur_pos / (n - 1)
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":219
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t n = len(self._sortedArray)
 *         if n > 1:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingQuantile.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_2result[] = "MovingQuantile.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_3result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_2result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_2result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingQuantile.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":226
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MQuantile}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_4__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_4__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":227
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MQuantile}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MQuantile_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":226
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MQuantile}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingQuantile.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_6__reduce_cython__[] = "MovingQuantile.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base._cur_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(7); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._sortedArray);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base._sortedArray);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._cur_pos, self._dependency, self._deque, self._isFull, self._sortedArray, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base._sortedArray != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._x) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingQuantile); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._sortedArray is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, None), state
 *     else:
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingQuantile__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingQuantile); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55178044);
    __Pyx_GIVEREF(__pyx_int_55178044);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_55178044);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingQuantile.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingQuantile__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_8__setstate_cython__[] = "MovingQuantile.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingQuantile__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingQuantile__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingQuantile, (type(self), 0x349f33c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingQuantile__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingQuantile.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":232
 * cdef class MovingAllTrue(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingAllTrue, self).__init__(window, x)
 *         self._countedTrue = 0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 232, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 232, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 232, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":233
 * 
 *     def __init__(self, window, x):
 *         super(MovingAllTrue, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._countedTrue = 0
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":234
 *     def __init__(self, window, x):
 *         super(MovingAllTrue, self).__init__(window, x)
 *         self._countedTrue = 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_countedTrue = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":232
 * cdef class MovingAllTrue(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingAllTrue, self).__init__(window, x)
 *         self._countedTrue = 0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":236
 *         self._countedTrue = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef int addedTrue
 *         cdef double popout
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  int __pyx_v_addedTrue;
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 236, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 236, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":240
 *         cdef double popout
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 *         if isnan(value):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":241
 * 
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 *         if isnan(value):
 *             return NAN
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":242
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         addedTrue = 0
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":243
 *         cdef double value = self._x.result()
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         addedTrue = 0
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 243, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":242
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         addedTrue = 0
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":244
 *         if isnan(value):
 *             return NAN
 *         addedTrue = 0             # <<<<<<<<<<<<<<
 * 
 *         if value:
 */
  __pyx_v_addedTrue = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":246
 *         addedTrue = 0
 * 
 *         if value:             # <<<<<<<<<<<<<<
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)
 */
  __pyx_t_5 = (__pyx_v_value != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":247
 * 
 *         if value:
 *             addedTrue += 1             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, False)
 *         if popout:
 */
    __pyx_v_addedTrue = (__pyx_v_addedTrue + 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":246
 *         addedTrue = 0
 * 
 *         if value:             # <<<<<<<<<<<<<<
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":248
 *         if value:
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)             # <<<<<<<<<<<<<<
 *         if popout:
 *             addedTrue -= 1
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = 0;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":249
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)
 *         if popout:             # <<<<<<<<<<<<<<
 *             addedTrue -= 1
 * 
 */
  __pyx_t_5 = (__pyx_v_popout != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":250
 *         popout = self._deque.dump(value, False)
 *         if popout:
 *             addedTrue -= 1             # <<<<<<<<<<<<<<
 * 
 *         self._countedTrue += addedTrue
 */
    __pyx_v_addedTrue = (__pyx_v_addedTrue - 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":249
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)
 *         if popout:             # <<<<<<<<<<<<<<
 *             addedTrue -= 1
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":252
 *             addedTrue -= 1
 * 
 *         self._countedTrue += addedTrue             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  __pyx_v_self->_countedTrue = (__pyx_v_self->_countedTrue + __pyx_v_addedTrue);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":253
 * 
 *         self._countedTrue += addedTrue
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":236
 *         self._countedTrue = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef int addedTrue
 *         cdef double popout
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_2push[] = "MovingAllTrue.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 236, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":255
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._countedTrue == self.size()
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 255, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 255, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":256
 * 
 *     cpdef double result(self):
 *         return self._countedTrue == self.size()             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = (__pyx_v_self->_countedTrue == ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.size(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self), 0));
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":255
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._countedTrue == self.size()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_4result[] = "MovingAllTrue.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":258
 *         return self._countedTrue == self.size()
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MAllTrue}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":259
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MAllTrue}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MAllTrue_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":258
 *         return self._countedTrue == self.size()
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MAllTrue}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":80
 * cdef class MovingAllTrue(SingleValuedValueHolder):
 * 
 *     cdef public size_t _countedTrue             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->_countedTrue); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue._countedTrue.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 80, __pyx_L1_error)
  __pyx_v_self->_countedTrue = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue._countedTrue.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_8__reduce_cython__[] = "MovingAllTrue.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._countedTrue, self._dependency, self._deque, self._isFull, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->_countedTrue); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._countedTrue, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._countedTrue, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._countedTrue, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingAllTrue); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_24234567);
    __Pyx_GIVEREF(__pyx_int_24234567);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_24234567);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, None), state
 *     else:
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingAllTrue__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingAllTrue); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_24234567);
    __Pyx_GIVEREF(__pyx_int_24234567);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_24234567);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingAllTrue__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_10__setstate_cython__[] = "MovingAllTrue.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingAllTrue__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAllTrue__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingAllTrue, (type(self), 0x171ca47, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingAllTrue__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":264
 * cdef class MovingAnyTrue(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingAnyTrue, self).__init__(window, x)
 *         self._countedTrue = 0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 264, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 264, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 264, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":265
 * 
 *     def __init__(self, window, x):
 *         super(MovingAnyTrue, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._countedTrue = 0
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":266
 *     def __init__(self, window, x):
 *         super(MovingAnyTrue, self).__init__(window, x)
 *         self._countedTrue = 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_countedTrue = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":264
 * cdef class MovingAnyTrue(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingAnyTrue, self).__init__(window, x)
 *         self._countedTrue = 0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":268
 *         self._countedTrue = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef int addedTrue
 *         cdef double popout
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  int __pyx_v_addedTrue;
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 268, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":272
 *         cdef double popout
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 *         if isnan(value):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":273
 * 
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 *         if isnan(value):
 *             return NAN
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":274
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         addedTrue = 0
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":275
 *         cdef double value = self._x.result()
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         addedTrue = 0
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":274
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         addedTrue = 0
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":276
 *         if isnan(value):
 *             return NAN
 *         addedTrue = 0             # <<<<<<<<<<<<<<
 * 
 *         if value:
 */
  __pyx_v_addedTrue = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":278
 *         addedTrue = 0
 * 
 *         if value:             # <<<<<<<<<<<<<<
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)
 */
  __pyx_t_5 = (__pyx_v_value != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":279
 * 
 *         if value:
 *             addedTrue += 1             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, False)
 *         if popout:
 */
    __pyx_v_addedTrue = (__pyx_v_addedTrue + 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":278
 *         addedTrue = 0
 * 
 *         if value:             # <<<<<<<<<<<<<<
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":280
 *         if value:
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)             # <<<<<<<<<<<<<<
 *         if popout:
 *             addedTrue -= 1
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = 0;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":281
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)
 *         if popout:             # <<<<<<<<<<<<<<
 *             addedTrue -= 1
 * 
 */
  __pyx_t_5 = (__pyx_v_popout != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":282
 *         popout = self._deque.dump(value, False)
 *         if popout:
 *             addedTrue -= 1             # <<<<<<<<<<<<<<
 * 
 *         self._countedTrue += addedTrue
 */
    __pyx_v_addedTrue = (__pyx_v_addedTrue - 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":281
 *             addedTrue += 1
 *         popout = self._deque.dump(value, False)
 *         if popout:             # <<<<<<<<<<<<<<
 *             addedTrue -= 1
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":284
 *             addedTrue -= 1
 * 
 *         self._countedTrue += addedTrue             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  __pyx_v_self->_countedTrue = (__pyx_v_self->_countedTrue + __pyx_v_addedTrue);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":285
 * 
 *         self._countedTrue += addedTrue
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":268
 *         self._countedTrue = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef int addedTrue
 *         cdef double popout
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_2push[] = "MovingAnyTrue.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 268, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":287
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._countedTrue != 0
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 287, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 287, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 287, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":288
 * 
 *     cpdef double result(self):
 *         return self._countedTrue != 0             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = (__pyx_v_self->_countedTrue != 0);
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":287
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._countedTrue != 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_4result[] = "MovingAnyTrue.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":290
 *         return self._countedTrue != 0
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MAnyTrue}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":291
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MAnyTrue}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MAnyTrue_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":290
 *         return self._countedTrue != 0
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MAnyTrue}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":88
 * cdef class MovingAnyTrue(SingleValuedValueHolder):
 * 
 *     cdef public size_t _countedTrue             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->_countedTrue); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue._countedTrue.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 88, __pyx_L1_error)
  __pyx_v_self->_countedTrue = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue._countedTrue.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_8__reduce_cython__[] = "MovingAnyTrue.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._countedTrue, self._dependency, self._deque, self._isFull, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->_countedTrue); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._countedTrue, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._countedTrue, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._countedTrue, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingAnyTrue); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_24234567);
    __Pyx_GIVEREF(__pyx_int_24234567);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_24234567);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, None), state
 *     else:
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingAnyTrue__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingAnyTrue); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_24234567);
    __Pyx_GIVEREF(__pyx_int_24234567);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_24234567);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingAnyTrue__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_10__setstate_cython__[] = "MovingAnyTrue.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingAnyTrue__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAnyTrue__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingAnyTrue, (type(self), 0x171ca47, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingAnyTrue__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":296
 * cdef class MovingSum(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingSum, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 296, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 296, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 296, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":297
 * 
 *     def __init__(self, window, x):
 *         super(MovingSum, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningSum = 0.0
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":298
 *     def __init__(self, window, x):
 *         super(MovingSum, self).__init__(window, x)
 *         self._runningSum = 0.0             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_runningSum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":296
 * cdef class MovingSum(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingSum, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":300
 *         self._runningSum = 0.0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 300, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":302
 *     cpdef push(self, dict data):
 *         cdef double popout
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 *         if isnan(value):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":303
 *         cdef double popout
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 *         if isnan(value):
 *             return NAN
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":304
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":305
 *         cdef double value = self._x.result()
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, 0.)
 *         self._runningSum = self._runningSum - popout + value
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":304
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":306
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value, 0.)             # <<<<<<<<<<<<<<
 *         self._runningSum = self._runningSum - popout + value
 * 
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = 0.;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":307
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 *         self._runningSum = self._runningSum - popout + value             # <<<<<<<<<<<<<<
 * 
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_v_self->_runningSum = ((__pyx_v_self->_runningSum - __pyx_v_popout) + __pyx_v_value);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":309
 *         self._runningSum = self._runningSum - popout + value
 * 
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":300
 *         self._runningSum = 0.0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_2push[] = "MovingSum.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 300, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":311
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningSum
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 311, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 311, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":312
 * 
 *     cpdef double result(self):
 *         return self._runningSum             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = __pyx_v_self->_runningSum;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":311
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningSum
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_4result[] = "MovingSum.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":314
 *         return self._runningSum
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MSum}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":315
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MSum}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MSum_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":314
 *         return self._runningSum
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MSum}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":96
 * cdef class MovingSum(SingleValuedValueHolder):
 * 
 *     cdef public double _runningSum             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum._runningSum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 96, __pyx_L1_error)
  __pyx_v_self->_runningSum = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum._runningSum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_8__reduce_cython__[] = "MovingSum.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningSum, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingSum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_128252030);
    __Pyx_GIVEREF(__pyx_int_128252030);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_128252030);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, None), state
 *     else:
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingSum__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingSum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_128252030);
    __Pyx_GIVEREF(__pyx_int_128252030);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_128252030);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingSum__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_10__setstate_cython__[] = "MovingSum.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingSum__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSum__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingSum, (type(self), 0x7a4f87e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingSum__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":320
 * cdef class MovingAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingAverage, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 320, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 320, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 320, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":321
 * 
 *     def __init__(self, window, x):
 *         super(MovingAverage, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningSum = 0.0
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":322
 *     def __init__(self, window, x):
 *         super(MovingAverage, self).__init__(window, x)
 *         self._runningSum = 0.0             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_runningSum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":320
 * cdef class MovingAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingAverage, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":324
 *         self._runningSum = 0.0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 324, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 324, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":327
 *         cdef double popout
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":328
 * 
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(value):
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":330
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":331
 * 
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, 0.)
 *         self._runningSum += value - popout
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":330
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":332
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value, 0.)             # <<<<<<<<<<<<<<
 *         self._runningSum += value - popout
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = 0.;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":333
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 *         self._runningSum += value - popout             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  __pyx_v_self->_runningSum = (__pyx_v_self->_runningSum + (__pyx_v_value - __pyx_v_popout));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":334
 *         popout = self._deque.dump(value, 0.)
 *         self._runningSum += value - popout
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":324
 *         self._runningSum = 0.0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_2push[] = "MovingAverage.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 324, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":337
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t size = self.size()
 *         if size:
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_v_size;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 337, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 337, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":338
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         cdef size_t size = self.size()             # <<<<<<<<<<<<<<
 *         if size:
 *             return self._runningSum / size
 */
  __pyx_v_size = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.size(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self), 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":339
 *     cpdef double result(self):
 *         cdef size_t size = self.size()
 *         if size:             # <<<<<<<<<<<<<<
 *             return self._runningSum / size
 *         else:
 */
  __pyx_t_6 = (__pyx_v_size != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":340
 *         cdef size_t size = self.size()
 *         if size:
 *             return self._runningSum / size             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    __pyx_r = (__pyx_v_self->_runningSum / __pyx_v_size);
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":339
 *     cpdef double result(self):
 *         cdef size_t size = self.size()
 *         if size:             # <<<<<<<<<<<<<<
 *             return self._runningSum / size
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":342
 *             return self._runningSum / size
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":337
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t size = self.size()
 *         if size:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_4result[] = "MovingAverage.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":344
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MA}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":345
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MA}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MA_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":344
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MA}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":104
 * cdef class MovingAverage(SingleValuedValueHolder):
 * 
 *     cdef public double _runningSum             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage._runningSum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 104, __pyx_L1_error)
  __pyx_v_self->_runningSum = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage._runningSum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_8__reduce_cython__[] = "MovingAverage.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningSum, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingAverage); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_128252030);
    __Pyx_GIVEREF(__pyx_int_128252030);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_128252030);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, None), state
 *     else:
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingAverage__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingAverage); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_128252030);
    __Pyx_GIVEREF(__pyx_int_128252030);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_128252030);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingAverage__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_10__setstate_cython__[] = "MovingAverage.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingAverage__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAverage__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingAverage, (type(self), 0x7a4f87e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingAverage__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":350
 * cdef class MovingDecay(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingDecay, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 350, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 350, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 350, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":351
 * 
 *     def __init__(self, window, x):
 *         super(MovingDecay, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningSum = 0.0
 *         self._runningWeightedSum = 0.0
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":352
 *     def __init__(self, window, x):
 *         super(MovingDecay, self).__init__(window, x)
 *         self._runningSum = 0.0             # <<<<<<<<<<<<<<
 *         self._runningWeightedSum = 0.0
 *         self._newestValue = 0.
 */
  __pyx_v_self->_runningSum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":353
 *         super(MovingDecay, self).__init__(window, x)
 *         self._runningSum = 0.0
 *         self._runningWeightedSum = 0.0             # <<<<<<<<<<<<<<
 *         self._newestValue = 0.
 * 
 */
  __pyx_v_self->_runningWeightedSum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":354
 *         self._runningSum = 0.0
 *         self._runningWeightedSum = 0.0
 *         self._newestValue = 0.             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_newestValue = 0.;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":350
 * cdef class MovingDecay(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingDecay, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":356
 *         self._newestValue = 0.
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         cdef double k = self._window
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_k;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  double __pyx_t_7;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 356, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 356, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":358
 *     cpdef push(self, dict data):
 *         cdef double popout
 *         cdef double k = self._window             # <<<<<<<<<<<<<<
 * 
 *         self._x.push(data)
 */
  __pyx_t_5 = __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window;
  __pyx_v_k = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":360
 *         cdef double k = self._window
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":361
 * 
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(value):
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":363
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  __pyx_t_6 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":364
 * 
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *         popout = self._deque.dump(value, 0.)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":363
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":366
 *             return NAN
 * 
 *         popout = self._deque.dump(value, 0.)             # <<<<<<<<<<<<<<
 *         self._runningWeightedSum = self._runningWeightedSum - self._runningSum + k * value
 *         self._runningSum += value - popout
 */
  __pyx_t_8.__pyx_n = 1;
  __pyx_t_8.__pyx_default = 0.;
  __pyx_t_7 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_8); 
  __pyx_v_popout = __pyx_t_7;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":367
 * 
 *         popout = self._deque.dump(value, 0.)
 *         self._runningWeightedSum = self._runningWeightedSum - self._runningSum + k * value             # <<<<<<<<<<<<<<
 *         self._runningSum += value - popout
 *         self._newestValue = value
 */
  __pyx_v_self->_runningWeightedSum = ((__pyx_v_self->_runningWeightedSum - __pyx_v_self->_runningSum) + (__pyx_v_k * __pyx_v_value));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":368
 *         popout = self._deque.dump(value, 0.)
 *         self._runningWeightedSum = self._runningWeightedSum - self._runningSum + k * value
 *         self._runningSum += value - popout             # <<<<<<<<<<<<<<
 *         self._newestValue = value
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_v_self->_runningSum = (__pyx_v_self->_runningSum + (__pyx_v_value - __pyx_v_popout));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":369
 *         self._runningWeightedSum = self._runningWeightedSum - self._runningSum + k * value
 *         self._runningSum += value - popout
 *         self._newestValue = value             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  __pyx_v_self->_newestValue = __pyx_v_value;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":370
 *         self._runningSum += value - popout
 *         self._newestValue = value
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_9) {
  } else {
    __pyx_t_6 = __pyx_t_9;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_9 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_6 = __pyx_t_9;
  __pyx_L4_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":356
 *         self._newestValue = 0.
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         cdef double k = self._window
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_2push[] = "MovingDecay.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 356, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":373
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double k = self._window
 *         cdef size_t s = self.size()
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_v_k;
  size_t __pyx_v_s;
  double __pyx_v_c;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 373, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 373, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":374
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         cdef double k = self._window             # <<<<<<<<<<<<<<
 *         cdef size_t s = self.size()
 *         cdef double c = consecutive_int_sum(k - s + 1, k)
 */
  __pyx_t_6 = __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window;
  __pyx_v_k = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":375
 *     cpdef double result(self):
 *         cdef double k = self._window
 *         cdef size_t s = self.size()             # <<<<<<<<<<<<<<
 *         cdef double c = consecutive_int_sum(k - s + 1, k)
 *         if s:
 */
  __pyx_v_s = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.size(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self), 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":376
 *         cdef double k = self._window
 *         cdef size_t s = self.size()
 *         cdef double c = consecutive_int_sum(k - s + 1, k)             # <<<<<<<<<<<<<<
 *         if s:
 *             return self._runningWeightedSum / c
 */
  __pyx_v_c = __pyx_f_6ultron_6sentry_4Math_4udfs_consecutive_int_sum(((__pyx_v_k - __pyx_v_s) + 1.0), __pyx_v_k);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":377
 *         cdef size_t s = self.size()
 *         cdef double c = consecutive_int_sum(k - s + 1, k)
 *         if s:             # <<<<<<<<<<<<<<
 *             return self._runningWeightedSum / c
 *         else:
 */
  __pyx_t_7 = (__pyx_v_s != 0);
  if (__pyx_t_7) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":378
 *         cdef double c = consecutive_int_sum(k - s + 1, k)
 *         if s:
 *             return self._runningWeightedSum / c             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    __pyx_r = (__pyx_v_self->_runningWeightedSum / __pyx_v_c);
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":377
 *         cdef size_t s = self.size()
 *         cdef double c = consecutive_int_sum(k - s + 1, k)
 *         if s:             # <<<<<<<<<<<<<<
 *             return self._runningWeightedSum / c
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":380
 *             return self._runningWeightedSum / c
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":373
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double k = self._window
 *         cdef size_t s = self.size()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_4result[] = "MovingDecay.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":382
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MADecay}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":383
 * 
 *     def __str__(self):
 *         return "\\mathrm{{MADecay}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_MADecay_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":382
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{MADecay}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":111
 * 
 * cdef class MovingDecay(SingleValuedValueHolder):
 *     cdef public double _runningSum             # <<<<<<<<<<<<<<
 *     cdef double _runningWeightedSum
 *     cdef double _newestValue
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay._runningSum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 111, __pyx_L1_error)
  __pyx_v_self->_runningSum = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay._runningSum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_8__reduce_cython__[] = "MovingDecay.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._newestValue, self._runningSum, self._runningWeightedSum, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_newestValue); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->_runningWeightedSum); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(8); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_6, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_6, 7, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._newestValue, self._runningSum, self._runningWeightedSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._newestValue, self._runningSum, self._runningWeightedSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._newestValue, self._runningSum, self._runningWeightedSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_MovingDecay); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_188027916);
    __Pyx_GIVEREF(__pyx_int_188027916);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_188027916);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, None), state
 *     else:
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingDecay__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_MovingDecay); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_188027916);
    __Pyx_GIVEREF(__pyx_int_188027916);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_188027916);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingDecay__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_10__setstate_cython__[] = "MovingDecay.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingDecay__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingDecay__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingDecay, (type(self), 0xb35140c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingDecay__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":388
 * cdef class MovingPositiveAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingPositiveAverage, self).__init__(window, x)
 *         self._runningPositiveSum = 0.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 388, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 388, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 388, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":389
 * 
 *     def __init__(self, window, x):
 *         super(MovingPositiveAverage, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningPositiveSum = 0.0
 *         self._runningPositiveCount = 0
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":390
 *     def __init__(self, window, x):
 *         super(MovingPositiveAverage, self).__init__(window, x)
 *         self._runningPositiveSum = 0.0             # <<<<<<<<<<<<<<
 *         self._runningPositiveCount = 0
 * 
 */
  __pyx_v_self->_runningPositiveSum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":391
 *         super(MovingPositiveAverage, self).__init__(window, x)
 *         self._runningPositiveSum = 0.0
 *         self._runningPositiveCount = 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_runningPositiveCount = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":388
 * cdef class MovingPositiveAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingPositiveAverage, self).__init__(window, x)
 *         self._runningPositiveSum = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":393
 *         self._runningPositiveCount = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 393, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":396
 *         cdef double popout
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":397
 * 
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(value):
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":399
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, -1.)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":400
 * 
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, -1.)
 *         if value > 0.0:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":399
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, -1.)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":401
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value, -1.)             # <<<<<<<<<<<<<<
 *         if value > 0.0:
 *             self._runningPositiveCount += 1
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = -1.;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":402
 *             return NAN
 *         popout = self._deque.dump(value, -1.)
 *         if value > 0.0:             # <<<<<<<<<<<<<<
 *             self._runningPositiveCount += 1
 *             self._runningPositiveSum += value
 */
  __pyx_t_5 = ((__pyx_v_value > 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":403
 *         popout = self._deque.dump(value, -1.)
 *         if value > 0.0:
 *             self._runningPositiveCount += 1             # <<<<<<<<<<<<<<
 *             self._runningPositiveSum += value
 * 
 */
    __pyx_v_self->_runningPositiveCount = (__pyx_v_self->_runningPositiveCount + 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":404
 *         if value > 0.0:
 *             self._runningPositiveCount += 1
 *             self._runningPositiveSum += value             # <<<<<<<<<<<<<<
 * 
 *         if popout > 0.0:
 */
    __pyx_v_self->_runningPositiveSum = (__pyx_v_self->_runningPositiveSum + __pyx_v_value);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":402
 *             return NAN
 *         popout = self._deque.dump(value, -1.)
 *         if value > 0.0:             # <<<<<<<<<<<<<<
 *             self._runningPositiveCount += 1
 *             self._runningPositiveSum += value
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":406
 *             self._runningPositiveSum += value
 * 
 *         if popout > 0.0:             # <<<<<<<<<<<<<<
 *             self._runningPositiveCount -= 1
 *             self._runningPositiveSum -= popout
 */
  __pyx_t_5 = ((__pyx_v_popout > 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":407
 * 
 *         if popout > 0.0:
 *             self._runningPositiveCount -= 1             # <<<<<<<<<<<<<<
 *             self._runningPositiveSum -= popout
 *         self._isFull = self._isFull or self._deque.isFull()
 */
    __pyx_v_self->_runningPositiveCount = (__pyx_v_self->_runningPositiveCount - 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":408
 *         if popout > 0.0:
 *             self._runningPositiveCount -= 1
 *             self._runningPositiveSum -= popout             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
    __pyx_v_self->_runningPositiveSum = (__pyx_v_self->_runningPositiveSum - __pyx_v_popout);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":406
 *             self._runningPositiveSum += value
 * 
 *         if popout > 0.0:             # <<<<<<<<<<<<<<
 *             self._runningPositiveCount -= 1
 *             self._runningPositiveSum -= popout
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":409
 *             self._runningPositiveCount -= 1
 *             self._runningPositiveSum -= popout
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":393
 *         self._runningPositiveCount = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_2push[] = "MovingPositiveAverage.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 393, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":412
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._runningPositiveCount == 0:
 *             return 0.0
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 412, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 412, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":413
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         if self._runningPositiveCount == 0:             # <<<<<<<<<<<<<<
 *             return 0.0
 *         else:
 */
  __pyx_t_6 = ((__pyx_v_self->_runningPositiveCount == 0) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":414
 *     cpdef double result(self):
 *         if self._runningPositiveCount == 0:
 *             return 0.0             # <<<<<<<<<<<<<<
 *         else:
 *             return self._runningPositiveSum / self._runningPositiveCount
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":413
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         if self._runningPositiveCount == 0:             # <<<<<<<<<<<<<<
 *             return 0.0
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":416
 *             return 0.0
 *         else:
 *             return self._runningPositiveSum / self._runningPositiveCount             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = (__pyx_v_self->_runningPositiveSum / __pyx_v_self->_runningPositiveCount);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":412
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._runningPositiveCount == 0:
 *             return 0.0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_4result[] = "MovingPositiveAverage.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":418
 *             return self._runningPositiveSum / self._runningPositiveCount
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mposavg}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":419
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mposavg}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mposavg_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":418
 *             return self._runningPositiveSum / self._runningPositiveCount
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mposavg}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":121
 * cdef class MovingPositiveAverage(SingleValuedValueHolder):
 * 
 *     cdef public double _runningPositiveSum             # <<<<<<<<<<<<<<
 *     cdef public int _runningPositiveCount
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningPositiveSum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage._runningPositiveSum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 121, __pyx_L1_error)
  __pyx_v_self->_runningPositiveSum = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage._runningPositiveSum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":122
 * 
 *     cdef public double _runningPositiveSum
 *     cdef public int _runningPositiveCount             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_runningPositiveCount); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage._runningPositiveCount.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 122, __pyx_L1_error)
  __pyx_v_self->_runningPositiveCount = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage._runningPositiveCount.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_8__reduce_cython__[] = "MovingPositiveAverage.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningPositiveCount, self._runningPositiveSum, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->_runningPositiveCount); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_runningPositiveSum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(7); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_5, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_5, 6, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningPositiveCount, self._runningPositiveSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_5 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v__dict = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningPositiveCount, self._runningPositiveSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningPositiveCount, self._runningPositiveSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, None), state
 */
  /*else*/ {
    __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_8 = (__pyx_t_6 != 0);
    __pyx_t_7 = __pyx_t_8;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_7;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, None), state
 *     else:
 */
  __pyx_t_7 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_MovingPositiveAve); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_130343579);
    __Pyx_GIVEREF(__pyx_int_130343579);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_130343579);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, None), state
 *     else:
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingPositiveAverage__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingPositiveAve); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_130343579);
    __Pyx_GIVEREF(__pyx_int_130343579);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_130343579);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingPositiveAverage__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_10__setstate_cython__[] = "MovingPositiveAverage.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingPositiveAverage__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingPositiveAverage__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingPositiveAverage, (type(self), 0x7c4e29b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingPositiveAverage__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":424
 * cdef class MovingPositiveDifferenceAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingPositiveDifferenceAverage, self).__init__(window, x)
 *         cdef Accumulator runningPositive = PositivePart(build_holder(x) - Shift(1, build_holder(x)))
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 424, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 424, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 424, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *__pyx_v_runningPositive = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":425
 * 
 *     def __init__(self, window, x):
 *         super(MovingPositiveDifferenceAverage, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         cdef Accumulator runningPositive = PositivePart(build_holder(x) - Shift(1, build_holder(x)))
 *         self._runningAverage = MovingAverage(window, x=runningPositive)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":426
 *     def __init__(self, window, x):
 *         super(MovingPositiveDifferenceAverage, self).__init__(window, x)
 *         cdef Accumulator runningPositive = PositivePart(build_holder(x) - Shift(1, build_holder(x)))             # <<<<<<<<<<<<<<
 *         self._runningAverage = MovingAverage(window, x=runningPositive)
 * 
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_int_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift), __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Subtract(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart), __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_runningPositive = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":427
 *         super(MovingPositiveDifferenceAverage, self).__init__(window, x)
 *         cdef Accumulator runningPositive = PositivePart(build_holder(x) - Shift(1, build_holder(x)))
 *         self._runningAverage = MovingAverage(window, x=runningPositive)             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_window);
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_x, ((PyObject *)__pyx_v_runningPositive)) < 0) __PYX_ERR(0, 427, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage), __pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_runningAverage);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_runningAverage));
  __pyx_v_self->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":424
 * cdef class MovingPositiveDifferenceAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingPositiveDifferenceAverage, self).__init__(window, x)
 *         cdef Accumulator runningPositive = PositivePart(build_holder(x) - Shift(1, build_holder(x)))
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_runningPositive);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":429
 *         self._runningAverage = MovingAverage(window, x=runningPositive)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._runningAverage.push(data)
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 429, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":430
 * 
 *     cpdef push(self, dict data):
 *         self._runningAverage.push(data)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_runningAverage->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_runningAverage), __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":431
 *     cpdef push(self, dict data):
 *         self._runningAverage.push(data)
 *         self._isFull = self._isFull or self._runningAverage.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_runningAverage->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_runningAverage), 0) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":429
 *         self._runningAverage = MovingAverage(window, x=runningPositive)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._runningAverage.push(data)
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_2push[] = "MovingPositiveDifferenceAverage.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 429, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":433
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningAverage.result()
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 433, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 433, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 433, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":434
 * 
 *     cpdef double result(self):
 *         return self._runningAverage.result()             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_runningAverage->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_runningAverage), 0);
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":433
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningAverage.result()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_4result[] = "MovingPositiveDifferenceAverage.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":436
 *         return self._runningAverage.result()
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mposdiffavg}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":437
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mposdiffavg}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mposdiffavg_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":436
 *         return self._runningAverage.result()
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mposdiffavg}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":130
 * cdef class MovingPositiveDifferenceAverage(SingleValuedValueHolder):
 * 
 *     cdef public MovingAverage _runningAverage             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_runningAverage));
  __pyx_r = ((PyObject *)__pyx_v_self->_runningAverage);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage))))) __PYX_ERR(1, 130, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_runningAverage);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_runningAverage));
  __pyx_v_self->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage._runningAverage.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_runningAverage);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_runningAverage));
  __pyx_v_self->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_8__reduce_cython__[] = "MovingPositiveDifferenceAverage.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningAverage, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(6); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_runningAverage));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_runningAverage));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->_runningAverage));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_3, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningAverage, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningAverage, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningAverage, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_runningAverage) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingPositiveDif); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_48560527);
    __Pyx_GIVEREF(__pyx_int_48560527);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_48560527);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, None), state
 *     else:
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_MovingPositiveDif); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_48560527);
    __Pyx_GIVEREF(__pyx_int_48560527);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_48560527);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_10__setstate_cython__[] = "MovingPositiveDifferenceAverage.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingPositiveDifferenceAverage__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingPositiveDifferenceAverage, (type(self), 0x2e4f98f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":442
 * cdef class MovingNegativeDifferenceAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingNegativeDifferenceAverage, self).__init__(window, x)
 *         cdef Accumulator runningNegative = NegativePart(build_holder(x) - Shift(1, build_holder(x)))
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 442, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 442, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 442, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *__pyx_v_runningNegative = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":443
 * 
 *     def __init__(self, window, x):
 *         super(MovingNegativeDifferenceAverage, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         cdef Accumulator runningNegative = NegativePart(build_holder(x) - Shift(1, build_holder(x)))
 *         self._runningAverage = MovingAverage(window, x=runningNegative)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 443, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":444
 *     def __init__(self, window, x):
 *         super(MovingNegativeDifferenceAverage, self).__init__(window, x)
 *         cdef Accumulator runningNegative = NegativePart(build_holder(x) - Shift(1, build_holder(x)))             # <<<<<<<<<<<<<<
 *         self._runningAverage = MovingAverage(window, x=runningNegative)
 * 
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_int_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift), __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Subtract(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart), __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_runningNegative = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":445
 *         super(MovingNegativeDifferenceAverage, self).__init__(window, x)
 *         cdef Accumulator runningNegative = NegativePart(build_holder(x) - Shift(1, build_holder(x)))
 *         self._runningAverage = MovingAverage(window, x=runningNegative)             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_window);
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_x, ((PyObject *)__pyx_v_runningNegative)) < 0) __PYX_ERR(0, 445, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage), __pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_runningAverage);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_runningAverage));
  __pyx_v_self->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":442
 * cdef class MovingNegativeDifferenceAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingNegativeDifferenceAverage, self).__init__(window, x)
 *         cdef Accumulator runningNegative = NegativePart(build_holder(x) - Shift(1, build_holder(x)))
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_runningNegative);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":447
 *         self._runningAverage = MovingAverage(window, x=runningNegative)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._runningAverage.push(data)
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 447, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 447, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":448
 * 
 *     cpdef push(self, dict data):
 *         self._runningAverage.push(data)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_runningAverage->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_runningAverage), __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":449
 *     cpdef push(self, dict data):
 *         self._runningAverage.push(data)
 *         self._isFull = self._isFull or self._runningAverage.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_runningAverage->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_runningAverage), 0) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":447
 *         self._runningAverage = MovingAverage(window, x=runningNegative)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._runningAverage.push(data)
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_2push[] = "MovingNegativeDifferenceAverage.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 447, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":451
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningAverage.result()
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 451, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 451, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 451, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":452
 * 
 *     cpdef double result(self):
 *         return self._runningAverage.result()             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_runningAverage->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_runningAverage), 0);
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":451
 *         self._isFull = self._isFull or self._runningAverage.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningAverage.result()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_4result[] = "MovingNegativeDifferenceAverage.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":454
 *         return self._runningAverage.result()
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mnegdiffavg}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":455
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mnegdiffavg}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mnegdiffavg_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":454
 *         return self._runningAverage.result()
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mnegdiffavg}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":138
 * cdef class MovingNegativeDifferenceAverage(SingleValuedValueHolder):
 * 
 *     cdef public MovingAverage _runningAverage             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_runningAverage));
  __pyx_r = ((PyObject *)__pyx_v_self->_runningAverage);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage))))) __PYX_ERR(1, 138, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_runningAverage);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_runningAverage));
  __pyx_v_self->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage._runningAverage.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_runningAverage);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_runningAverage));
  __pyx_v_self->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_8__reduce_cython__[] = "MovingNegativeDifferenceAverage.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningAverage, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(6); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_runningAverage));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_runningAverage));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->_runningAverage));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_3, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningAverage, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningAverage, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningAverage, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_runningAverage) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingNegativeDif); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_48560527);
    __Pyx_GIVEREF(__pyx_int_48560527);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_48560527);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._runningAverage is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, None), state
 *     else:
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_MovingNegativeDif); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_48560527);
    __Pyx_GIVEREF(__pyx_int_48560527);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_48560527);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_10__setstate_cython__[] = "MovingNegativeDifferenceAverage.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeDifferenceAverage__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingNegativeDifferenceAverage, (type(self), 0x2e4f98f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":460
 * cdef class MovingRSI(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingRSI, self).__init__(window, x)
 *         self._posDiffAvg = MovingPositiveDifferenceAverage(window, x)
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 460, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 460, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 460, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":461
 * 
 *     def __init__(self, window, x):
 *         super(MovingRSI, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._posDiffAvg = MovingPositiveDifferenceAverage(window, x)
 *         self._negDiffAvg = MovingNegativeDifferenceAverage(window, x)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 461, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":462
 *     def __init__(self, window, x):
 *         super(MovingRSI, self).__init__(window, x)
 *         self._posDiffAvg = MovingPositiveDifferenceAverage(window, x)             # <<<<<<<<<<<<<<
 *         self._negDiffAvg = MovingNegativeDifferenceAverage(window, x)
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_window);
  __Pyx_INCREF(__pyx_v_x);
  __Pyx_GIVEREF(__pyx_v_x);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_x);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_posDiffAvg);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_posDiffAvg));
  __pyx_v_self->_posDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":463
 *         super(MovingRSI, self).__init__(window, x)
 *         self._posDiffAvg = MovingPositiveDifferenceAverage(window, x)
 *         self._negDiffAvg = MovingNegativeDifferenceAverage(window, x)             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_window);
  __Pyx_INCREF(__pyx_v_x);
  __Pyx_GIVEREF(__pyx_v_x);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_x);
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_negDiffAvg);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_negDiffAvg));
  __pyx_v_self->_negDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":460
 * cdef class MovingRSI(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingRSI, self).__init__(window, x)
 *         self._posDiffAvg = MovingPositiveDifferenceAverage(window, x)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":465
 *         self._negDiffAvg = MovingNegativeDifferenceAverage(window, x)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._posDiffAvg.push(data)
 *         self._negDiffAvg.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":466
 * 
 *     cpdef push(self, dict data):
 *         self._posDiffAvg.push(data)             # <<<<<<<<<<<<<<
 *         self._negDiffAvg.push(data)
 *         self._isFull = self._isFull or (self._posDiffAvg.isFull() and self._negDiffAvg.isFull())
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self->_posDiffAvg->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_posDiffAvg), __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":467
 *     cpdef push(self, dict data):
 *         self._posDiffAvg.push(data)
 *         self._negDiffAvg.push(data)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or (self._posDiffAvg.isFull() and self._negDiffAvg.isFull())
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self->_negDiffAvg->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_negDiffAvg), __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":468
 *         self._posDiffAvg.push(data)
 *         self._negDiffAvg.push(data)
 *         self._isFull = self._isFull or (self._posDiffAvg.isFull() and self._negDiffAvg.isFull())             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self->_posDiffAvg->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_posDiffAvg), 0) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self->_negDiffAvg->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_negDiffAvg), 0) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":465
 *         self._negDiffAvg = MovingNegativeDifferenceAverage(window, x)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._posDiffAvg.push(data)
 *         self._negDiffAvg.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_2push[] = "MovingRSI.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 465, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":471
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double nominator = self._posDiffAvg.result()
 *         cdef double denominator = nominator - self._negDiffAvg.result()
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_v_nominator;
  double __pyx_v_denominator;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 471, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 471, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 471, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":472
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         cdef double nominator = self._posDiffAvg.result()             # <<<<<<<<<<<<<<
 *         cdef double denominator = nominator - self._negDiffAvg.result()
 *         if denominator:
 */
  __pyx_v_nominator = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v_self->_posDiffAvg->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_posDiffAvg), 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":473
 *     cpdef double result(self):
 *         cdef double nominator = self._posDiffAvg.result()
 *         cdef double denominator = nominator - self._negDiffAvg.result()             # <<<<<<<<<<<<<<
 *         if denominator:
 *             return 100. * nominator / denominator
 */
  __pyx_v_denominator = (__pyx_v_nominator - ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v_self->_negDiffAvg->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_negDiffAvg), 0));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":474
 *         cdef double nominator = self._posDiffAvg.result()
 *         cdef double denominator = nominator - self._negDiffAvg.result()
 *         if denominator:             # <<<<<<<<<<<<<<
 *             return 100. * nominator / denominator
 *         else:
 */
  __pyx_t_6 = (__pyx_v_denominator != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":475
 *         cdef double denominator = nominator - self._negDiffAvg.result()
 *         if denominator:
 *             return 100. * nominator / denominator             # <<<<<<<<<<<<<<
 *         else:
 *             return 50.
 */
    __pyx_r = ((100. * __pyx_v_nominator) / __pyx_v_denominator);
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":474
 *         cdef double nominator = self._posDiffAvg.result()
 *         cdef double denominator = nominator - self._negDiffAvg.result()
 *         if denominator:             # <<<<<<<<<<<<<<
 *             return 100. * nominator / denominator
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":477
 *             return 100. * nominator / denominator
 *         else:
 *             return 50.             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = 50.;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":471
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double nominator = self._posDiffAvg.result()
 *         cdef double denominator = nominator - self._negDiffAvg.result()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_4result[] = "MovingRSI.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":479
 *             return 50.
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mrsi}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":480
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mrsi}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mrsi_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 480, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":479
 *             return 50.
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mrsi}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":146
 * cdef class MovingRSI(SingleValuedValueHolder):
 * 
 *     cdef public MovingPositiveDifferenceAverage _posDiffAvg             # <<<<<<<<<<<<<<
 *     cdef public MovingNegativeDifferenceAverage _negDiffAvg
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_posDiffAvg));
  __pyx_r = ((PyObject *)__pyx_v_self->_posDiffAvg);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage))))) __PYX_ERR(1, 146, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_posDiffAvg);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_posDiffAvg));
  __pyx_v_self->_posDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI._posDiffAvg.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_posDiffAvg);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_posDiffAvg));
  __pyx_v_self->_posDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":147
 * 
 *     cdef public MovingPositiveDifferenceAverage _posDiffAvg
 *     cdef public MovingNegativeDifferenceAverage _negDiffAvg             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_negDiffAvg));
  __pyx_r = ((PyObject *)__pyx_v_self->_negDiffAvg);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage))))) __PYX_ERR(1, 147, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_negDiffAvg);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_negDiffAvg));
  __pyx_v_self->_negDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI._negDiffAvg.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_negDiffAvg);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_negDiffAvg));
  __pyx_v_self->_negDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_8__reduce_cython__[] = "MovingRSI.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._negDiffAvg, self._posDiffAvg, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(7); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_negDiffAvg));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_negDiffAvg));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->_negDiffAvg));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_posDiffAvg));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_posDiffAvg));
  PyTuple_SET_ITEM(__pyx_t_3, 4, ((PyObject *)__pyx_v_self->_posDiffAvg));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._negDiffAvg, self._posDiffAvg, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._negDiffAvg, self._posDiffAvg, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._negDiffAvg is not None or self._posDiffAvg is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._negDiffAvg, self._posDiffAvg, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._negDiffAvg is not None or self._posDiffAvg is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_negDiffAvg) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_posDiffAvg) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    __pyx_t_5 = __pyx_t_6;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._negDiffAvg is not None or self._posDiffAvg is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._negDiffAvg is not None or self._posDiffAvg is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingRSI); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_172883081);
    __Pyx_GIVEREF(__pyx_int_172883081);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_172883081);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._negDiffAvg is not None or self._posDiffAvg is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, None), state
 *     else:
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingRSI__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_MovingRSI); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_172883081);
    __Pyx_GIVEREF(__pyx_int_172883081);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_172883081);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingRSI__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_10__setstate_cython__[] = "MovingRSI.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingRSI__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingRSI__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingRSI, (type(self), 0xa4dfc89, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingRSI__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":485
 * cdef class MovingNegativeAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingNegativeAverage, self).__init__(window, x)
 *         self._runningNegativeSum = 0.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 485, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 485, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 485, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":486
 * 
 *     def __init__(self, window, x):
 *         super(MovingNegativeAverage, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningNegativeSum = 0.0
 *         self._runningNegativeCount = 0
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":487
 *     def __init__(self, window, x):
 *         super(MovingNegativeAverage, self).__init__(window, x)
 *         self._runningNegativeSum = 0.0             # <<<<<<<<<<<<<<
 *         self._runningNegativeCount = 0
 * 
 */
  __pyx_v_self->_runningNegativeSum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":488
 *         super(MovingNegativeAverage, self).__init__(window, x)
 *         self._runningNegativeSum = 0.0
 *         self._runningNegativeCount = 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_runningNegativeCount = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":485
 * cdef class MovingNegativeAverage(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingNegativeAverage, self).__init__(window, x)
 *         self._runningNegativeSum = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":490
 *         self._runningNegativeCount = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 490, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 490, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":493
 *         cdef double popout
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":494
 * 
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(value):
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":496
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 1.)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":497
 * 
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, 1.)
 *         if value < 0.0:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":496
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 1.)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":498
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value, 1.)             # <<<<<<<<<<<<<<
 *         if value < 0.0:
 *             self._runningNegativeCount += 1
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = 1.;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":499
 *             return NAN
 *         popout = self._deque.dump(value, 1.)
 *         if value < 0.0:             # <<<<<<<<<<<<<<
 *             self._runningNegativeCount += 1
 *             self._runningNegativeSum += value
 */
  __pyx_t_5 = ((__pyx_v_value < 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":500
 *         popout = self._deque.dump(value, 1.)
 *         if value < 0.0:
 *             self._runningNegativeCount += 1             # <<<<<<<<<<<<<<
 *             self._runningNegativeSum += value
 * 
 */
    __pyx_v_self->_runningNegativeCount = (__pyx_v_self->_runningNegativeCount + 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":501
 *         if value < 0.0:
 *             self._runningNegativeCount += 1
 *             self._runningNegativeSum += value             # <<<<<<<<<<<<<<
 * 
 *         if popout < 0.0:
 */
    __pyx_v_self->_runningNegativeSum = (__pyx_v_self->_runningNegativeSum + __pyx_v_value);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":499
 *             return NAN
 *         popout = self._deque.dump(value, 1.)
 *         if value < 0.0:             # <<<<<<<<<<<<<<
 *             self._runningNegativeCount += 1
 *             self._runningNegativeSum += value
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":503
 *             self._runningNegativeSum += value
 * 
 *         if popout < 0.0:             # <<<<<<<<<<<<<<
 *             self._runningNegativeCount -= 1
 *             self._runningNegativeSum -= popout
 */
  __pyx_t_5 = ((__pyx_v_popout < 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":504
 * 
 *         if popout < 0.0:
 *             self._runningNegativeCount -= 1             # <<<<<<<<<<<<<<
 *             self._runningNegativeSum -= popout
 *         self._isFull = self._isFull or self._deque.isFull()
 */
    __pyx_v_self->_runningNegativeCount = (__pyx_v_self->_runningNegativeCount - 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":505
 *         if popout < 0.0:
 *             self._runningNegativeCount -= 1
 *             self._runningNegativeSum -= popout             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
    __pyx_v_self->_runningNegativeSum = (__pyx_v_self->_runningNegativeSum - __pyx_v_popout);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":503
 *             self._runningNegativeSum += value
 * 
 *         if popout < 0.0:             # <<<<<<<<<<<<<<
 *             self._runningNegativeCount -= 1
 *             self._runningNegativeSum -= popout
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":506
 *             self._runningNegativeCount -= 1
 *             self._runningNegativeSum -= popout
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":490
 *         self._runningNegativeCount = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_2push[] = "MovingNegativeAverage.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 490, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":509
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._runningNegativeCount:
 *             return self._runningNegativeSum / self._runningNegativeCount
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 509, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 509, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":510
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         if self._runningNegativeCount:             # <<<<<<<<<<<<<<
 *             return self._runningNegativeSum / self._runningNegativeCount
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->_runningNegativeCount != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":511
 *     cpdef double result(self):
 *         if self._runningNegativeCount:
 *             return self._runningNegativeSum / self._runningNegativeCount             # <<<<<<<<<<<<<<
 *         else:
 *             return 0.
 */
    __pyx_r = (__pyx_v_self->_runningNegativeSum / __pyx_v_self->_runningNegativeCount);
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":510
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         if self._runningNegativeCount:             # <<<<<<<<<<<<<<
 *             return self._runningNegativeSum / self._runningNegativeCount
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":513
 *             return self._runningNegativeSum / self._runningNegativeCount
 *         else:
 *             return 0.             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = 0.;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":509
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         if self._runningNegativeCount:
 *             return self._runningNegativeSum / self._runningNegativeCount
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_4result[] = "MovingNegativeAverage.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":515
 *             return 0.
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mnegavg}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":516
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mnegavg}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mnegavg_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":515
 *             return 0.
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mnegavg}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":155
 * cdef class MovingNegativeAverage(SingleValuedValueHolder):
 * 
 *     cdef public double _runningNegativeSum             # <<<<<<<<<<<<<<
 *     cdef public int _runningNegativeCount
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningNegativeSum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage._runningNegativeSum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 155, __pyx_L1_error)
  __pyx_v_self->_runningNegativeSum = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage._runningNegativeSum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":156
 * 
 *     cdef public double _runningNegativeSum
 *     cdef public int _runningNegativeCount             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_runningNegativeCount); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage._runningNegativeCount.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 156, __pyx_L1_error)
  __pyx_v_self->_runningNegativeCount = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage._runningNegativeCount.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_8__reduce_cython__[] = "MovingNegativeAverage.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningNegativeCount, self._runningNegativeSum, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->_runningNegativeCount); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_runningNegativeSum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(7); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_5, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_5, 6, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningNegativeCount, self._runningNegativeSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_5 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v__dict = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningNegativeCount, self._runningNegativeSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningNegativeCount, self._runningNegativeSum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, None), state
 */
  /*else*/ {
    __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_8 = (__pyx_t_6 != 0);
    __pyx_t_7 = __pyx_t_8;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_7;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, None), state
 *     else:
 */
  __pyx_t_7 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_MovingNegativeAve); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_162829789);
    __Pyx_GIVEREF(__pyx_int_162829789);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_162829789);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, None), state
 *     else:
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingNegativeAverage__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingNegativeAve); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_162829789);
    __Pyx_GIVEREF(__pyx_int_162829789);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_162829789);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingNegativeAverage__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_10__setstate_cython__[] = "MovingNegativeAverage.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingNegativeAverage__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeAverage__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingNegativeAverage, (type(self), 0x9b495dd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingNegativeAverage__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":521
 * cdef class MovingVariance(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x, isPopulation=False):             # <<<<<<<<<<<<<<
 *         super(MovingVariance, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_isPopulation = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,&__pyx_n_s_isPopulation,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(0, 521, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_isPopulation);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 521, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
    __pyx_v_isPopulation = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 521, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self), __pyx_v_window, __pyx_v_x, __pyx_v_isPopulation);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_isPopulation) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":522
 * 
 *     def __init__(self, window, x, isPopulation=False):
 *         super(MovingVariance, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningSum = 0.0
 *         self._runningSumSquare = 0.0
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 522, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 522, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 522, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 522, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":523
 *     def __init__(self, window, x, isPopulation=False):
 *         super(MovingVariance, self).__init__(window, x)
 *         self._runningSum = 0.0             # <<<<<<<<<<<<<<
 *         self._runningSumSquare = 0.0
 *         self._isPop = isPopulation
 */
  __pyx_v_self->_runningSum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":524
 *         super(MovingVariance, self).__init__(window, x)
 *         self._runningSum = 0.0
 *         self._runningSumSquare = 0.0             # <<<<<<<<<<<<<<
 *         self._isPop = isPopulation
 *         if not self._isPop:
 */
  __pyx_v_self->_runningSumSquare = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":525
 *         self._runningSum = 0.0
 *         self._runningSumSquare = 0.0
 *         self._isPop = isPopulation             # <<<<<<<<<<<<<<
 *         if not self._isPop:
 *             pyFinAssert(window >= 2, ValueError, "sampling variance can't be calculated with window size < 2")
 */
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_isPopulation); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 525, __pyx_L1_error)
  __pyx_v_self->_isPop = __pyx_t_4;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":526
 *         self._runningSumSquare = 0.0
 *         self._isPop = isPopulation
 *         if not self._isPop:             # <<<<<<<<<<<<<<
 *             pyFinAssert(window >= 2, ValueError, "sampling variance can't be calculated with window size < 2")
 * 
 */
  __pyx_t_6 = ((!(__pyx_v_self->_isPop != 0)) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":527
 *         self._isPop = isPopulation
 *         if not self._isPop:
 *             pyFinAssert(window >= 2, ValueError, "sampling variance can't be calculated with window size < 2")             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_window, __pyx_int_2, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 527, __pyx_L1_error)
    __pyx_t_7.__pyx_n = 1;
    __pyx_t_7.msg = __pyx_kp_s_sampling_variance_can_t_be_calcu;
    __pyx_t_4 = __pyx_f_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert(__pyx_t_1, __pyx_builtin_ValueError, 0, &__pyx_t_7); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 527, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":526
 *         self._runningSumSquare = 0.0
 *         self._isPop = isPopulation
 *         if not self._isPop:             # <<<<<<<<<<<<<<
 *             pyFinAssert(window >= 2, ValueError, "sampling variance can't be calculated with window size < 2")
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":521
 * cdef class MovingVariance(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x, isPopulation=False):             # <<<<<<<<<<<<<<
 *         super(MovingVariance, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":529
 *             pyFinAssert(window >= 2, ValueError, "sampling variance can't be calculated with window size < 2")
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 529, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 529, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":531
 *     cpdef push(self, dict data):
 *         cdef double popout
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 *         if isnan(value):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":532
 *         cdef double popout
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 *         if isnan(value):
 *             return NAN
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":533
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":534
 *         cdef double value = self._x.result()
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, 0.)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 534, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":533
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":535
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value, 0.)             # <<<<<<<<<<<<<<
 * 
 *         self._runningSum += value - popout
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = 0.;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":537
 *         popout = self._deque.dump(value, 0.)
 * 
 *         self._runningSum += value - popout             # <<<<<<<<<<<<<<
 *         self._runningSumSquare += value * value - popout * popout
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_v_self->_runningSum = (__pyx_v_self->_runningSum + (__pyx_v_value - __pyx_v_popout));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":538
 * 
 *         self._runningSum += value - popout
 *         self._runningSumSquare += value * value - popout * popout             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  __pyx_v_self->_runningSumSquare = (__pyx_v_self->_runningSumSquare + ((__pyx_v_value * __pyx_v_value) - (__pyx_v_popout * __pyx_v_popout)));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":539
 *         self._runningSum += value - popout
 *         self._runningSumSquare += value * value - popout * popout
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":529
 *             pyFinAssert(window >= 2, ValueError, "sampling variance can't be calculated with window size < 2")
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_2push[] = "MovingVariance.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 529, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":542
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t length = self._deque.size()
 *         cdef double tmp
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_v_length;
  double __pyx_v_tmp;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 542, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":543
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         cdef size_t length = self._deque.size()             # <<<<<<<<<<<<<<
 *         cdef double tmp
 * 
 */
  __pyx_v_length = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->size(__pyx_v_self->__pyx_base.__pyx_base._deque);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":546
 *         cdef double tmp
 * 
 *         if length == 0:             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  __pyx_t_6 = ((__pyx_v_length == 0) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":547
 * 
 *         if length == 0:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *         tmp = self._runningSumSquare - self._runningSum * self._runningSum / length
 */
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":546
 *         cdef double tmp
 * 
 *         if length == 0:             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":549
 *             return NAN
 * 
 *         tmp = self._runningSumSquare - self._runningSum * self._runningSum / length             # <<<<<<<<<<<<<<
 * 
 *         if self._isPop:
 */
  __pyx_v_tmp = (__pyx_v_self->_runningSumSquare - ((__pyx_v_self->_runningSum * __pyx_v_self->_runningSum) / __pyx_v_length));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":551
 *         tmp = self._runningSumSquare - self._runningSum * self._runningSum / length
 * 
 *         if self._isPop:             # <<<<<<<<<<<<<<
 *             return tmp / length
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->_isPop != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":552
 * 
 *         if self._isPop:
 *             return tmp / length             # <<<<<<<<<<<<<<
 *         else:
 *             if length >= 2:
 */
    __pyx_r = (__pyx_v_tmp / __pyx_v_length);
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":551
 *         tmp = self._runningSumSquare - self._runningSum * self._runningSum / length
 * 
 *         if self._isPop:             # <<<<<<<<<<<<<<
 *             return tmp / length
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":554
 *             return tmp / length
 *         else:
 *             if length >= 2:             # <<<<<<<<<<<<<<
 *                 return tmp / (length - 1)
 *             else:
 */
  /*else*/ {
    __pyx_t_6 = ((__pyx_v_length >= 2) != 0);
    if (__pyx_t_6) {

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":555
 *         else:
 *             if length >= 2:
 *                 return tmp / (length - 1)             # <<<<<<<<<<<<<<
 *             else:
 *                 return NAN
 */
      __pyx_r = (__pyx_v_tmp / (__pyx_v_length - 1));
      goto __pyx_L0;

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":554
 *             return tmp / length
 *         else:
 *             if length >= 2:             # <<<<<<<<<<<<<<
 *                 return tmp / (length - 1)
 *             else:
 */
    }

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":557
 *                 return tmp / (length - 1)
 *             else:
 *                 return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
    /*else*/ {
      __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
      goto __pyx_L0;
    }
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":542
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t length = self._deque.size()
 *         cdef double tmp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_4result[] = "MovingVariance.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":559
 *                 return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mvar}}({0}, {1}, {2})".format(self._window, str(self._x), self._isPop)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":560
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mvar}}({0}, {1}, {2})".format(self._window, str(self._x), self._isPop)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mvar_0_1_2, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->_isPop); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 560, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":559
 *                 return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mvar}}({0}, {1}, {2})".format(self._window, str(self._x), self._isPop)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":164
 * cdef class MovingVariance(SingleValuedValueHolder):
 * 
 *     cdef public double _runningSum             # <<<<<<<<<<<<<<
 *     cdef public double _runningSumSquare
 *     cdef public int _isPop
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance._runningSum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 164, __pyx_L1_error)
  __pyx_v_self->_runningSum = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance._runningSum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":165
 * 
 *     cdef public double _runningSum
 *     cdef public double _runningSumSquare             # <<<<<<<<<<<<<<
 *     cdef public int _isPop
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSumSquare); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance._runningSumSquare.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 165, __pyx_L1_error)
  __pyx_v_self->_runningSumSquare = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance._runningSumSquare.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":166
 *     cdef public double _runningSum
 *     cdef public double _runningSumSquare
 *     cdef public int _isPop             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_isPop); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance._isPop.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 166, __pyx_L1_error)
  __pyx_v_self->_isPop = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance._isPop.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_8__reduce_cython__[] = "MovingVariance.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningSum, self._runningSumSquare, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->_isPop); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->_runningSumSquare); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(8); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_6, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_6, 7, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningSum, self._runningSumSquare, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningSum, self._runningSumSquare, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningSum, self._runningSumSquare, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_MovingVariance); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_266717091);
    __Pyx_GIVEREF(__pyx_int_266717091);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_266717091);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, None), state
 *     else:
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingVariance__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_MovingVariance); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_266717091);
    __Pyx_GIVEREF(__pyx_int_266717091);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_266717091);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingVariance__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_10__setstate_cython__[] = "MovingVariance.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingVariance__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingVariance__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingVariance, (type(self), 0xfe5c7a3, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingVariance__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":565
 * cdef class MovingStandardDeviation(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x, isPopulation=False):             # <<<<<<<<<<<<<<
 *         super(MovingStandardDeviation, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_isPopulation = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,&__pyx_n_s_isPopulation,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(0, 565, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_isPopulation);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 565, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
    __pyx_v_isPopulation = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 565, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self), __pyx_v_window, __pyx_v_x, __pyx_v_isPopulation);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_isPopulation) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":566
 * 
 *     def __init__(self, window, x, isPopulation=False):
 *         super(MovingStandardDeviation, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningSum = 0.0
 *         self._runningSumSquare = 0.0
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 566, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 566, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 566, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 566, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":567
 *     def __init__(self, window, x, isPopulation=False):
 *         super(MovingStandardDeviation, self).__init__(window, x)
 *         self._runningSum = 0.0             # <<<<<<<<<<<<<<
 *         self._runningSumSquare = 0.0
 *         self._isPop = isPopulation
 */
  __pyx_v_self->_runningSum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":568
 *         super(MovingStandardDeviation, self).__init__(window, x)
 *         self._runningSum = 0.0
 *         self._runningSumSquare = 0.0             # <<<<<<<<<<<<<<
 *         self._isPop = isPopulation
 *         if not self._isPop:
 */
  __pyx_v_self->_runningSumSquare = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":569
 *         self._runningSum = 0.0
 *         self._runningSumSquare = 0.0
 *         self._isPop = isPopulation             # <<<<<<<<<<<<<<
 *         if not self._isPop:
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 */
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_isPopulation); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 569, __pyx_L1_error)
  __pyx_v_self->_isPop = __pyx_t_4;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":570
 *         self._runningSumSquare = 0.0
 *         self._isPop = isPopulation
 *         if not self._isPop:             # <<<<<<<<<<<<<<
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 * 
 */
  __pyx_t_6 = ((!(__pyx_v_self->_isPop != 0)) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":571
 *         self._isPop = isPopulation
 *         if not self._isPop:
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_window, __pyx_int_2, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 571, __pyx_L1_error)
    __pyx_t_7.__pyx_n = 1;
    __pyx_t_7.msg = __pyx_kp_s_sampling_standard_deviation_can;
    __pyx_t_4 = __pyx_f_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert(__pyx_t_1, __pyx_builtin_ValueError, 0, &__pyx_t_7); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 571, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":570
 *         self._runningSumSquare = 0.0
 *         self._isPop = isPopulation
 *         if not self._isPop:             # <<<<<<<<<<<<<<
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":565
 * cdef class MovingStandardDeviation(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x, isPopulation=False):             # <<<<<<<<<<<<<<
 *         super(MovingStandardDeviation, self).__init__(window, x)
 *         self._runningSum = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":573
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 573, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":575
 *     cpdef push(self, dict data):
 *         cdef double popout
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":576
 *         cdef double popout
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(value):
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":578
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":579
 * 
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, 0.)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":578
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 0.)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":580
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value, 0.)             # <<<<<<<<<<<<<<
 * 
 *         self._runningSum += value - popout
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = 0.;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":582
 *         popout = self._deque.dump(value, 0.)
 * 
 *         self._runningSum += value - popout             # <<<<<<<<<<<<<<
 *         self._runningSumSquare += value * value - popout * popout
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_v_self->_runningSum = (__pyx_v_self->_runningSum + (__pyx_v_value - __pyx_v_popout));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":583
 * 
 *         self._runningSum += value - popout
 *         self._runningSumSquare += value * value - popout * popout             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  __pyx_v_self->_runningSumSquare = (__pyx_v_self->_runningSumSquare + ((__pyx_v_value * __pyx_v_value) - (__pyx_v_popout * __pyx_v_popout)));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":584
 *         self._runningSum += value - popout
 *         self._runningSumSquare += value * value - popout * popout
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":573
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_2push[] = "MovingStandardDeviation.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 573, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":587
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t length = self._deque.size()
 *         cdef double tmp
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_v_length;
  double __pyx_v_tmp;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 587, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 587, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 587, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":588
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         cdef size_t length = self._deque.size()             # <<<<<<<<<<<<<<
 *         cdef double tmp
 * 
 */
  __pyx_v_length = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->size(__pyx_v_self->__pyx_base.__pyx_base._deque);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":591
 *         cdef double tmp
 * 
 *         if length == 0:             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  __pyx_t_6 = ((__pyx_v_length == 0) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":592
 * 
 *         if length == 0:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *         tmp = self._runningSumSquare - self._runningSum * self._runningSum / length
 */
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":591
 *         cdef double tmp
 * 
 *         if length == 0:             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":594
 *             return NAN
 * 
 *         tmp = self._runningSumSquare - self._runningSum * self._runningSum / length             # <<<<<<<<<<<<<<
 * 
 *         if self._isPop:
 */
  __pyx_v_tmp = (__pyx_v_self->_runningSumSquare - ((__pyx_v_self->_runningSum * __pyx_v_self->_runningSum) / __pyx_v_length));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":596
 *         tmp = self._runningSumSquare - self._runningSum * self._runningSum / length
 * 
 *         if self._isPop:             # <<<<<<<<<<<<<<
 *             return sqrt(tmp / length)
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->_isPop != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":597
 * 
 *         if self._isPop:
 *             return sqrt(tmp / length)             # <<<<<<<<<<<<<<
 *         else:
 *             if length >= 2:
 */
    __pyx_r = sqrt((__pyx_v_tmp / __pyx_v_length));
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":596
 *         tmp = self._runningSumSquare - self._runningSum * self._runningSum / length
 * 
 *         if self._isPop:             # <<<<<<<<<<<<<<
 *             return sqrt(tmp / length)
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":599
 *             return sqrt(tmp / length)
 *         else:
 *             if length >= 2:             # <<<<<<<<<<<<<<
 *                 return sqrt(tmp / (length - 1))
 *             else:
 */
  /*else*/ {
    __pyx_t_6 = ((__pyx_v_length >= 2) != 0);
    if (__pyx_t_6) {

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":600
 *         else:
 *             if length >= 2:
 *                 return sqrt(tmp / (length - 1))             # <<<<<<<<<<<<<<
 *             else:
 *                 return NAN
 */
      __pyx_r = sqrt((__pyx_v_tmp / (__pyx_v_length - 1)));
      goto __pyx_L0;

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":599
 *             return sqrt(tmp / length)
 *         else:
 *             if length >= 2:             # <<<<<<<<<<<<<<
 *                 return sqrt(tmp / (length - 1))
 *             else:
 */
    }

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":602
 *                 return sqrt(tmp / (length - 1))
 *             else:
 *                 return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
    /*else*/ {
      __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
      goto __pyx_L0;
    }
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":587
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t length = self._deque.size()
 *         cdef double tmp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_4result[] = "MovingStandardDeviation.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":604
 *                 return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mstd}}({0}, {1}, {2})".format(self._window, str(self._x), self._isPop)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":605
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mstd}}({0}, {1}, {2})".format(self._window, str(self._x), self._isPop)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mstd_0_1_2, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->_isPop); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":604
 *                 return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mstd}}({0}, {1}, {2})".format(self._window, str(self._x), self._isPop)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":174
 * cdef class MovingStandardDeviation(SingleValuedValueHolder):
 * 
 *     cdef public double _runningSum             # <<<<<<<<<<<<<<
 *     cdef public double _runningSumSquare
 *     cdef public int _isPop
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation._runningSum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 174, __pyx_L1_error)
  __pyx_v_self->_runningSum = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation._runningSum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":175
 * 
 *     cdef public double _runningSum
 *     cdef public double _runningSumSquare             # <<<<<<<<<<<<<<
 *     cdef public int _isPop
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSumSquare); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation._runningSumSquare.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 175, __pyx_L1_error)
  __pyx_v_self->_runningSumSquare = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation._runningSumSquare.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":176
 *     cdef public double _runningSum
 *     cdef public double _runningSumSquare
 *     cdef public int _isPop             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_isPop); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation._isPop.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 176, __pyx_L1_error)
  __pyx_v_self->_isPop = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation._isPop.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_8__reduce_cython__[] = "MovingStandardDeviation.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningSum, self._runningSumSquare, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->_isPop); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_runningSum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->_runningSumSquare); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(8); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_6, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_6, 7, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningSum, self._runningSumSquare, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_6 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_v__dict = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningSum, self._runningSumSquare, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_7 = (__pyx_v__dict != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v__dict);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningSum, self._runningSumSquare, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, None), state
 */
  /*else*/ {
    __pyx_t_7 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_7 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_7 = (__pyx_t_9 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_9 = (__pyx_t_7 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_8;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, None), state
 *     else:
 */
  __pyx_t_8 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_8) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_MovingStandardDev); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_266717091);
    __Pyx_GIVEREF(__pyx_int_266717091);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_266717091);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, None), state
 *     else:
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingStandardDeviation__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_MovingStandardDev); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_266717091);
    __Pyx_GIVEREF(__pyx_int_266717091);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_266717091);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingStandardDeviation__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_10__setstate_cython__[] = "MovingStandardDeviation.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingStandardDeviation__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingStandardDeviation__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingStandardDeviation, (type(self), 0xfe5c7a3, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingStandardDeviation__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":610
 * cdef class MovingNegativeVariance(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x, isPopulation=0):             # <<<<<<<<<<<<<<
 *         super(MovingNegativeVariance, self).__init__(window, x)
 *         self._runningNegativeSum = 0.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_isPopulation = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,&__pyx_n_s_isPopulation,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)__pyx_int_0);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(0, 610, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_isPopulation);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 610, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
    __pyx_v_isPopulation = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 610, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self), __pyx_v_window, __pyx_v_x, __pyx_v_isPopulation);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_isPopulation) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":611
 * 
 *     def __init__(self, window, x, isPopulation=0):
 *         super(MovingNegativeVariance, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningNegativeSum = 0.0
 *         self._runningNegativeSumSquare = 0.0
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 611, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 611, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 611, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 611, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 611, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 611, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 611, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":612
 *     def __init__(self, window, x, isPopulation=0):
 *         super(MovingNegativeVariance, self).__init__(window, x)
 *         self._runningNegativeSum = 0.0             # <<<<<<<<<<<<<<
 *         self._runningNegativeSumSquare = 0.0
 *         self._runningNegativeCount = 0
 */
  __pyx_v_self->_runningNegativeSum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":613
 *         super(MovingNegativeVariance, self).__init__(window, x)
 *         self._runningNegativeSum = 0.0
 *         self._runningNegativeSumSquare = 0.0             # <<<<<<<<<<<<<<
 *         self._runningNegativeCount = 0
 *         self._isPop = isPopulation
 */
  __pyx_v_self->_runningNegativeSumSquare = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":614
 *         self._runningNegativeSum = 0.0
 *         self._runningNegativeSumSquare = 0.0
 *         self._runningNegativeCount = 0             # <<<<<<<<<<<<<<
 *         self._isPop = isPopulation
 *         if not self._isPop:
 */
  __pyx_v_self->_runningNegativeCount = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":615
 *         self._runningNegativeSumSquare = 0.0
 *         self._runningNegativeCount = 0
 *         self._isPop = isPopulation             # <<<<<<<<<<<<<<
 *         if not self._isPop:
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 */
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_isPopulation); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 615, __pyx_L1_error)
  __pyx_v_self->_isPop = __pyx_t_4;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":616
 *         self._runningNegativeCount = 0
 *         self._isPop = isPopulation
 *         if not self._isPop:             # <<<<<<<<<<<<<<
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 * 
 */
  __pyx_t_6 = ((!(__pyx_v_self->_isPop != 0)) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":617
 *         self._isPop = isPopulation
 *         if not self._isPop:
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_window, __pyx_int_2, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 617, __pyx_L1_error)
    __pyx_t_7.__pyx_n = 1;
    __pyx_t_7.msg = __pyx_kp_s_sampling_standard_deviation_can;
    __pyx_t_4 = __pyx_f_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert(__pyx_t_1, __pyx_builtin_ValueError, 0, &__pyx_t_7); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 617, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":616
 *         self._runningNegativeCount = 0
 *         self._isPop = isPopulation
 *         if not self._isPop:             # <<<<<<<<<<<<<<
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":610
 * cdef class MovingNegativeVariance(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x, isPopulation=0):             # <<<<<<<<<<<<<<
 *         super(MovingNegativeVariance, self).__init__(window, x)
 *         self._runningNegativeSum = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":619
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 619, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":621
 *     cpdef push(self, dict data):
 *         cdef double popout
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":622
 *         cdef double popout
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(value):
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":624
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 1.)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":625
 * 
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, 1.)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 625, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":624
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 1.)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":626
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value, 1.)             # <<<<<<<<<<<<<<
 * 
 *         if value < 0:
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = 1.;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":628
 *         popout = self._deque.dump(value, 1.)
 * 
 *         if value < 0:             # <<<<<<<<<<<<<<
 *             self._runningNegativeSum += value
 *             self._runningNegativeSumSquare += value * value
 */
  __pyx_t_5 = ((__pyx_v_value < 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":629
 * 
 *         if value < 0:
 *             self._runningNegativeSum += value             # <<<<<<<<<<<<<<
 *             self._runningNegativeSumSquare += value * value
 *             self._runningNegativeCount += 1
 */
    __pyx_v_self->_runningNegativeSum = (__pyx_v_self->_runningNegativeSum + __pyx_v_value);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":630
 *         if value < 0:
 *             self._runningNegativeSum += value
 *             self._runningNegativeSumSquare += value * value             # <<<<<<<<<<<<<<
 *             self._runningNegativeCount += 1
 *         if popout < 0:
 */
    __pyx_v_self->_runningNegativeSumSquare = (__pyx_v_self->_runningNegativeSumSquare + (__pyx_v_value * __pyx_v_value));

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":631
 *             self._runningNegativeSum += value
 *             self._runningNegativeSumSquare += value * value
 *             self._runningNegativeCount += 1             # <<<<<<<<<<<<<<
 *         if popout < 0:
 *             self._runningNegativeSum -= popout
 */
    __pyx_v_self->_runningNegativeCount = (__pyx_v_self->_runningNegativeCount + 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":628
 *         popout = self._deque.dump(value, 1.)
 * 
 *         if value < 0:             # <<<<<<<<<<<<<<
 *             self._runningNegativeSum += value
 *             self._runningNegativeSumSquare += value * value
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":632
 *             self._runningNegativeSumSquare += value * value
 *             self._runningNegativeCount += 1
 *         if popout < 0:             # <<<<<<<<<<<<<<
 *             self._runningNegativeSum -= popout
 *             self._runningNegativeSumSquare -= popout * popout
 */
  __pyx_t_5 = ((__pyx_v_popout < 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":633
 *             self._runningNegativeCount += 1
 *         if popout < 0:
 *             self._runningNegativeSum -= popout             # <<<<<<<<<<<<<<
 *             self._runningNegativeSumSquare -= popout * popout
 *             self._runningNegativeCount -= 1
 */
    __pyx_v_self->_runningNegativeSum = (__pyx_v_self->_runningNegativeSum - __pyx_v_popout);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":634
 *         if popout < 0:
 *             self._runningNegativeSum -= popout
 *             self._runningNegativeSumSquare -= popout * popout             # <<<<<<<<<<<<<<
 *             self._runningNegativeCount -= 1
 *         self._isFull = self._isFull or self._deque.isFull()
 */
    __pyx_v_self->_runningNegativeSumSquare = (__pyx_v_self->_runningNegativeSumSquare - (__pyx_v_popout * __pyx_v_popout));

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":635
 *             self._runningNegativeSum -= popout
 *             self._runningNegativeSumSquare -= popout * popout
 *             self._runningNegativeCount -= 1             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
    __pyx_v_self->_runningNegativeCount = (__pyx_v_self->_runningNegativeCount - 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":632
 *             self._runningNegativeSumSquare += value * value
 *             self._runningNegativeCount += 1
 *         if popout < 0:             # <<<<<<<<<<<<<<
 *             self._runningNegativeSum -= popout
 *             self._runningNegativeSumSquare -= popout * popout
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":636
 *             self._runningNegativeSumSquare -= popout * popout
 *             self._runningNegativeCount -= 1
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":619
 *             pyFinAssert(window >= 2, ValueError, "sampling standard deviation can't be calculated with window size < 2")
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_2push[] = "MovingNegativeVariance.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 619, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":639
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 * 
 *         cdef int length
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_v_length;
  double __pyx_v_tmp;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 639, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 639, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":644
 *         cdef double tmp
 * 
 *         if self._isPop:             # <<<<<<<<<<<<<<
 *             if self._runningNegativeCount >= 1:
 *                 length = self._runningNegativeCount
 */
  __pyx_t_6 = (__pyx_v_self->_isPop != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":645
 * 
 *         if self._isPop:
 *             if self._runningNegativeCount >= 1:             # <<<<<<<<<<<<<<
 *                 length = self._runningNegativeCount
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length
 */
    __pyx_t_6 = ((__pyx_v_self->_runningNegativeCount >= 1) != 0);
    if (__pyx_t_6) {

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":646
 *         if self._isPop:
 *             if self._runningNegativeCount >= 1:
 *                 length = self._runningNegativeCount             # <<<<<<<<<<<<<<
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length
 *                 return tmp / length
 */
      __pyx_t_7 = __pyx_v_self->_runningNegativeCount;
      __pyx_v_length = __pyx_t_7;

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":647
 *             if self._runningNegativeCount >= 1:
 *                 length = self._runningNegativeCount
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length             # <<<<<<<<<<<<<<
 *                 return tmp / length
 *             else:
 */
      __pyx_v_tmp = (__pyx_v_self->_runningNegativeSumSquare - ((__pyx_v_self->_runningNegativeSum * __pyx_v_self->_runningNegativeSum) / __pyx_v_length));

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":648
 *                 length = self._runningNegativeCount
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length
 *                 return tmp / length             # <<<<<<<<<<<<<<
 *             else:
 *                 return NAN
 */
      __pyx_r = (__pyx_v_tmp / __pyx_v_length);
      goto __pyx_L0;

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":645
 * 
 *         if self._isPop:
 *             if self._runningNegativeCount >= 1:             # <<<<<<<<<<<<<<
 *                 length = self._runningNegativeCount
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length
 */
    }

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":650
 *                 return tmp / length
 *             else:
 *                 return NAN             # <<<<<<<<<<<<<<
 *         else:
 *             if self._runningNegativeCount >= 2:
 */
    /*else*/ {
      __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
      goto __pyx_L0;
    }

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":644
 *         cdef double tmp
 * 
 *         if self._isPop:             # <<<<<<<<<<<<<<
 *             if self._runningNegativeCount >= 1:
 *                 length = self._runningNegativeCount
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":652
 *                 return NAN
 *         else:
 *             if self._runningNegativeCount >= 2:             # <<<<<<<<<<<<<<
 *                 length = self._runningNegativeCount
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length
 */
  /*else*/ {
    __pyx_t_6 = ((__pyx_v_self->_runningNegativeCount >= 2) != 0);
    if (__pyx_t_6) {

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":653
 *         else:
 *             if self._runningNegativeCount >= 2:
 *                 length = self._runningNegativeCount             # <<<<<<<<<<<<<<
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length
 *                 return tmp / (length - 1)
 */
      __pyx_t_7 = __pyx_v_self->_runningNegativeCount;
      __pyx_v_length = __pyx_t_7;

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":654
 *             if self._runningNegativeCount >= 2:
 *                 length = self._runningNegativeCount
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length             # <<<<<<<<<<<<<<
 *                 return tmp / (length - 1)
 *             else:
 */
      __pyx_v_tmp = (__pyx_v_self->_runningNegativeSumSquare - ((__pyx_v_self->_runningNegativeSum * __pyx_v_self->_runningNegativeSum) / __pyx_v_length));

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":655
 *                 length = self._runningNegativeCount
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length
 *                 return tmp / (length - 1)             # <<<<<<<<<<<<<<
 *             else:
 *                 return NAN
 */
      __pyx_r = (__pyx_v_tmp / (__pyx_v_length - 1));
      goto __pyx_L0;

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":652
 *                 return NAN
 *         else:
 *             if self._runningNegativeCount >= 2:             # <<<<<<<<<<<<<<
 *                 length = self._runningNegativeCount
 *                 tmp = self._runningNegativeSumSquare - self._runningNegativeSum * self._runningNegativeSum / length
 */
    }

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":657
 *                 return tmp / (length - 1)
 *             else:
 *                 return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
    /*else*/ {
      __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
      goto __pyx_L0;
    }
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":639
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 * 
 *         cdef int length
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_4result[] = "MovingNegativeVariance.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":659
 *                 return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mnegvar}}({0}, {1}, {2})".format(self._window, str(self._x), self._isPop)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":660
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mnegvar}}({0}, {1}, {2})".format(self._window, str(self._x), self._isPop)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mnegvar_0_1_2, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 660, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 660, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 660, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->_isPop); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 660, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 660, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 660, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 660, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 660, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":659
 *                 return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mnegvar}}({0}, {1}, {2})".format(self._window, str(self._x), self._isPop)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":184
 * cdef class MovingNegativeVariance(SingleValuedValueHolder):
 * 
 *     cdef public double _runningNegativeSum             # <<<<<<<<<<<<<<
 *     cdef public double _runningNegativeSumSquare
 *     cdef public int _runningNegativeCount
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningNegativeSum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance._runningNegativeSum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 184, __pyx_L1_error)
  __pyx_v_self->_runningNegativeSum = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance._runningNegativeSum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":185
 * 
 *     cdef public double _runningNegativeSum
 *     cdef public double _runningNegativeSumSquare             # <<<<<<<<<<<<<<
 *     cdef public int _runningNegativeCount
 *     cdef public int _isPop
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningNegativeSumSquare); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance._runningNegativeSumSquare.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 185, __pyx_L1_error)
  __pyx_v_self->_runningNegativeSumSquare = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance._runningNegativeSumSquare.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":186
 *     cdef public double _runningNegativeSum
 *     cdef public double _runningNegativeSumSquare
 *     cdef public int _runningNegativeCount             # <<<<<<<<<<<<<<
 *     cdef public int _isPop
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_runningNegativeCount); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance._runningNegativeCount.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 186, __pyx_L1_error)
  __pyx_v_self->_runningNegativeCount = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance._runningNegativeCount.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":187
 *     cdef public double _runningNegativeSumSquare
 *     cdef public int _runningNegativeCount
 *     cdef public int _isPop             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_isPop); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance._isPop.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 187, __pyx_L1_error)
  __pyx_v_self->_isPop = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance._isPop.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_8__reduce_cython__[] = "MovingNegativeVariance.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningNegativeCount, self._runningNegativeSum, self._runningNegativeSumSquare, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->_isPop); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->_runningNegativeCount); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->_runningNegativeSum); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->_runningNegativeSumSquare); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyTuple_New(9); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_7, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_7, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_7, 5, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 6, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_7, 7, __pyx_t_6);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_7, 8, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningNegativeCount, self._runningNegativeSum, self._runningNegativeSumSquare, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_7 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_v__dict = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningNegativeCount, self._runningNegativeSum, self._runningNegativeSumSquare, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_8 = (__pyx_v__dict != Py_None);
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v__dict);
    __pyx_t_6 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._isPop, self._runningNegativeCount, self._runningNegativeSum, self._runningNegativeSumSquare, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, None), state
 */
  /*else*/ {
    __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_10 = (__pyx_t_8 != 0);
    __pyx_t_9 = __pyx_t_10;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_9;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, None), state
 *     else:
 */
  __pyx_t_9 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_9) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_MovingNegativeVar); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_7, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_126295376);
    __Pyx_GIVEREF(__pyx_int_126295376);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_int_126295376);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_7, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_7);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, None), state
 *     else:
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingNegativeVariance__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_MovingNegativeVar); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_7, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_126295376);
    __Pyx_GIVEREF(__pyx_int_126295376);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_int_126295376);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_v_state);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_7);
    __pyx_t_5 = 0;
    __pyx_t_7 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingNegativeVariance__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_10__setstate_cython__[] = "MovingNegativeVariance.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingNegativeVariance__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeVariance__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingNegativeVariance, (type(self), 0x7871d50, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingNegativeVariance__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":665
 * cdef class MovingCountedPositive(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingCountedPositive, self).__init__(window, x)
 *         self._counts = 0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 665, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 665, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 665, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":666
 * 
 *     def __init__(self, window, x):
 *         super(MovingCountedPositive, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._counts = 0
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 666, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 666, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 666, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":667
 *     def __init__(self, window, x):
 *         super(MovingCountedPositive, self).__init__(window, x)
 *         self._counts = 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_counts = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":665
 * cdef class MovingCountedPositive(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingCountedPositive, self).__init__(window, x)
 *         self._counts = 0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":669
 *         self._counts = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 669, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 669, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":671
 *     cpdef push(self, dict data):
 *         cdef double popout
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":672
 *         cdef double popout
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(value):
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":674
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, -1.)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":675
 * 
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, -1.)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 675, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":674
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, -1.)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":676
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value, -1.)             # <<<<<<<<<<<<<<
 * 
 *         if value > 0:
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = -1.;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":678
 *         popout = self._deque.dump(value, -1.)
 * 
 *         if value > 0:             # <<<<<<<<<<<<<<
 *             self._counts += 1
 *         if popout > 0:
 */
  __pyx_t_5 = ((__pyx_v_value > 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":679
 * 
 *         if value > 0:
 *             self._counts += 1             # <<<<<<<<<<<<<<
 *         if popout > 0:
 *             self._counts -= 1
 */
    __pyx_v_self->_counts = (__pyx_v_self->_counts + 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":678
 *         popout = self._deque.dump(value, -1.)
 * 
 *         if value > 0:             # <<<<<<<<<<<<<<
 *             self._counts += 1
 *         if popout > 0:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":680
 *         if value > 0:
 *             self._counts += 1
 *         if popout > 0:             # <<<<<<<<<<<<<<
 *             self._counts -= 1
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_t_5 = ((__pyx_v_popout > 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":681
 *             self._counts += 1
 *         if popout > 0:
 *             self._counts -= 1             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
    __pyx_v_self->_counts = (__pyx_v_self->_counts - 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":680
 *         if value > 0:
 *             self._counts += 1
 *         if popout > 0:             # <<<<<<<<<<<<<<
 *             self._counts -= 1
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":682
 *         if popout > 0:
 *             self._counts -= 1
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":669
 *         self._counts = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_2push[] = "MovingCountedPositive.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 669, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":684
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._counts
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 684, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 684, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 684, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":685
 * 
 *     cpdef double result(self):
 *         return self._counts             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = __pyx_v_self->_counts;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":684
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._counts
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_4result[] = "MovingCountedPositive.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":687
 *         return self._counts
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mposcount}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":688
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mposcount}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mposcount_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 688, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 688, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":687
 *         return self._counts
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mposcount}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":195
 * cdef class MovingCountedPositive(SingleValuedValueHolder):
 * 
 *     cdef public int _counts             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_counts); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive._counts.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 195, __pyx_L1_error)
  __pyx_v_self->_counts = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive._counts.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_8__reduce_cython__[] = "MovingCountedPositive.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._counts, self._dependency, self._deque, self._isFull, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_counts); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._counts, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._counts, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._counts, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingCountedPosi); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_251250905);
    __Pyx_GIVEREF(__pyx_int_251250905);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_251250905);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, None), state
 *     else:
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingCountedPositive__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingCountedPosi); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_251250905);
    __Pyx_GIVEREF(__pyx_int_251250905);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_251250905);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingCountedPositive__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_10__setstate_cython__[] = "MovingCountedPositive.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingCountedPositive__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCountedPositive__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingCountedPositive, (type(self), 0xef9c8d9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingCountedPositive__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":693
 * cdef class MovingCountedNegative(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingCountedNegative, self).__init__(window, x)
 *         self._counts = 0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 693, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 693, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 693, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":694
 * 
 *     def __init__(self, window, x):
 *         super(MovingCountedNegative, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._counts = 0
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 694, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":695
 *     def __init__(self, window, x):
 *         super(MovingCountedNegative, self).__init__(window, x)
 *         self._counts = 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_counts = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":693
 * cdef class MovingCountedNegative(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingCountedNegative, self).__init__(window, x)
 *         self._counts = 0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":697
 *         self._counts = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  double __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 697, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 697, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":699
 *     cpdef push(self, dict data):
 *         cdef double popout
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 699, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":700
 *         cdef double popout
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(value):
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":702
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 1)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":703
 * 
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value, 1)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 703, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":702
 *         cdef double value = self._x.result()
 * 
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value, 1)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":704
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value, 1)             # <<<<<<<<<<<<<<
 * 
 *         if value < 0:
 */
  __pyx_t_7.__pyx_n = 1;
  __pyx_t_7.__pyx_default = 1.0;
  __pyx_t_6 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, &__pyx_t_7); 
  __pyx_v_popout = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":706
 *         popout = self._deque.dump(value, 1)
 * 
 *         if value < 0:             # <<<<<<<<<<<<<<
 *             self._counts += 1
 *         if popout < 0:
 */
  __pyx_t_5 = ((__pyx_v_value < 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":707
 * 
 *         if value < 0:
 *             self._counts += 1             # <<<<<<<<<<<<<<
 *         if popout < 0:
 *             self._counts -= 1
 */
    __pyx_v_self->_counts = (__pyx_v_self->_counts + 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":706
 *         popout = self._deque.dump(value, 1)
 * 
 *         if value < 0:             # <<<<<<<<<<<<<<
 *             self._counts += 1
 *         if popout < 0:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":708
 *         if value < 0:
 *             self._counts += 1
 *         if popout < 0:             # <<<<<<<<<<<<<<
 *             self._counts -= 1
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_t_5 = ((__pyx_v_popout < 0.0) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":709
 *             self._counts += 1
 *         if popout < 0:
 *             self._counts -= 1             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
    __pyx_v_self->_counts = (__pyx_v_self->_counts - 1);

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":708
 *         if value < 0:
 *             self._counts += 1
 *         if popout < 0:             # <<<<<<<<<<<<<<
 *             self._counts -= 1
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":710
 *         if popout < 0:
 *             self._counts -= 1
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_8) {
  } else {
    __pyx_t_5 = __pyx_t_8;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_8 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_8;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":697
 *         self._counts = 0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_2push[] = "MovingCountedNegative.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 697, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 697, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":712
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._counts
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 712, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 712, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":713
 * 
 *     cpdef double result(self):
 *         return self._counts             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = __pyx_v_self->_counts;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":712
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._counts
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_4result[] = "MovingCountedNegative.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":715
 *         return self._counts
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mnegcount}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":716
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mnegcount}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mnegcount_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":715
 *         return self._counts
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mnegcount}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":203
 * cdef class MovingCountedNegative(SingleValuedValueHolder):
 * 
 *     cdef public int _counts             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_counts); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative._counts.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 203, __pyx_L1_error)
  __pyx_v_self->_counts = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative._counts.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_8__reduce_cython__[] = "MovingCountedNegative.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._counts, self._dependency, self._deque, self._isFull, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_counts); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._counts, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._counts, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._counts, self._dependency, self._deque, self._isFull, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingCountedNega); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_251250905);
    __Pyx_GIVEREF(__pyx_int_251250905);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_251250905);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, None), state
 *     else:
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingCountedNegative__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingCountedNega); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_251250905);
    __Pyx_GIVEREF(__pyx_int_251250905);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_251250905);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingCountedNegative__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_10__setstate_cython__[] = "MovingCountedNegative.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingCountedNegative__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCountedNegative__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingCountedNegative, (type(self), 0xef9c8d9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingCountedNegative__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":722
 * cdef class MovingCorrelation(StatefulValueHolder):
 * 
 *     def __init__(self, window, x, y):             # <<<<<<<<<<<<<<
 *         super(MovingCorrelation, self).__init__(window)
 *         self._runningSumLeft = 0.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(0, 722, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(0, 722, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 722, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
    __pyx_v_y = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 722, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self), __pyx_v_window, __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":723
 * 
 *     def __init__(self, window, x, y):
 *         super(MovingCorrelation, self).__init__(window)             # <<<<<<<<<<<<<<
 *         self._runningSumLeft = 0.0
 *         self._runningSumRight = 0.0
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":724
 *     def __init__(self, window, x, y):
 *         super(MovingCorrelation, self).__init__(window)
 *         self._runningSumLeft = 0.0             # <<<<<<<<<<<<<<
 *         self._runningSumRight = 0.0
 *         self._runningSumSquareLeft = 0.0
 */
  __pyx_v_self->_runningSumLeft = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":725
 *         super(MovingCorrelation, self).__init__(window)
 *         self._runningSumLeft = 0.0
 *         self._runningSumRight = 0.0             # <<<<<<<<<<<<<<
 *         self._runningSumSquareLeft = 0.0
 *         self._runningSumSquareRight = 0.0
 */
  __pyx_v_self->_runningSumRight = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":726
 *         self._runningSumLeft = 0.0
 *         self._runningSumRight = 0.0
 *         self._runningSumSquareLeft = 0.0             # <<<<<<<<<<<<<<
 *         self._runningSumSquareRight = 0.0
 *         self._runningSumCrossSquare = 0.0
 */
  __pyx_v_self->_runningSumSquareLeft = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":727
 *         self._runningSumRight = 0.0
 *         self._runningSumSquareLeft = 0.0
 *         self._runningSumSquareRight = 0.0             # <<<<<<<<<<<<<<
 *         self._runningSumCrossSquare = 0.0
 *         self._x = build_holder(x)
 */
  __pyx_v_self->_runningSumSquareRight = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":728
 *         self._runningSumSquareLeft = 0.0
 *         self._runningSumSquareRight = 0.0
 *         self._runningSumCrossSquare = 0.0             # <<<<<<<<<<<<<<
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)
 */
  __pyx_v_self->_runningSumCrossSquare = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":729
 *         self._runningSumSquareRight = 0.0
 *         self._runningSumCrossSquare = 0.0
 *         self._x = build_holder(x)             # <<<<<<<<<<<<<<
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 729, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 729, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":730
 *         self._runningSumCrossSquare = 0.0
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)             # <<<<<<<<<<<<<<
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_y, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_y);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_y));
  __pyx_v_self->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":731
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)             # <<<<<<<<<<<<<<
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 *         self._deque_y = Deque(window)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_y), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 731, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
  } else {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.__pyx_base._window = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":732
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))             # <<<<<<<<<<<<<<
 *         self._deque_y = Deque(window)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_dependency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_y), __pyx_n_s_dependency); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PySet_New(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_List(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __pyx_v_self->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":733
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 *         self._deque_y = Deque(window)             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque), __pyx_v_window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_deque_y);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_deque_y));
  __pyx_v_self->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":722
 * cdef class MovingCorrelation(StatefulValueHolder):
 * 
 *     def __init__(self, window, x, y):             # <<<<<<<<<<<<<<
 *         super(MovingCorrelation, self).__init__(window)
 *         self._runningSumLeft = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":735
 *         self._deque_y = Deque(window)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double headLeft
 *         cdef double headRight
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_headLeft;
  double __pyx_v_headRight;
  double __pyx_v_x;
  double __pyx_v_y;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  double __pyx_t_7;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 735, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 735, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":739
 *         cdef double headRight
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double x = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->push(__pyx_v_self->_x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":740
 * 
 *         self._x.push(data)
 *         cdef double x = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         self._y.push(data)
 */
  __pyx_v_x = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->result(__pyx_v_self->_x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":742
 *         cdef double x = self._x.result()
 * 
 *         self._y.push(data)             # <<<<<<<<<<<<<<
 *         cdef double y = self._y.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_y->__pyx_vtab)->push(__pyx_v_self->_y, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":743
 * 
 *         self._y.push(data)
 *         cdef double y = self._y.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(x) or isnan(y):
 */
  __pyx_v_y = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_y->__pyx_vtab)->result(__pyx_v_self->_y, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":745
 *         cdef double y = self._y.result()
 * 
 *         if isnan(x) or isnan(y):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  __pyx_t_6 = (isnan(__pyx_v_x) != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = (isnan(__pyx_v_y) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":746
 * 
 *         if isnan(x) or isnan(y):
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *         headLeft = self._deque.dump(x, 0.)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":745
 *         cdef double y = self._y.result()
 * 
 *         if isnan(x) or isnan(y):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":748
 *             return NAN
 * 
 *         headLeft = self._deque.dump(x, 0.)             # <<<<<<<<<<<<<<
 *         headRight = self._deque_y.dump(y, 0.)
 * 
 */
  __pyx_t_8.__pyx_n = 1;
  __pyx_t_8.__pyx_default = 0.;
  __pyx_t_7 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base._deque, __pyx_v_x, &__pyx_t_8); 
  __pyx_v_headLeft = __pyx_t_7;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":749
 * 
 *         headLeft = self._deque.dump(x, 0.)
 *         headRight = self._deque_y.dump(y, 0.)             # <<<<<<<<<<<<<<
 * 
 *         # updating cached values
 */
  __pyx_t_8.__pyx_n = 1;
  __pyx_t_8.__pyx_default = 0.;
  __pyx_t_7 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->_deque_y->__pyx_vtab)->dump(__pyx_v_self->_deque_y, __pyx_v_y, &__pyx_t_8); 
  __pyx_v_headRight = __pyx_t_7;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":752
 * 
 *         # updating cached values
 *         self._runningSumLeft = self._runningSumLeft - headLeft + x             # <<<<<<<<<<<<<<
 *         self._runningSumRight = self._runningSumRight - headRight + y
 *         self._runningSumSquareLeft = self._runningSumSquareLeft - headLeft * headLeft + x * x
 */
  __pyx_v_self->_runningSumLeft = ((__pyx_v_self->_runningSumLeft - __pyx_v_headLeft) + __pyx_v_x);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":753
 *         # updating cached values
 *         self._runningSumLeft = self._runningSumLeft - headLeft + x
 *         self._runningSumRight = self._runningSumRight - headRight + y             # <<<<<<<<<<<<<<
 *         self._runningSumSquareLeft = self._runningSumSquareLeft - headLeft * headLeft + x * x
 *         self._runningSumSquareRight = self._runningSumSquareRight - headRight * headRight + y * y
 */
  __pyx_v_self->_runningSumRight = ((__pyx_v_self->_runningSumRight - __pyx_v_headRight) + __pyx_v_y);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":754
 *         self._runningSumLeft = self._runningSumLeft - headLeft + x
 *         self._runningSumRight = self._runningSumRight - headRight + y
 *         self._runningSumSquareLeft = self._runningSumSquareLeft - headLeft * headLeft + x * x             # <<<<<<<<<<<<<<
 *         self._runningSumSquareRight = self._runningSumSquareRight - headRight * headRight + y * y
 *         self._runningSumCrossSquare = self._runningSumCrossSquare - headLeft * headRight + x * y
 */
  __pyx_v_self->_runningSumSquareLeft = ((__pyx_v_self->_runningSumSquareLeft - (__pyx_v_headLeft * __pyx_v_headLeft)) + (__pyx_v_x * __pyx_v_x));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":755
 *         self._runningSumRight = self._runningSumRight - headRight + y
 *         self._runningSumSquareLeft = self._runningSumSquareLeft - headLeft * headLeft + x * x
 *         self._runningSumSquareRight = self._runningSumSquareRight - headRight * headRight + y * y             # <<<<<<<<<<<<<<
 *         self._runningSumCrossSquare = self._runningSumCrossSquare - headLeft * headRight + x * y
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_v_self->_runningSumSquareRight = ((__pyx_v_self->_runningSumSquareRight - (__pyx_v_headRight * __pyx_v_headRight)) + (__pyx_v_y * __pyx_v_y));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":756
 *         self._runningSumSquareLeft = self._runningSumSquareLeft - headLeft * headLeft + x * x
 *         self._runningSumSquareRight = self._runningSumSquareRight - headRight * headRight + y * y
 *         self._runningSumCrossSquare = self._runningSumCrossSquare - headLeft * headRight + x * y             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  __pyx_v_self->_runningSumCrossSquare = ((__pyx_v_self->_runningSumCrossSquare - (__pyx_v_headLeft * __pyx_v_headRight)) + (__pyx_v_x * __pyx_v_y));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":757
 *         self._runningSumSquareRight = self._runningSumSquareRight - headRight * headRight + y * y
 *         self._runningSumCrossSquare = self._runningSumCrossSquare - headLeft * headRight + x * y
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":735
 *         self._deque_y = Deque(window)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double headLeft
 *         cdef double headRight
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_2push[] = "MovingCorrelation.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 735, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":759
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t n = self.size()
 *         cdef double nominator
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, int __pyx_skip_dispatch) {
  size_t __pyx_v_n;
  double __pyx_v_nominator;
  double __pyx_v_denominator;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_isClose __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 759, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 759, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 759, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":760
 * 
 *     cpdef double result(self):
 *         cdef size_t n = self.size()             # <<<<<<<<<<<<<<
 *         cdef double nominator
 *         cdef double denominator
 */
  __pyx_v_n = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.size(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v_self), 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":763
 *         cdef double nominator
 *         cdef double denominator
 *         if n >= 2:             # <<<<<<<<<<<<<<
 *             nominator = n * self._runningSumCrossSquare - self._runningSumLeft * self._runningSumRight
 *             denominator = (n * self._runningSumSquareLeft - self._runningSumLeft * self._runningSumLeft) \
 */
  __pyx_t_6 = ((__pyx_v_n >= 2) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":764
 *         cdef double denominator
 *         if n >= 2:
 *             nominator = n * self._runningSumCrossSquare - self._runningSumLeft * self._runningSumRight             # <<<<<<<<<<<<<<
 *             denominator = (n * self._runningSumSquareLeft - self._runningSumLeft * self._runningSumLeft) \
 *                           * (n * self._runningSumSquareRight - self._runningSumRight * self._runningSumRight)
 */
    __pyx_v_nominator = ((__pyx_v_n * __pyx_v_self->_runningSumCrossSquare) - (__pyx_v_self->_runningSumLeft * __pyx_v_self->_runningSumRight));

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":766
 *             nominator = n * self._runningSumCrossSquare - self._runningSumLeft * self._runningSumRight
 *             denominator = (n * self._runningSumSquareLeft - self._runningSumLeft * self._runningSumLeft) \
 *                           * (n * self._runningSumSquareRight - self._runningSumRight * self._runningSumRight)             # <<<<<<<<<<<<<<
 *             if not isClose(denominator, 0.):
 *                 denominator = sqrt(denominator)
 */
    __pyx_v_denominator = (((__pyx_v_n * __pyx_v_self->_runningSumSquareLeft) - (__pyx_v_self->_runningSumLeft * __pyx_v_self->_runningSumLeft)) * ((__pyx_v_n * __pyx_v_self->_runningSumSquareRight) - (__pyx_v_self->_runningSumRight * __pyx_v_self->_runningSumRight)));

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":767
 *             denominator = (n * self._runningSumSquareLeft - self._runningSumLeft * self._runningSumLeft) \
 *                           * (n * self._runningSumSquareRight - self._runningSumRight * self._runningSumRight)
 *             if not isClose(denominator, 0.):             # <<<<<<<<<<<<<<
 *                 denominator = sqrt(denominator)
 *                 return nominator / denominator
 */
    __pyx_t_7.__pyx_n = 1;
    __pyx_t_7.b = 0.;
    __pyx_t_6 = __pyx_f_6ultron_6sentry_9Utilities_7Asserts_isClose(__pyx_v_denominator, 0, &__pyx_t_7); 
    __pyx_t_8 = ((!(__pyx_t_6 != 0)) != 0);
    if (__pyx_t_8) {

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":768
 *                           * (n * self._runningSumSquareRight - self._runningSumRight * self._runningSumRight)
 *             if not isClose(denominator, 0.):
 *                 denominator = sqrt(denominator)             # <<<<<<<<<<<<<<
 *                 return nominator / denominator
 *             else:
 */
      __pyx_v_denominator = sqrt(__pyx_v_denominator);

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":769
 *             if not isClose(denominator, 0.):
 *                 denominator = sqrt(denominator)
 *                 return nominator / denominator             # <<<<<<<<<<<<<<
 *             else:
 *                 return 0.0
 */
      if (unlikely(__pyx_v_denominator == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 769, __pyx_L1_error)
      }
      __pyx_r = (__pyx_v_nominator / __pyx_v_denominator);
      goto __pyx_L0;

      /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":767
 *             denominator = (n * self._runningSumSquareLeft - self._runningSumLeft * self._runningSumLeft) \
 *                           * (n * self._runningSumSquareRight - self._runningSumRight * self._runningSumRight)
 *             if not isClose(denominator, 0.):             # <<<<<<<<<<<<<<
 *                 denominator = sqrt(denominator)
 *                 return nominator / denominator
 */
    }

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":771
 *                 return nominator / denominator
 *             else:
 *                 return 0.0             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    /*else*/ {
      __pyx_r = 0.0;
      goto __pyx_L0;
    }

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":763
 *         cdef double nominator
 *         cdef double denominator
 *         if n >= 2:             # <<<<<<<<<<<<<<
 *             nominator = n * self._runningSumCrossSquare - self._runningSumLeft * self._runningSumRight
 *             denominator = (n * self._runningSumSquareLeft - self._runningSumLeft * self._runningSumLeft) \
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":773
 *                 return 0.0
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":759
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef size_t n = self.size()
 *         cdef double nominator
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_4result[] = "MovingCorrelation.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 759, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":775
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mcorr}}({0}, {1}, {2})".format(self._window, str(self._x), str(self._y))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":776
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mcorr}}({0}, {1}, {2})".format(self._window, str(self._x), str(self._y))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mcorr_0_1_2, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->_x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->_y)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":775
 *             return NAN
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mcorr}}({0}, {1}, {2})".format(self._window, str(self._x), str(self._y))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":211
 * cdef class MovingCorrelation(StatefulValueHolder):
 * 
 *     cdef public double _runningSumLeft             # <<<<<<<<<<<<<<
 *     cdef public double _runningSumRight
 *     cdef public double _runningSumSquareLeft
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSumLeft); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumLeft.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 211, __pyx_L1_error)
  __pyx_v_self->_runningSumLeft = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumLeft.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":212
 * 
 *     cdef public double _runningSumLeft
 *     cdef public double _runningSumRight             # <<<<<<<<<<<<<<
 *     cdef public double _runningSumSquareLeft
 *     cdef public double _runningSumSquareRight
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSumRight); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumRight.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 212, __pyx_L1_error)
  __pyx_v_self->_runningSumRight = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumRight.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":213
 *     cdef public double _runningSumLeft
 *     cdef public double _runningSumRight
 *     cdef public double _runningSumSquareLeft             # <<<<<<<<<<<<<<
 *     cdef public double _runningSumSquareRight
 *     cdef public double _runningSumCrossSquare
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSumSquareLeft); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumSquareLeft.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 213, __pyx_L1_error)
  __pyx_v_self->_runningSumSquareLeft = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumSquareLeft.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":214
 *     cdef public double _runningSumRight
 *     cdef public double _runningSumSquareLeft
 *     cdef public double _runningSumSquareRight             # <<<<<<<<<<<<<<
 *     cdef public double _runningSumCrossSquare
 *     cdef Accumulator _x
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSumSquareRight); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumSquareRight.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 214, __pyx_L1_error)
  __pyx_v_self->_runningSumSquareRight = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumSquareRight.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":215
 *     cdef public double _runningSumSquareLeft
 *     cdef public double _runningSumSquareRight
 *     cdef public double _runningSumCrossSquare             # <<<<<<<<<<<<<<
 *     cdef Accumulator _x
 *     cdef Accumulator _y
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningSumCrossSquare); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumCrossSquare.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 215, __pyx_L1_error)
  __pyx_v_self->_runningSumCrossSquare = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation._runningSumCrossSquare.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_8__reduce_cython__[] = "MovingCorrelation.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._runningSumCrossSquare, self._runningSumLeft, self._runningSumRight, self._runningSumSquareLeft, self._runningSumSquareRight, self._window, self._x, self._y)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_runningSumCrossSquare); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_runningSumLeft); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->_runningSumRight); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->_runningSumSquareLeft); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->_runningSumSquareRight); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(12); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_8, 1, ((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_deque_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_deque_y));
  PyTuple_SET_ITEM(__pyx_t_8, 2, ((PyObject *)__pyx_v_self->_deque_y));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_8, 3, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_8, 4, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_8, 5, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 6, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_8, 7, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 8, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_8, 9, __pyx_t_7);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_x));
  PyTuple_SET_ITEM(__pyx_t_8, 10, ((PyObject *)__pyx_v_self->_x));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_y));
  PyTuple_SET_ITEM(__pyx_t_8, 11, ((PyObject *)__pyx_v_self->_y));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._runningSumCrossSquare, self._runningSumLeft, self._runningSumRight, self._runningSumSquareLeft, self._runningSumSquareRight, self._window, self._x, self._y)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_8 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v__dict = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._runningSumCrossSquare, self._runningSumLeft, self._runningSumRight, self._runningSumSquareLeft, self._runningSumSquareRight, self._window, self._x, self._y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_9 = (__pyx_v__dict != Py_None);
  __pyx_t_10 = (__pyx_t_9 != 0);
  if (__pyx_t_10) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v__dict);
    __pyx_t_7 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_7));
    __pyx_t_7 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._runningSumCrossSquare, self._runningSumLeft, self._runningSumRight, self._runningSumSquareLeft, self._runningSumSquareRight, self._window, self._x, self._y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, None), state
 */
  /*else*/ {
    __pyx_t_9 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_11 = (((PyObject *)__pyx_v_self->__pyx_base._deque) != Py_None);
    __pyx_t_9 = (__pyx_t_11 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_10 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (((PyObject *)__pyx_v_self->_deque_y) != Py_None);
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_11 = (((PyObject *)__pyx_v_self->_x) != Py_None);
    __pyx_t_9 = (__pyx_t_11 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_10 = __pyx_t_9;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_9 = (((PyObject *)__pyx_v_self->_y) != Py_None);
    __pyx_t_11 = (__pyx_t_9 != 0);
    __pyx_t_10 = __pyx_t_11;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_10;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, None), state
 *     else:
 */
  __pyx_t_10 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_10) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_pyx_unpickle_MovingCorrelation); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_195870223);
    __Pyx_GIVEREF(__pyx_int_195870223);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_int_195870223);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_8, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_8);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, None), state
 *     else:
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingCorrelation__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_MovingCorrelation); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_195870223);
    __Pyx_GIVEREF(__pyx_int_195870223);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_int_195870223);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_v_state);
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_8);
    __pyx_t_6 = 0;
    __pyx_t_8 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingCorrelation__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_10__setstate_cython__[] = "MovingCorrelation.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingCorrelation__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCorrelation__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingCorrelation, (type(self), 0xbacbe0f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingCorrelation__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":781
 * cdef class MovingProduct(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingProduct, self).__init__(window, x)
 *         self._runningProduct = 1.0
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 781, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 781, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 781, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":782
 * 
 *     def __init__(self, window, x):
 *         super(MovingProduct, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningProduct = 1.0
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 782, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 782, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 782, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 782, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 782, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 782, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 782, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":783
 *     def __init__(self, window, x):
 *         super(MovingProduct, self).__init__(window, x)
 *         self._runningProduct = 1.0             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_runningProduct = 1.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":781
 * cdef class MovingProduct(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingProduct, self).__init__(window, x)
 *         self._runningProduct = 1.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":785
 *         self._runningProduct = 1.0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 785, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 785, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":787
 *     cpdef push(self, dict data):
 *         cdef double popout
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 *         if isnan(value):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":788
 *         cdef double popout
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 *         if isnan(value):
 *             return NAN
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":789
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":790
 *         cdef double value = self._x.result()
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value)
 *         if not isnan(popout):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 790, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":789
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":791
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value)             # <<<<<<<<<<<<<<
 *         if not isnan(popout):
 *             self._runningProduct *= value / popout
 */
  __pyx_v_popout = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, NULL);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":792
 *             return NAN
 *         popout = self._deque.dump(value)
 *         if not isnan(popout):             # <<<<<<<<<<<<<<
 *             self._runningProduct *= value / popout
 *         else:
 */
  __pyx_t_5 = ((!(isnan(__pyx_v_popout) != 0)) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":793
 *         popout = self._deque.dump(value)
 *         if not isnan(popout):
 *             self._runningProduct *= value / popout             # <<<<<<<<<<<<<<
 *         else:
 *             self._runningProduct *= value
 */
    if (unlikely(__pyx_v_popout == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 793, __pyx_L1_error)
    }
    __pyx_v_self->_runningProduct = (__pyx_v_self->_runningProduct * (__pyx_v_value / __pyx_v_popout));

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":792
 *             return NAN
 *         popout = self._deque.dump(value)
 *         if not isnan(popout):             # <<<<<<<<<<<<<<
 *             self._runningProduct *= value / popout
 *         else:
 */
    goto __pyx_L4;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":795
 *             self._runningProduct *= value / popout
 *         else:
 *             self._runningProduct *= value             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  /*else*/ {
    __pyx_v_self->_runningProduct = (__pyx_v_self->_runningProduct * __pyx_v_value);
  }
  __pyx_L4:;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":796
 *         else:
 *             self._runningProduct *= value
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L5_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":785
 *         self._runningProduct = 1.0
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_2push[] = "MovingProduct.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 785, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 785, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":798
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningProduct
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 798, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 798, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 798, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":799
 * 
 *     cpdef double result(self):
 *         return self._runningProduct             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = __pyx_v_self->_runningProduct;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":798
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningProduct
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_4result[] = "MovingProduct.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":801
 *         return self._runningProduct
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mprod}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_7__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_7__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_6__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_6__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":802
 * 
 *     def __str__(self):
 *         return "\\mathrm{{mprod}}({0}, {1})".format(self._window, str(self._x))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_mprod_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 802, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 802, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 802, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 802, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 802, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 802, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 802, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":801
 *         return self._runningProduct
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{mprod}}({0}, {1})".format(self._window, str(self._x))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":226
 * cdef class MovingProduct(SingleValuedValueHolder):
 * 
 *     cdef public double _runningProduct             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningProduct); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct._runningProduct.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 226, __pyx_L1_error)
  __pyx_v_self->_runningProduct = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct._runningProduct.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_8__reduce_cython__[] = "MovingProduct.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_8__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_8__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningProduct, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_runningProduct); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningProduct, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningProduct, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningProduct, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingProduct); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_34816434);
    __Pyx_GIVEREF(__pyx_int_34816434);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_34816434);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, None), state
 *     else:
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingProduct__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingProduct); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_34816434);
    __Pyx_GIVEREF(__pyx_int_34816434);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_34816434);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingProduct__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_10__setstate_cython__[] = "MovingProduct.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_10__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_10__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingProduct__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingProduct__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingProduct, (type(self), 0x21341b2, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingProduct__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":807
 * cdef class MACD(Accumulator):
 * 
 *     def __init__(self, short_win, long_win, x, method=XAverage):             # <<<<<<<<<<<<<<
 *         super(MACD, self).__init__()
 *         self._short_average = method(window=short_win, x=x)
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_short_win = 0;
  PyObject *__pyx_v_long_win = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_method = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_short_win,&__pyx_n_s_long_win,&__pyx_n_s_x,&__pyx_n_s_method,0};
    PyObject* values[4] = {0,0,0,0};
    values[3] = __pyx_k_;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_short_win)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_long_win)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, 1); __PYX_ERR(0, 807, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, 2); __PYX_ERR(0, 807, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_method);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 807, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_short_win = values[0];
    __pyx_v_long_win = values[1];
    __pyx_v_x = values[2];
    __pyx_v_method = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 807, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self), __pyx_v_short_win, __pyx_v_long_win, __pyx_v_x, __pyx_v_method);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_short_win, PyObject *__pyx_v_long_win, PyObject *__pyx_v_x, PyObject *__pyx_v_method) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":808
 * 
 *     def __init__(self, short_win, long_win, x, method=XAverage):
 *         super(MACD, self).__init__()             # <<<<<<<<<<<<<<
 *         self._short_average = method(window=short_win, x=x)
 *         self._long_average = method(window=long_win, x=x)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":809
 *     def __init__(self, short_win, long_win, x, method=XAverage):
 *         super(MACD, self).__init__()
 *         self._short_average = method(window=short_win, x=x)             # <<<<<<<<<<<<<<
 *         self._long_average = method(window=long_win, x=x)
 *         self._isFull = False
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 809, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_window, __pyx_v_short_win) < 0) __PYX_ERR(0, 809, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_x, __pyx_v_x) < 0) __PYX_ERR(0, 809, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_v_method, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 809, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 809, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_short_average);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_short_average));
  __pyx_v_self->_short_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":810
 *         super(MACD, self).__init__()
 *         self._short_average = method(window=short_win, x=x)
 *         self._long_average = method(window=long_win, x=x)             # <<<<<<<<<<<<<<
 *         self._isFull = False
 *         self._window = max(self._short_average.window, self._long_average.window)
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 810, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_window, __pyx_v_long_win) < 0) __PYX_ERR(0, 810, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_x, __pyx_v_x) < 0) __PYX_ERR(0, 810, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_v_method, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 810, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 810, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_long_average);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_long_average));
  __pyx_v_self->_long_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":811
 *         self._short_average = method(window=short_win, x=x)
 *         self._long_average = method(window=long_win, x=x)
 *         self._isFull = False             # <<<<<<<<<<<<<<
 *         self._window = max(self._short_average.window, self._long_average.window)
 *         self._dependency = self._short_average._dependency
 */
  __pyx_v_self->__pyx_base._isFull = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":812
 *         self._long_average = method(window=long_win, x=x)
 *         self._isFull = False
 *         self._window = max(self._short_average.window, self._long_average.window)             # <<<<<<<<<<<<<<
 *         self._dependency = self._short_average._dependency
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_long_average), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 812, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_short_average), __pyx_n_s_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 812, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 812, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 812, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = __pyx_t_1;
  } else {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = __pyx_t_2;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_3); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 812, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_self->__pyx_base._window = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":813
 *         self._isFull = False
 *         self._window = max(self._short_average.window, self._long_average.window)
 *         self._dependency = self._short_average._dependency             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_t_3 = __pyx_v_self->_short_average->_dependency;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v_self->__pyx_base._dependency);
  __pyx_v_self->__pyx_base._dependency = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":807
 * cdef class MACD(Accumulator):
 * 
 *     def __init__(self, short_win, long_win, x, method=XAverage):             # <<<<<<<<<<<<<<
 *         super(MACD, self).__init__()
 *         self._short_average = method(window=short_win, x=x)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":815
 *         self._dependency = self._short_average._dependency
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._short_average.push(data)
 *         self._long_average.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 815, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 815, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":816
 * 
 *     cpdef push(self, dict data):
 *         self._short_average.push(data)             # <<<<<<<<<<<<<<
 *         self._long_average.push(data)
 *         self._isFull = self._isFull or (self._short_average.isFull() and self._long_average.isFull())
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_short_average->__pyx_vtab)->push(__pyx_v_self->_short_average, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 816, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":817
 *     cpdef push(self, dict data):
 *         self._short_average.push(data)
 *         self._long_average.push(data)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or (self._short_average.isFull() and self._long_average.isFull())
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_long_average->__pyx_vtab)->push(__pyx_v_self->_long_average, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 817, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":818
 *         self._short_average.push(data)
 *         self._long_average.push(data)
 *         self._isFull = self._isFull or (self._short_average.isFull() and self._long_average.isFull())             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_short_average->__pyx_vtab)->isFull(__pyx_v_self->_short_average, 0) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_long_average->__pyx_vtab)->isFull(__pyx_v_self->_long_average, 0) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_v_self->__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":815
 *         self._dependency = self._short_average._dependency
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._short_average.push(data)
 *         self._long_average.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_2push[] = "MACD.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 815, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 815, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":820
 *         self._isFull = self._isFull or (self._short_average.isFull() and self._long_average.isFull())
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._short_average.result() - self._long_average.result()
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 820, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 820, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 820, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":821
 * 
 *     cpdef double result(self):
 *         return self._short_average.result() - self._long_average.result()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_short_average->__pyx_vtab)->result(__pyx_v_self->_short_average, 0) - ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_long_average->__pyx_vtab)->result(__pyx_v_self->_long_average, 0));
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":820
 *         self._isFull = self._isFull or (self._short_average.isFull() and self._long_average.isFull())
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._short_average.result() - self._long_average.result()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_4result[] = "MACD.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":234
 * cdef class MACD(Accumulator):
 * 
 *     cdef public Accumulator _short_average             # <<<<<<<<<<<<<<
 *     cdef public Accumulator _long_average
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_short_average));
  __pyx_r = ((PyObject *)__pyx_v_self->_short_average);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 234, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_short_average);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_short_average));
  __pyx_v_self->_short_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD._short_average.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_short_average);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_short_average));
  __pyx_v_self->_short_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":235
 * 
 *     cdef public Accumulator _short_average
 *     cdef public Accumulator _long_average             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_long_average));
  __pyx_r = ((PyObject *)__pyx_v_self->_long_average);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 235, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_long_average);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_long_average));
  __pyx_v_self->_long_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD._long_average.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_long_average);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_long_average));
  __pyx_v_self->_long_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_6__reduce_cython__[] = "MACD.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._isFull, self._long_average, self._short_average, self._window)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(5); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_long_average));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_long_average));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->_long_average));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_short_average));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_short_average));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->_short_average));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._isFull, self._long_average, self._short_average, self._window)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._isFull, self._long_average, self._short_average, self._window)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._long_average is not None or self._short_average is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._isFull, self._long_average, self._short_average, self._window)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._long_average is not None or self._short_average is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_long_average) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_short_average) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    __pyx_t_5 = __pyx_t_6;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._long_average is not None or self._short_average is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._long_average is not None or self._short_average is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MACD); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_88594310);
    __Pyx_GIVEREF(__pyx_int_88594310);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_88594310);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._long_average is not None or self._short_average is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, None), state
 *     else:
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MACD__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_MACD); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_88594310);
    __Pyx_GIVEREF(__pyx_int_88594310);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_88594310);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MACD__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_8__setstate_cython__[] = "MACD.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MACD__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MACD__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MACD, (type(self), 0x547d786, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MACD__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":830
 * cdef class MovingLogReturn(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingLogReturn, self).__init__(window, x)
 *         self._runningReturn = NAN
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 830, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 830, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 830, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":831
 * 
 *     def __init__(self, window, x):
 *         super(MovingLogReturn, self).__init__(window, x)             # <<<<<<<<<<<<<<
 *         self._runningReturn = NAN
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_window, __pyx_v_x};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_window);
    __Pyx_GIVEREF(__pyx_v_window);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_window);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":832
 *     def __init__(self, window, x):
 *         super(MovingLogReturn, self).__init__(window, x)
 *         self._runningReturn = NAN             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_v_self->_runningReturn = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":830
 * cdef class MovingLogReturn(SingleValuedValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingLogReturn, self).__init__(window, x)
 *         self._runningReturn = NAN
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":835
 * 
 *     @cython.cdivision(True)
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_popout;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 835, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 835, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":837
 *     cpdef push(self, dict data):
 *         cdef double popout
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double value = self._x.result()
 *         if isnan(value):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->push(__pyx_v_self->__pyx_base._x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":838
 *         cdef double popout
 *         self._x.push(data)
 *         cdef double value = self._x.result()             # <<<<<<<<<<<<<<
 *         if isnan(value):
 *             return NAN
 */
  __pyx_v_value = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->__pyx_base._x->__pyx_vtab)->result(__pyx_v_self->__pyx_base._x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":839
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value)
 */
  __pyx_t_5 = (isnan(__pyx_v_value) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":840
 *         cdef double value = self._x.result()
 *         if isnan(value):
 *             return NAN             # <<<<<<<<<<<<<<
 *         popout = self._deque.dump(value)
 *         if not isnan(popout):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 840, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":839
 *         self._x.push(data)
 *         cdef double value = self._x.result()
 *         if isnan(value):             # <<<<<<<<<<<<<<
 *             return NAN
 *         popout = self._deque.dump(value)
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":841
 *         if isnan(value):
 *             return NAN
 *         popout = self._deque.dump(value)             # <<<<<<<<<<<<<<
 *         if not isnan(popout):
 *             self._runningReturn = log(value / popout)
 */
  __pyx_v_popout = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base.__pyx_base._deque, __pyx_v_value, NULL);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":842
 *             return NAN
 *         popout = self._deque.dump(value)
 *         if not isnan(popout):             # <<<<<<<<<<<<<<
 *             self._runningReturn = log(value / popout)
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_t_5 = ((!(isnan(__pyx_v_popout) != 0)) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":843
 *         popout = self._deque.dump(value)
 *         if not isnan(popout):
 *             self._runningReturn = log(value / popout)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
    __pyx_v_self->_runningReturn = log((__pyx_v_value / __pyx_v_popout));

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":842
 *             return NAN
 *         popout = self._deque.dump(value)
 *         if not isnan(popout):             # <<<<<<<<<<<<<<
 *             self._runningReturn = log(value / popout)
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":844
 *         if not isnan(popout):
 *             self._runningReturn = log(value / popout)
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base.__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base.__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L5_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":835
 * 
 *     @cython.cdivision(True)
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double popout
 *         self._x.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_2push[] = "MovingLogReturn.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 835, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 835, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":846
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningReturn
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 846, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 846, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":847
 * 
 *     cpdef double result(self):
 *         return self._runningReturn             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_self->_runningReturn;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":846
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningReturn
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_4result[] = "MovingLogReturn.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":243
 * cdef class MovingLogReturn(SingleValuedValueHolder):
 * 
 *     cdef public double _runningReturn             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_runningReturn); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn._runningReturn.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 243, __pyx_L1_error)
  __pyx_v_self->_runningReturn = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn._runningReturn.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_6__reduce_cython__[] = "MovingLogReturn.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningReturn, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_runningReturn); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._x));
  PyTuple_SET_ITEM(__pyx_t_4, 5, ((PyObject *)__pyx_v_self->__pyx_base._x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._isFull, self._runningReturn, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningReturn, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._isFull, self._runningReturn, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._deque) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._x) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingLogReturn); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_131110952);
    __Pyx_GIVEREF(__pyx_int_131110952);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_131110952);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, None), state
 *     else:
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingLogReturn__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingLogReturn); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_131110952);
    __Pyx_GIVEREF(__pyx_int_131110952);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_131110952);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingLogReturn__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_8__setstate_cython__[] = "MovingLogReturn.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingLogReturn__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingLogReturn__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingLogReturn, (type(self), 0x7d09828, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingLogReturn__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":852
 * cdef class MovingSharp(StatefulValueHolder):
 * 
 *     def __init__(self, window, x, y):             # <<<<<<<<<<<<<<
 *         super(MovingSharp, self).__init__(window)
 *         self._mean = MovingAverage(window, x='x')
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(0, 852, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(0, 852, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 852, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
    __pyx_v_y = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 852, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self), __pyx_v_window, __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":853
 * 
 *     def __init__(self, window, x, y):
 *         super(MovingSharp, self).__init__(window)             # <<<<<<<<<<<<<<
 *         self._mean = MovingAverage(window, x='x')
 *         self._var = MovingVariance(window, x='x', isPopulation=False)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":854
 *     def __init__(self, window, x, y):
 *         super(MovingSharp, self).__init__(window)
 *         self._mean = MovingAverage(window, x='x')             # <<<<<<<<<<<<<<
 *         self._var = MovingVariance(window, x='x', isPopulation=False)
 *         self._x = build_holder(x)
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 854, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_window);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 854, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_x, __pyx_n_s_x) < 0) __PYX_ERR(0, 854, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage), __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 854, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_mean);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_mean));
  __pyx_v_self->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":855
 *         super(MovingSharp, self).__init__(window)
 *         self._mean = MovingAverage(window, x='x')
 *         self._var = MovingVariance(window, x='x', isPopulation=False)             # <<<<<<<<<<<<<<
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)
 */
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 855, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_window);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 855, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_x, __pyx_n_s_x) < 0) __PYX_ERR(0, 855, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_isPopulation, Py_False) < 0) __PYX_ERR(0, 855, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance), __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 855, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_var);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_var));
  __pyx_v_self->_var = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":856
 *         self._mean = MovingAverage(window, x='x')
 *         self._var = MovingVariance(window, x='x', isPopulation=False)
 *         self._x = build_holder(x)             # <<<<<<<<<<<<<<
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 856, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":857
 *         self._var = MovingVariance(window, x='x', isPopulation=False)
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)             # <<<<<<<<<<<<<<
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_y, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 857, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 857, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_y);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_y));
  __pyx_v_self->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":858
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)             # <<<<<<<<<<<<<<
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 *         self._deque_y = Deque(window)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_y), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 858, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
  } else {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 858, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.__pyx_base._window = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":859
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))             # <<<<<<<<<<<<<<
 *         self._deque_y = Deque(window)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_dependency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_y), __pyx_n_s_dependency); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PySet_New(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_List(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __pyx_v_self->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":860
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 *         self._deque_y = Deque(window)             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque), __pyx_v_window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 860, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_deque_y);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_deque_y));
  __pyx_v_self->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":852
 * cdef class MovingSharp(StatefulValueHolder):
 * 
 *     def __init__(self, window, x, y):             # <<<<<<<<<<<<<<
 *         super(MovingSharp, self).__init__(window)
 *         self._mean = MovingAverage(window, x='x')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":862
 *         self._deque_y = Deque(window)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 * 
 *         cdef tuple value
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_ret;
  double __pyx_v_benchmark;
  PyObject *__pyx_v_new_data = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 862, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 862, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":869
 *         cdef dict new_data
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         ret = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->push(__pyx_v_self->_x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 869, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":870
 * 
 *         self._x.push(data)
 *         ret = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         self._y.push(data)
 */
  __pyx_v_ret = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->result(__pyx_v_self->_x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":872
 *         ret = self._x.result()
 * 
 *         self._y.push(data)             # <<<<<<<<<<<<<<
 *         benchmark = self._y.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_y->__pyx_vtab)->push(__pyx_v_self->_y, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 872, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":873
 * 
 *         self._y.push(data)
 *         benchmark = self._y.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(ret) or isnan(benchmark):
 */
  __pyx_v_benchmark = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_y->__pyx_vtab)->result(__pyx_v_self->_y, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":875
 *         benchmark = self._y.result()
 * 
 *         if isnan(ret) or isnan(benchmark):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  __pyx_t_6 = (isnan(__pyx_v_ret) != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = (isnan(__pyx_v_benchmark) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":876
 * 
 *         if isnan(ret) or isnan(benchmark):
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *         new_data = {'x': ret - benchmark}
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 876, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":875
 *         benchmark = self._y.result()
 * 
 *         if isnan(ret) or isnan(benchmark):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":878
 *             return NAN
 * 
 *         new_data = {'x': ret - benchmark}             # <<<<<<<<<<<<<<
 *         self._mean.push(new_data)
 *         self._var.push(new_data)
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 878, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble((__pyx_v_ret - __pyx_v_benchmark)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 878, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_x, __pyx_t_2) < 0) __PYX_ERR(0, 878, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_new_data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":879
 * 
 *         new_data = {'x': ret - benchmark}
 *         self._mean.push(new_data)             # <<<<<<<<<<<<<<
 *         self._var.push(new_data)
 *         self._isFull = self._isFull or (self._mean.isFull() and self._var.isFull())
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_mean->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_mean), __pyx_v_new_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 879, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":880
 *         new_data = {'x': ret - benchmark}
 *         self._mean.push(new_data)
 *         self._var.push(new_data)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or (self._mean.isFull() and self._var.isFull())
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self->_var->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_var), __pyx_v_new_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":881
 *         self._mean.push(new_data)
 *         self._var.push(new_data)
 *         self._isFull = self._isFull or (self._mean.isFull() and self._var.isFull())             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_mean->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_mean), 0) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self->_var->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_var), 0) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":862
 *         self._deque_y = Deque(window)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 * 
 *         cdef tuple value
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_new_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_2push[] = "MovingSharp.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 862, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":884
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double tmp = self._var.result()
 *         if tmp != 0.:
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_v_tmp;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 884, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 884, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":885
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         cdef double tmp = self._var.result()             # <<<<<<<<<<<<<<
 *         if tmp != 0.:
 *             return self._mean.result() / sqrt(tmp)
 */
  __pyx_v_tmp = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v_self->_var->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_var), 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":886
 *     cpdef double result(self):
 *         cdef double tmp = self._var.result()
 *         if tmp != 0.:             # <<<<<<<<<<<<<<
 *             return self._mean.result() / sqrt(tmp)
 *         else:
 */
  __pyx_t_6 = ((__pyx_v_tmp != 0.) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":887
 *         cdef double tmp = self._var.result()
 *         if tmp != 0.:
 *             return self._mean.result() / sqrt(tmp)             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    __pyx_r = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_mean->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_mean), 0) / sqrt(__pyx_v_tmp));
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":886
 *     cpdef double result(self):
 *         cdef double tmp = self._var.result()
 *         if tmp != 0.:             # <<<<<<<<<<<<<<
 *             return self._mean.result() / sqrt(tmp)
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":889
 *             return self._mean.result() / sqrt(tmp)
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":884
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double tmp = self._var.result()
 *         if tmp != 0.:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4result[] = "MovingSharp.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":251
 * cdef class MovingSharp(StatefulValueHolder):
 * 
 *     cdef public MovingAverage _mean             # <<<<<<<<<<<<<<
 *     cdef public MovingVariance _var
 *     cdef Accumulator _x
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_mean));
  __pyx_r = ((PyObject *)__pyx_v_self->_mean);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage))))) __PYX_ERR(1, 251, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_mean);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_mean));
  __pyx_v_self->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp._mean.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_mean);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_mean));
  __pyx_v_self->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":252
 * 
 *     cdef public MovingAverage _mean
 *     cdef public MovingVariance _var             # <<<<<<<<<<<<<<
 *     cdef Accumulator _x
 *     cdef Accumulator _y
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_var));
  __pyx_r = ((PyObject *)__pyx_v_self->_var);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance))))) __PYX_ERR(1, 252, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_var);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_var));
  __pyx_v_self->_var = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp._var.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_var);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_var));
  __pyx_v_self->_var = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_6__reduce_cython__[] = "MovingSharp.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._mean, self._var, self._window, self._x, self._y)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(9); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_deque_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_deque_y));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->_deque_y));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_mean));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_mean));
  PyTuple_SET_ITEM(__pyx_t_3, 4, ((PyObject *)__pyx_v_self->_mean));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_var));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_var));
  PyTuple_SET_ITEM(__pyx_t_3, 5, ((PyObject *)__pyx_v_self->_var));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 6, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_x));
  PyTuple_SET_ITEM(__pyx_t_3, 7, ((PyObject *)__pyx_v_self->_x));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_y));
  PyTuple_SET_ITEM(__pyx_t_3, 8, ((PyObject *)__pyx_v_self->_y));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._mean, self._var, self._window, self._x, self._y)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._mean, self._var, self._window, self._x, self._y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._var is not None or self._x is not None or self._y is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._mean, self._var, self._window, self._x, self._y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._var is not None or self._x is not None or self._y is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._deque) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_deque_y) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_mean) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_var) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_x) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_y) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    __pyx_t_5 = __pyx_t_6;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._var is not None or self._x is not None or self._y is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._var is not None or self._x is not None or self._y is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingSharp); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_224517490);
    __Pyx_GIVEREF(__pyx_int_224517490);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_224517490);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._var is not None or self._x is not None or self._y is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, None), state
 *     else:
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingSharp__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_MovingSharp); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_224517490);
    __Pyx_GIVEREF(__pyx_int_224517490);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_224517490);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingSharp__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_8__setstate_cython__[] = "MovingSharp.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingSharp__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSharp__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingSharp, (type(self), 0xd61dd72, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingSharp__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":894
 * cdef class MovingSortino(StatefulValueHolder):
 * 
 *     def __init__(self, window, x, y):             # <<<<<<<<<<<<<<
 *         super(MovingSortino, self).__init__(window)
 *         self._mean = MovingAverage(window, x='x')
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(0, 894, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(0, 894, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 894, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
    __pyx_v_y = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 894, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self), __pyx_v_window, __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":895
 * 
 *     def __init__(self, window, x, y):
 *         super(MovingSortino, self).__init__(window)             # <<<<<<<<<<<<<<
 *         self._mean = MovingAverage(window, x='x')
 *         self._negativeVar = MovingNegativeVariance(window, x='x')
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":896
 *     def __init__(self, window, x, y):
 *         super(MovingSortino, self).__init__(window)
 *         self._mean = MovingAverage(window, x='x')             # <<<<<<<<<<<<<<
 *         self._negativeVar = MovingNegativeVariance(window, x='x')
 *         self._x = build_holder(x)
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 896, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_window);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 896, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_x, __pyx_n_s_x) < 0) __PYX_ERR(0, 896, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage), __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 896, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_mean);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_mean));
  __pyx_v_self->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":897
 *         super(MovingSortino, self).__init__(window)
 *         self._mean = MovingAverage(window, x='x')
 *         self._negativeVar = MovingNegativeVariance(window, x='x')             # <<<<<<<<<<<<<<
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)
 */
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_window);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_x, __pyx_n_s_x) < 0) __PYX_ERR(0, 897, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance), __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_negativeVar);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_negativeVar));
  __pyx_v_self->_negativeVar = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":898
 *         self._mean = MovingAverage(window, x='x')
 *         self._negativeVar = MovingNegativeVariance(window, x='x')
 *         self._x = build_holder(x)             # <<<<<<<<<<<<<<
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 898, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":899
 *         self._negativeVar = MovingNegativeVariance(window, x='x')
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)             # <<<<<<<<<<<<<<
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_y, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_y);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_y));
  __pyx_v_self->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":900
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)             # <<<<<<<<<<<<<<
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 *         self._deque_y = Deque(window)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_y), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 900, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 900, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 900, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 900, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 900, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 900, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
  } else {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 900, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.__pyx_base._window = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":901
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))             # <<<<<<<<<<<<<<
 *         self._deque_y = Deque(window)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_dependency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_y), __pyx_n_s_dependency); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PySet_New(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_List(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __pyx_v_self->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":902
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 *         self._deque_y = Deque(window)             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque), __pyx_v_window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_deque_y);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_deque_y));
  __pyx_v_self->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":894
 * cdef class MovingSortino(StatefulValueHolder):
 * 
 *     def __init__(self, window, x, y):             # <<<<<<<<<<<<<<
 *         super(MovingSortino, self).__init__(window)
 *         self._mean = MovingAverage(window, x='x')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":904
 *         self._deque_y = Deque(window)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef tuple value
 *         cdef double ret
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_ret;
  double __pyx_v_benchmark;
  PyObject *__pyx_v_new_data = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 904, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 904, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":910
 *         cdef dict new_data
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         ret = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->push(__pyx_v_self->_x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 910, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":911
 * 
 *         self._x.push(data)
 *         ret = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         self._y.push(data)
 */
  __pyx_v_ret = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->result(__pyx_v_self->_x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":913
 *         ret = self._x.result()
 * 
 *         self._y.push(data)             # <<<<<<<<<<<<<<
 *         benchmark = self._y.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_y->__pyx_vtab)->push(__pyx_v_self->_y, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 913, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":914
 * 
 *         self._y.push(data)
 *         benchmark = self._y.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(ret) or isnan(benchmark):
 */
  __pyx_v_benchmark = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_y->__pyx_vtab)->result(__pyx_v_self->_y, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":916
 *         benchmark = self._y.result()
 * 
 *         if isnan(ret) or isnan(benchmark):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  __pyx_t_6 = (isnan(__pyx_v_ret) != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = (isnan(__pyx_v_benchmark) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":917
 * 
 *         if isnan(ret) or isnan(benchmark):
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *         new_data = {'x': ret - benchmark}
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 917, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":916
 *         benchmark = self._y.result()
 * 
 *         if isnan(ret) or isnan(benchmark):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":919
 *             return NAN
 * 
 *         new_data = {'x': ret - benchmark}             # <<<<<<<<<<<<<<
 *         self._mean.push(new_data)
 *         self._negativeVar.push(new_data)
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble((__pyx_v_ret - __pyx_v_benchmark)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_x, __pyx_t_2) < 0) __PYX_ERR(0, 919, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_new_data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":920
 * 
 *         new_data = {'x': ret - benchmark}
 *         self._mean.push(new_data)             # <<<<<<<<<<<<<<
 *         self._negativeVar.push(new_data)
 *         self._isFull = self._isFull or (self._negativeVar.isFull() and self._mean.isFull())
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_mean->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_mean), __pyx_v_new_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 920, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":921
 *         new_data = {'x': ret - benchmark}
 *         self._mean.push(new_data)
 *         self._negativeVar.push(new_data)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or (self._negativeVar.isFull() and self._mean.isFull())
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self->_negativeVar->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_negativeVar), __pyx_v_new_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 921, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":922
 *         self._mean.push(new_data)
 *         self._negativeVar.push(new_data)
 *         self._isFull = self._isFull or (self._negativeVar.isFull() and self._mean.isFull())             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self->_negativeVar->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_negativeVar), 0) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_mean->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_mean), 0) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L6_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":904
 *         self._deque_y = Deque(window)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef tuple value
 *         cdef double ret
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_new_data);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_2push[] = "MovingSortino.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 904, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 904, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":925
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double tmp = self._negativeVar.result()
 *         if tmp != 0.:
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_v_tmp;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 925, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 925, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 925, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":926
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         cdef double tmp = self._negativeVar.result()             # <<<<<<<<<<<<<<
 *         if tmp != 0.:
 *             return self._mean.result() /sqrt(self._negativeVar.result())
 */
  __pyx_v_tmp = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self->_negativeVar->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_negativeVar), 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":927
 *     cpdef double result(self):
 *         cdef double tmp = self._negativeVar.result()
 *         if tmp != 0.:             # <<<<<<<<<<<<<<
 *             return self._mean.result() /sqrt(self._negativeVar.result())
 *         else:
 */
  __pyx_t_6 = ((__pyx_v_tmp != 0.) != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":928
 *         cdef double tmp = self._negativeVar.result()
 *         if tmp != 0.:
 *             return self._mean.result() /sqrt(self._negativeVar.result())             # <<<<<<<<<<<<<<
 *         else:
 *             return NAN
 */
    __pyx_r = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v_self->_mean->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_mean), 0) / sqrt(((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v_self->_negativeVar->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_negativeVar), 0)));
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":927
 *     cpdef double result(self):
 *         cdef double tmp = self._negativeVar.result()
 *         if tmp != 0.:             # <<<<<<<<<<<<<<
 *             return self._mean.result() /sqrt(self._negativeVar.result())
 *         else:
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":930
 *             return self._mean.result() /sqrt(self._negativeVar.result())
 *         else:
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":925
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double tmp = self._negativeVar.result()
 *         if tmp != 0.:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_4result[] = "MovingSortino.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 925, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":263
 * cdef class MovingSortino(StatefulValueHolder):
 * 
 *     cdef public MovingAverage _mean             # <<<<<<<<<<<<<<
 *     cdef public MovingNegativeVariance _negativeVar
 *     cdef Accumulator _x
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_mean));
  __pyx_r = ((PyObject *)__pyx_v_self->_mean);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage))))) __PYX_ERR(1, 263, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_mean);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_mean));
  __pyx_v_self->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino._mean.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_mean);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_mean));
  __pyx_v_self->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":264
 * 
 *     cdef public MovingAverage _mean
 *     cdef public MovingNegativeVariance _negativeVar             # <<<<<<<<<<<<<<
 *     cdef Accumulator _x
 *     cdef Accumulator _y
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_negativeVar));
  __pyx_r = ((PyObject *)__pyx_v_self->_negativeVar);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance))))) __PYX_ERR(1, 264, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_negativeVar);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_negativeVar));
  __pyx_v_self->_negativeVar = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino._negativeVar.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_4__del__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_4__del__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_negativeVar);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_negativeVar));
  __pyx_v_self->_negativeVar = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_6__reduce_cython__[] = "MovingSortino.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._mean, self._negativeVar, self._window, self._x, self._y)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(9); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_deque_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_deque_y));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->_deque_y));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_mean));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_mean));
  PyTuple_SET_ITEM(__pyx_t_3, 4, ((PyObject *)__pyx_v_self->_mean));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_negativeVar));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_negativeVar));
  PyTuple_SET_ITEM(__pyx_t_3, 5, ((PyObject *)__pyx_v_self->_negativeVar));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 6, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_x));
  PyTuple_SET_ITEM(__pyx_t_3, 7, ((PyObject *)__pyx_v_self->_x));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_y));
  PyTuple_SET_ITEM(__pyx_t_3, 8, ((PyObject *)__pyx_v_self->_y));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._mean, self._negativeVar, self._window, self._x, self._y)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._mean, self._negativeVar, self._window, self._x, self._y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._negativeVar is not None or self._x is not None or self._y is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._deque_y, self._isFull, self._mean, self._negativeVar, self._window, self._x, self._y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._negativeVar is not None or self._x is not None or self._y is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._deque) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_deque_y) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_mean) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_negativeVar) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_x) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_y) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    __pyx_t_5 = __pyx_t_6;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._negativeVar is not None or self._x is not None or self._y is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._negativeVar is not None or self._x is not None or self._y is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingSortino); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_67199698);
    __Pyx_GIVEREF(__pyx_int_67199698);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_67199698);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._mean is not None or self._negativeVar is not None or self._x is not None or self._y is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, None), state
 *     else:
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingSortino__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_MovingSortino); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_67199698);
    __Pyx_GIVEREF(__pyx_int_67199698);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_67199698);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingSortino__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_8__setstate_cython__[] = "MovingSortino.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingSortino__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSortino__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingSortino, (type(self), 0x40162d2, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingSortino__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":935
 * cdef class MovingDrawdown(StatefulValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingDrawdown, self).__init__(window)
 *         self._maxer = MovingMax(window + 1, x='x')
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 935, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 935, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 935, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDrawdown.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":936
 * 
 *     def __init__(self, window, x):
 *         super(MovingDrawdown, self).__init__(window)             # <<<<<<<<<<<<<<
 *         self._maxer = MovingMax(window + 1, x='x')
 *         self._maxer.push(dict(x=0.0))
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":937
 *     def __init__(self, window, x):
 *         super(MovingDrawdown, self).__init__(window)
 *         self._maxer = MovingMax(window + 1, x='x')             # <<<<<<<<<<<<<<
 *         self._maxer.push(dict(x=0.0))
 *         self._x = build_holder(x)
 */
  __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_window, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_x, __pyx_n_s_x) < 0) __PYX_ERR(0, 937, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax), __pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_maxer);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_maxer));
  __pyx_v_self->_maxer = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":938
 *         super(MovingDrawdown, self).__init__(window)
 *         self._maxer = MovingMax(window + 1, x='x')
 *         self._maxer.push(dict(x=0.0))             # <<<<<<<<<<<<<<
 *         self._x = build_holder(x)
 *         self._runningCum = 0.0
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 938, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_x, __pyx_float_0_0) < 0) __PYX_ERR(0, 938, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v_self->_maxer->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_maxer), ((PyObject*)__pyx_t_3), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 938, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":939
 *         self._maxer = MovingMax(window + 1, x='x')
 *         self._maxer.push(dict(x=0.0))
 *         self._x = build_holder(x)             # <<<<<<<<<<<<<<
 *         self._runningCum = 0.0
 *         self._currentMax = NAN
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 939, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 939, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":940
 *         self._maxer.push(dict(x=0.0))
 *         self._x = build_holder(x)
 *         self._runningCum = 0.0             # <<<<<<<<<<<<<<
 *         self._currentMax = NAN
 * 
 */
  __pyx_v_self->_runningCum = 0.0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":941
 *         self._x = build_holder(x)
 *         self._runningCum = 0.0
 *         self._currentMax = NAN             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_v_self->_currentMax = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":935
 * cdef class MovingDrawdown(StatefulValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingDrawdown, self).__init__(window)
 *         self._maxer = MovingMax(window + 1, x='x')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDrawdown.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":943
 *         self._currentMax = NAN
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._x.push(data)
 *         cdef double ret = self._x.result()
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 943, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 943, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":944
 * 
 *     cpdef push(self, dict data):
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double ret = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->push(__pyx_v_self->_x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 944, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":945
 *     cpdef push(self, dict data):
 *         self._x.push(data)
 *         cdef double ret = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(ret):
 */
  __pyx_v_ret = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->result(__pyx_v_self->_x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":947
 *         cdef double ret = self._x.result()
 * 
 *         if isnan(ret):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  __pyx_t_5 = (isnan(__pyx_v_ret) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":948
 * 
 *         if isnan(ret):
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *         self._runningCum += ret
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 948, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":947
 *         cdef double ret = self._x.result()
 * 
 *         if isnan(ret):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":950
 *             return NAN
 * 
 *         self._runningCum += ret             # <<<<<<<<<<<<<<
 *         self._maxer.push(dict(x=self._runningCum))
 *         self._currentMax = self._maxer.result()
 */
  __pyx_v_self->_runningCum = (__pyx_v_self->_runningCum + __pyx_v_ret);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":951
 * 
 *         self._runningCum += ret
 *         self._maxer.push(dict(x=self._runningCum))             # <<<<<<<<<<<<<<
 *         self._currentMax = self._maxer.result()
 *         self._isFull = self._isFull or self._maxer.isFull()
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_runningCum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_x, __pyx_t_2) < 0) __PYX_ERR(0, 951, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v_self->_maxer->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_maxer), ((PyObject*)__pyx_t_1), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":952
 *         self._runningCum += ret
 *         self._maxer.push(dict(x=self._runningCum))
 *         self._currentMax = self._maxer.result()             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._maxer.isFull()
 * 
 */
  __pyx_v_self->_currentMax = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v_self->_maxer->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_maxer), 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":953
 *         self._maxer.push(dict(x=self._runningCum))
 *         self._currentMax = self._maxer.result()
 *         self._isFull = self._isFull or self._maxer.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v_self->_maxer->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.__pyx_base.isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_maxer), 0) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":943
 *         self._currentMax = NAN
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._x.push(data)
 *         cdef double ret = self._x.result()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDrawdown.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_2push[] = "MovingDrawdown.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 943, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 943, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDrawdown.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":955
 *         self._isFull = self._isFull or self._maxer.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningCum - self._currentMax
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 955, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 955, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 955, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":956
 * 
 *     cpdef double result(self):
 *         return self._runningCum - self._currentMax             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (__pyx_v_self->_runningCum - __pyx_v_self->_currentMax);
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":955
 *         self._isFull = self._isFull or self._maxer.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._runningCum - self._currentMax
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDrawdown.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_4result[] = "MovingDrawdown.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 955, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDrawdown.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_6__reduce_cython__[] = "MovingDrawdown.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._currentMax, self._dependency, self._deque, self._isFull, self._maxer, self._runningCum, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_currentMax); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_runningCum); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(8); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_5, 2, ((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_maxer));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_maxer));
  PyTuple_SET_ITEM(__pyx_t_5, 4, ((PyObject *)__pyx_v_self->_maxer));
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 6, __pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_x));
  PyTuple_SET_ITEM(__pyx_t_5, 7, ((PyObject *)__pyx_v_self->_x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._currentMax, self._dependency, self._deque, self._isFull, self._maxer, self._runningCum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_5 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v__dict = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "(tree fragment)":7
 *     state = (self._currentMax, self._dependency, self._deque, self._isFull, self._maxer, self._runningCum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._maxer is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._currentMax, self._dependency, self._deque, self._isFull, self._maxer, self._runningCum, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._maxer is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, None), state
 */
  /*else*/ {
    __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (((PyObject *)__pyx_v_self->__pyx_base._deque) != Py_None);
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_maxer) != Py_None);
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (((PyObject *)__pyx_v_self->_x) != Py_None);
    __pyx_t_6 = (__pyx_t_8 != 0);
    __pyx_t_7 = __pyx_t_6;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_7;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._maxer is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, None), state
 *     else:
 */
  __pyx_t_7 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._maxer is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_MovingDrawdown); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_31274558);
    __Pyx_GIVEREF(__pyx_int_31274558);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_31274558);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._maxer is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, None), state
 *     else:
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingDrawdown__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_MovingDrawdown); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_31274558);
    __Pyx_GIVEREF(__pyx_int_31274558);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_31274558);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDrawdown.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingDrawdown__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_8__setstate_cython__[] = "MovingDrawdown.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingDrawdown__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingDrawdown__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingDrawdown, (type(self), 0x1dd363e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingDrawdown__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDrawdown.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":961
 * cdef class MovingMaxDrawdown(StatefulValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingMaxDrawdown, self).__init__(window)
 *         self._drawdownCalculator = MovingDrawdown(window, 'x')
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 961, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 961, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 961, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMaxDrawdown.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)__pyx_v_self), __pyx_v_window, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":962
 * 
 *     def __init__(self, window, x):
 *         super(MovingMaxDrawdown, self).__init__(window)             # <<<<<<<<<<<<<<
 *         self._drawdownCalculator = MovingDrawdown(window, 'x')
 *         self._minimer = MovingMin(window, 'x')
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":963
 *     def __init__(self, window, x):
 *         super(MovingMaxDrawdown, self).__init__(window)
 *         self._drawdownCalculator = MovingDrawdown(window, 'x')             # <<<<<<<<<<<<<<
 *         self._minimer = MovingMin(window, 'x')
 *         self._x = build_holder(x)
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_window);
  __Pyx_INCREF(__pyx_n_s_x);
  __Pyx_GIVEREF(__pyx_n_s_x);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_x);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_drawdownCalculator);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_drawdownCalculator));
  __pyx_v_self->_drawdownCalculator = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":964
 *         super(MovingMaxDrawdown, self).__init__(window)
 *         self._drawdownCalculator = MovingDrawdown(window, 'x')
 *         self._minimer = MovingMin(window, 'x')             # <<<<<<<<<<<<<<
 *         self._x = build_holder(x)
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 964, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_window);
  __Pyx_GIVEREF(__pyx_v_window);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_window);
  __Pyx_INCREF(__pyx_n_s_x);
  __Pyx_GIVEREF(__pyx_n_s_x);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_x);
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 964, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_minimer);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_minimer));
  __pyx_v_self->_minimer = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":965
 *         self._drawdownCalculator = MovingDrawdown(window, 'x')
 *         self._minimer = MovingMin(window, 'x')
 *         self._x = build_holder(x)             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 965, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 965, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":961
 * cdef class MovingMaxDrawdown(StatefulValueHolder):
 * 
 *     def __init__(self, window, x):             # <<<<<<<<<<<<<<
 *         super(MovingMaxDrawdown, self).__init__(window)
 *         self._drawdownCalculator = MovingDrawdown(window, 'x')
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMaxDrawdown.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":967
 *         self._x = build_holder(x)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._x.push(data)
 *         cdef double ret = self._x.result()
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_ret;
  double __pyx_v_draw_down;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 967, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 967, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":968
 * 
 *     cpdef push(self, dict data):
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         cdef double ret = self._x.result()
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->push(__pyx_v_self->_x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":969
 *     cpdef push(self, dict data):
 *         self._x.push(data)
 *         cdef double ret = self._x.result()             # <<<<<<<<<<<<<<
 * 
 *         if isnan(ret):
 */
  __pyx_v_ret = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->result(__pyx_v_self->_x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":971
 *         cdef double ret = self._x.result()
 * 
 *         if isnan(ret):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  __pyx_t_5 = (isnan(__pyx_v_ret) != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":972
 * 
 *         if isnan(ret):
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *         self._drawdownCalculator.push(dict(x=ret))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 972, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":971
 *         cdef double ret = self._x.result()
 * 
 *         if isnan(ret):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":974
 *             return NAN
 * 
 *         self._drawdownCalculator.push(dict(x=ret))             # <<<<<<<<<<<<<<
 *         cdef double draw_down = self._drawdownCalculator.result()
 *         self._minimer.push(dict(x=draw_down))
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_ret); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_x, __pyx_t_2) < 0) __PYX_ERR(0, 974, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_v_self->_drawdownCalculator->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_drawdownCalculator), ((PyObject*)__pyx_t_1), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":975
 * 
 *         self._drawdownCalculator.push(dict(x=ret))
 *         cdef double draw_down = self._drawdownCalculator.result()             # <<<<<<<<<<<<<<
 *         self._minimer.push(dict(x=draw_down))
 *         self._deque.dump(draw_down)
 */
  __pyx_v_draw_down = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_v_self->_drawdownCalculator->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_drawdownCalculator), 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":976
 *         self._drawdownCalculator.push(dict(x=ret))
 *         cdef double draw_down = self._drawdownCalculator.result()
 *         self._minimer.push(dict(x=draw_down))             # <<<<<<<<<<<<<<
 *         self._deque.dump(draw_down)
 *         self._isFull = self._isFull or self._deque.isFull()
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 976, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_draw_down); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 976, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_x, __pyx_t_1) < 0) __PYX_ERR(0, 976, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_v_self->_minimer->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.__pyx_base.push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_minimer), ((PyObject*)__pyx_t_2), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 976, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":977
 *         cdef double draw_down = self._drawdownCalculator.result()
 *         self._minimer.push(dict(x=draw_down))
 *         self._deque.dump(draw_down)             # <<<<<<<<<<<<<<
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 */
  (void)(((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base._deque, __pyx_v_draw_down, NULL));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":978
 *         self._minimer.push(dict(x=draw_down))
 *         self._deque.dump(draw_down)
 *         self._isFull = self._isFull or self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self):
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base._isFull != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = (((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base._deque) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  __pyx_v_self->__pyx_base.__pyx_base._isFull = __pyx_t_5;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":967
 *         self._x = build_holder(x)
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._x.push(data)
 *         cdef double ret = self._x.result()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMaxDrawdown.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_2push[] = "MovingMaxDrawdown.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 967, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 967, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMaxDrawdown.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":980
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._minimer.result()
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 980, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_5result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 980, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 980, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":981
 * 
 *     cpdef double result(self):
 *         return self._minimer.result()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_v_self->_minimer->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.__pyx_base.__pyx_base.result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_minimer), 0);
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":980
 *         self._isFull = self._isFull or self._deque.isFull()
 * 
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         return self._minimer.result()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMaxDrawdown.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_4result[] = "MovingMaxDrawdown.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_5result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_4result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_4result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 980, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMaxDrawdown.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_6__reduce_cython__[] = "MovingMaxDrawdown.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_6__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_6__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._drawdownCalculator, self._isFull, self._minimer, self._window, self._x)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(7); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_drawdownCalculator));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_drawdownCalculator));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->_drawdownCalculator));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_minimer));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_minimer));
  PyTuple_SET_ITEM(__pyx_t_3, 4, ((PyObject *)__pyx_v_self->_minimer));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_x));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->_x));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._deque, self._drawdownCalculator, self._isFull, self._minimer, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._drawdownCalculator, self._isFull, self._minimer, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._drawdownCalculator is not None or self._minimer is not None or self._x is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._deque, self._drawdownCalculator, self._isFull, self._minimer, self._window, self._x)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._drawdownCalculator is not None or self._minimer is not None or self._x is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._deque) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_drawdownCalculator) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_minimer) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_x) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    __pyx_t_5 = __pyx_t_6;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._drawdownCalculator is not None or self._minimer is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._drawdownCalculator is not None or self._minimer is not None or self._x is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_MovingMaxDrawdown); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_130223719);
    __Pyx_GIVEREF(__pyx_int_130223719);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_130223719);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._drawdownCalculator is not None or self._minimer is not None or self._x is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, None), state
 *     else:
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingMaxDrawdown__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_MovingMaxDrawdown); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_130223719);
    __Pyx_GIVEREF(__pyx_int_130223719);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_130223719);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMaxDrawdown.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingMaxDrawdown__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_8__setstate_cython__[] = "MovingMaxDrawdown.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_8__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_8__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingMaxDrawdown__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMaxDrawdown__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingMaxDrawdown, (type(self), 0x7c30e67, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingMaxDrawdown__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMaxDrawdown.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":986
 * cdef class MovingResidue(StatefulValueHolder):
 * 
 *     def __init__(self, window, x, y):             # <<<<<<<<<<<<<<
 *         super(MovingResidue, self).__init__(window)
 *         self._cross = 0.
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_window = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_window,&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_window)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(0, 986, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(0, 986, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 986, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_window = values[0];
    __pyx_v_x = values[1];
    __pyx_v_y = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 986, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue___init__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self), __pyx_v_window, __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue___init__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_window, PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":987
 * 
 *     def __init__(self, window, x, y):
 *         super(MovingResidue, self).__init__(window)             # <<<<<<<<<<<<<<
 *         self._cross = 0.
 *         self._xsquare = 0.
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_window) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_window);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":988
 *     def __init__(self, window, x, y):
 *         super(MovingResidue, self).__init__(window)
 *         self._cross = 0.             # <<<<<<<<<<<<<<
 *         self._xsquare = 0.
 *         self._lasty = NAN
 */
  __pyx_v_self->_cross = 0.;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":989
 *         super(MovingResidue, self).__init__(window)
 *         self._cross = 0.
 *         self._xsquare = 0.             # <<<<<<<<<<<<<<
 *         self._lasty = NAN
 *         self._lastx = NAN
 */
  __pyx_v_self->_xsquare = 0.;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":990
 *         self._cross = 0.
 *         self._xsquare = 0.
 *         self._lasty = NAN             # <<<<<<<<<<<<<<
 *         self._lastx = NAN
 *         self._x = build_holder(x)
 */
  __pyx_v_self->_lasty = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":991
 *         self._xsquare = 0.
 *         self._lasty = NAN
 *         self._lastx = NAN             # <<<<<<<<<<<<<<
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)
 */
  __pyx_v_self->_lastx = __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":992
 *         self._lasty = NAN
 *         self._lastx = NAN
 *         self._x = build_holder(x)             # <<<<<<<<<<<<<<
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 992, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 992, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_x));
  __pyx_v_self->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":993
 *         self._lastx = NAN
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)             # <<<<<<<<<<<<<<
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 */
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder(__pyx_v_y, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 993, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(0, 993, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_y);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_y));
  __pyx_v_self->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":994
 *         self._x = build_holder(x)
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)             # <<<<<<<<<<<<<<
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 *         self._deque_y = Deque(window)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_y), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_v_window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 994, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
  } else {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 994, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->__pyx_base.__pyx_base._window = __pyx_t_6;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":995
 *         self._y = build_holder(y)
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))             # <<<<<<<<<<<<<<
 *         self._deque_y = Deque(window)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_x), __pyx_n_s_dependency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_y), __pyx_n_s_dependency); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PySet_New(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_List(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __pyx_v_self->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":996
 *         self._window = max(self._x.window + window, self._y.window + window)
 *         self._dependency = list(set(self._x.dependency + self._y.dependency))
 *         self._deque_y = Deque(window)             # <<<<<<<<<<<<<<
 * 
 *     @cython.boundscheck(False)
 */
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque), __pyx_v_window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 996, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_deque_y);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_deque_y));
  __pyx_v_self->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":986
 * cdef class MovingResidue(StatefulValueHolder):
 * 
 *     def __init__(self, window, x, y):             # <<<<<<<<<<<<<<
 *         super(MovingResidue, self).__init__(window)
 *         self._cross = 0.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1000
 *     @cython.boundscheck(False)
 *     @cython.wraparound(False)
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double x
 *         cdef double y
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  double __pyx_v_x;
  double __pyx_v_y;
  double __pyx_v_head_y;
  double __pyx_v_head_x;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  double __pyx_t_7;
  struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_4impl_5Deque_dump __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1000, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1000, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1006
 *         cdef double head_x
 * 
 *         self._x.push(data)             # <<<<<<<<<<<<<<
 *         x = self._x.result()
 *         self._y.push(data)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->push(__pyx_v_self->_x, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1006, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1007
 * 
 *         self._x.push(data)
 *         x = self._x.result()             # <<<<<<<<<<<<<<
 *         self._y.push(data)
 *         y = self._y.result()
 */
  __pyx_v_x = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_x->__pyx_vtab)->result(__pyx_v_self->_x, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1008
 *         self._x.push(data)
 *         x = self._x.result()
 *         self._y.push(data)             # <<<<<<<<<<<<<<
 *         y = self._y.result()
 *         if isnan(x) or isnan(y):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_y->__pyx_vtab)->push(__pyx_v_self->_y, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1008, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1009
 *         x = self._x.result()
 *         self._y.push(data)
 *         y = self._y.result()             # <<<<<<<<<<<<<<
 *         if isnan(x) or isnan(y):
 *             return NAN
 */
  __pyx_v_y = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_v_self->_y->__pyx_vtab)->result(__pyx_v_self->_y, 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1010
 *         self._y.push(data)
 *         y = self._y.result()
 *         if isnan(x) or isnan(y):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  __pyx_t_6 = (isnan(__pyx_v_x) != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = (isnan(__pyx_v_y) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1011
 *         y = self._y.result()
 *         if isnan(x) or isnan(y):
 *             return NAN             # <<<<<<<<<<<<<<
 * 
 *         head_x = self._deque.dump(x, 0.)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyFloat_FromDouble(__pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1011, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1010
 *         self._y.push(data)
 *         y = self._y.result()
 *         if isnan(x) or isnan(y):             # <<<<<<<<<<<<<<
 *             return NAN
 * 
 */
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1013
 *             return NAN
 * 
 *         head_x = self._deque.dump(x, 0.)             # <<<<<<<<<<<<<<
 *         head_y = self._deque_y.dump(y, 0.)
 * 
 */
  __pyx_t_8.__pyx_n = 1;
  __pyx_t_8.__pyx_default = 0.;
  __pyx_t_7 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->dump(__pyx_v_self->__pyx_base._deque, __pyx_v_x, &__pyx_t_8); 
  __pyx_v_head_x = __pyx_t_7;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1014
 * 
 *         head_x = self._deque.dump(x, 0.)
 *         head_y = self._deque_y.dump(y, 0.)             # <<<<<<<<<<<<<<
 * 
 *         self._cross += x * y - head_x * head_y
 */
  __pyx_t_8.__pyx_n = 1;
  __pyx_t_8.__pyx_default = 0.;
  __pyx_t_7 = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->_deque_y->__pyx_vtab)->dump(__pyx_v_self->_deque_y, __pyx_v_y, &__pyx_t_8); 
  __pyx_v_head_y = __pyx_t_7;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1016
 *         head_y = self._deque_y.dump(y, 0.)
 * 
 *         self._cross += x * y - head_x * head_y             # <<<<<<<<<<<<<<
 *         self._xsquare += x * x - head_x * head_x
 * 
 */
  __pyx_v_self->_cross = (__pyx_v_self->_cross + ((__pyx_v_x * __pyx_v_y) - (__pyx_v_head_x * __pyx_v_head_y)));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1017
 * 
 *         self._cross += x * y - head_x * head_y
 *         self._xsquare += x * x - head_x * head_x             # <<<<<<<<<<<<<<
 * 
 *         self._lastx = x
 */
  __pyx_v_self->_xsquare = (__pyx_v_self->_xsquare + ((__pyx_v_x * __pyx_v_x) - (__pyx_v_head_x * __pyx_v_head_x)));

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1019
 *         self._xsquare += x * x - head_x * head_x
 * 
 *         self._lastx = x             # <<<<<<<<<<<<<<
 *         self._lasty = y
 * 
 */
  __pyx_v_self->_lastx = __pyx_v_x;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1020
 * 
 *         self._lastx = x
 *         self._lasty = y             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint isFull(self):
 */
  __pyx_v_self->_lasty = __pyx_v_y;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1000
 *     @cython.boundscheck(False)
 *     @cython.wraparound(False)
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         cdef double x
 *         cdef double y
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_2push[] = "MovingResidue.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 1000, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_2push(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_2push(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1000, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1022
 *         self._lasty = y
 * 
 *     cpdef bint isFull(self):             # <<<<<<<<<<<<<<
 *         return self._deque.isFull()
 * 
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_5isFull(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_isFull(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isFull", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_isFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1022, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_5isFull)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1022, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1022, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1023
 * 
 *     cpdef bint isFull(self):
 *         return self._deque.isFull()             # <<<<<<<<<<<<<<
 * 
 *     @cython.cdivision(True)
 */
  __pyx_r = ((struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_v_self->__pyx_base._deque->__pyx_vtab)->isFull(__pyx_v_self->__pyx_base._deque);
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1022
 *         self._lasty = y
 * 
 *     cpdef bint isFull(self):             # <<<<<<<<<<<<<<
 *         return self._deque.isFull()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.isFull", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_5isFull(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_4isFull[] = "MovingResidue.isFull(self) -> bool";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_5isFull(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isFull (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_4isFull(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_4isFull(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isFull", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_isFull(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1022, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.isFull", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1026
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double coef = self._cross / self._xsquare
 *         return self._lasty - coef * self._lastx
 */

static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_7result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_v_coef;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1026, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_7result)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1026, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1026, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1027
 *     @cython.cdivision(True)
 *     cpdef double result(self):
 *         cdef double coef = self._cross / self._xsquare             # <<<<<<<<<<<<<<
 *         return self._lasty - coef * self._lastx
 * 
 */
  __pyx_v_coef = (__pyx_v_self->_cross / __pyx_v_self->_xsquare);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1028
 *     cpdef double result(self):
 *         cdef double coef = self._cross / self._xsquare
 *         return self._lasty - coef * self._lastx             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_r = (__pyx_v_self->_lasty - (__pyx_v_coef * __pyx_v_self->_lastx));
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1026
 * 
 *     @cython.cdivision(True)
 *     cpdef double result(self):             # <<<<<<<<<<<<<<
 *         cdef double coef = self._cross / self._xsquare
 *         return self._lasty - coef * self._lastx
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.result", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_7result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6result[] = "MovingResidue.result(self) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_7result(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("result (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6result(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6result(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("result", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_result(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1026, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.result", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1030
 *         return self._lasty - coef * self._lastx
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{Res}}({0}, {1}, {2})".format(self._window, str(self._x), str(self._y))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8__str__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8__str__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1031
 * 
 *     def __str__(self):
 *         return "\\mathrm{{Res}}({0}, {1}, {2})".format(self._window, str(self._x), str(self._y))             # <<<<<<<<<<<<<<
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_Res_0_1_2, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->_x)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->_y)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1031, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1031, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1031, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1031, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1030
 *         return self._lasty - coef * self._lastx
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\\mathrm{{Res}}({0}, {1}, {2})".format(self._window, str(self._x), str(self._y))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":296
 * cdef class MovingResidue(StatefulValueHolder):
 * 
 *     cdef public double _cross             # <<<<<<<<<<<<<<
 *     cdef public double _xsquare
 *     cdef public double _lastx
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_cross); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue._cross.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 296, __pyx_L1_error)
  __pyx_v_self->_cross = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue._cross.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":297
 * 
 *     cdef public double _cross
 *     cdef public double _xsquare             # <<<<<<<<<<<<<<
 *     cdef public double _lastx
 *     cdef public double _lasty
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_xsquare); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue._xsquare.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 297, __pyx_L1_error)
  __pyx_v_self->_xsquare = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue._xsquare.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":298
 *     cdef public double _cross
 *     cdef public double _xsquare
 *     cdef public double _lastx             # <<<<<<<<<<<<<<
 *     cdef public double _lasty
 *     cdef Accumulator _x
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_lastx); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue._lastx.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 298, __pyx_L1_error)
  __pyx_v_self->_lastx = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue._lastx.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pxd":299
 *     cdef public double _xsquare
 *     cdef public double _lastx
 *     cdef public double _lasty             # <<<<<<<<<<<<<<
 *     cdef Accumulator _x
 *     cdef Accumulator _y
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty___get__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty___get__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_lasty); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue._lasty.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty_2__set__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty_2__set__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 299, __pyx_L1_error)
  __pyx_v_self->_lasty = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue._lasty.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_10__reduce_cython__[] = "MovingResidue.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._cross, self._dependency, self._deque, self._deque_y, self._isFull, self._lastx, self._lasty, self._window, self._x, self._xsquare, self._y)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_cross); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base._isFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_lastx); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->_lasty); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->_xsquare); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyTuple_New(11); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._deque));
  PyTuple_SET_ITEM(__pyx_t_7, 2, ((PyObject *)__pyx_v_self->__pyx_base._deque));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_deque_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_deque_y));
  PyTuple_SET_ITEM(__pyx_t_7, 3, ((PyObject *)__pyx_v_self->_deque_y));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_7, 4, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 5, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_7, 6, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 7, __pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_x));
  PyTuple_SET_ITEM(__pyx_t_7, 8, ((PyObject *)__pyx_v_self->_x));
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_7, 9, __pyx_t_6);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_y));
  PyTuple_SET_ITEM(__pyx_t_7, 10, ((PyObject *)__pyx_v_self->_y));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._cross, self._dependency, self._deque, self._deque_y, self._isFull, self._lastx, self._lasty, self._window, self._x, self._xsquare, self._y)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_7 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_v__dict = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "(tree fragment)":7
 *     state = (self._cross, self._dependency, self._deque, self._deque_y, self._isFull, self._lastx, self._lasty, self._window, self._x, self._xsquare, self._y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_8 = (__pyx_v__dict != Py_None);
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v__dict);
    __pyx_t_6 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._cross, self._dependency, self._deque, self._deque_y, self._isFull, self._lastx, self._lasty, self._window, self._x, self._xsquare, self._y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, None), state
 */
  /*else*/ {
    __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (((PyObject *)__pyx_v_self->__pyx_base._deque) != Py_None);
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (((PyObject *)__pyx_v_self->_deque_y) != Py_None);
    __pyx_t_10 = (__pyx_t_8 != 0);
    if (!__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_10 = (((PyObject *)__pyx_v_self->_x) != Py_None);
    __pyx_t_8 = (__pyx_t_10 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (((PyObject *)__pyx_v_self->_y) != Py_None);
    __pyx_t_10 = (__pyx_t_8 != 0);
    __pyx_t_9 = __pyx_t_10;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_9;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, None), state
 *     else:
 */
  __pyx_t_9 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_9) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None
 *     if use_setstate:
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_MovingResidue); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_7, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_182436700);
    __Pyx_GIVEREF(__pyx_int_182436700);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_int_182436700);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_7, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_7);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._deque is not None or self._deque_y is not None or self._x is not None or self._y is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, None), state
 *     else:
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingResidue__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_MovingResidue); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_7, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_182436700);
    __Pyx_GIVEREF(__pyx_int_182436700);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_int_182436700);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_v_state);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_7);
    __pyx_t_5 = 0;
    __pyx_t_7 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingResidue__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_12__setstate_cython__[] = "MovingResidue.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_MovingResidue__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingResidue__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_MovingResidue, (type(self), 0xadfc35c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_MovingResidue__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_StatefulValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_1__pyx_unpickle_StatefulValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_StatefulValueHolder[] = "__pyx_unpickle_StatefulValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_1__pyx_unpickle_StatefulValueHolder = {"__pyx_unpickle_StatefulValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_1__pyx_unpickle_StatefulValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_StatefulValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_1__pyx_unpickle_StatefulValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_StatefulValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StatefulValueHolder", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StatefulValueHolder", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_StatefulValueHolder") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StatefulValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_StatefulValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_StatefulValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_StatefulValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StatefulValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x93f2e8c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93f2e8c = (_dependency, _deque, _isFull, _window))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x93f2e8c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x93f2e8c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93f2e8c = (_dependency, _deque, _isFull, _window))" % __pyx_checksum)
 *     __pyx_result = StatefulValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x93f2e8c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93f2e8c = (_dependency, _deque, _isFull, _window))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = StatefulValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x93, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x93f2e8c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93f2e8c = (_dependency, _deque, _isFull, _window))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93f2e8c = (_dependency, _deque, _isFull, _window))" % __pyx_checksum)
 *     __pyx_result = StatefulValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StatefulValueHolder__set_state(<StatefulValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93f2e8c = (_dependency, _deque, _isFull, _window))" % __pyx_checksum)
 *     __pyx_result = StatefulValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StatefulValueHolder__set_state(<StatefulValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = StatefulValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StatefulValueHolder__set_state(<StatefulValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_StatefulValueHolder__set_state(StatefulValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_StatefulValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x93f2e8c = (_dependency, _deque, _isFull, _window))" % __pyx_checksum)
 *     __pyx_result = StatefulValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StatefulValueHolder__set_state(<StatefulValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StatefulValueHolder__set_state(<StatefulValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_StatefulValueHolder__set_state(StatefulValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_StatefulValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_StatefulValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_StatefulValueHolder__set_state(<StatefulValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StatefulValueHolder__set_state(StatefulValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_StatefulValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StatefulValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_StatefulValueHolder__set_state(StatefulValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_deque));
  __pyx_v___pyx_result->_deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._window = __pyx_t_3;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_StatefulValueHolder__set_state(StatefulValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 4) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_StatefulValueHolder__set_state(StatefulValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_StatefulValueHolder__set_state(<StatefulValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StatefulValueHolder__set_state(StatefulValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_StatefulValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Shift(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_3__pyx_unpickle_Shift(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_2__pyx_unpickle_Shift[] = "__pyx_unpickle_Shift(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_3__pyx_unpickle_Shift = {"__pyx_unpickle_Shift", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_3__pyx_unpickle_Shift, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_2__pyx_unpickle_Shift};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_3__pyx_unpickle_Shift(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Shift (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Shift", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Shift", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Shift") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Shift", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_Shift", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_2__pyx_unpickle_Shift(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_2__pyx_unpickle_Shift(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Shift", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x62254c8:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x62254c8 = (_dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x62254c8) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x62254c8:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x62254c8 = (_dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 *     __pyx_result = Shift.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x62254c8:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x62254c8 = (_dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Shift.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x62, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x62254c8:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x62254c8 = (_dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x62254c8 = (_dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 *     __pyx_result = Shift.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Shift__set_state(<Shift> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x62254c8 = (_dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 *     __pyx_result = Shift.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Shift__set_state(<Shift> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = Shift.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Shift__set_state(<Shift> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Shift__set_state(Shift __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_Shift__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x62254c8 = (_dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 *     __pyx_result = Shift.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Shift__set_state(<Shift> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Shift__set_state(<Shift> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Shift__set_state(Shift __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._popout = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Shift(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_Shift", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Shift__set_state(<Shift> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Shift__set_state(Shift __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._popout = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_Shift__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  double __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Shift__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Shift__set_state(Shift __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._popout = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_popout = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_x));
  __pyx_v___pyx_result->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Shift__set_state(Shift __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._popout = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 6) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_2 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._popout = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_Shift__set_state(Shift __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._popout = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_Shift__set_state(<Shift> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Shift__set_state(Shift __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._popout = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_Shift__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Delta(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5__pyx_unpickle_Delta(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4__pyx_unpickle_Delta[] = "__pyx_unpickle_Delta(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5__pyx_unpickle_Delta = {"__pyx_unpickle_Delta", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5__pyx_unpickle_Delta, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4__pyx_unpickle_Delta};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5__pyx_unpickle_Delta(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Delta (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Delta", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Delta", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Delta") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Delta", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_Delta", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4__pyx_unpickle_Delta(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4__pyx_unpickle_Delta(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Delta", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb597db2:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb597db2 = (_current, _dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xb597db2) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb597db2:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb597db2 = (_current, _dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 *     __pyx_result = Delta.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xb597db2:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb597db2 = (_current, _dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Delta.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xb5, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb597db2:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb597db2 = (_current, _dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb597db2 = (_current, _dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 *     __pyx_result = Delta.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Delta__set_state(<Delta> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb597db2 = (_current, _dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 *     __pyx_result = Delta.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Delta__set_state(<Delta> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = Delta.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Delta__set_state(<Delta> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Delta__set_state(Delta __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_Delta__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb597db2 = (_current, _dependency, _deque, _isFull, _popout, _window, _x))" % __pyx_checksum)
 *     __pyx_result = Delta.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Delta__set_state(<Delta> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Delta__set_state(<Delta> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Delta__set_state(Delta __pyx_result, tuple __pyx_state):
 *     __pyx_result._current = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._popout = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Delta(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_Delta", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Delta__set_state(<Delta> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Delta__set_state(Delta __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._current = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._popout = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_Delta__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Delta__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Delta__set_state(Delta __pyx_result, tuple __pyx_state):
 *     __pyx_result._current = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._popout = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_current = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_popout = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_x));
  __pyx_v___pyx_result->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Delta__set_state(Delta __pyx_result, tuple __pyx_state):
 *     __pyx_result._current = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._popout = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 7) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._current = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._popout = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_Delta__set_state(Delta __pyx_result, tuple __pyx_state):
 *     __pyx_result._current = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._popout = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_Delta__set_state(<Delta> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Delta__set_state(Delta __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._current = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._popout = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_Delta__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_SingleValuedValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_7__pyx_unpickle_SingleValuedValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_6__pyx_unpickle_SingleValuedValueHolder[] = "__pyx_unpickle_SingleValuedValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_7__pyx_unpickle_SingleValuedValueHolder = {"__pyx_unpickle_SingleValuedValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_7__pyx_unpickle_SingleValuedValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_6__pyx_unpickle_SingleValuedValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_7__pyx_unpickle_SingleValuedValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_SingleValuedValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SingleValuedValueHolder", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SingleValuedValueHolder", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_SingleValuedValueHolder") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SingleValuedValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_SingleValuedValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_6__pyx_unpickle_SingleValuedValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_6__pyx_unpickle_SingleValuedValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SingleValuedValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7ea5ed3:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7ea5ed3 = (_dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7ea5ed3) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7ea5ed3:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7ea5ed3 = (_dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = SingleValuedValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7ea5ed3:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7ea5ed3 = (_dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = SingleValuedValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7e, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7ea5ed3:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7ea5ed3 = (_dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7ea5ed3 = (_dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = SingleValuedValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SingleValuedValueHolder__set_state(<SingleValuedValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7ea5ed3 = (_dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = SingleValuedValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SingleValuedValueHolder__set_state(<SingleValuedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = SingleValuedValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SingleValuedValueHolder__set_state(<SingleValuedValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_SingleValuedValueHolder__set_state(SingleValuedValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_SingleValuedValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7ea5ed3 = (_dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = SingleValuedValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SingleValuedValueHolder__set_state(<SingleValuedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SingleValuedValueHolder__set_state(<SingleValuedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_SingleValuedValueHolder__set_state(SingleValuedValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]; __pyx_result._x = __pyx_state[4]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_SingleValuedValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_SingleValuedValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_SingleValuedValueHolder__set_state(<SingleValuedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SingleValuedValueHolder__set_state(SingleValuedValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]; __pyx_result._x = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_SingleValuedValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SingleValuedValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_SingleValuedValueHolder__set_state(SingleValuedValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]; __pyx_result._x = __pyx_state[4]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_x));
  __pyx_v___pyx_result->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_SingleValuedValueHolder__set_state(SingleValuedValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]; __pyx_result._x = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 5) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]; __pyx_result._x = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_SingleValuedValueHolder__set_state(SingleValuedValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]; __pyx_result._x = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_SingleValuedValueHolder__set_state(<SingleValuedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SingleValuedValueHolder__set_state(SingleValuedValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]; __pyx_result._x = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_SingleValuedValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_SortedValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9__pyx_unpickle_SortedValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_8__pyx_unpickle_SortedValueHolder[] = "__pyx_unpickle_SortedValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9__pyx_unpickle_SortedValueHolder = {"__pyx_unpickle_SortedValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9__pyx_unpickle_SortedValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_8__pyx_unpickle_SortedValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9__pyx_unpickle_SortedValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_SortedValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SortedValueHolder", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SortedValueHolder", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_SortedValueHolder") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_SortedValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_SortedValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_8__pyx_unpickle_SortedValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_8__pyx_unpickle_SortedValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SortedValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x349f33c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = SortedValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = SortedValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x34, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = SortedValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SortedValueHolder__set_state(<SortedValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = SortedValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SortedValueHolder__set_state(<SortedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = SortedValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SortedValueHolder__set_state(<SortedValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_SortedValueHolder__set_state(SortedValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_SortedValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = SortedValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_SortedValueHolder__set_state(<SortedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_SortedValueHolder__set_state(<SortedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_SortedValueHolder__set_state(SortedValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_SortedValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_SortedValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_SortedValueHolder__set_state(<SortedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SortedValueHolder__set_state(SortedValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_SortedValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_SortedValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_SortedValueHolder__set_state(SortedValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_cur_pos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_sortedArray);
  __Pyx_DECREF(__pyx_v___pyx_result->_sortedArray);
  __pyx_v___pyx_result->_sortedArray = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_SortedValueHolder__set_state(SortedValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 7) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_SortedValueHolder__set_state(SortedValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_SortedValueHolder__set_state(<SortedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SortedValueHolder__set_state(SortedValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_SortedValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingMax(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11__pyx_unpickle_MovingMax(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10__pyx_unpickle_MovingMax[] = "__pyx_unpickle_MovingMax(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11__pyx_unpickle_MovingMax = {"__pyx_unpickle_MovingMax", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11__pyx_unpickle_MovingMax, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10__pyx_unpickle_MovingMax};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11__pyx_unpickle_MovingMax(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingMax (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingMax", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingMax", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingMax") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingMax", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingMax", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10__pyx_unpickle_MovingMax(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10__pyx_unpickle_MovingMax(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingMax", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x349f33c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMax.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingMax.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x34, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMax.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingMax__set_state(<MovingMax> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMax.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingMax__set_state(<MovingMax> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingMax.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingMax__set_state(<MovingMax> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMax__set_state(MovingMax __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMax__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMax.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingMax__set_state(<MovingMax> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingMax__set_state(<MovingMax> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingMax__set_state(MovingMax __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingMax(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingMax", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingMax__set_state(<MovingMax> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMax__set_state(MovingMax __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMax__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingMax__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMax__set_state(MovingMax __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._cur_pos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __pyx_v___pyx_result->__pyx_base._sortedArray = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingMax__set_state(MovingMax __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 7) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingMax__set_state(MovingMax __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingMax__set_state(<MovingMax> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMax__set_state(MovingMax __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingMax__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingArgMax(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13__pyx_unpickle_MovingArgMax(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12__pyx_unpickle_MovingArgMax[] = "__pyx_unpickle_MovingArgMax(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13__pyx_unpickle_MovingArgMax = {"__pyx_unpickle_MovingArgMax", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13__pyx_unpickle_MovingArgMax, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12__pyx_unpickle_MovingArgMax};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13__pyx_unpickle_MovingArgMax(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingArgMax (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingArgMax", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingArgMax", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingArgMax") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingArgMax", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingArgMax", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12__pyx_unpickle_MovingArgMax(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12__pyx_unpickle_MovingArgMax(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingArgMax", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x349f33c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingArgMax.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingArgMax.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x34, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingArgMax.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingArgMax__set_state(<MovingArgMax> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingArgMax.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingArgMax__set_state(<MovingArgMax> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingArgMax.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingArgMax__set_state(<MovingArgMax> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMax__set_state(MovingArgMax __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingArgMax__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingArgMax.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingArgMax__set_state(<MovingArgMax> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingArgMax__set_state(<MovingArgMax> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingArgMax__set_state(MovingArgMax __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingArgMax(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingArgMax", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingArgMax__set_state(<MovingArgMax> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMax__set_state(MovingArgMax __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingArgMax__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingArgMax__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMax__set_state(MovingArgMax __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._cur_pos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __pyx_v___pyx_result->__pyx_base._sortedArray = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingArgMax__set_state(MovingArgMax __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 7) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingArgMax__set_state(MovingArgMax __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingArgMax__set_state(<MovingArgMax> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMax__set_state(MovingArgMax __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingArgMax__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingMin(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15__pyx_unpickle_MovingMin(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14__pyx_unpickle_MovingMin[] = "__pyx_unpickle_MovingMin(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15__pyx_unpickle_MovingMin = {"__pyx_unpickle_MovingMin", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15__pyx_unpickle_MovingMin, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14__pyx_unpickle_MovingMin};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15__pyx_unpickle_MovingMin(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingMin (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingMin", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingMin", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingMin") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingMin", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingMin", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14__pyx_unpickle_MovingMin(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14__pyx_unpickle_MovingMin(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingMin", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x349f33c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMin.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingMin.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x34, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMin.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingMin__set_state(<MovingMin> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMin.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingMin__set_state(<MovingMin> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingMin.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingMin__set_state(<MovingMin> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMin__set_state(MovingMin __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMin__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMin.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingMin__set_state(<MovingMin> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingMin__set_state(<MovingMin> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingMin__set_state(MovingMin __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingMin(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingMin", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingMin__set_state(<MovingMin> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMin__set_state(MovingMin __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMin__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingMin__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMin__set_state(MovingMin __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._cur_pos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __pyx_v___pyx_result->__pyx_base._sortedArray = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingMin__set_state(MovingMin __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 7) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingMin__set_state(MovingMin __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingMin__set_state(<MovingMin> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMin__set_state(MovingMin __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingMin__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingArgMin(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17__pyx_unpickle_MovingArgMin(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_16__pyx_unpickle_MovingArgMin[] = "__pyx_unpickle_MovingArgMin(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17__pyx_unpickle_MovingArgMin = {"__pyx_unpickle_MovingArgMin", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17__pyx_unpickle_MovingArgMin, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_16__pyx_unpickle_MovingArgMin};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17__pyx_unpickle_MovingArgMin(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingArgMin (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingArgMin", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingArgMin", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingArgMin") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingArgMin", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingArgMin", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_16__pyx_unpickle_MovingArgMin(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_16__pyx_unpickle_MovingArgMin(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingArgMin", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x349f33c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingArgMin.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingArgMin.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x34, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingArgMin.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingArgMin__set_state(<MovingArgMin> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingArgMin.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingArgMin__set_state(<MovingArgMin> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingArgMin.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingArgMin__set_state(<MovingArgMin> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMin__set_state(MovingArgMin __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingArgMin__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingArgMin.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingArgMin__set_state(<MovingArgMin> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingArgMin__set_state(<MovingArgMin> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingArgMin__set_state(MovingArgMin __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingArgMin(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingArgMin", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingArgMin__set_state(<MovingArgMin> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMin__set_state(MovingArgMin __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingArgMin__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingArgMin__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMin__set_state(MovingArgMin __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._cur_pos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __pyx_v___pyx_result->__pyx_base._sortedArray = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingArgMin__set_state(MovingArgMin __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 7) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingArgMin__set_state(MovingArgMin __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingArgMin__set_state(<MovingArgMin> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMin__set_state(MovingArgMin __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingArgMin__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingRank(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19__pyx_unpickle_MovingRank(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_18__pyx_unpickle_MovingRank[] = "__pyx_unpickle_MovingRank(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19__pyx_unpickle_MovingRank = {"__pyx_unpickle_MovingRank", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19__pyx_unpickle_MovingRank, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_18__pyx_unpickle_MovingRank};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19__pyx_unpickle_MovingRank(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingRank (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingRank", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingRank", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingRank") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingRank", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingRank", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_18__pyx_unpickle_MovingRank(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_18__pyx_unpickle_MovingRank(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingRank", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x349f33c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingRank.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingRank.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x34, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingRank.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingRank__set_state(<MovingRank> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingRank.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingRank__set_state(<MovingRank> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingRank.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingRank__set_state(<MovingRank> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingRank__set_state(MovingRank __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingRank__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingRank.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingRank__set_state(<MovingRank> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingRank__set_state(<MovingRank> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingRank__set_state(MovingRank __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingRank(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingRank", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingRank__set_state(<MovingRank> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingRank__set_state(MovingRank __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingRank__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingRank__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingRank__set_state(MovingRank __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._cur_pos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __pyx_v___pyx_result->__pyx_base._sortedArray = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingRank__set_state(MovingRank __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 7) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingRank__set_state(MovingRank __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingRank__set_state(<MovingRank> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingRank__set_state(MovingRank __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingRank__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingQuantile(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21__pyx_unpickle_MovingQuantile(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_20__pyx_unpickle_MovingQuantile[] = "__pyx_unpickle_MovingQuantile(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21__pyx_unpickle_MovingQuantile = {"__pyx_unpickle_MovingQuantile", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21__pyx_unpickle_MovingQuantile, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_20__pyx_unpickle_MovingQuantile};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21__pyx_unpickle_MovingQuantile(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingQuantile (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingQuantile", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingQuantile", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingQuantile") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingQuantile", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingQuantile", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_20__pyx_unpickle_MovingQuantile(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_20__pyx_unpickle_MovingQuantile(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingQuantile", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x349f33c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingQuantile.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x349f33c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingQuantile.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x34, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x349f33c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingQuantile.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingQuantile__set_state(<MovingQuantile> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingQuantile.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingQuantile__set_state(<MovingQuantile> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingQuantile.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingQuantile__set_state(<MovingQuantile> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingQuantile__set_state(MovingQuantile __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingQuantile__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x349f33c = (_cur_pos, _dependency, _deque, _isFull, _sortedArray, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingQuantile.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingQuantile__set_state(<MovingQuantile> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingQuantile__set_state(<MovingQuantile> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingQuantile__set_state(MovingQuantile __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingQuantile(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingQuantile", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingQuantile__set_state(<MovingQuantile> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingQuantile__set_state(MovingQuantile __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingQuantile__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingQuantile__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingQuantile__set_state(MovingQuantile __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._cur_pos = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._sortedArray);
  __pyx_v___pyx_result->__pyx_base._sortedArray = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingQuantile__set_state(MovingQuantile __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 7) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingQuantile__set_state(MovingQuantile __pyx_result, tuple __pyx_state):
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingQuantile__set_state(<MovingQuantile> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingQuantile__set_state(MovingQuantile __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingQuantile__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingAllTrue(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23__pyx_unpickle_MovingAllTrue(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22__pyx_unpickle_MovingAllTrue[] = "__pyx_unpickle_MovingAllTrue(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23__pyx_unpickle_MovingAllTrue = {"__pyx_unpickle_MovingAllTrue", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23__pyx_unpickle_MovingAllTrue, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22__pyx_unpickle_MovingAllTrue};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23__pyx_unpickle_MovingAllTrue(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingAllTrue (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingAllTrue", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingAllTrue", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingAllTrue") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingAllTrue", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingAllTrue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22__pyx_unpickle_MovingAllTrue(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22__pyx_unpickle_MovingAllTrue(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingAllTrue", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x171ca47:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x171ca47) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x171ca47:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAllTrue.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x171ca47:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingAllTrue.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x17, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x171ca47:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAllTrue.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingAllTrue__set_state(<MovingAllTrue> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAllTrue.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingAllTrue__set_state(<MovingAllTrue> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingAllTrue.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingAllTrue__set_state(<MovingAllTrue> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAllTrue__set_state(MovingAllTrue __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAllTrue__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAllTrue.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingAllTrue__set_state(<MovingAllTrue> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingAllTrue__set_state(<MovingAllTrue> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingAllTrue__set_state(MovingAllTrue __pyx_result, tuple __pyx_state):
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingAllTrue(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingAllTrue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingAllTrue__set_state(<MovingAllTrue> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAllTrue__set_state(MovingAllTrue __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAllTrue__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingAllTrue__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAllTrue__set_state(MovingAllTrue __pyx_result, tuple __pyx_state):
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_countedTrue = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingAllTrue__set_state(MovingAllTrue __pyx_result, tuple __pyx_state):
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 6) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingAllTrue__set_state(MovingAllTrue __pyx_result, tuple __pyx_state):
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingAllTrue__set_state(<MovingAllTrue> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAllTrue__set_state(MovingAllTrue __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingAllTrue__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingAnyTrue(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_25__pyx_unpickle_MovingAnyTrue(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_24__pyx_unpickle_MovingAnyTrue[] = "__pyx_unpickle_MovingAnyTrue(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_25__pyx_unpickle_MovingAnyTrue = {"__pyx_unpickle_MovingAnyTrue", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_25__pyx_unpickle_MovingAnyTrue, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_24__pyx_unpickle_MovingAnyTrue};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_25__pyx_unpickle_MovingAnyTrue(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingAnyTrue (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingAnyTrue", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingAnyTrue", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingAnyTrue") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingAnyTrue", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingAnyTrue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_24__pyx_unpickle_MovingAnyTrue(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_24__pyx_unpickle_MovingAnyTrue(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingAnyTrue", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x171ca47:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x171ca47) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x171ca47:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAnyTrue.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x171ca47:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingAnyTrue.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x17, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x171ca47:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAnyTrue.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingAnyTrue__set_state(<MovingAnyTrue> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAnyTrue.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingAnyTrue__set_state(<MovingAnyTrue> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingAnyTrue.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingAnyTrue__set_state(<MovingAnyTrue> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAnyTrue__set_state(MovingAnyTrue __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAnyTrue__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x171ca47 = (_countedTrue, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAnyTrue.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingAnyTrue__set_state(<MovingAnyTrue> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingAnyTrue__set_state(<MovingAnyTrue> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingAnyTrue__set_state(MovingAnyTrue __pyx_result, tuple __pyx_state):
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingAnyTrue(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingAnyTrue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingAnyTrue__set_state(<MovingAnyTrue> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAnyTrue__set_state(MovingAnyTrue __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAnyTrue__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingAnyTrue__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAnyTrue__set_state(MovingAnyTrue __pyx_result, tuple __pyx_state):
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_countedTrue = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingAnyTrue__set_state(MovingAnyTrue __pyx_result, tuple __pyx_state):
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 6) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingAnyTrue__set_state(MovingAnyTrue __pyx_result, tuple __pyx_state):
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingAnyTrue__set_state(<MovingAnyTrue> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAnyTrue__set_state(MovingAnyTrue __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingAnyTrue__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingSum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_27__pyx_unpickle_MovingSum(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_26__pyx_unpickle_MovingSum[] = "__pyx_unpickle_MovingSum(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_27__pyx_unpickle_MovingSum = {"__pyx_unpickle_MovingSum", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_27__pyx_unpickle_MovingSum, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_26__pyx_unpickle_MovingSum};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_27__pyx_unpickle_MovingSum(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingSum (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingSum", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingSum", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingSum") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingSum", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingSum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_26__pyx_unpickle_MovingSum(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_26__pyx_unpickle_MovingSum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingSum", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7a4f87e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7a4f87e) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7a4f87e:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingSum.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7a4f87e:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingSum.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7a, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7a4f87e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingSum.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingSum__set_state(<MovingSum> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingSum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingSum__set_state(<MovingSum> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingSum.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingSum__set_state(<MovingSum> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSum__set_state(MovingSum __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSum__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingSum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingSum__set_state(<MovingSum> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingSum__set_state(<MovingSum> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingSum__set_state(MovingSum __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingSum(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingSum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingSum__set_state(<MovingSum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSum__set_state(MovingSum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSum__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  double __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingSum__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSum__set_state(MovingSum __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSum = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingSum__set_state(MovingSum __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 6) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_2 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingSum__set_state(MovingSum __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingSum__set_state(<MovingSum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSum__set_state(MovingSum __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingSum__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_29__pyx_unpickle_MovingAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_28__pyx_unpickle_MovingAverage[] = "__pyx_unpickle_MovingAverage(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_29__pyx_unpickle_MovingAverage = {"__pyx_unpickle_MovingAverage", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_29__pyx_unpickle_MovingAverage, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_28__pyx_unpickle_MovingAverage};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_29__pyx_unpickle_MovingAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingAverage (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingAverage", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingAverage", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingAverage") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingAverage", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_28__pyx_unpickle_MovingAverage(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_28__pyx_unpickle_MovingAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingAverage", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7a4f87e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7a4f87e) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7a4f87e:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAverage.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7a4f87e:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7a, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7a4f87e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAverage.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingAverage__set_state(<MovingAverage> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingAverage__set_state(<MovingAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingAverage__set_state(<MovingAverage> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAverage__set_state(MovingAverage __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAverage__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a4f87e = (_dependency, _deque, _isFull, _runningSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingAverage__set_state(<MovingAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingAverage__set_state(<MovingAverage> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingAverage__set_state(MovingAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingAverage__set_state(<MovingAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAverage__set_state(MovingAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  double __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingAverage__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAverage__set_state(MovingAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSum = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingAverage__set_state(MovingAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 6) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_2 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingAverage__set_state(MovingAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingAverage__set_state(<MovingAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAverage__set_state(MovingAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingAverage__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingDecay(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31__pyx_unpickle_MovingDecay(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_30__pyx_unpickle_MovingDecay[] = "__pyx_unpickle_MovingDecay(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31__pyx_unpickle_MovingDecay = {"__pyx_unpickle_MovingDecay", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31__pyx_unpickle_MovingDecay, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_30__pyx_unpickle_MovingDecay};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31__pyx_unpickle_MovingDecay(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingDecay (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingDecay", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingDecay", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingDecay") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingDecay", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingDecay", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_30__pyx_unpickle_MovingDecay(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_30__pyx_unpickle_MovingDecay(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingDecay", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb35140c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb35140c = (_dependency, _deque, _isFull, _newestValue, _runningSum, _runningWeightedSum, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xb35140c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb35140c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb35140c = (_dependency, _deque, _isFull, _newestValue, _runningSum, _runningWeightedSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingDecay.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xb35140c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb35140c = (_dependency, _deque, _isFull, _newestValue, _runningSum, _runningWeightedSum, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingDecay.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xb3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xb35140c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb35140c = (_dependency, _deque, _isFull, _newestValue, _runningSum, _runningWeightedSum, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb35140c = (_dependency, _deque, _isFull, _newestValue, _runningSum, _runningWeightedSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingDecay.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingDecay__set_state(<MovingDecay> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb35140c = (_dependency, _deque, _isFull, _newestValue, _runningSum, _runningWeightedSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingDecay.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingDecay__set_state(<MovingDecay> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingDecay.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingDecay__set_state(<MovingDecay> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingDecay__set_state(MovingDecay __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingDecay__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb35140c = (_dependency, _deque, _isFull, _newestValue, _runningSum, _runningWeightedSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingDecay.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingDecay__set_state(<MovingDecay> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingDecay__set_state(<MovingDecay> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingDecay__set_state(MovingDecay __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._newestValue = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningWeightedSum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingDecay(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingDecay", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingDecay__set_state(<MovingDecay> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingDecay__set_state(MovingDecay __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._newestValue = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningWeightedSum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingDecay__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  double __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingDecay__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingDecay__set_state(MovingDecay __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._newestValue = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningWeightedSum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_newestValue = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSum = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningWeightedSum = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingDecay__set_state(MovingDecay __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._newestValue = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningWeightedSum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 8) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_2 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._newestValue = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningWeightedSum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingDecay__set_state(MovingDecay __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._newestValue = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningWeightedSum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingDecay__set_state(<MovingDecay> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingDecay__set_state(MovingDecay __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._newestValue = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningWeightedSum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingDecay__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingPositiveAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_33__pyx_unpickle_MovingPositiveAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_32__pyx_unpickle_MovingPositiveAverage[] = "__pyx_unpickle_MovingPositiveAverage(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_33__pyx_unpickle_MovingPositiveAverage = {"__pyx_unpickle_MovingPositiveAverage", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_33__pyx_unpickle_MovingPositiveAverage, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_32__pyx_unpickle_MovingPositiveAverage};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_33__pyx_unpickle_MovingPositiveAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingPositiveAverage (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingPositiveAverage", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingPositiveAverage", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingPositiveAverage") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingPositiveAverage", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingPositiveAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_32__pyx_unpickle_MovingPositiveAverage(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_32__pyx_unpickle_MovingPositiveAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingPositiveAverage", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7c4e29b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c4e29b = (_dependency, _deque, _isFull, _runningPositiveCount, _runningPositiveSum, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7c4e29b) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7c4e29b:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c4e29b = (_dependency, _deque, _isFull, _runningPositiveCount, _runningPositiveSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingPositiveAverage.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7c4e29b:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c4e29b = (_dependency, _deque, _isFull, _runningPositiveCount, _runningPositiveSum, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingPositiveAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7c, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7c4e29b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c4e29b = (_dependency, _deque, _isFull, _runningPositiveCount, _runningPositiveSum, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c4e29b = (_dependency, _deque, _isFull, _runningPositiveCount, _runningPositiveSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingPositiveAverage.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingPositiveAverage__set_state(<MovingPositiveAverage> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c4e29b = (_dependency, _deque, _isFull, _runningPositiveCount, _runningPositiveSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingPositiveAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingPositiveAverage__set_state(<MovingPositiveAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingPositiveAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingPositiveAverage__set_state(<MovingPositiveAverage> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingPositiveAverage__set_state(MovingPositiveAverage __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingPositiveAverage__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c4e29b = (_dependency, _deque, _isFull, _runningPositiveCount, _runningPositiveSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingPositiveAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingPositiveAverage__set_state(<MovingPositiveAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingPositiveAverage__set_state(<MovingPositiveAverage> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingPositiveAverage__set_state(MovingPositiveAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningPositiveCount = __pyx_state[3]; __pyx_result._runningPositiveSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingPositiveAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingPositiveAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingPositiveAverage__set_state(<MovingPositiveAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingPositiveAverage__set_state(MovingPositiveAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningPositiveCount = __pyx_state[3]; __pyx_result._runningPositiveSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingPositiveAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  double __pyx_t_4;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingPositiveAverage__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingPositiveAverage__set_state(MovingPositiveAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningPositiveCount = __pyx_state[3]; __pyx_result._runningPositiveSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningPositiveCount = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningPositiveSum = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingPositiveAverage__set_state(MovingPositiveAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningPositiveCount = __pyx_state[3]; __pyx_result._runningPositiveSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_6 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_6 > 7) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningPositiveCount = __pyx_state[3]; __pyx_result._runningPositiveSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingPositiveAverage__set_state(MovingPositiveAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningPositiveCount = __pyx_state[3]; __pyx_result._runningPositiveSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingPositiveAverage__set_state(<MovingPositiveAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingPositiveAverage__set_state(MovingPositiveAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningPositiveCount = __pyx_state[3]; __pyx_result._runningPositiveSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingPositiveAverage__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingPositiveDifferenceAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_35__pyx_unpickle_MovingPositiveDifferenceAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_34__pyx_unpickle_MovingPositiveDifferenceAverage[] = "__pyx_unpickle_MovingPositiveDifferenceAverage(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_35__pyx_unpickle_MovingPositiveDifferenceAverage = {"__pyx_unpickle_MovingPositiveDifferenceAverage", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_35__pyx_unpickle_MovingPositiveDifferenceAverage, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_34__pyx_unpickle_MovingPositiveDifferenceAverage};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_35__pyx_unpickle_MovingPositiveDifferenceAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingPositiveDifferenceAverage (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingPositiveDifferenceAverage", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingPositiveDifferenceAverage", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingPositiveDifferenceAverage") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingPositiveDifferenceAverage", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingPositiveDifferenceAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_34__pyx_unpickle_MovingPositiveDifferenceAverage(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_34__pyx_unpickle_MovingPositiveDifferenceAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingPositiveDifferenceAverage", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x2e4f98f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x2e4f98f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x2e4f98f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingPositiveDifferenceAverage.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x2e4f98f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingPositiveDifferenceAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x2e, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x2e4f98f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingPositiveDifferenceAverage.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(<MovingPositiveDifferenceAverage> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingPositiveDifferenceAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(<MovingPositiveDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingPositiveDifferenceAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(<MovingPositiveDifferenceAverage> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(MovingPositiveDifferenceAverage __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingPositiveDifferenceAverage__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingPositiveDifferenceAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(<MovingPositiveDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(<MovingPositiveDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(MovingPositiveDifferenceAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingPositiveDifferenceAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingPositiveDifferenceAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(<MovingPositiveDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(MovingPositiveDifferenceAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingPositiveDifferenceAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingPositiveDifferenceAverage__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(MovingPositiveDifferenceAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_runningAverage);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_runningAverage));
  __pyx_v___pyx_result->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(MovingPositiveDifferenceAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 6) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(MovingPositiveDifferenceAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(<MovingPositiveDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingPositiveDifferenceAverage__set_state(MovingPositiveDifferenceAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingPositiveDifferenceAverage__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingNegativeDifferenceAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_37__pyx_unpickle_MovingNegativeDifferenceAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_36__pyx_unpickle_MovingNegativeDifferenceAverage[] = "__pyx_unpickle_MovingNegativeDifferenceAverage(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_37__pyx_unpickle_MovingNegativeDifferenceAverage = {"__pyx_unpickle_MovingNegativeDifferenceAverage", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_37__pyx_unpickle_MovingNegativeDifferenceAverage, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_36__pyx_unpickle_MovingNegativeDifferenceAverage};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_37__pyx_unpickle_MovingNegativeDifferenceAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingNegativeDifferenceAverage (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingNegativeDifferenceAverage", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingNegativeDifferenceAverage", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingNegativeDifferenceAverage") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingNegativeDifferenceAverage", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingNegativeDifferenceAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_36__pyx_unpickle_MovingNegativeDifferenceAverage(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_36__pyx_unpickle_MovingNegativeDifferenceAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingNegativeDifferenceAverage", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x2e4f98f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x2e4f98f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x2e4f98f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeDifferenceAverage.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x2e4f98f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingNegativeDifferenceAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x2e, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x2e4f98f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeDifferenceAverage.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(<MovingNegativeDifferenceAverage> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeDifferenceAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(<MovingNegativeDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingNegativeDifferenceAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(<MovingNegativeDifferenceAverage> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(MovingNegativeDifferenceAverage __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeDifferenceAverage__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2e4f98f = (_dependency, _deque, _isFull, _runningAverage, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeDifferenceAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(<MovingNegativeDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(<MovingNegativeDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(MovingNegativeDifferenceAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingNegativeDifferenceAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingNegativeDifferenceAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(<MovingNegativeDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(MovingNegativeDifferenceAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeDifferenceAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingNegativeDifferenceAverage__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(MovingNegativeDifferenceAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_runningAverage);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_runningAverage));
  __pyx_v___pyx_result->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(MovingNegativeDifferenceAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 6) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(MovingNegativeDifferenceAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(<MovingNegativeDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(MovingNegativeDifferenceAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingNegativeDifferenceAverage__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingRSI(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_39__pyx_unpickle_MovingRSI(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_38__pyx_unpickle_MovingRSI[] = "__pyx_unpickle_MovingRSI(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_39__pyx_unpickle_MovingRSI = {"__pyx_unpickle_MovingRSI", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_39__pyx_unpickle_MovingRSI, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_38__pyx_unpickle_MovingRSI};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_39__pyx_unpickle_MovingRSI(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingRSI (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingRSI", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingRSI", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingRSI") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingRSI", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingRSI", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_38__pyx_unpickle_MovingRSI(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_38__pyx_unpickle_MovingRSI(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingRSI", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xa4dfc89:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xa4dfc89 = (_dependency, _deque, _isFull, _negDiffAvg, _posDiffAvg, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xa4dfc89) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xa4dfc89:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xa4dfc89 = (_dependency, _deque, _isFull, _negDiffAvg, _posDiffAvg, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingRSI.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xa4dfc89:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xa4dfc89 = (_dependency, _deque, _isFull, _negDiffAvg, _posDiffAvg, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingRSI.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xa4, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xa4dfc89:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xa4dfc89 = (_dependency, _deque, _isFull, _negDiffAvg, _posDiffAvg, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xa4dfc89 = (_dependency, _deque, _isFull, _negDiffAvg, _posDiffAvg, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingRSI.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingRSI__set_state(<MovingRSI> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xa4dfc89 = (_dependency, _deque, _isFull, _negDiffAvg, _posDiffAvg, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingRSI.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingRSI__set_state(<MovingRSI> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingRSI.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingRSI__set_state(<MovingRSI> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingRSI__set_state(MovingRSI __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingRSI__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xa4dfc89 = (_dependency, _deque, _isFull, _negDiffAvg, _posDiffAvg, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingRSI.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingRSI__set_state(<MovingRSI> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingRSI__set_state(<MovingRSI> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingRSI__set_state(MovingRSI __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._negDiffAvg = __pyx_state[3]; __pyx_result._posDiffAvg = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingRSI(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingRSI", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingRSI__set_state(<MovingRSI> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingRSI__set_state(MovingRSI __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._negDiffAvg = __pyx_state[3]; __pyx_result._posDiffAvg = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingRSI__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingRSI__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingRSI__set_state(MovingRSI __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._negDiffAvg = __pyx_state[3]; __pyx_result._posDiffAvg = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_negDiffAvg);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_negDiffAvg));
  __pyx_v___pyx_result->_negDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_posDiffAvg);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_posDiffAvg));
  __pyx_v___pyx_result->_posDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingRSI__set_state(MovingRSI __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._negDiffAvg = __pyx_state[3]; __pyx_result._posDiffAvg = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 7) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._negDiffAvg = __pyx_state[3]; __pyx_result._posDiffAvg = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingRSI__set_state(MovingRSI __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._negDiffAvg = __pyx_state[3]; __pyx_result._posDiffAvg = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingRSI__set_state(<MovingRSI> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingRSI__set_state(MovingRSI __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._negDiffAvg = __pyx_state[3]; __pyx_result._posDiffAvg = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingRSI__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingNegativeAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_41__pyx_unpickle_MovingNegativeAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_40__pyx_unpickle_MovingNegativeAverage[] = "__pyx_unpickle_MovingNegativeAverage(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_41__pyx_unpickle_MovingNegativeAverage = {"__pyx_unpickle_MovingNegativeAverage", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_41__pyx_unpickle_MovingNegativeAverage, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_40__pyx_unpickle_MovingNegativeAverage};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_41__pyx_unpickle_MovingNegativeAverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingNegativeAverage (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingNegativeAverage", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingNegativeAverage", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingNegativeAverage") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingNegativeAverage", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingNegativeAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_40__pyx_unpickle_MovingNegativeAverage(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_40__pyx_unpickle_MovingNegativeAverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingNegativeAverage", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x9b495dd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9b495dd = (_dependency, _deque, _isFull, _runningNegativeCount, _runningNegativeSum, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x9b495dd) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x9b495dd:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9b495dd = (_dependency, _deque, _isFull, _runningNegativeCount, _runningNegativeSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeAverage.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x9b495dd:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9b495dd = (_dependency, _deque, _isFull, _runningNegativeCount, _runningNegativeSum, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingNegativeAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x9b, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x9b495dd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9b495dd = (_dependency, _deque, _isFull, _runningNegativeCount, _runningNegativeSum, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9b495dd = (_dependency, _deque, _isFull, _runningNegativeCount, _runningNegativeSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeAverage.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingNegativeAverage__set_state(<MovingNegativeAverage> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9b495dd = (_dependency, _deque, _isFull, _runningNegativeCount, _runningNegativeSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingNegativeAverage__set_state(<MovingNegativeAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingNegativeAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingNegativeAverage__set_state(<MovingNegativeAverage> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeAverage__set_state(MovingNegativeAverage __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeAverage__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9b495dd = (_dependency, _deque, _isFull, _runningNegativeCount, _runningNegativeSum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeAverage.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingNegativeAverage__set_state(<MovingNegativeAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingNegativeAverage__set_state(<MovingNegativeAverage> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingNegativeAverage__set_state(MovingNegativeAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningNegativeCount = __pyx_state[3]; __pyx_result._runningNegativeSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingNegativeAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingNegativeAverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingNegativeAverage__set_state(<MovingNegativeAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeAverage__set_state(MovingNegativeAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningNegativeCount = __pyx_state[3]; __pyx_result._runningNegativeSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeAverage__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  double __pyx_t_4;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingNegativeAverage__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeAverage__set_state(MovingNegativeAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningNegativeCount = __pyx_state[3]; __pyx_result._runningNegativeSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningNegativeCount = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningNegativeSum = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingNegativeAverage__set_state(MovingNegativeAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningNegativeCount = __pyx_state[3]; __pyx_result._runningNegativeSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_6 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_6 > 7) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningNegativeCount = __pyx_state[3]; __pyx_result._runningNegativeSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingNegativeAverage__set_state(MovingNegativeAverage __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningNegativeCount = __pyx_state[3]; __pyx_result._runningNegativeSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingNegativeAverage__set_state(<MovingNegativeAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeAverage__set_state(MovingNegativeAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningNegativeCount = __pyx_state[3]; __pyx_result._runningNegativeSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingNegativeAverage__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingVariance(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_43__pyx_unpickle_MovingVariance(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_42__pyx_unpickle_MovingVariance[] = "__pyx_unpickle_MovingVariance(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_43__pyx_unpickle_MovingVariance = {"__pyx_unpickle_MovingVariance", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_43__pyx_unpickle_MovingVariance, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_42__pyx_unpickle_MovingVariance};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_43__pyx_unpickle_MovingVariance(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingVariance (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingVariance", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingVariance", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingVariance") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingVariance", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingVariance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_42__pyx_unpickle_MovingVariance(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_42__pyx_unpickle_MovingVariance(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingVariance", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xfe5c7a3:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xfe5c7a3) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xfe5c7a3:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingVariance.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xfe5c7a3:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingVariance.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xfe, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xfe5c7a3:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingVariance.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingVariance__set_state(<MovingVariance> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingVariance.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingVariance__set_state(<MovingVariance> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingVariance.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingVariance__set_state(<MovingVariance> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingVariance__set_state(MovingVariance __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingVariance__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingVariance.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingVariance__set_state(<MovingVariance> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingVariance__set_state(<MovingVariance> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingVariance__set_state(MovingVariance __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingVariance(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingVariance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingVariance__set_state(<MovingVariance> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingVariance__set_state(MovingVariance __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingVariance__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  double __pyx_t_4;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingVariance__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingVariance__set_state(MovingVariance __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_isPop = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSum = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSumSquare = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingVariance__set_state(MovingVariance __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_6 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_6 > 8) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingVariance__set_state(MovingVariance __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingVariance__set_state(<MovingVariance> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingVariance__set_state(MovingVariance __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingVariance__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingStandardDeviation(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_45__pyx_unpickle_MovingStandardDeviation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_44__pyx_unpickle_MovingStandardDeviation[] = "__pyx_unpickle_MovingStandardDeviation(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_45__pyx_unpickle_MovingStandardDeviation = {"__pyx_unpickle_MovingStandardDeviation", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_45__pyx_unpickle_MovingStandardDeviation, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_44__pyx_unpickle_MovingStandardDeviation};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_45__pyx_unpickle_MovingStandardDeviation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingStandardDeviation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingStandardDeviation", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingStandardDeviation", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingStandardDeviation") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingStandardDeviation", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingStandardDeviation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_44__pyx_unpickle_MovingStandardDeviation(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_44__pyx_unpickle_MovingStandardDeviation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingStandardDeviation", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xfe5c7a3:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xfe5c7a3) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xfe5c7a3:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingStandardDeviation.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xfe5c7a3:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingStandardDeviation.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xfe, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xfe5c7a3:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingStandardDeviation.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingStandardDeviation__set_state(<MovingStandardDeviation> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingStandardDeviation.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingStandardDeviation__set_state(<MovingStandardDeviation> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingStandardDeviation.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingStandardDeviation__set_state(<MovingStandardDeviation> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingStandardDeviation__set_state(MovingStandardDeviation __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingStandardDeviation__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xfe5c7a3 = (_dependency, _deque, _isFull, _isPop, _runningSum, _runningSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingStandardDeviation.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingStandardDeviation__set_state(<MovingStandardDeviation> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingStandardDeviation__set_state(<MovingStandardDeviation> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingStandardDeviation__set_state(MovingStandardDeviation __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingStandardDeviation(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingStandardDeviation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingStandardDeviation__set_state(<MovingStandardDeviation> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingStandardDeviation__set_state(MovingStandardDeviation __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingStandardDeviation__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  double __pyx_t_4;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingStandardDeviation__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingStandardDeviation__set_state(MovingStandardDeviation __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_isPop = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSum = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSumSquare = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingStandardDeviation__set_state(MovingStandardDeviation __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_6 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_6 > 8) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingStandardDeviation__set_state(MovingStandardDeviation __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingStandardDeviation__set_state(<MovingStandardDeviation> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingStandardDeviation__set_state(MovingStandardDeviation __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingStandardDeviation__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingNegativeVariance(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_47__pyx_unpickle_MovingNegativeVariance(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_46__pyx_unpickle_MovingNegativeVariance[] = "__pyx_unpickle_MovingNegativeVariance(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_47__pyx_unpickle_MovingNegativeVariance = {"__pyx_unpickle_MovingNegativeVariance", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_47__pyx_unpickle_MovingNegativeVariance, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_46__pyx_unpickle_MovingNegativeVariance};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_47__pyx_unpickle_MovingNegativeVariance(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingNegativeVariance (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingNegativeVariance", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingNegativeVariance", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingNegativeVariance") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingNegativeVariance", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingNegativeVariance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_46__pyx_unpickle_MovingNegativeVariance(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_46__pyx_unpickle_MovingNegativeVariance(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingNegativeVariance", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7871d50:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7871d50 = (_dependency, _deque, _isFull, _isPop, _runningNegativeCount, _runningNegativeSum, _runningNegativeSumSquare, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7871d50) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7871d50:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7871d50 = (_dependency, _deque, _isFull, _isPop, _runningNegativeCount, _runningNegativeSum, _runningNegativeSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeVariance.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7871d50:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7871d50 = (_dependency, _deque, _isFull, _isPop, _runningNegativeCount, _runningNegativeSum, _runningNegativeSumSquare, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingNegativeVariance.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x78, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7871d50:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7871d50 = (_dependency, _deque, _isFull, _isPop, _runningNegativeCount, _runningNegativeSum, _runningNegativeSumSquare, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7871d50 = (_dependency, _deque, _isFull, _isPop, _runningNegativeCount, _runningNegativeSum, _runningNegativeSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeVariance.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingNegativeVariance__set_state(<MovingNegativeVariance> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7871d50 = (_dependency, _deque, _isFull, _isPop, _runningNegativeCount, _runningNegativeSum, _runningNegativeSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeVariance.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingNegativeVariance__set_state(<MovingNegativeVariance> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingNegativeVariance.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingNegativeVariance__set_state(<MovingNegativeVariance> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeVariance__set_state(MovingNegativeVariance __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeVariance__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7871d50 = (_dependency, _deque, _isFull, _isPop, _runningNegativeCount, _runningNegativeSum, _runningNegativeSumSquare, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingNegativeVariance.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingNegativeVariance__set_state(<MovingNegativeVariance> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingNegativeVariance__set_state(<MovingNegativeVariance> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingNegativeVariance__set_state(MovingNegativeVariance __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningNegativeCount = __pyx_state[4]; __pyx_result._runningNegativeSum = __pyx_state[5]; __pyx_result._runningNegativeSumSquare = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingNegativeVariance(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingNegativeVariance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingNegativeVariance__set_state(<MovingNegativeVariance> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeVariance__set_state(MovingNegativeVariance __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningNegativeCount = __pyx_state[4]; __pyx_result._runningNegativeSum = __pyx_state[5]; __pyx_result._runningNegativeSumSquare = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingNegativeVariance__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  double __pyx_t_4;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingNegativeVariance__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeVariance__set_state(MovingNegativeVariance __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningNegativeCount = __pyx_state[4]; __pyx_result._runningNegativeSum = __pyx_state[5]; __pyx_result._runningNegativeSumSquare = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_isPop = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningNegativeCount = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningNegativeSum = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningNegativeSumSquare = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingNegativeVariance__set_state(MovingNegativeVariance __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningNegativeCount = __pyx_state[4]; __pyx_result._runningNegativeSum = __pyx_state[5]; __pyx_result._runningNegativeSumSquare = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_6 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_6 > 9) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_2 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningNegativeCount = __pyx_state[4]; __pyx_result._runningNegativeSum = __pyx_state[5]; __pyx_result._runningNegativeSumSquare = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])             # <<<<<<<<<<<<<<
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingNegativeVariance__set_state(MovingNegativeVariance __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningNegativeCount = __pyx_state[4]; __pyx_result._runningNegativeSum = __pyx_state[5]; __pyx_result._runningNegativeSumSquare = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingNegativeVariance__set_state(<MovingNegativeVariance> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeVariance__set_state(MovingNegativeVariance __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningNegativeCount = __pyx_state[4]; __pyx_result._runningNegativeSum = __pyx_state[5]; __pyx_result._runningNegativeSumSquare = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingNegativeVariance__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingCountedPositive(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_49__pyx_unpickle_MovingCountedPositive(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_48__pyx_unpickle_MovingCountedPositive[] = "__pyx_unpickle_MovingCountedPositive(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_49__pyx_unpickle_MovingCountedPositive = {"__pyx_unpickle_MovingCountedPositive", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_49__pyx_unpickle_MovingCountedPositive, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_48__pyx_unpickle_MovingCountedPositive};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_49__pyx_unpickle_MovingCountedPositive(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingCountedPositive (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingCountedPositive", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingCountedPositive", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingCountedPositive") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingCountedPositive", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingCountedPositive", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_48__pyx_unpickle_MovingCountedPositive(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_48__pyx_unpickle_MovingCountedPositive(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingCountedPositive", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xef9c8d9:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xef9c8d9) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xef9c8d9:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingCountedPositive.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xef9c8d9:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingCountedPositive.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xef, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xef9c8d9:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingCountedPositive.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingCountedPositive__set_state(<MovingCountedPositive> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingCountedPositive.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingCountedPositive__set_state(<MovingCountedPositive> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingCountedPositive.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingCountedPositive__set_state(<MovingCountedPositive> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCountedPositive__set_state(MovingCountedPositive __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCountedPositive__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingCountedPositive.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingCountedPositive__set_state(<MovingCountedPositive> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingCountedPositive__set_state(<MovingCountedPositive> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingCountedPositive__set_state(MovingCountedPositive __pyx_result, tuple __pyx_state):
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingCountedPositive(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingCountedPositive", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingCountedPositive__set_state(<MovingCountedPositive> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCountedPositive__set_state(MovingCountedPositive __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCountedPositive__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingCountedPositive__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCountedPositive__set_state(MovingCountedPositive __pyx_result, tuple __pyx_state):
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_counts = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingCountedPositive__set_state(MovingCountedPositive __pyx_result, tuple __pyx_state):
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 6) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingCountedPositive__set_state(MovingCountedPositive __pyx_result, tuple __pyx_state):
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingCountedPositive__set_state(<MovingCountedPositive> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCountedPositive__set_state(MovingCountedPositive __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingCountedPositive__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingCountedNegative(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_51__pyx_unpickle_MovingCountedNegative(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_50__pyx_unpickle_MovingCountedNegative[] = "__pyx_unpickle_MovingCountedNegative(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_51__pyx_unpickle_MovingCountedNegative = {"__pyx_unpickle_MovingCountedNegative", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_51__pyx_unpickle_MovingCountedNegative, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_50__pyx_unpickle_MovingCountedNegative};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_51__pyx_unpickle_MovingCountedNegative(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingCountedNegative (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingCountedNegative", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingCountedNegative", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingCountedNegative") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingCountedNegative", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingCountedNegative", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_50__pyx_unpickle_MovingCountedNegative(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_50__pyx_unpickle_MovingCountedNegative(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingCountedNegative", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xef9c8d9:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xef9c8d9) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xef9c8d9:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingCountedNegative.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xef9c8d9:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingCountedNegative.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xef, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xef9c8d9:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingCountedNegative.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingCountedNegative__set_state(<MovingCountedNegative> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingCountedNegative.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingCountedNegative__set_state(<MovingCountedNegative> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingCountedNegative.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingCountedNegative__set_state(<MovingCountedNegative> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCountedNegative__set_state(MovingCountedNegative __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCountedNegative__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xef9c8d9 = (_counts, _dependency, _deque, _isFull, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingCountedNegative.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingCountedNegative__set_state(<MovingCountedNegative> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingCountedNegative__set_state(<MovingCountedNegative> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingCountedNegative__set_state(MovingCountedNegative __pyx_result, tuple __pyx_state):
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingCountedNegative(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingCountedNegative", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingCountedNegative__set_state(<MovingCountedNegative> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCountedNegative__set_state(MovingCountedNegative __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCountedNegative__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingCountedNegative__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCountedNegative__set_state(MovingCountedNegative __pyx_result, tuple __pyx_state):
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_counts = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingCountedNegative__set_state(MovingCountedNegative __pyx_result, tuple __pyx_state):
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 6) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingCountedNegative__set_state(MovingCountedNegative __pyx_result, tuple __pyx_state):
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingCountedNegative__set_state(<MovingCountedNegative> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCountedNegative__set_state(MovingCountedNegative __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingCountedNegative__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingCorrelation(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_53__pyx_unpickle_MovingCorrelation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_52__pyx_unpickle_MovingCorrelation[] = "__pyx_unpickle_MovingCorrelation(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_53__pyx_unpickle_MovingCorrelation = {"__pyx_unpickle_MovingCorrelation", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_53__pyx_unpickle_MovingCorrelation, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_52__pyx_unpickle_MovingCorrelation};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_53__pyx_unpickle_MovingCorrelation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingCorrelation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingCorrelation", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingCorrelation", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingCorrelation") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingCorrelation", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingCorrelation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_52__pyx_unpickle_MovingCorrelation(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_52__pyx_unpickle_MovingCorrelation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingCorrelation", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xbacbe0f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xbacbe0f = (_dependency, _deque, _deque_y, _isFull, _runningSumCrossSquare, _runningSumLeft, _runningSumRight, _runningSumSquareLeft, _runningSumSquareRight, _window, _x, _y))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xbacbe0f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xbacbe0f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xbacbe0f = (_dependency, _deque, _deque_y, _isFull, _runningSumCrossSquare, _runningSumLeft, _runningSumRight, _runningSumSquareLeft, _runningSumSquareRight, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingCorrelation.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xbacbe0f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xbacbe0f = (_dependency, _deque, _deque_y, _isFull, _runningSumCrossSquare, _runningSumLeft, _runningSumRight, _runningSumSquareLeft, _runningSumSquareRight, _window, _x, _y))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingCorrelation.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xba, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xbacbe0f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xbacbe0f = (_dependency, _deque, _deque_y, _isFull, _runningSumCrossSquare, _runningSumLeft, _runningSumRight, _runningSumSquareLeft, _runningSumSquareRight, _window, _x, _y))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xbacbe0f = (_dependency, _deque, _deque_y, _isFull, _runningSumCrossSquare, _runningSumLeft, _runningSumRight, _runningSumSquareLeft, _runningSumSquareRight, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingCorrelation.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingCorrelation__set_state(<MovingCorrelation> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xbacbe0f = (_dependency, _deque, _deque_y, _isFull, _runningSumCrossSquare, _runningSumLeft, _runningSumRight, _runningSumSquareLeft, _runningSumSquareRight, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingCorrelation.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingCorrelation__set_state(<MovingCorrelation> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingCorrelation.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingCorrelation__set_state(<MovingCorrelation> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCorrelation__set_state(MovingCorrelation __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCorrelation__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xbacbe0f = (_dependency, _deque, _deque_y, _isFull, _runningSumCrossSquare, _runningSumLeft, _runningSumRight, _runningSumSquareLeft, _runningSumSquareRight, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingCorrelation.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingCorrelation__set_state(<MovingCorrelation> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingCorrelation__set_state(<MovingCorrelation> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingCorrelation__set_state(MovingCorrelation __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._runningSumCrossSquare = __pyx_state[4]; __pyx_result._runningSumLeft = __pyx_state[5]; __pyx_result._runningSumRight = __pyx_state[6]; __pyx_result._runningSumSquareLeft = __pyx_state[7]; __pyx_result._runningSumSquareRight = __pyx_state[8]; __pyx_result._window = __pyx_state[9]; __pyx_result._x = __pyx_state[10]; __pyx_result._y = __pyx_state[11]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingCorrelation(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingCorrelation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingCorrelation__set_state(<MovingCorrelation> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCorrelation__set_state(MovingCorrelation __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._runningSumCrossSquare = __pyx_state[4]; __pyx_result._runningSumLeft = __pyx_state[5]; __pyx_result._runningSumRight = __pyx_state[6]; __pyx_result._runningSumSquareLeft = __pyx_state[7]; __pyx_result._runningSumSquareRight = __pyx_state[8]; __pyx_result._window = __pyx_state[9]; __pyx_result._x = __pyx_state[10]; __pyx_result._y = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingCorrelation__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  double __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingCorrelation__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCorrelation__set_state(MovingCorrelation __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._runningSumCrossSquare = __pyx_state[4]; __pyx_result._runningSumLeft = __pyx_state[5]; __pyx_result._runningSumRight = __pyx_state[6]; __pyx_result._runningSumSquareLeft = __pyx_state[7]; __pyx_result._runningSumSquareRight = __pyx_state[8]; __pyx_result._window = __pyx_state[9]; __pyx_result._x = __pyx_state[10]; __pyx_result._y = __pyx_state[11]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[12])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_deque_y);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_deque_y));
  __pyx_v___pyx_result->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSumCrossSquare = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSumLeft = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSumRight = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSumSquareLeft = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningSumSquareRight = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_x));
  __pyx_v___pyx_result->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_y);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_y));
  __pyx_v___pyx_result->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingCorrelation__set_state(MovingCorrelation __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._runningSumCrossSquare = __pyx_state[4]; __pyx_result._runningSumLeft = __pyx_state[5]; __pyx_result._runningSumRight = __pyx_state[6]; __pyx_result._runningSumSquareLeft = __pyx_state[7]; __pyx_result._runningSumSquareRight = __pyx_state[8]; __pyx_result._window = __pyx_state[9]; __pyx_result._x = __pyx_state[10]; __pyx_result._y = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[12])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 12) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_2 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._runningSumCrossSquare = __pyx_state[4]; __pyx_result._runningSumLeft = __pyx_state[5]; __pyx_result._runningSumRight = __pyx_state[6]; __pyx_result._runningSumSquareLeft = __pyx_state[7]; __pyx_result._runningSumSquareRight = __pyx_state[8]; __pyx_result._window = __pyx_state[9]; __pyx_result._x = __pyx_state[10]; __pyx_result._y = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[12])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingCorrelation__set_state(MovingCorrelation __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._runningSumCrossSquare = __pyx_state[4]; __pyx_result._runningSumLeft = __pyx_state[5]; __pyx_result._runningSumRight = __pyx_state[6]; __pyx_result._runningSumSquareLeft = __pyx_state[7]; __pyx_result._runningSumSquareRight = __pyx_state[8]; __pyx_result._window = __pyx_state[9]; __pyx_result._x = __pyx_state[10]; __pyx_result._y = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[12])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingCorrelation__set_state(<MovingCorrelation> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCorrelation__set_state(MovingCorrelation __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._runningSumCrossSquare = __pyx_state[4]; __pyx_result._runningSumLeft = __pyx_state[5]; __pyx_result._runningSumRight = __pyx_state[6]; __pyx_result._runningSumSquareLeft = __pyx_state[7]; __pyx_result._runningSumSquareRight = __pyx_state[8]; __pyx_result._window = __pyx_state[9]; __pyx_result._x = __pyx_state[10]; __pyx_result._y = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingCorrelation__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingProduct(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_55__pyx_unpickle_MovingProduct(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_54__pyx_unpickle_MovingProduct[] = "__pyx_unpickle_MovingProduct(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_55__pyx_unpickle_MovingProduct = {"__pyx_unpickle_MovingProduct", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_55__pyx_unpickle_MovingProduct, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_54__pyx_unpickle_MovingProduct};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_55__pyx_unpickle_MovingProduct(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingProduct (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingProduct", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingProduct", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingProduct") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingProduct", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingProduct", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_54__pyx_unpickle_MovingProduct(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_54__pyx_unpickle_MovingProduct(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingProduct", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x21341b2:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x21341b2 = (_dependency, _deque, _isFull, _runningProduct, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x21341b2) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x21341b2:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x21341b2 = (_dependency, _deque, _isFull, _runningProduct, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingProduct.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x21341b2:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x21341b2 = (_dependency, _deque, _isFull, _runningProduct, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingProduct.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x21, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x21341b2:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x21341b2 = (_dependency, _deque, _isFull, _runningProduct, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x21341b2 = (_dependency, _deque, _isFull, _runningProduct, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingProduct.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingProduct__set_state(<MovingProduct> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x21341b2 = (_dependency, _deque, _isFull, _runningProduct, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingProduct.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingProduct__set_state(<MovingProduct> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingProduct.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingProduct__set_state(<MovingProduct> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingProduct__set_state(MovingProduct __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingProduct__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x21341b2 = (_dependency, _deque, _isFull, _runningProduct, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingProduct.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingProduct__set_state(<MovingProduct> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingProduct__set_state(<MovingProduct> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingProduct__set_state(MovingProduct __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningProduct = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingProduct(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingProduct", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingProduct__set_state(<MovingProduct> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingProduct__set_state(MovingProduct __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningProduct = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingProduct__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  double __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingProduct__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingProduct__set_state(MovingProduct __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningProduct = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningProduct = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingProduct__set_state(MovingProduct __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningProduct = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 6) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_2 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningProduct = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingProduct__set_state(MovingProduct __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningProduct = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingProduct__set_state(<MovingProduct> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingProduct__set_state(MovingProduct __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningProduct = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingProduct__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MACD(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_57__pyx_unpickle_MACD(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_56__pyx_unpickle_MACD[] = "__pyx_unpickle_MACD(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_57__pyx_unpickle_MACD = {"__pyx_unpickle_MACD", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_57__pyx_unpickle_MACD, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_56__pyx_unpickle_MACD};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_57__pyx_unpickle_MACD(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MACD (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MACD", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MACD", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MACD") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MACD", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MACD", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_56__pyx_unpickle_MACD(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_56__pyx_unpickle_MACD(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MACD", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x547d786:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x547d786 = (_dependency, _isFull, _long_average, _short_average, _window))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x547d786) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x547d786:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x547d786 = (_dependency, _isFull, _long_average, _short_average, _window))" % __pyx_checksum)
 *     __pyx_result = MACD.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x547d786:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x547d786 = (_dependency, _isFull, _long_average, _short_average, _window))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MACD.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x54, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x547d786:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x547d786 = (_dependency, _isFull, _long_average, _short_average, _window))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x547d786 = (_dependency, _isFull, _long_average, _short_average, _window))" % __pyx_checksum)
 *     __pyx_result = MACD.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MACD__set_state(<MACD> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x547d786 = (_dependency, _isFull, _long_average, _short_average, _window))" % __pyx_checksum)
 *     __pyx_result = MACD.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MACD__set_state(<MACD> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MACD.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MACD__set_state(<MACD> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MACD__set_state(MACD __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MACD__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x547d786 = (_dependency, _isFull, _long_average, _short_average, _window))" % __pyx_checksum)
 *     __pyx_result = MACD.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MACD__set_state(<MACD> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MACD__set_state(<MACD> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MACD__set_state(MACD __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._isFull = __pyx_state[1]; __pyx_result._long_average = __pyx_state[2]; __pyx_result._short_average = __pyx_state[3]; __pyx_result._window = __pyx_state[4]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MACD(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MACD", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MACD__set_state(<MACD> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MACD__set_state(MACD __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._isFull = __pyx_state[1]; __pyx_result._long_average = __pyx_state[2]; __pyx_result._short_average = __pyx_state[3]; __pyx_result._window = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MACD__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MACD__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MACD__set_state(MACD __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._isFull = __pyx_state[1]; __pyx_result._long_average = __pyx_state[2]; __pyx_result._short_average = __pyx_state[3]; __pyx_result._window = __pyx_state[4]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_long_average);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_long_average));
  __pyx_v___pyx_result->_long_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_short_average);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_short_average));
  __pyx_v___pyx_result->_short_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._window = __pyx_t_3;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MACD__set_state(MACD __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._isFull = __pyx_state[1]; __pyx_result._long_average = __pyx_state[2]; __pyx_result._short_average = __pyx_state[3]; __pyx_result._window = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 5) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._isFull = __pyx_state[1]; __pyx_result._long_average = __pyx_state[2]; __pyx_result._short_average = __pyx_state[3]; __pyx_result._window = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MACD__set_state(MACD __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._isFull = __pyx_state[1]; __pyx_result._long_average = __pyx_state[2]; __pyx_result._short_average = __pyx_state[3]; __pyx_result._window = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MACD__set_state(<MACD> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MACD__set_state(MACD __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._isFull = __pyx_state[1]; __pyx_result._long_average = __pyx_state[2]; __pyx_result._short_average = __pyx_state[3]; __pyx_result._window = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MACD__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingLogReturn(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_59__pyx_unpickle_MovingLogReturn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_58__pyx_unpickle_MovingLogReturn[] = "__pyx_unpickle_MovingLogReturn(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_59__pyx_unpickle_MovingLogReturn = {"__pyx_unpickle_MovingLogReturn", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_59__pyx_unpickle_MovingLogReturn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_58__pyx_unpickle_MovingLogReturn};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_59__pyx_unpickle_MovingLogReturn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingLogReturn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingLogReturn", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingLogReturn", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingLogReturn") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingLogReturn", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingLogReturn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_58__pyx_unpickle_MovingLogReturn(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_58__pyx_unpickle_MovingLogReturn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingLogReturn", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7d09828:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d09828 = (_dependency, _deque, _isFull, _runningReturn, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7d09828) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7d09828:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d09828 = (_dependency, _deque, _isFull, _runningReturn, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingLogReturn.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7d09828:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d09828 = (_dependency, _deque, _isFull, _runningReturn, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingLogReturn.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7d, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7d09828:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d09828 = (_dependency, _deque, _isFull, _runningReturn, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d09828 = (_dependency, _deque, _isFull, _runningReturn, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingLogReturn.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingLogReturn__set_state(<MovingLogReturn> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d09828 = (_dependency, _deque, _isFull, _runningReturn, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingLogReturn.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingLogReturn__set_state(<MovingLogReturn> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingLogReturn.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingLogReturn__set_state(<MovingLogReturn> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingLogReturn__set_state(MovingLogReturn __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingLogReturn__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d09828 = (_dependency, _deque, _isFull, _runningReturn, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingLogReturn.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingLogReturn__set_state(<MovingLogReturn> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingLogReturn__set_state(<MovingLogReturn> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingLogReturn__set_state(MovingLogReturn __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningReturn = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingLogReturn(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingLogReturn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingLogReturn__set_state(<MovingLogReturn> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingLogReturn__set_state(MovingLogReturn __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningReturn = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingLogReturn__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  double __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingLogReturn__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingLogReturn__set_state(MovingLogReturn __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningReturn = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningReturn = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._x));
  __pyx_v___pyx_result->__pyx_base._x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingLogReturn__set_state(MovingLogReturn __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningReturn = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 6) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_2 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningReturn = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingLogReturn__set_state(MovingLogReturn __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningReturn = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingLogReturn__set_state(<MovingLogReturn> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingLogReturn__set_state(MovingLogReturn __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningReturn = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingLogReturn__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingSharp(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_61__pyx_unpickle_MovingSharp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_60__pyx_unpickle_MovingSharp[] = "__pyx_unpickle_MovingSharp(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_61__pyx_unpickle_MovingSharp = {"__pyx_unpickle_MovingSharp", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_61__pyx_unpickle_MovingSharp, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_60__pyx_unpickle_MovingSharp};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_61__pyx_unpickle_MovingSharp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingSharp (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingSharp", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingSharp", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingSharp") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingSharp", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingSharp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_60__pyx_unpickle_MovingSharp(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_60__pyx_unpickle_MovingSharp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingSharp", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd61dd72:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd61dd72 = (_dependency, _deque, _deque_y, _isFull, _mean, _var, _window, _x, _y))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xd61dd72) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd61dd72:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd61dd72 = (_dependency, _deque, _deque_y, _isFull, _mean, _var, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingSharp.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xd61dd72:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd61dd72 = (_dependency, _deque, _deque_y, _isFull, _mean, _var, _window, _x, _y))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingSharp.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xd6, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xd61dd72:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd61dd72 = (_dependency, _deque, _deque_y, _isFull, _mean, _var, _window, _x, _y))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd61dd72 = (_dependency, _deque, _deque_y, _isFull, _mean, _var, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingSharp.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingSharp__set_state(<MovingSharp> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd61dd72 = (_dependency, _deque, _deque_y, _isFull, _mean, _var, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingSharp.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingSharp__set_state(<MovingSharp> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingSharp.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingSharp__set_state(<MovingSharp> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSharp__set_state(MovingSharp __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSharp__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd61dd72 = (_dependency, _deque, _deque_y, _isFull, _mean, _var, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingSharp.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingSharp__set_state(<MovingSharp> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingSharp__set_state(<MovingSharp> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingSharp__set_state(MovingSharp __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._var = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingSharp(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingSharp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingSharp__set_state(<MovingSharp> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSharp__set_state(MovingSharp __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._var = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSharp__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingSharp__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSharp__set_state(MovingSharp __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._var = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_deque_y);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_deque_y));
  __pyx_v___pyx_result->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_mean);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_mean));
  __pyx_v___pyx_result->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_var);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_var));
  __pyx_v___pyx_result->_var = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_x));
  __pyx_v___pyx_result->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_y);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_y));
  __pyx_v___pyx_result->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingSharp__set_state(MovingSharp __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._var = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 9) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._var = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingSharp__set_state(MovingSharp __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._var = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingSharp__set_state(<MovingSharp> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSharp__set_state(MovingSharp __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._var = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingSharp__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingSortino(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_63__pyx_unpickle_MovingSortino(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_62__pyx_unpickle_MovingSortino[] = "__pyx_unpickle_MovingSortino(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_63__pyx_unpickle_MovingSortino = {"__pyx_unpickle_MovingSortino", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_63__pyx_unpickle_MovingSortino, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_62__pyx_unpickle_MovingSortino};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_63__pyx_unpickle_MovingSortino(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingSortino (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingSortino", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingSortino", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingSortino") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingSortino", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingSortino", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_62__pyx_unpickle_MovingSortino(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_62__pyx_unpickle_MovingSortino(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingSortino", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x40162d2:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x40162d2 = (_dependency, _deque, _deque_y, _isFull, _mean, _negativeVar, _window, _x, _y))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x40162d2) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x40162d2:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x40162d2 = (_dependency, _deque, _deque_y, _isFull, _mean, _negativeVar, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingSortino.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x40162d2:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x40162d2 = (_dependency, _deque, _deque_y, _isFull, _mean, _negativeVar, _window, _x, _y))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingSortino.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x40, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x40162d2:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x40162d2 = (_dependency, _deque, _deque_y, _isFull, _mean, _negativeVar, _window, _x, _y))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x40162d2 = (_dependency, _deque, _deque_y, _isFull, _mean, _negativeVar, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingSortino.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingSortino__set_state(<MovingSortino> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x40162d2 = (_dependency, _deque, _deque_y, _isFull, _mean, _negativeVar, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingSortino.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingSortino__set_state(<MovingSortino> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingSortino.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingSortino__set_state(<MovingSortino> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSortino__set_state(MovingSortino __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSortino__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x40162d2 = (_dependency, _deque, _deque_y, _isFull, _mean, _negativeVar, _window, _x, _y))" % __pyx_checksum)
 *     __pyx_result = MovingSortino.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingSortino__set_state(<MovingSortino> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingSortino__set_state(<MovingSortino> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingSortino__set_state(MovingSortino __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._negativeVar = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingSortino(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingSortino", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingSortino__set_state(<MovingSortino> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSortino__set_state(MovingSortino __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._negativeVar = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingSortino__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingSortino__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSortino__set_state(MovingSortino __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._negativeVar = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_deque_y);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_deque_y));
  __pyx_v___pyx_result->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_mean);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_mean));
  __pyx_v___pyx_result->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_negativeVar);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_negativeVar));
  __pyx_v___pyx_result->_negativeVar = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_x));
  __pyx_v___pyx_result->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_y);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_y));
  __pyx_v___pyx_result->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingSortino__set_state(MovingSortino __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._negativeVar = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 9) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._negativeVar = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingSortino__set_state(MovingSortino __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._negativeVar = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingSortino__set_state(<MovingSortino> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSortino__set_state(MovingSortino __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._negativeVar = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingSortino__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingDrawdown(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_65__pyx_unpickle_MovingDrawdown(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_64__pyx_unpickle_MovingDrawdown[] = "__pyx_unpickle_MovingDrawdown(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_65__pyx_unpickle_MovingDrawdown = {"__pyx_unpickle_MovingDrawdown", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_65__pyx_unpickle_MovingDrawdown, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_64__pyx_unpickle_MovingDrawdown};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_65__pyx_unpickle_MovingDrawdown(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingDrawdown (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingDrawdown", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingDrawdown", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingDrawdown") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingDrawdown", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingDrawdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_64__pyx_unpickle_MovingDrawdown(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_64__pyx_unpickle_MovingDrawdown(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingDrawdown", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x1dd363e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x1dd363e = (_currentMax, _dependency, _deque, _isFull, _maxer, _runningCum, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x1dd363e) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x1dd363e:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x1dd363e = (_currentMax, _dependency, _deque, _isFull, _maxer, _runningCum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingDrawdown.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x1dd363e:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x1dd363e = (_currentMax, _dependency, _deque, _isFull, _maxer, _runningCum, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingDrawdown.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x1d, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x1dd363e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x1dd363e = (_currentMax, _dependency, _deque, _isFull, _maxer, _runningCum, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x1dd363e = (_currentMax, _dependency, _deque, _isFull, _maxer, _runningCum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingDrawdown.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingDrawdown__set_state(<MovingDrawdown> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x1dd363e = (_currentMax, _dependency, _deque, _isFull, _maxer, _runningCum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingDrawdown.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingDrawdown__set_state(<MovingDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingDrawdown.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingDrawdown__set_state(<MovingDrawdown> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingDrawdown__set_state(MovingDrawdown __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingDrawdown__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x1dd363e = (_currentMax, _dependency, _deque, _isFull, _maxer, _runningCum, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingDrawdown.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingDrawdown__set_state(<MovingDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingDrawdown__set_state(<MovingDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingDrawdown__set_state(MovingDrawdown __pyx_result, tuple __pyx_state):
 *     __pyx_result._currentMax = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._maxer = __pyx_state[4]; __pyx_result._runningCum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingDrawdown(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingDrawdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingDrawdown__set_state(<MovingDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingDrawdown__set_state(MovingDrawdown __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._currentMax = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._maxer = __pyx_state[4]; __pyx_result._runningCum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingDrawdown__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingDrawdown__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingDrawdown__set_state(MovingDrawdown __pyx_result, tuple __pyx_state):
 *     __pyx_result._currentMax = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._maxer = __pyx_state[4]; __pyx_result._runningCum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_currentMax = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_maxer);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_maxer));
  __pyx_v___pyx_result->_maxer = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_runningCum = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_x));
  __pyx_v___pyx_result->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingDrawdown__set_state(MovingDrawdown __pyx_result, tuple __pyx_state):
 *     __pyx_result._currentMax = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._maxer = __pyx_state[4]; __pyx_result._runningCum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 8) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._currentMax = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._maxer = __pyx_state[4]; __pyx_result._runningCum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingDrawdown__set_state(MovingDrawdown __pyx_result, tuple __pyx_state):
 *     __pyx_result._currentMax = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._maxer = __pyx_state[4]; __pyx_result._runningCum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingDrawdown__set_state(<MovingDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingDrawdown__set_state(MovingDrawdown __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._currentMax = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._maxer = __pyx_state[4]; __pyx_result._runningCum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingDrawdown__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingMaxDrawdown(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_67__pyx_unpickle_MovingMaxDrawdown(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_66__pyx_unpickle_MovingMaxDrawdown[] = "__pyx_unpickle_MovingMaxDrawdown(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_67__pyx_unpickle_MovingMaxDrawdown = {"__pyx_unpickle_MovingMaxDrawdown", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_67__pyx_unpickle_MovingMaxDrawdown, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_66__pyx_unpickle_MovingMaxDrawdown};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_67__pyx_unpickle_MovingMaxDrawdown(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingMaxDrawdown (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingMaxDrawdown", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingMaxDrawdown", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingMaxDrawdown") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingMaxDrawdown", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingMaxDrawdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_66__pyx_unpickle_MovingMaxDrawdown(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_66__pyx_unpickle_MovingMaxDrawdown(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingMaxDrawdown", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7c30e67:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c30e67 = (_dependency, _deque, _drawdownCalculator, _isFull, _minimer, _window, _x))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7c30e67) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7c30e67:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c30e67 = (_dependency, _deque, _drawdownCalculator, _isFull, _minimer, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMaxDrawdown.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7c30e67:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c30e67 = (_dependency, _deque, _drawdownCalculator, _isFull, _minimer, _window, _x))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingMaxDrawdown.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7c_2, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7c30e67:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c30e67 = (_dependency, _deque, _drawdownCalculator, _isFull, _minimer, _window, _x))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c30e67 = (_dependency, _deque, _drawdownCalculator, _isFull, _minimer, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMaxDrawdown.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingMaxDrawdown__set_state(<MovingMaxDrawdown> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c30e67 = (_dependency, _deque, _drawdownCalculator, _isFull, _minimer, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMaxDrawdown.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingMaxDrawdown__set_state(<MovingMaxDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingMaxDrawdown.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingMaxDrawdown__set_state(<MovingMaxDrawdown> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMaxDrawdown__set_state(MovingMaxDrawdown __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMaxDrawdown__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7c30e67 = (_dependency, _deque, _drawdownCalculator, _isFull, _minimer, _window, _x))" % __pyx_checksum)
 *     __pyx_result = MovingMaxDrawdown.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingMaxDrawdown__set_state(<MovingMaxDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingMaxDrawdown__set_state(<MovingMaxDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingMaxDrawdown__set_state(MovingMaxDrawdown __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._drawdownCalculator = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._minimer = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingMaxDrawdown(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingMaxDrawdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingMaxDrawdown__set_state(<MovingMaxDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMaxDrawdown__set_state(MovingMaxDrawdown __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._drawdownCalculator = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._minimer = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingMaxDrawdown__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingMaxDrawdown__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMaxDrawdown__set_state(MovingMaxDrawdown __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._drawdownCalculator = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._minimer = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_drawdownCalculator);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_drawdownCalculator));
  __pyx_v___pyx_result->_drawdownCalculator = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._isFull = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_minimer);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_minimer));
  __pyx_v___pyx_result->_minimer = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_x));
  __pyx_v___pyx_result->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingMaxDrawdown__set_state(MovingMaxDrawdown __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._drawdownCalculator = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._minimer = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 7) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._drawdownCalculator = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._minimer = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingMaxDrawdown__set_state(MovingMaxDrawdown __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._drawdownCalculator = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._minimer = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingMaxDrawdown__set_state(<MovingMaxDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingMaxDrawdown__set_state(MovingMaxDrawdown __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._drawdownCalculator = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._minimer = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingMaxDrawdown__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_MovingResidue(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_69__pyx_unpickle_MovingResidue(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_68__pyx_unpickle_MovingResidue[] = "__pyx_unpickle_MovingResidue(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_69__pyx_unpickle_MovingResidue = {"__pyx_unpickle_MovingResidue", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_69__pyx_unpickle_MovingResidue, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_68__pyx_unpickle_MovingResidue};
static PyObject *__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_69__pyx_unpickle_MovingResidue(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingResidue (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingResidue", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingResidue", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_MovingResidue") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_MovingResidue", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingResidue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_68__pyx_unpickle_MovingResidue(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_68__pyx_unpickle_MovingResidue(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingResidue", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xadfc35c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xadfc35c = (_cross, _dependency, _deque, _deque_y, _isFull, _lastx, _lasty, _window, _x, _xsquare, _y))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xadfc35c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xadfc35c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xadfc35c = (_cross, _dependency, _deque, _deque_y, _isFull, _lastx, _lasty, _window, _x, _xsquare, _y))" % __pyx_checksum)
 *     __pyx_result = MovingResidue.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xadfc35c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xadfc35c = (_cross, _dependency, _deque, _deque_y, _isFull, _lastx, _lasty, _window, _x, _xsquare, _y))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = MovingResidue.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xad, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xadfc35c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xadfc35c = (_cross, _dependency, _deque, _deque_y, _isFull, _lastx, _lasty, _window, _x, _xsquare, _y))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xadfc35c = (_cross, _dependency, _deque, _deque_y, _isFull, _lastx, _lasty, _window, _x, _xsquare, _y))" % __pyx_checksum)
 *     __pyx_result = MovingResidue.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingResidue__set_state(<MovingResidue> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xadfc35c = (_cross, _dependency, _deque, _deque_y, _isFull, _lastx, _lasty, _window, _x, _xsquare, _y))" % __pyx_checksum)
 *     __pyx_result = MovingResidue.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingResidue__set_state(<MovingResidue> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = MovingResidue.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingResidue__set_state(<MovingResidue> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingResidue__set_state(MovingResidue __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingResidue__set_state(((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xadfc35c = (_cross, _dependency, _deque, _deque_y, _isFull, _lastx, _lasty, _window, _x, _xsquare, _y))" % __pyx_checksum)
 *     __pyx_result = MovingResidue.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_MovingResidue__set_state(<MovingResidue> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_MovingResidue__set_state(<MovingResidue> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_MovingResidue__set_state(MovingResidue __pyx_result, tuple __pyx_state):
 *     __pyx_result._cross = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._deque_y = __pyx_state[3]; __pyx_result._isFull = __pyx_state[4]; __pyx_result._lastx = __pyx_state[5]; __pyx_result._lasty = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]; __pyx_result._xsquare = __pyx_state[9]; __pyx_result._y = __pyx_state[10]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingResidue(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingResidue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_MovingResidue__set_state(<MovingResidue> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingResidue__set_state(MovingResidue __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cross = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._deque_y = __pyx_state[3]; __pyx_result._isFull = __pyx_state[4]; __pyx_result._lastx = __pyx_state[5]; __pyx_result._lasty = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]; __pyx_result._xsquare = __pyx_state[9]; __pyx_result._y = __pyx_state[10]
 *     if len(__pyx_state) > 11 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators___pyx_unpickle_MovingResidue__set_state(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_MovingResidue__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingResidue__set_state(MovingResidue __pyx_result, tuple __pyx_state):
 *     __pyx_result._cross = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._deque_y = __pyx_state[3]; __pyx_result._isFull = __pyx_state[4]; __pyx_result._lastx = __pyx_state[5]; __pyx_result._lasty = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]; __pyx_result._xsquare = __pyx_state[9]; __pyx_result._y = __pyx_state[10]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 11 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[11])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_cross = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._deque);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._deque));
  __pyx_v___pyx_result->__pyx_base._deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_deque_y);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_deque_y));
  __pyx_v___pyx_result->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._isFull = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_lastx = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_lasty = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_x);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_x));
  __pyx_v___pyx_result->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_xsquare = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_y);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_y));
  __pyx_v___pyx_result->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingResidue__set_state(MovingResidue __pyx_result, tuple __pyx_state):
 *     __pyx_result._cross = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._deque_y = __pyx_state[3]; __pyx_result._isFull = __pyx_state[4]; __pyx_result._lastx = __pyx_state[5]; __pyx_result._lasty = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]; __pyx_result._xsquare = __pyx_state[9]; __pyx_result._y = __pyx_state[10]
 *     if len(__pyx_state) > 11 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[11])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 11) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._cross = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._deque_y = __pyx_state[3]; __pyx_result._isFull = __pyx_state[4]; __pyx_result._lastx = __pyx_state[5]; __pyx_result._lasty = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]; __pyx_result._xsquare = __pyx_state[9]; __pyx_result._y = __pyx_state[10]
 *     if len(__pyx_state) > 11 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[11])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_MovingResidue__set_state(MovingResidue __pyx_result, tuple __pyx_state):
 *     __pyx_result._cross = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._deque_y = __pyx_state[3]; __pyx_result._isFull = __pyx_state[4]; __pyx_result._lastx = __pyx_state[5]; __pyx_result._lasty = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]; __pyx_result._xsquare = __pyx_state[9]; __pyx_result._y = __pyx_state[10]
 *     if len(__pyx_state) > 11 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[11])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingResidue__set_state(<MovingResidue> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingResidue__set_state(MovingResidue __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cross = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._deque_y = __pyx_state[3]; __pyx_result._isFull = __pyx_state[4]; __pyx_result._lastx = __pyx_state[5]; __pyx_result._lasty = __pyx_state[6]; __pyx_result._window = __pyx_state[7]; __pyx_result._x = __pyx_state[8]; __pyx_result._xsquare = __pyx_state[9]; __pyx_result._y = __pyx_state[10]
 *     if len(__pyx_state) > 11 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Math.Accumulators.StatefulAccumulators.__pyx_unpickle_MovingResidue__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":735
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":738
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 738, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":741
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":744
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":747
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":751
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":753
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":869
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":870
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":873
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":875
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":876
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":882
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 882, __pyx_L3_error)

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":883
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(3, 883, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 884, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(3, 884, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":888
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 888, __pyx_L3_error)

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":889
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(3, 889, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 890, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(3, 890, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":894
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 894, __pyx_L3_error)

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":895
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(3, 895, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":896
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 896, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(3, 896, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *p;
  PyObject *o = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  p->_deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_deque);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator)) __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)o;
  e = ((likely(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator)) ? ((__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_traverse) ? __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder)); if (e) return e;
  if (p->_deque) {
    e = (*v)(((PyObject *)p->_deque), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *)o;
  if (likely(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator)) { if (__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_clear) __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder);
  tmp = ((PyObject*)p->_deque);
  p->_deque = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder__deque(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder__deque(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6_deque_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder[] = {
  {"size", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_3size, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_2size},
  {"isFull", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_5isFull, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_4isFull},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_8__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder[] = {
  {(char *)"_deque", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder__deque, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder__deque, (char *)"_deque: ultron.sentry.Math.Accumulators.impl.Deque", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.StatefulValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "StatefulValueHolder(window)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift;
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_x);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o, v, a); if (e) return e;
  if (p->_x) {
    e = (*v)(((PyObject *)p->_x), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
  tmp = ((PyObject*)p->_x);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift__popout(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift__popout(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7_popout_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift__x(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift__x(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2_x_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_4result},
  {"lag", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_7lag, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_6lag},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_12__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift[] = {
  {(char *)"_popout", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift__popout, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift__popout, (char *)"_popout: 'double'", 0},
  {(char *)"_x", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift__x, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift__x, (char *)"_x: ultron.sentry.Math.Accumulators.IAccumulators.Accumulator", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.Shift", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Shift(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta;
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_x);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o, v, a); if (e) return e;
  if (p->_x) {
    e = (*v)(((PyObject *)p->_x), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
  tmp = ((PyObject*)p->_x);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta__popout(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta__popout(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7_popout_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta__x(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta__x(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2_x_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_4result},
  {"lag", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_7lag, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_6lag},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_12__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta[] = {
  {(char *)"_popout", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta__popout, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta__popout, (char *)"_popout: 'double'", 0},
  {(char *)"_x", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta__x, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta__x, (char *)"_x: ultron.sentry.Math.Accumulators.IAccumulators.Accumulator", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.Delta", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Delta(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_x);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o, v, a); if (e) return e;
  if (p->_x) {
    e = (*v)(((PyObject *)p->_x), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
  tmp = ((PyObject*)p->_x);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder__x(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder__x(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2_x_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_3__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_5__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_4__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder[] = {
  {(char *)"_x", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder__x, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder__x, (char *)"_x: ultron.sentry.Math.Accumulators.IAccumulators.Accumulator", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.SingleValuedValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "SingleValuedValueHolder(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23SingleValuedValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  p->_sortedArray = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_sortedArray);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o, v, a); if (e) return e;
  if (p->_sortedArray) {
    e = (*v)(p->_sortedArray, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o);
  tmp = ((PyObject*)p->_sortedArray);
  p->_sortedArray = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_2push},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_5__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_7__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_6__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.SortedValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "SortedValueHolder(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax[] = {
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_3result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_2result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_8__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMax", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingMax(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax[] = {
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_3result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_2result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_8__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMax", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingArgMax(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin[] = {
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_3result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_2result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_8__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMin", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingMin(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin[] = {
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_3result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_2result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_8__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingArgMin", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingArgMin(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank[] = {
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_3result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_2result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_8__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRank", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingRank(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile[] = {
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_3result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_2result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_8__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingQuantile", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingQuantile(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue__countedTrue(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue__countedTrue(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_12_countedTrue_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue[] = {
  {(char *)"_countedTrue", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue__countedTrue, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue__countedTrue, (char *)"_countedTrue: 'size_t'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAllTrue", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingAllTrue(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue__countedTrue(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue__countedTrue(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_12_countedTrue_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue[] = {
  {(char *)"_countedTrue", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue__countedTrue, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue__countedTrue, (char *)"_countedTrue: 'size_t'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAnyTrue", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingAnyTrue(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum__runningSum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum__runningSum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11_runningSum_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum[] = {
  {(char *)"_runningSum", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum__runningSum, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum__runningSum, (char *)"_runningSum: 'double'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSum", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingSum(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage__runningSum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage__runningSum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11_runningSum_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage[] = {
  {(char *)"_runningSum", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage__runningSum, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage__runningSum, (char *)"_runningSum: 'double'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingAverage(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay__runningSum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay__runningSum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11_runningSum_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay[] = {
  {(char *)"_runningSum", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay__runningSum, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay__runningSum, (char *)"_runningSum: 'double'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDecay", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingDecay(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage__runningPositiveSum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage__runningPositiveSum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_19_runningPositiveSum_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage__runningPositiveCount(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage__runningPositiveCount(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_21_runningPositiveCount_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage[] = {
  {(char *)"_runningPositiveSum", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage__runningPositiveSum, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage__runningPositiveSum, (char *)"_runningPositiveSum: 'double'", 0},
  {(char *)"_runningPositiveCount", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage__runningPositiveCount, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage__runningPositiveCount, (char *)"_runningPositiveCount: 'int'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveAverage", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingPositiveAverage(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage;
  p->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_runningAverage);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o, v, a); if (e) return e;
  if (p->_runningAverage) {
    e = (*v)(((PyObject *)p->_runningAverage), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o);
  tmp = ((PyObject*)p->_runningAverage);
  p->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage__runningAverage(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage__runningAverage(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_15_runningAverage_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage[] = {
  {(char *)"_runningAverage", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage__runningAverage, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage__runningAverage, (char *)"_runningAverage: ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingPositiveDifferenceAverage(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage;
  p->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_runningAverage);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o, v, a); if (e) return e;
  if (p->_runningAverage) {
    e = (*v)(((PyObject *)p->_runningAverage), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o);
  tmp = ((PyObject*)p->_runningAverage);
  p->_runningAverage = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage__runningAverage(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage__runningAverage(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_15_runningAverage_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage[] = {
  {(char *)"_runningAverage", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage__runningAverage, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage__runningAverage, (char *)"_runningAverage: ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingNegativeDifferenceAverage(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI;
  p->_posDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)Py_None); Py_INCREF(Py_None);
  p->_negDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_posDiffAvg);
  Py_CLEAR(p->_negDiffAvg);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o, v, a); if (e) return e;
  if (p->_posDiffAvg) {
    e = (*v)(((PyObject *)p->_posDiffAvg), a); if (e) return e;
  }
  if (p->_negDiffAvg) {
    e = (*v)(((PyObject *)p->_negDiffAvg), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(o);
  tmp = ((PyObject*)p->_posDiffAvg);
  p->_posDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_negDiffAvg);
  p->_negDiffAvg = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI__posDiffAvg(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI__posDiffAvg(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_posDiffAvg_5__del__(o);
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI__negDiffAvg(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI__negDiffAvg(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11_negDiffAvg_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI[] = {
  {(char *)"_posDiffAvg", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI__posDiffAvg, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI__posDiffAvg, (char *)"_posDiffAvg: ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingPositiveDifferenceAverage", 0},
  {(char *)"_negDiffAvg", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI__negDiffAvg, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI__negDiffAvg, (char *)"_negDiffAvg: ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeDifferenceAverage", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingRSI", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingRSI(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage__runningNegativeSum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage__runningNegativeSum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_19_runningNegativeSum_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage__runningNegativeCount(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage__runningNegativeCount(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_21_runningNegativeCount_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage[] = {
  {(char *)"_runningNegativeSum", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage__runningNegativeSum, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage__runningNegativeSum, (char *)"_runningNegativeSum: 'double'", 0},
  {(char *)"_runningNegativeCount", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage__runningNegativeCount, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage__runningNegativeCount, (char *)"_runningNegativeCount: 'int'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeAverage", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingNegativeAverage(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__runningSum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__runningSum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11_runningSum_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__runningSumSquare(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__runningSumSquare(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_17_runningSumSquare_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__isPop(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__isPop(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_6_isPop_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance[] = {
  {(char *)"_runningSum", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__runningSum, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__runningSum, (char *)"_runningSum: 'double'", 0},
  {(char *)"_runningSumSquare", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__runningSumSquare, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__runningSumSquare, (char *)"_runningSumSquare: 'double'", 0},
  {(char *)"_isPop", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__isPop, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance__isPop, (char *)"_isPop: 'int'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingVariance(window, x, isPopulation=False)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__runningSum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__runningSum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11_runningSum_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__runningSumSquare(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__runningSumSquare(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_17_runningSumSquare_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__isPop(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__isPop(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_6_isPop_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation[] = {
  {(char *)"_runningSum", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__runningSum, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__runningSum, (char *)"_runningSum: 'double'", 0},
  {(char *)"_runningSumSquare", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__runningSumSquare, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__runningSumSquare, (char *)"_runningSumSquare: 'double'", 0},
  {(char *)"_isPop", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__isPop, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation__isPop, (char *)"_isPop: 'int'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingStandardDeviation", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingStandardDeviation(window, x, isPopulation=False)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeSum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeSum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_19_runningNegativeSum_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeSumSquare(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeSumSquare(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_25_runningNegativeSumSquare_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeCount(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeCount(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_21_runningNegativeCount_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__isPop(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__isPop(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_6_isPop_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance[] = {
  {(char *)"_runningNegativeSum", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeSum, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeSum, (char *)"_runningNegativeSum: 'double'", 0},
  {(char *)"_runningNegativeSumSquare", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeSumSquare, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeSumSquare, (char *)"_runningNegativeSumSquare: 'double'", 0},
  {(char *)"_runningNegativeCount", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeCount, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__runningNegativeCount, (char *)"_runningNegativeCount: 'int'", 0},
  {(char *)"_isPop", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__isPop, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance__isPop, (char *)"_isPop: 'int'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingNegativeVariance(window, x, isPopulation=0)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive__counts(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive__counts(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7_counts_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive[] = {
  {(char *)"_counts", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive__counts, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive__counts, (char *)"_counts: 'int'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedPositive", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingCountedPositive(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative__counts(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative__counts(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7_counts_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative[] = {
  {(char *)"_counts", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative__counts, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative__counts, (char *)"_counts: 'int'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCountedNegative", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingCountedNegative(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation;
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  p->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  p->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_x);
  Py_CLEAR(p->_y);
  Py_CLEAR(p->_deque_y);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o, v, a); if (e) return e;
  if (p->_x) {
    e = (*v)(((PyObject *)p->_x), a); if (e) return e;
  }
  if (p->_y) {
    e = (*v)(((PyObject *)p->_y), a); if (e) return e;
  }
  if (p->_deque_y) {
    e = (*v)(((PyObject *)p->_deque_y), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
  tmp = ((PyObject*)p->_x);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_y);
  p->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_deque_y);
  p->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumLeft(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumLeft(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_15_runningSumLeft_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumRight(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumRight(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_16_runningSumRight_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumSquareLeft(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumSquareLeft(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_21_runningSumSquareLeft_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumSquareRight(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumSquareRight(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumSquareRight_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumCrossSquare(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumCrossSquare(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_22_runningSumCrossSquare_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation[] = {
  {(char *)"_runningSumLeft", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumLeft, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumLeft, (char *)"_runningSumLeft: 'double'", 0},
  {(char *)"_runningSumRight", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumRight, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumRight, (char *)"_runningSumRight: 'double'", 0},
  {(char *)"_runningSumSquareLeft", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumSquareLeft, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumSquareLeft, (char *)"_runningSumSquareLeft: 'double'", 0},
  {(char *)"_runningSumSquareRight", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumSquareRight, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumSquareRight, (char *)"_runningSumSquareRight: 'double'", 0},
  {(char *)"_runningSumCrossSquare", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumCrossSquare, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation__runningSumCrossSquare, (char *)"_runningSumCrossSquare: 'double'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingCorrelation", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingCorrelation(window, x, y)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct__runningProduct(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct__runningProduct(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_15_runningProduct_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_9__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_11__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct[] = {
  {(char *)"_runningProduct", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct__runningProduct, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct__runningProduct, (char *)"_runningProduct: 'double'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingProduct", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_7__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingProduct(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *p;
  PyObject *o = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD;
  p->_short_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  p->_long_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_short_average);
  Py_CLEAR(p->_long_average);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator)) __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)o;
  e = ((likely(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator)) ? ((__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_traverse) ? __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD)); if (e) return e;
  if (p->_short_average) {
    e = (*v)(((PyObject *)p->_short_average), a); if (e) return e;
  }
  if (p->_long_average) {
    e = (*v)(((PyObject *)p->_long_average), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD *)o;
  if (likely(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator)) { if (__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_clear) __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD);
  tmp = ((PyObject*)p->_short_average);
  p->_short_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_long_average);
  p->_long_average = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD__short_average(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD__short_average(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_14_short_average_5__del__(o);
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD__long_average(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD__long_average(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_13_long_average_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_8__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD[] = {
  {(char *)"_short_average", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD__short_average, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD__short_average, (char *)"_short_average: ultron.sentry.Math.Accumulators.IAccumulators.Accumulator", 0},
  {(char *)"_long_average", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD__long_average, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD__long_average, (char *)"_long_average: ultron.sentry.Math.Accumulators.IAccumulators.Accumulator", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MACD", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MACD(short_win, long_win, x, method=XAverage)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn *)o);
  p->__pyx_base.__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn;
  return o;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn__runningReturn(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn__runningReturn(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_14_runningReturn_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_8__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn[] = {
  {(char *)"_runningReturn", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn__runningReturn, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn__runningReturn, (char *)"_runningReturn: 'double'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingLogReturn", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingLogReturn(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp;
  p->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None); Py_INCREF(Py_None);
  p->_var = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)Py_None); Py_INCREF(Py_None);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  p->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  p->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_mean);
  Py_CLEAR(p->_var);
  Py_CLEAR(p->_x);
  Py_CLEAR(p->_y);
  Py_CLEAR(p->_deque_y);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o, v, a); if (e) return e;
  if (p->_mean) {
    e = (*v)(((PyObject *)p->_mean), a); if (e) return e;
  }
  if (p->_var) {
    e = (*v)(((PyObject *)p->_var), a); if (e) return e;
  }
  if (p->_x) {
    e = (*v)(((PyObject *)p->_x), a); if (e) return e;
  }
  if (p->_y) {
    e = (*v)(((PyObject *)p->_y), a); if (e) return e;
  }
  if (p->_deque_y) {
    e = (*v)(((PyObject *)p->_deque_y), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
  tmp = ((PyObject*)p->_mean);
  p->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_var);
  p->_var = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_x);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_y);
  p->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_deque_y);
  p->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp__mean(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp__mean(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5_mean_5__del__(o);
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp__var(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp__var(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4_var_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_8__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp[] = {
  {(char *)"_mean", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp__mean, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp__mean, (char *)"_mean: ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage", 0},
  {(char *)"_var", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp__var, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp__var, (char *)"_var: ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingVariance", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSharp", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingSharp(window, x, y)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino;
  p->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None); Py_INCREF(Py_None);
  p->_negativeVar = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)Py_None); Py_INCREF(Py_None);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  p->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  p->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_mean);
  Py_CLEAR(p->_negativeVar);
  Py_CLEAR(p->_x);
  Py_CLEAR(p->_y);
  Py_CLEAR(p->_deque_y);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o, v, a); if (e) return e;
  if (p->_mean) {
    e = (*v)(((PyObject *)p->_mean), a); if (e) return e;
  }
  if (p->_negativeVar) {
    e = (*v)(((PyObject *)p->_negativeVar), a); if (e) return e;
  }
  if (p->_x) {
    e = (*v)(((PyObject *)p->_x), a); if (e) return e;
  }
  if (p->_y) {
    e = (*v)(((PyObject *)p->_y), a); if (e) return e;
  }
  if (p->_deque_y) {
    e = (*v)(((PyObject *)p->_deque_y), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
  tmp = ((PyObject*)p->_mean);
  p->_mean = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_negativeVar);
  p->_negativeVar = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_x);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_y);
  p->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_deque_y);
  p->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino__mean(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino__mean(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5_mean_5__del__(o);
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino__negativeVar(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino__negativeVar(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_12_negativeVar_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_8__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino[] = {
  {(char *)"_mean", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino__mean, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino__mean, (char *)"_mean: ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingAverage", 0},
  {(char *)"_negativeVar", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino__negativeVar, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino__negativeVar, (char *)"_negativeVar: ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingNegativeVariance", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingSortino", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingSortino(window, x, y)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown;
  p->_maxer = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)Py_None); Py_INCREF(Py_None);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_maxer);
  Py_CLEAR(p->_x);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o, v, a); if (e) return e;
  if (p->_maxer) {
    e = (*v)(((PyObject *)p->_maxer), a); if (e) return e;
  }
  if (p->_x) {
    e = (*v)(((PyObject *)p->_x), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
  tmp = ((PyObject*)p->_maxer);
  p->_maxer = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_x);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_8__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingDrawdown", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingDrawdown(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown;
  p->_drawdownCalculator = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)Py_None); Py_INCREF(Py_None);
  p->_minimer = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)Py_None); Py_INCREF(Py_None);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_drawdownCalculator);
  Py_CLEAR(p->_minimer);
  Py_CLEAR(p->_x);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o, v, a); if (e) return e;
  if (p->_drawdownCalculator) {
    e = (*v)(((PyObject *)p->_drawdownCalculator), a); if (e) return e;
  }
  if (p->_minimer) {
    e = (*v)(((PyObject *)p->_minimer), a); if (e) return e;
  }
  if (p->_x) {
    e = (*v)(((PyObject *)p->_x), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
  tmp = ((PyObject*)p->_drawdownCalculator);
  p->_drawdownCalculator = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_minimer);
  p->_minimer = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_x);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_2push},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_5result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_4result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_7__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_9__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_8__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingMaxDrawdown", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingMaxDrawdown(window, x)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue;

static PyObject *__pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue;
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  p->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  p->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_x);
  Py_CLEAR(p->_y);
  Py_CLEAR(p->_deque_y);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
}

static int __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)o;
  e = __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o, v, a); if (e) return e;
  if (p->_x) {
    e = (*v)(((PyObject *)p->_x), a); if (e) return e;
  }
  if (p->_y) {
    e = (*v)(((PyObject *)p->_y), a); if (e) return e;
  }
  if (p->_deque_y) {
    e = (*v)(((PyObject *)p->_deque_y), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *p = (struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue *)o;
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder(o);
  tmp = ((PyObject*)p->_x);
  p->_x = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_y);
  p->_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_deque_y);
  p->_deque_y = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__cross(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__cross(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_cross_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__xsquare(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__xsquare(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_8_xsquare_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__lastx(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__lastx(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lastx_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__lasty(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__lasty(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6_lasty_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_3push, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_2push},
  {"isFull", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_5isFull, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_4isFull},
  {"result", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_7result, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_6result},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_12__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue[] = {
  {(char *)"_cross", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__cross, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__cross, (char *)"_cross: 'double'", 0},
  {(char *)"_xsquare", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__xsquare, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__xsquare, (char *)"_xsquare: 'double'", 0},
  {(char *)"_lastx", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__lastx, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__lastx, (char *)"_lastx: 'double'", 0},
  {(char *)"_lasty", __pyx_getprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__lasty, __pyx_setprop_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue__lasty, (char *)"_lasty: 'double'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Math.Accumulators.StatefulAccumulators.MovingResidue", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "MovingResidue(window, x, y)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_StatefulAccumulators(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_StatefulAccumulators},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "StatefulAccumulators",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_Delta, __pyx_k_Delta, sizeof(__pyx_k_Delta), 0, 0, 1, 1},
  {&__pyx_kp_s_Delta_window_value_should_not_be, __pyx_k_Delta_window_value_should_not_be, sizeof(__pyx_k_Delta_window_value_should_not_be), 0, 0, 1, 0},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x17, __pyx_k_Incompatible_checksums_s_vs_0x17, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x17), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x1d, __pyx_k_Incompatible_checksums_s_vs_0x1d, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x1d), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x21, __pyx_k_Incompatible_checksums_s_vs_0x21, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x21), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x2e, __pyx_k_Incompatible_checksums_s_vs_0x2e, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x2e), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x34, __pyx_k_Incompatible_checksums_s_vs_0x34, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x34), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x40, __pyx_k_Incompatible_checksums_s_vs_0x40, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x40), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x54, __pyx_k_Incompatible_checksums_s_vs_0x54, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x54), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x62, __pyx_k_Incompatible_checksums_s_vs_0x62, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x62), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x78, __pyx_k_Incompatible_checksums_s_vs_0x78, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x78), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x7a, __pyx_k_Incompatible_checksums_s_vs_0x7a, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x7a), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x7c, __pyx_k_Incompatible_checksums_s_vs_0x7c, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x7c), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x7c_2, __pyx_k_Incompatible_checksums_s_vs_0x7c_2, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x7c_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x7d, __pyx_k_Incompatible_checksums_s_vs_0x7d, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x7d), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x7e, __pyx_k_Incompatible_checksums_s_vs_0x7e, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x7e), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x93, __pyx_k_Incompatible_checksums_s_vs_0x93, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x93), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x9b, __pyx_k_Incompatible_checksums_s_vs_0x9b, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x9b), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xa4, __pyx_k_Incompatible_checksums_s_vs_0xa4, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xa4), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xad, __pyx_k_Incompatible_checksums_s_vs_0xad, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xad), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xb3, __pyx_k_Incompatible_checksums_s_vs_0xb3, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xb3), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xb5, __pyx_k_Incompatible_checksums_s_vs_0xb5, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xb5), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xba, __pyx_k_Incompatible_checksums_s_vs_0xba, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xba), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xd6, __pyx_k_Incompatible_checksums_s_vs_0xd6, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xd6), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xef, __pyx_k_Incompatible_checksums_s_vs_0xef, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xef), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xfe, __pyx_k_Incompatible_checksums_s_vs_0xfe, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xfe), 0, 0, 1, 0},
  {&__pyx_n_s_MACD, __pyx_k_MACD, sizeof(__pyx_k_MACD), 0, 0, 1, 1},
  {&__pyx_n_s_MovingAllTrue, __pyx_k_MovingAllTrue, sizeof(__pyx_k_MovingAllTrue), 0, 0, 1, 1},
  {&__pyx_n_s_MovingAnyTrue, __pyx_k_MovingAnyTrue, sizeof(__pyx_k_MovingAnyTrue), 0, 0, 1, 1},
  {&__pyx_n_s_MovingArgMax, __pyx_k_MovingArgMax, sizeof(__pyx_k_MovingArgMax), 0, 0, 1, 1},
  {&__pyx_n_s_MovingArgMin, __pyx_k_MovingArgMin, sizeof(__pyx_k_MovingArgMin), 0, 0, 1, 1},
  {&__pyx_n_s_MovingAverage, __pyx_k_MovingAverage, sizeof(__pyx_k_MovingAverage), 0, 0, 1, 1},
  {&__pyx_n_s_MovingCorrelation, __pyx_k_MovingCorrelation, sizeof(__pyx_k_MovingCorrelation), 0, 0, 1, 1},
  {&__pyx_n_s_MovingCountedNegative, __pyx_k_MovingCountedNegative, sizeof(__pyx_k_MovingCountedNegative), 0, 0, 1, 1},
  {&__pyx_n_s_MovingCountedPositive, __pyx_k_MovingCountedPositive, sizeof(__pyx_k_MovingCountedPositive), 0, 0, 1, 1},
  {&__pyx_n_s_MovingDecay, __pyx_k_MovingDecay, sizeof(__pyx_k_MovingDecay), 0, 0, 1, 1},
  {&__pyx_n_s_MovingDrawdown, __pyx_k_MovingDrawdown, sizeof(__pyx_k_MovingDrawdown), 0, 0, 1, 1},
  {&__pyx_n_s_MovingLogReturn, __pyx_k_MovingLogReturn, sizeof(__pyx_k_MovingLogReturn), 0, 0, 1, 1},
  {&__pyx_n_s_MovingMax, __pyx_k_MovingMax, sizeof(__pyx_k_MovingMax), 0, 0, 1, 1},
  {&__pyx_n_s_MovingMaxDrawdown, __pyx_k_MovingMaxDrawdown, sizeof(__pyx_k_MovingMaxDrawdown), 0, 0, 1, 1},
  {&__pyx_n_s_MovingMin, __pyx_k_MovingMin, sizeof(__pyx_k_MovingMin), 0, 0, 1, 1},
  {&__pyx_n_s_MovingNegativeAverage, __pyx_k_MovingNegativeAverage, sizeof(__pyx_k_MovingNegativeAverage), 0, 0, 1, 1},
  {&__pyx_n_s_MovingNegativeDifferenceAverage, __pyx_k_MovingNegativeDifferenceAverage, sizeof(__pyx_k_MovingNegativeDifferenceAverage), 0, 0, 1, 1},
  {&__pyx_n_s_MovingNegativeVariance, __pyx_k_MovingNegativeVariance, sizeof(__pyx_k_MovingNegativeVariance), 0, 0, 1, 1},
  {&__pyx_n_s_MovingPositiveAverage, __pyx_k_MovingPositiveAverage, sizeof(__pyx_k_MovingPositiveAverage), 0, 0, 1, 1},
  {&__pyx_n_s_MovingPositiveDifferenceAverage, __pyx_k_MovingPositiveDifferenceAverage, sizeof(__pyx_k_MovingPositiveDifferenceAverage), 0, 0, 1, 1},
  {&__pyx_n_s_MovingProduct, __pyx_k_MovingProduct, sizeof(__pyx_k_MovingProduct), 0, 0, 1, 1},
  {&__pyx_n_s_MovingQuantile, __pyx_k_MovingQuantile, sizeof(__pyx_k_MovingQuantile), 0, 0, 1, 1},
  {&__pyx_n_s_MovingRSI, __pyx_k_MovingRSI, sizeof(__pyx_k_MovingRSI), 0, 0, 1, 1},
  {&__pyx_n_s_MovingRank, __pyx_k_MovingRank, sizeof(__pyx_k_MovingRank), 0, 0, 1, 1},
  {&__pyx_n_s_MovingResidue, __pyx_k_MovingResidue, sizeof(__pyx_k_MovingResidue), 0, 0, 1, 1},
  {&__pyx_n_s_MovingSharp, __pyx_k_MovingSharp, sizeof(__pyx_k_MovingSharp), 0, 0, 1, 1},
  {&__pyx_n_s_MovingSortino, __pyx_k_MovingSortino, sizeof(__pyx_k_MovingSortino), 0, 0, 1, 1},
  {&__pyx_n_s_MovingStandardDeviation, __pyx_k_MovingStandardDeviation, sizeof(__pyx_k_MovingStandardDeviation), 0, 0, 1, 1},
  {&__pyx_n_s_MovingSum, __pyx_k_MovingSum, sizeof(__pyx_k_MovingSum), 0, 0, 1, 1},
  {&__pyx_n_s_MovingVariance, __pyx_k_MovingVariance, sizeof(__pyx_k_MovingVariance), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_Shift, __pyx_k_Shift, sizeof(__pyx_k_Shift), 0, 0, 1, 1},
  {&__pyx_n_s_SingleValuedValueHolder, __pyx_k_SingleValuedValueHolder, sizeof(__pyx_k_SingleValuedValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_SortedValueHolder, __pyx_k_SortedValueHolder, sizeof(__pyx_k_SortedValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_StatefulValueHolder, __pyx_k_StatefulValueHolder, sizeof(__pyx_k_StatefulValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_bisect, __pyx_k_bisect, sizeof(__pyx_k_bisect), 0, 0, 1, 1},
  {&__pyx_n_s_bisect_left, __pyx_k_bisect_left, sizeof(__pyx_k_bisect_left), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_deepcopy, __pyx_k_deepcopy, sizeof(__pyx_k_deepcopy), 0, 0, 1, 1},
  {&__pyx_n_s_dependency, __pyx_k_dependency, sizeof(__pyx_k_dependency), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_isFull, __pyx_k_isFull, sizeof(__pyx_k_isFull), 0, 0, 1, 1},
  {&__pyx_n_s_isPopulation, __pyx_k_isPopulation, sizeof(__pyx_k_isPopulation), 0, 0, 1, 1},
  {&__pyx_n_s_lag, __pyx_k_lag, sizeof(__pyx_k_lag), 0, 0, 1, 1},
  {&__pyx_n_s_long_win, __pyx_k_long_win, sizeof(__pyx_k_long_win), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_kp_s_mathrm_Delta_0_1, __pyx_k_mathrm_Delta_0_1, sizeof(__pyx_k_mathrm_Delta_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MADecay_0_1, __pyx_k_mathrm_MADecay_0_1, sizeof(__pyx_k_mathrm_MADecay_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MA_0_1, __pyx_k_mathrm_MA_0_1, sizeof(__pyx_k_mathrm_MA_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MAllTrue_0_1, __pyx_k_mathrm_MAllTrue_0_1, sizeof(__pyx_k_mathrm_MAllTrue_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MAnyTrue_0_1, __pyx_k_mathrm_MAnyTrue_0_1, sizeof(__pyx_k_mathrm_MAnyTrue_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MArgMax_0_1, __pyx_k_mathrm_MArgMax_0_1, sizeof(__pyx_k_mathrm_MArgMax_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MArgMin_0_1, __pyx_k_mathrm_MArgMin_0_1, sizeof(__pyx_k_mathrm_MArgMin_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MMax_0_1, __pyx_k_mathrm_MMax_0_1, sizeof(__pyx_k_mathrm_MMax_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MMin_0_1, __pyx_k_mathrm_MMin_0_1, sizeof(__pyx_k_mathrm_MMin_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MQuantile_0_1, __pyx_k_mathrm_MQuantile_0_1, sizeof(__pyx_k_mathrm_MQuantile_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MRank_0_1, __pyx_k_mathrm_MRank_0_1, sizeof(__pyx_k_mathrm_MRank_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_MSum_0_1, __pyx_k_mathrm_MSum_0_1, sizeof(__pyx_k_mathrm_MSum_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_Res_0_1_2, __pyx_k_mathrm_Res_0_1_2, sizeof(__pyx_k_mathrm_Res_0_1_2), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_Shift_0_1, __pyx_k_mathrm_Shift_0_1, sizeof(__pyx_k_mathrm_Shift_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mcorr_0_1_2, __pyx_k_mathrm_mcorr_0_1_2, sizeof(__pyx_k_mathrm_mcorr_0_1_2), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mnegavg_0_1, __pyx_k_mathrm_mnegavg_0_1, sizeof(__pyx_k_mathrm_mnegavg_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mnegcount_0_1, __pyx_k_mathrm_mnegcount_0_1, sizeof(__pyx_k_mathrm_mnegcount_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mnegdiffavg_0_1, __pyx_k_mathrm_mnegdiffavg_0_1, sizeof(__pyx_k_mathrm_mnegdiffavg_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mnegvar_0_1_2, __pyx_k_mathrm_mnegvar_0_1_2, sizeof(__pyx_k_mathrm_mnegvar_0_1_2), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mposavg_0_1, __pyx_k_mathrm_mposavg_0_1, sizeof(__pyx_k_mathrm_mposavg_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mposcount_0_1, __pyx_k_mathrm_mposcount_0_1, sizeof(__pyx_k_mathrm_mposcount_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mposdiffavg_0_1, __pyx_k_mathrm_mposdiffavg_0_1, sizeof(__pyx_k_mathrm_mposdiffavg_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mprod_0_1, __pyx_k_mathrm_mprod_0_1, sizeof(__pyx_k_mathrm_mprod_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mrsi_0_1, __pyx_k_mathrm_mrsi_0_1, sizeof(__pyx_k_mathrm_mrsi_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mstd_0_1_2, __pyx_k_mathrm_mstd_0_1_2, sizeof(__pyx_k_mathrm_mstd_0_1_2), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_mvar_0_1_2, __pyx_k_mathrm_mvar_0_1_2, sizeof(__pyx_k_mathrm_mvar_0_1_2), 0, 0, 1, 0},
  {&__pyx_n_s_method, __pyx_k_method, sizeof(__pyx_k_method), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_s_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 0, 1, 0},
  {&__pyx_kp_s_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 0, 1, 0},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_push, __pyx_k_push, sizeof(__pyx_k_push), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Delta, __pyx_k_pyx_unpickle_Delta, sizeof(__pyx_k_pyx_unpickle_Delta), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MACD, __pyx_k_pyx_unpickle_MACD, sizeof(__pyx_k_pyx_unpickle_MACD), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingAllTrue, __pyx_k_pyx_unpickle_MovingAllTrue, sizeof(__pyx_k_pyx_unpickle_MovingAllTrue), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingAnyTrue, __pyx_k_pyx_unpickle_MovingAnyTrue, sizeof(__pyx_k_pyx_unpickle_MovingAnyTrue), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingArgMax, __pyx_k_pyx_unpickle_MovingArgMax, sizeof(__pyx_k_pyx_unpickle_MovingArgMax), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingArgMin, __pyx_k_pyx_unpickle_MovingArgMin, sizeof(__pyx_k_pyx_unpickle_MovingArgMin), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingAverage, __pyx_k_pyx_unpickle_MovingAverage, sizeof(__pyx_k_pyx_unpickle_MovingAverage), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingCorrelation, __pyx_k_pyx_unpickle_MovingCorrelation, sizeof(__pyx_k_pyx_unpickle_MovingCorrelation), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingCountedNega, __pyx_k_pyx_unpickle_MovingCountedNega, sizeof(__pyx_k_pyx_unpickle_MovingCountedNega), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingCountedPosi, __pyx_k_pyx_unpickle_MovingCountedPosi, sizeof(__pyx_k_pyx_unpickle_MovingCountedPosi), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingDecay, __pyx_k_pyx_unpickle_MovingDecay, sizeof(__pyx_k_pyx_unpickle_MovingDecay), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingDrawdown, __pyx_k_pyx_unpickle_MovingDrawdown, sizeof(__pyx_k_pyx_unpickle_MovingDrawdown), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingLogReturn, __pyx_k_pyx_unpickle_MovingLogReturn, sizeof(__pyx_k_pyx_unpickle_MovingLogReturn), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingMax, __pyx_k_pyx_unpickle_MovingMax, sizeof(__pyx_k_pyx_unpickle_MovingMax), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingMaxDrawdown, __pyx_k_pyx_unpickle_MovingMaxDrawdown, sizeof(__pyx_k_pyx_unpickle_MovingMaxDrawdown), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingMin, __pyx_k_pyx_unpickle_MovingMin, sizeof(__pyx_k_pyx_unpickle_MovingMin), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingNegativeAve, __pyx_k_pyx_unpickle_MovingNegativeAve, sizeof(__pyx_k_pyx_unpickle_MovingNegativeAve), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingNegativeDif, __pyx_k_pyx_unpickle_MovingNegativeDif, sizeof(__pyx_k_pyx_unpickle_MovingNegativeDif), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingNegativeVar, __pyx_k_pyx_unpickle_MovingNegativeVar, sizeof(__pyx_k_pyx_unpickle_MovingNegativeVar), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingPositiveAve, __pyx_k_pyx_unpickle_MovingPositiveAve, sizeof(__pyx_k_pyx_unpickle_MovingPositiveAve), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingPositiveDif, __pyx_k_pyx_unpickle_MovingPositiveDif, sizeof(__pyx_k_pyx_unpickle_MovingPositiveDif), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingProduct, __pyx_k_pyx_unpickle_MovingProduct, sizeof(__pyx_k_pyx_unpickle_MovingProduct), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingQuantile, __pyx_k_pyx_unpickle_MovingQuantile, sizeof(__pyx_k_pyx_unpickle_MovingQuantile), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingRSI, __pyx_k_pyx_unpickle_MovingRSI, sizeof(__pyx_k_pyx_unpickle_MovingRSI), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingRank, __pyx_k_pyx_unpickle_MovingRank, sizeof(__pyx_k_pyx_unpickle_MovingRank), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingResidue, __pyx_k_pyx_unpickle_MovingResidue, sizeof(__pyx_k_pyx_unpickle_MovingResidue), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingSharp, __pyx_k_pyx_unpickle_MovingSharp, sizeof(__pyx_k_pyx_unpickle_MovingSharp), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingSortino, __pyx_k_pyx_unpickle_MovingSortino, sizeof(__pyx_k_pyx_unpickle_MovingSortino), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingStandardDev, __pyx_k_pyx_unpickle_MovingStandardDev, sizeof(__pyx_k_pyx_unpickle_MovingStandardDev), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingSum, __pyx_k_pyx_unpickle_MovingSum, sizeof(__pyx_k_pyx_unpickle_MovingSum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_MovingVariance, __pyx_k_pyx_unpickle_MovingVariance, sizeof(__pyx_k_pyx_unpickle_MovingVariance), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Shift, __pyx_k_pyx_unpickle_Shift, sizeof(__pyx_k_pyx_unpickle_Shift), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_SingleValuedValue, __pyx_k_pyx_unpickle_SingleValuedValue, sizeof(__pyx_k_pyx_unpickle_SingleValuedValue), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_SortedValueHolder, __pyx_k_pyx_unpickle_SortedValueHolder, sizeof(__pyx_k_pyx_unpickle_SortedValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_StatefulValueHold, __pyx_k_pyx_unpickle_StatefulValueHold, sizeof(__pyx_k_pyx_unpickle_StatefulValueHold), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_result, __pyx_k_result, sizeof(__pyx_k_result), 0, 0, 1, 1},
  {&__pyx_kp_s_sampling_standard_deviation_can, __pyx_k_sampling_standard_deviation_can, sizeof(__pyx_k_sampling_standard_deviation_can), 0, 0, 1, 0},
  {&__pyx_kp_s_sampling_variance_can_t_be_calcu, __pyx_k_sampling_variance_can_t_be_calcu, sizeof(__pyx_k_sampling_variance_can_t_be_calcu), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_kp_s_shift_value_should_not_be_less_t, __pyx_k_shift_value_should_not_be_less_t, sizeof(__pyx_k_shift_value_should_not_be_less_t), 0, 0, 1, 0},
  {&__pyx_n_s_short_win, __pyx_k_short_win, sizeof(__pyx_k_short_win), 0, 0, 1, 1},
  {&__pyx_n_s_six, __pyx_k_six, sizeof(__pyx_k_six), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_ultron_sentry_Math_Accumulators, __pyx_k_ultron_sentry_Math_Accumulators, sizeof(__pyx_k_ultron_sentry_Math_Accumulators), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_window, __pyx_k_window, sizeof(__pyx_k_window), 0, 0, 1, 1},
  {&__pyx_kp_s_window_length_should_be_greater, __pyx_k_window_length_should_be_greater, sizeof(__pyx_k_window_length_should_be_greater), 0, 0, 1, 0},
  {&__pyx_kp_s_window_parameter_should_be_a_pos, __pyx_k_window_parameter_should_be_a_pos, sizeof(__pyx_k_window_parameter_should_be_a_pos), 0, 0, 1, 0},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 33, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(3, 884, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(3, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(3, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "(tree fragment)":1
 * def __pyx_unpickle_StatefulValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__4 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_StatefulValueHold, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__6 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Shift, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__8 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Delta, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__10 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __pyx_codeobj__11 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_SingleValuedValue, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__11)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__12 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);
  __pyx_codeobj__13 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__12, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_SortedValueHolder, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__13)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__14 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);
  __pyx_codeobj__15 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__14, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingMax, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__15)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__16 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);
  __pyx_codeobj__17 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__16, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingArgMax, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__17)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__18 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);
  __pyx_codeobj__19 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__18, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingMin, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__19)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__20 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);
  __pyx_codeobj__21 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__20, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingArgMin, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__21)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__22 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);
  __pyx_codeobj__23 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__22, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingRank, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__23)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__24 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_codeobj__25 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__24, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingQuantile, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__25)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__26 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);
  __pyx_codeobj__27 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__26, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingAllTrue, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__27)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__28 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_codeobj__29 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingAnyTrue, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__29)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__30 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);
  __pyx_codeobj__31 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingSum, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__31)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__32 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__32, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingAverage, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__34 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingDecay, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__36 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__36, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingPositiveAve, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__38 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__38, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingPositiveDif, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__40 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);
  __pyx_codeobj__41 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__40, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingNegativeDif, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__41)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__42 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);
  __pyx_codeobj__43 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__42, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingRSI, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__43)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__44 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);
  __pyx_codeobj__45 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__44, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingNegativeAve, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__45)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__46 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);
  __pyx_codeobj__47 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__46, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingVariance, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__47)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__48 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);
  __pyx_codeobj__49 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__48, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingStandardDev, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__49)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__50 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);
  __pyx_codeobj__51 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__50, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingNegativeVar, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__51)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__52 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);
  __pyx_codeobj__53 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__52, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingCountedPosi, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__53)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__54 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);
  __pyx_codeobj__55 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__54, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingCountedNega, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__55)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__56 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);
  __pyx_codeobj__57 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__56, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingCorrelation, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__57)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__58 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);
  __pyx_codeobj__59 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__58, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingProduct, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__59)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__60 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);
  __pyx_codeobj__61 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__60, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MACD, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__61)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__62 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);
  __pyx_codeobj__63 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__62, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingLogReturn, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__63)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__64 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);
  __pyx_codeobj__65 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__64, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingSharp, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__65)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__66 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__66)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);
  __pyx_codeobj__67 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__66, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingSortino, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__67)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__68 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);
  __pyx_codeobj__69 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__68, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingDrawdown, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__69)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__70 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);
  __pyx_codeobj__71 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__70, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingMaxDrawdown, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__71)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__72 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__72)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);
  __pyx_codeobj__73 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__72, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_MovingResidue, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__73)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_0 = PyFloat_FromDouble(0.0); if (unlikely(!__pyx_float_0_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_24234567 = PyInt_FromLong(24234567L); if (unlikely(!__pyx_int_24234567)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_31274558 = PyInt_FromLong(31274558L); if (unlikely(!__pyx_int_31274558)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_34816434 = PyInt_FromLong(34816434L); if (unlikely(!__pyx_int_34816434)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_48560527 = PyInt_FromLong(48560527L); if (unlikely(!__pyx_int_48560527)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_55178044 = PyInt_FromLong(55178044L); if (unlikely(!__pyx_int_55178044)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_67199698 = PyInt_FromLong(67199698L); if (unlikely(!__pyx_int_67199698)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_88594310 = PyInt_FromLong(88594310L); if (unlikely(!__pyx_int_88594310)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_102913224 = PyInt_FromLong(102913224L); if (unlikely(!__pyx_int_102913224)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_126295376 = PyInt_FromLong(126295376L); if (unlikely(!__pyx_int_126295376)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_128252030 = PyInt_FromLong(128252030L); if (unlikely(!__pyx_int_128252030)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_130223719 = PyInt_FromLong(130223719L); if (unlikely(!__pyx_int_130223719)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_130343579 = PyInt_FromLong(130343579L); if (unlikely(!__pyx_int_130343579)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_131110952 = PyInt_FromLong(131110952L); if (unlikely(!__pyx_int_131110952)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_132800211 = PyInt_FromLong(132800211L); if (unlikely(!__pyx_int_132800211)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_155135628 = PyInt_FromLong(155135628L); if (unlikely(!__pyx_int_155135628)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_162829789 = PyInt_FromLong(162829789L); if (unlikely(!__pyx_int_162829789)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_172883081 = PyInt_FromLong(172883081L); if (unlikely(!__pyx_int_172883081)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_182436700 = PyInt_FromLong(182436700L); if (unlikely(!__pyx_int_182436700)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_188027916 = PyInt_FromLong(188027916L); if (unlikely(!__pyx_int_188027916)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_190414258 = PyInt_FromLong(190414258L); if (unlikely(!__pyx_int_190414258)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_195870223 = PyInt_FromLong(195870223L); if (unlikely(!__pyx_int_195870223)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_224517490 = PyInt_FromLong(224517490L); if (unlikely(!__pyx_int_224517490)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_251250905 = PyInt_FromLong(251250905L); if (unlikely(!__pyx_int_251250905)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_266717091 = PyInt_FromLong(266717091L); if (unlikely(!__pyx_int_266717091)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.Accumulators.IAccumulators"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Accumulator", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder.__pyx_base.isFull = (int (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_isFull;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder.size = (size_t (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19StatefulValueHolder_size;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_StatefulValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_result;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.lag = (int (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Shift_lag;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Shift, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Shift;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_result;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.lag = (int (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5Delta_lag;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta) < 0) __PYX_ERR(0, 71, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta) < 0) __PYX_ERR(0, 71, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Delta, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta) < 0) __PYX_ERR(0, 71, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta) < 0) __PYX_ERR(0, 71, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_Delta;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder) < 0) __PYX_ERR(0, 101, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder) < 0) __PYX_ERR(0, 101, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_SingleValuedValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder) < 0) __PYX_ERR(0, 101, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder) < 0) __PYX_ERR(0, 101, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17SortedValueHolder_push;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_SortedValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax.__pyx_base.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMax_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax) < 0) __PYX_ERR(0, 137, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax) < 0) __PYX_ERR(0, 137, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingMax, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax) < 0) __PYX_ERR(0, 137, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax) < 0) __PYX_ERR(0, 137, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMax;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax.__pyx_base.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMax_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingArgMax, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMax;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin.__pyx_base.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingMin_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin) < 0) __PYX_ERR(0, 170, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin) < 0) __PYX_ERR(0, 170, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingMin, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin) < 0) __PYX_ERR(0, 170, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin) < 0) __PYX_ERR(0, 170, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMin;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin.__pyx_base.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_12MovingArgMin_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin) < 0) __PYX_ERR(0, 184, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin) < 0) __PYX_ERR(0, 184, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingArgMin, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin) < 0) __PYX_ERR(0, 184, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin) < 0) __PYX_ERR(0, 184, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingArgMin;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank.__pyx_base.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_10MovingRank_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank) < 0) __PYX_ERR(0, 203, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank) < 0) __PYX_ERR(0, 203, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingRank, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank) < 0) __PYX_ERR(0, 203, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank) < 0) __PYX_ERR(0, 203, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRank;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile.__pyx_base.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingQuantile_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SortedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile) < 0) __PYX_ERR(0, 214, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile) < 0) __PYX_ERR(0, 214, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingQuantile, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile) < 0) __PYX_ERR(0, 214, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile) < 0) __PYX_ERR(0, 214, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingQuantile;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAllTrue_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue) < 0) __PYX_ERR(0, 230, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue) < 0) __PYX_ERR(0, 230, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingAllTrue, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue) < 0) __PYX_ERR(0, 230, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue) < 0) __PYX_ERR(0, 230, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAllTrue;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAnyTrue_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingAnyTrue, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAnyTrue;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingSum_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum) < 0) __PYX_ERR(0, 294, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum) < 0) __PYX_ERR(0, 294, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingSum, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum) < 0) __PYX_ERR(0, 294, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum) < 0) __PYX_ERR(0, 294, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSum;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingAverage_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage) < 0) __PYX_ERR(0, 318, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage) < 0) __PYX_ERR(0, 318, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingAverage, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage) < 0) __PYX_ERR(0, 318, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage) < 0) __PYX_ERR(0, 318, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingAverage;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingDecay_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay) < 0) __PYX_ERR(0, 348, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay) < 0) __PYX_ERR(0, 348, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingDecay, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay) < 0) __PYX_ERR(0, 348, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay) < 0) __PYX_ERR(0, 348, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDecay;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingPositiveAverage_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage) < 0) __PYX_ERR(0, 386, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage) < 0) __PYX_ERR(0, 386, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingPositiveAverage, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage) < 0) __PYX_ERR(0, 386, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage) < 0) __PYX_ERR(0, 386, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveAverage;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingPositiveDifferenceAverage_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage) < 0) __PYX_ERR(0, 422, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage) < 0) __PYX_ERR(0, 422, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingPositiveDifferenceAverage, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage) < 0) __PYX_ERR(0, 422, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage) < 0) __PYX_ERR(0, 422, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingPositiveDifferenceAverage;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31MovingNegativeDifferenceAverage_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage) < 0) __PYX_ERR(0, 440, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage) < 0) __PYX_ERR(0, 440, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingNegativeDifferenceAverage, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage) < 0) __PYX_ERR(0, 440, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage) < 0) __PYX_ERR(0, 440, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeDifferenceAverage;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9MovingRSI_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI) < 0) __PYX_ERR(0, 458, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI) < 0) __PYX_ERR(0, 458, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingRSI, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI) < 0) __PYX_ERR(0, 458, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI) < 0) __PYX_ERR(0, 458, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingRSI;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingNegativeAverage_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage) < 0) __PYX_ERR(0, 483, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage) < 0) __PYX_ERR(0, 483, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingNegativeAverage, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage) < 0) __PYX_ERR(0, 483, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage) < 0) __PYX_ERR(0, 483, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeAverage;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingVariance_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance) < 0) __PYX_ERR(0, 519, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance) < 0) __PYX_ERR(0, 519, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingVariance, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance) < 0) __PYX_ERR(0, 519, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance) < 0) __PYX_ERR(0, 519, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingVariance;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23MovingStandardDeviation_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation) < 0) __PYX_ERR(0, 563, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation) < 0) __PYX_ERR(0, 563, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingStandardDeviation, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation) < 0) __PYX_ERR(0, 563, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation) < 0) __PYX_ERR(0, 563, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingStandardDeviation;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_22MovingNegativeVariance_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance) < 0) __PYX_ERR(0, 608, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance) < 0) __PYX_ERR(0, 608, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingNegativeVariance, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance) < 0) __PYX_ERR(0, 608, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance) < 0) __PYX_ERR(0, 608, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingNegativeVariance;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedPositive_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive) < 0) __PYX_ERR(0, 663, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive) < 0) __PYX_ERR(0, 663, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingCountedPositive, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive) < 0) __PYX_ERR(0, 663, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive) < 0) __PYX_ERR(0, 663, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedPositive;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21MovingCountedNegative_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative) < 0) __PYX_ERR(0, 691, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative) < 0) __PYX_ERR(0, 691, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingCountedNegative, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative) < 0) __PYX_ERR(0, 691, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative) < 0) __PYX_ERR(0, 691, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCountedNegative;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingCorrelation_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation) < 0) __PYX_ERR(0, 720, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation) < 0) __PYX_ERR(0, 720, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingCorrelation, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation) < 0) __PYX_ERR(0, 720, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation) < 0) __PYX_ERR(0, 720, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingCorrelation;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingProduct_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct) < 0) __PYX_ERR(0, 779, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct) < 0) __PYX_ERR(0, 779, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingProduct, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct) < 0) __PYX_ERR(0, 779, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct) < 0) __PYX_ERR(0, 779, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingProduct;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_4MACD_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD) < 0) __PYX_ERR(0, 805, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD) < 0) __PYX_ERR(0, 805, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MACD, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD) < 0) __PYX_ERR(0, 805, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD) < 0) __PYX_ERR(0, 805, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MACD;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn.__pyx_base.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn.__pyx_base.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15MovingLogReturn_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_SingleValuedValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn) < 0) __PYX_ERR(0, 828, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn) < 0) __PYX_ERR(0, 828, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingLogReturn, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn) < 0) __PYX_ERR(0, 828, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn) < 0) __PYX_ERR(0, 828, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingLogReturn;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11MovingSharp_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp) < 0) __PYX_ERR(0, 850, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp) < 0) __PYX_ERR(0, 850, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingSharp, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp) < 0) __PYX_ERR(0, 850, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp) < 0) __PYX_ERR(0, 850, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSharp;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingSortino_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino) < 0) __PYX_ERR(0, 892, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino) < 0) __PYX_ERR(0, 892, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingSortino, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino) < 0) __PYX_ERR(0, 892, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino) < 0) __PYX_ERR(0, 892, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingSortino;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_14MovingDrawdown_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown) < 0) __PYX_ERR(0, 933, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown) < 0) __PYX_ERR(0, 933, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingDrawdown, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown) < 0) __PYX_ERR(0, 933, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown) < 0) __PYX_ERR(0, 933, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingDrawdown;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17MovingMaxDrawdown_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown) < 0) __PYX_ERR(0, 959, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown) < 0) __PYX_ERR(0, 959, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingMaxDrawdown, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown) < 0) __PYX_ERR(0, 959, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown) < 0) __PYX_ERR(0, 959, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingMaxDrawdown;
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue = &__pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.__pyx_base.__pyx_base.isFull = (int (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_isFull;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.__pyx_base.__pyx_base.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_push;
  __pyx_vtable_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.__pyx_base.__pyx_base.result = (double (*)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13MovingResidue_result;
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.tp_base = __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_StatefulValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue) < 0) __PYX_ERR(0, 984, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.tp_dictoffset && __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue.tp_dict, __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue) < 0) __PYX_ERR(0, 984, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MovingResidue, (PyObject *)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue) < 0) __PYX_ERR(0, 984, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue) < 0) __PYX_ERR(0, 984, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue = &__pyx_type_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_MovingResidue;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(3, 199, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(3, 222, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(3, 226, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(3, 238, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(3, 764, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.Accumulators.impl"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.impl", "Deque", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_4impl_Deque), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque) __PYX_ERR(5, 4, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_4impl_Deque = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_4impl_Deque*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_4impl_Deque->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_4impl_Deque)) __PYX_ERR(5, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.Distributions.NormalDistribution"); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Distributions.NormalDistribution", "InverseCumulativeNormal", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal) __PYX_ERR(6, 3, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal)) __PYX_ERR(6, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.Accumulators.IAccumulators"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "IAccumulator", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator) __PYX_ERR(7, 6, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Negative", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative) __PYX_ERR(7, 25, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative)) __PYX_ERR(7, 25, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "CombinedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder) __PYX_ERR(7, 33, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder)) __PYX_ERR(7, 33, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "AddedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder) __PYX_ERR(7, 41, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder)) __PYX_ERR(7, 41, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "MinusedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder) __PYX_ERR(7, 46, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder)) __PYX_ERR(7, 46, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "MultipliedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder) __PYX_ERR(7, 51, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder)) __PYX_ERR(7, 51, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "DividedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder) __PYX_ERR(7, 56, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder)) __PYX_ERR(7, 56, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "LtOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder) __PYX_ERR(7, 61, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder)) __PYX_ERR(7, 61, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "LeOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder) __PYX_ERR(7, 66, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder)) __PYX_ERR(7, 66, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "GtOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder) __PYX_ERR(7, 71, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder)) __PYX_ERR(7, 71, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "GeOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder) __PYX_ERR(7, 76, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder)) __PYX_ERR(7, 76, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "EqOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder) __PYX_ERR(7, 81, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder)) __PYX_ERR(7, 81, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "NeOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder) __PYX_ERR(7, 86, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder)) __PYX_ERR(7, 86, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Identity", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity) __PYX_ERR(7, 91, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity)) __PYX_ERR(7, 91, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Current", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current) __PYX_ERR(7, 99, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current)) __PYX_ERR(7, 99, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Latest", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest) __PYX_ERR(7, 107, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest)) __PYX_ERR(7, 107, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "CompoundedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder) __PYX_ERR(7, 115, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder)) __PYX_ERR(7, 115, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "IIF", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF) __PYX_ERR(7, 124, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF)) __PYX_ERR(7, 124, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "BasicFunction", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction) __PYX_ERR(7, 134, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction)) __PYX_ERR(7, 134, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Exp", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp) __PYX_ERR(7, 142, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp)) __PYX_ERR(7, 142, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Log", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log) __PYX_ERR(7, 147, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log)) __PYX_ERR(7, 147, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Sqrt", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt) __PYX_ERR(7, 152, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt)) __PYX_ERR(7, 152, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Pow", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow) __PYX_ERR(7, 157, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow)) __PYX_ERR(7, 157, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Abs", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs) __PYX_ERR(7, 164, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs)) __PYX_ERR(7, 164, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Sign", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign) __PYX_ERR(7, 169, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign)) __PYX_ERR(7, 169, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Acos", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos) __PYX_ERR(7, 174, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos)) __PYX_ERR(7, 174, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Acosh", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh) __PYX_ERR(7, 179, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh)) __PYX_ERR(7, 179, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Asin", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin) __PYX_ERR(7, 184, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin)) __PYX_ERR(7, 184, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Asinh", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh) __PYX_ERR(7, 189, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh)) __PYX_ERR(7, 189, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "NormInv", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv) __PYX_ERR(7, 194, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv)) __PYX_ERR(7, 194, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Ceil", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil) __PYX_ERR(7, 201, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil)) __PYX_ERR(7, 201, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Floor", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor) __PYX_ERR(7, 206, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor)) __PYX_ERR(7, 206, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Round", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round) __PYX_ERR(7, 211, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round)) __PYX_ERR(7, 211, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Sigmoid", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid) __PYX_ERR(7, 215, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid)) __PYX_ERR(7, 215, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Tanh", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh) __PYX_ERR(7, 219, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh)) __PYX_ERR(7, 219, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.Accumulators.StatelessAccumulators"); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "Diff", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff) __PYX_ERR(8, 7, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Diff)) __PYX_ERR(8, 7, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "SimpleReturn", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn) __PYX_ERR(8, 17, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_SimpleReturn)) __PYX_ERR(8, 17, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "LogReturn", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn) __PYX_ERR(8, 28, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_LogReturn)) __PYX_ERR(8, 28, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "PositivePart", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart) __PYX_ERR(8, 39, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_PositivePart)) __PYX_ERR(8, 39, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "NegativePart", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart) __PYX_ERR(8, 48, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_NegativePart)) __PYX_ERR(8, 48, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "Max", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max) __PYX_ERR(8, 57, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Max)) __PYX_ERR(8, 57, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "Maximum", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum) __PYX_ERR(8, 67, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Maximum)) __PYX_ERR(8, 67, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "Min", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min) __PYX_ERR(8, 77, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Min)) __PYX_ERR(8, 77, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "Minimum", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum) __PYX_ERR(8, 87, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Minimum)) __PYX_ERR(8, 87, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "Sum", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum) __PYX_ERR(8, 98, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Sum)) __PYX_ERR(8, 98, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "Average", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average) __PYX_ERR(8, 108, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Average)) __PYX_ERR(8, 108, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "XAverage", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage) __PYX_ERR(8, 118, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage)) __PYX_ERR(8, 118, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "Variance", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance) __PYX_ERR(8, 129, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Variance)) __PYX_ERR(8, 129, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.StatelessAccumulators", "Product", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product) __PYX_ERR(8, 141, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_Product)) __PYX_ERR(8, 141, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.MathConstants"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "DBL_MIN", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_DBL_MIN, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "QL_EPSILON", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_QL_EPSILON, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_SQRT2", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT2, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_SQRT_2", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT_2, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_SQRTPI", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRTPI, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_1_SQRTPI", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_1_SQRTPI, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_PI", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_PI, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "NAN", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_NAN, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.Accumulators.IAccumulators"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction(__pyx_t_1, "build_holder", (void (**)(void))&__pyx_f_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_build_holder, "PyObject *(PyObject *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Utilities.Asserts"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction(__pyx_t_1, "pyFinAssert", (void (**)(void))&__pyx_f_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert, "int (PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_pyFinAssert *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "isClose", (void (**)(void))&__pyx_f_6ultron_6sentry_9Utilities_7Asserts_isClose, "int (double, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_9Utilities_7Asserts_isClose *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.udfs"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction(__pyx_t_1, "consecutive_int_sum", (void (**)(void))&__pyx_f_6ultron_6sentry_4Math_4udfs_consecutive_int_sum, "double (double, double)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initStatefulAccumulators(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initStatefulAccumulators(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_StatefulAccumulators(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_StatefulAccumulators(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_StatefulAccumulators(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'StatefulAccumulators' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_StatefulAccumulators(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("StatefulAccumulators", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ultron__sentry__Math__Accumulators__StatefulAccumulators) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ultron.sentry.Math.Accumulators.StatefulAccumulators")) {
      if (unlikely(PyDict_SetItemString(modules, "ultron.sentry.Math.Accumulators.StatefulAccumulators", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_function_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":4
 * 
 * 
 * import copy             # <<<<<<<<<<<<<<
 * import bisect
 * import six
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_copy, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_copy, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":5
 * 
 * import copy
 * import bisect             # <<<<<<<<<<<<<<
 * import six
 * import numpy as np
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_bisect, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bisect, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":6
 * import copy
 * import bisect
 * import six             # <<<<<<<<<<<<<<
 * import numpy as np
 * cimport numpy as np
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_six, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_six, __pyx_t_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":7
 * import bisect
 * import six
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * cimport cython
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":13
 * from libc.math cimport log
 * from libc.math cimport sqrt
 * from copy import deepcopy             # <<<<<<<<<<<<<<
 * from ultron.sentry.Math.Accumulators.IAccumulators cimport Accumulator
 * from ultron.sentry.Math.Accumulators.IAccumulators cimport Latest
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_deepcopy);
  __Pyx_GIVEREF(__pyx_n_s_deepcopy);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_deepcopy);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_copy, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_deepcopy, __pyx_t_1) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":807
 * cdef class MACD(Accumulator):
 * 
 *     def __init__(self, short_win, long_win, x, method=XAverage):             # <<<<<<<<<<<<<<
 *         super(MACD, self).__init__()
 *         self._short_average = method(window=short_win, x=x)
 */
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage));
  __pyx_k_ = ((PyObject *)__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage);
  __Pyx_GIVEREF(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_21StatelessAccumulators_XAverage);

  /* "(tree fragment)":1
 * def __pyx_unpickle_StatefulValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_1__pyx_unpickle_StatefulValueHolder, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_StatefulValueHold, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_StatefulValueHolder__set_state(<StatefulValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StatefulValueHolder__set_state(StatefulValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._window = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_3__pyx_unpickle_Shift, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Shift, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Delta(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_5__pyx_unpickle_Delta, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Delta, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_Delta__set_state(<Delta> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Delta__set_state(Delta __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._current = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._popout = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_7__pyx_unpickle_SingleValuedValueHolder, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_SingleValuedValue, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_SortedValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_9__pyx_unpickle_SortedValueHolder, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_SortedValueHolder, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_SortedValueHolder__set_state(<SortedValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_SortedValueHolder__set_state(SortedValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_11__pyx_unpickle_MovingMax, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingMax, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingArgMax(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_13__pyx_unpickle_MovingArgMax, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingArgMax, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingArgMax__set_state(<MovingArgMax> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMax__set_state(MovingArgMax __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_15__pyx_unpickle_MovingMin, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingMin, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingArgMin(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_17__pyx_unpickle_MovingArgMin, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingArgMin, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingArgMin__set_state(<MovingArgMin> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingArgMin__set_state(MovingArgMin __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_19__pyx_unpickle_MovingRank, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingRank, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingQuantile(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_21__pyx_unpickle_MovingQuantile, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingQuantile, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingQuantile__set_state(<MovingQuantile> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingQuantile__set_state(MovingQuantile __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._cur_pos = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._sortedArray = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_23__pyx_unpickle_MovingAllTrue, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingAllTrue, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingAnyTrue(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_25__pyx_unpickle_MovingAnyTrue, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingAnyTrue, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingAnyTrue__set_state(<MovingAnyTrue> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAnyTrue__set_state(MovingAnyTrue __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._countedTrue = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_27__pyx_unpickle_MovingSum, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingSum, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_29__pyx_unpickle_MovingAverage, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingAverage, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingAverage__set_state(<MovingAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingAverage__set_state(MovingAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningSum = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_31__pyx_unpickle_MovingDecay, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingDecay, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingPositiveAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_33__pyx_unpickle_MovingPositiveAverage, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingPositiveAve, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingPositiveAverage__set_state(<MovingPositiveAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingPositiveAverage__set_state(MovingPositiveAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningPositiveCount = __pyx_state[3]; __pyx_result._runningPositiveSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_35__pyx_unpickle_MovingPositiveDifferenceAverage, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingPositiveDif, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingNegativeDifferenceAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_37__pyx_unpickle_MovingNegativeDifferenceAverage, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingNegativeDif, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(<MovingNegativeDifferenceAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeDifferenceAverage__set_state(MovingNegativeDifferenceAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningAverage = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_39__pyx_unpickle_MovingRSI, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingRSI, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingNegativeAverage(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_41__pyx_unpickle_MovingNegativeAverage, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingNegativeAve, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingNegativeAverage__set_state(<MovingNegativeAverage> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingNegativeAverage__set_state(MovingNegativeAverage __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._runningNegativeCount = __pyx_state[3]; __pyx_result._runningNegativeSum = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result._x = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_43__pyx_unpickle_MovingVariance, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingVariance, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingStandardDeviation(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_45__pyx_unpickle_MovingStandardDeviation, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingStandardDev, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingStandardDeviation__set_state(<MovingStandardDeviation> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingStandardDeviation__set_state(MovingStandardDeviation __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._isFull = __pyx_state[2]; __pyx_result._isPop = __pyx_state[3]; __pyx_result._runningSum = __pyx_state[4]; __pyx_result._runningSumSquare = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_47__pyx_unpickle_MovingNegativeVariance, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingNegativeVar, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingCountedPositive(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_49__pyx_unpickle_MovingCountedPositive, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingCountedPosi, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingCountedPositive__set_state(<MovingCountedPositive> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCountedPositive__set_state(MovingCountedPositive __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._counts = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._window = __pyx_state[4]; __pyx_result._x = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_51__pyx_unpickle_MovingCountedNegative, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingCountedNega, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingCorrelation(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_53__pyx_unpickle_MovingCorrelation, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingCorrelation, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingCorrelation__set_state(<MovingCorrelation> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingCorrelation__set_state(MovingCorrelation __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._runningSumCrossSquare = __pyx_state[4]; __pyx_result._runningSumLeft = __pyx_state[5]; __pyx_result._runningSumRight = __pyx_state[6]; __pyx_result._runningSumSquareLeft = __pyx_state[7]; __pyx_result._runningSumSquareRight = __pyx_state[8]; __pyx_result._window = __pyx_state[9]; __pyx_result._x = __pyx_state[10]; __pyx_result._y = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_55__pyx_unpickle_MovingProduct, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingProduct, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MACD(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_57__pyx_unpickle_MACD, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MACD, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MACD__set_state(<MACD> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MACD__set_state(MACD __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._isFull = __pyx_state[1]; __pyx_result._long_average = __pyx_state[2]; __pyx_result._short_average = __pyx_state[3]; __pyx_result._window = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_59__pyx_unpickle_MovingLogReturn, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingLogReturn, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingSharp(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_61__pyx_unpickle_MovingSharp, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingSharp, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingSharp__set_state(<MovingSharp> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingSharp__set_state(MovingSharp __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._deque = __pyx_state[1]; __pyx_result._deque_y = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._mean = __pyx_state[4]; __pyx_result._var = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]; __pyx_result._y = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_63__pyx_unpickle_MovingSortino, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingSortino, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingDrawdown(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_65__pyx_unpickle_MovingDrawdown, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingDrawdown, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_MovingDrawdown__set_state(<MovingDrawdown> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_MovingDrawdown__set_state(MovingDrawdown __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._currentMax = __pyx_state[0]; __pyx_result._dependency = __pyx_state[1]; __pyx_result._deque = __pyx_state[2]; __pyx_result._isFull = __pyx_state[3]; __pyx_result._maxer = __pyx_state[4]; __pyx_result._runningCum = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result._x = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_67__pyx_unpickle_MovingMaxDrawdown, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingMaxDrawdown, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_MovingResidue(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_4Math_12Accumulators_20StatefulAccumulators_69__pyx_unpickle_MovingResidue, NULL, __pyx_n_s_ultron_sentry_Math_Accumulators); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_MovingResidue, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ultron/sentry/Math/Accumulators/StatefulAccumulators.pyx":1
 * # -*- coding: utf-8 -*-             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init ultron.sentry.Math.Accumulators.StatefulAccumulators", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ultron.sentry.Math.Accumulators.StatefulAccumulators");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* DelItemInt */
static int __Pyx_DelItem_Generic(PyObject *o, PyObject *j) {
    int r;
    if (!j) return -1;
    r = PyObject_DelItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_DelItemInt_Fast(PyObject *o, Py_ssize_t i,
                                               CYTHON_UNUSED int is_list, CYTHON_NCP_UNUSED int wraparound) {
#if !CYTHON_USE_TYPE_SLOTS
    if (is_list || PySequence_Check(o)) {
        return PySequence_DelItem(o, i);
    }
#else
    PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
    if (likely(m && m->sq_ass_item)) {
        if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
            Py_ssize_t l = m->sq_length(o);
            if (likely(l >= 0)) {
                i += l;
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    return -1;
                PyErr_Clear();
            }
        }
        return m->sq_ass_item(o, i, (PyObject *)NULL);
    }
#endif
    return __Pyx_DelItem_Generic(o, PyInt_FromSsize_t(i));
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* CallNextTpDealloc */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
    const size_t neg_one = (size_t) ((size_t) 0 - (size_t) 1), const_zero = (size_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
