/* Generated by Cython 0.29.21 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "ultron.sentry.Analysis.CrossSectionValueHolders",
        "sources": [
            "ultron/sentry/Analysis/CrossSectionValueHolders.pyx"
        ]
    },
    "module_name": "ultron.sentry.Analysis.CrossSectionValueHolders"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_21"
#define CYTHON_HEX_VERSION 0x001D15F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ultron__sentry__Analysis__CrossSectionValueHolders
#define __PYX_HAVE_API__ultron__sentry__Analysis__CrossSectionValueHolders
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"

    /* NumPy API declarations from "numpy/__init__.pxd" */
    
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "ultron/sentry/Analysis/CrossSectionValueHolders.pyx",
  "stringsource",
  "__init__.pxd",
  "type.pxd",
  "ultron/sentry/Analysis/SeriesValues.pxd",
  "ultron/sentry/Math/Distributions/NormalDistribution.pxd",
  "ultron/sentry/Math/Accumulators/IAccumulators.pxd",
  "ultron/sentry/Analysis/SecurityValueHolders.pxd",
};

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":689
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":690
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":691
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":692
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":696
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":697
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":698
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":699
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":703
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":704
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":713
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":714
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":715
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":717
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":718
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":719
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":721
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":722
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":724
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":725
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":726
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/
struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues;
struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid;
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder;
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":728
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":729
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":730
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":732
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_rank;
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n;
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n;
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n_percentile;
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n_percentile;
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_zscore;
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_mean;
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_fillna;
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_percentile;

/* "ultron/sentry/Analysis/SeriesValues.pxd":13
 *     cpdef SeriesValues mask(self, np.ndarray flags)
 *     cpdef list index(self)
 *     cpdef SeriesValues rank(self, SeriesValues groups=*)             # <<<<<<<<<<<<<<
 *     cpdef SeriesValues top_n(self, int n, SeriesValues groups=*)
 *     cpdef SeriesValues bottom_n(self, int n, SeriesValues groups=*)
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_rank {
  int __pyx_n;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *groups;
};

/* "ultron/sentry/Analysis/SeriesValues.pxd":14
 *     cpdef list index(self)
 *     cpdef SeriesValues rank(self, SeriesValues groups=*)
 *     cpdef SeriesValues top_n(self, int n, SeriesValues groups=*)             # <<<<<<<<<<<<<<
 *     cpdef SeriesValues bottom_n(self, int n, SeriesValues groups=*)
 *     cpdef SeriesValues top_n_percentile(self, double n, SeriesValues groups=*)
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n {
  int __pyx_n;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *groups;
};

/* "ultron/sentry/Analysis/SeriesValues.pxd":15
 *     cpdef SeriesValues rank(self, SeriesValues groups=*)
 *     cpdef SeriesValues top_n(self, int n, SeriesValues groups=*)
 *     cpdef SeriesValues bottom_n(self, int n, SeriesValues groups=*)             # <<<<<<<<<<<<<<
 *     cpdef SeriesValues top_n_percentile(self, double n, SeriesValues groups=*)
 *     cpdef SeriesValues bottom_n_percentile(self, double n, SeriesValues groups=*)
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n {
  int __pyx_n;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *groups;
};

/* "ultron/sentry/Analysis/SeriesValues.pxd":16
 *     cpdef SeriesValues top_n(self, int n, SeriesValues groups=*)
 *     cpdef SeriesValues bottom_n(self, int n, SeriesValues groups=*)
 *     cpdef SeriesValues top_n_percentile(self, double n, SeriesValues groups=*)             # <<<<<<<<<<<<<<
 *     cpdef SeriesValues bottom_n_percentile(self, double n, SeriesValues groups=*)
 *     cpdef SeriesValues zscore(self, SeriesValues groups=*)
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n_percentile {
  int __pyx_n;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *groups;
};

/* "ultron/sentry/Analysis/SeriesValues.pxd":17
 *     cpdef SeriesValues bottom_n(self, int n, SeriesValues groups=*)
 *     cpdef SeriesValues top_n_percentile(self, double n, SeriesValues groups=*)
 *     cpdef SeriesValues bottom_n_percentile(self, double n, SeriesValues groups=*)             # <<<<<<<<<<<<<<
 *     cpdef SeriesValues zscore(self, SeriesValues groups=*)
 *     cpdef SeriesValues unit(self)
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n_percentile {
  int __pyx_n;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *groups;
};

/* "ultron/sentry/Analysis/SeriesValues.pxd":18
 *     cpdef SeriesValues top_n_percentile(self, double n, SeriesValues groups=*)
 *     cpdef SeriesValues bottom_n_percentile(self, double n, SeriesValues groups=*)
 *     cpdef SeriesValues zscore(self, SeriesValues groups=*)             # <<<<<<<<<<<<<<
 *     cpdef SeriesValues unit(self)
 * 
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_zscore {
  int __pyx_n;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *groups;
};

/* "ultron/sentry/Analysis/SeriesValues.pxd":21
 *     cpdef SeriesValues unit(self)
 * 
 *     cpdef SeriesValues mean(self, SeriesValues groups=*)             # <<<<<<<<<<<<<<
 *     cpdef SeriesValues fillna(self, SeriesValues groups=*)
 *     cpdef SeriesValues percentile(self, SeriesValues groups=*)
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_mean {
  int __pyx_n;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *groups;
};

/* "ultron/sentry/Analysis/SeriesValues.pxd":22
 * 
 *     cpdef SeriesValues mean(self, SeriesValues groups=*)
 *     cpdef SeriesValues fillna(self, SeriesValues groups=*)             # <<<<<<<<<<<<<<
 *     cpdef SeriesValues percentile(self, SeriesValues groups=*)
 *     cpdef double dot(self, SeriesValues right)
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_fillna {
  int __pyx_n;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *groups;
};

/* "ultron/sentry/Analysis/SeriesValues.pxd":23
 *     cpdef SeriesValues mean(self, SeriesValues groups=*)
 *     cpdef SeriesValues fillna(self, SeriesValues groups=*)
 *     cpdef SeriesValues percentile(self, SeriesValues groups=*)             # <<<<<<<<<<<<<<
 *     cpdef double dot(self, SeriesValues right)
 *     cpdef SeriesValues res(self, SeriesValues right)
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_percentile {
  int __pyx_n;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *groups;
};
struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_11Accumulator_transform;

/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":19
 *     cpdef push(self, dict data)
 *     cpdef double result(self)
 *     cpdef transform(self, data, str name=*, bint to_sort=*)             # <<<<<<<<<<<<<<
 * 
 * cdef bint isanumber(a)
 */
struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_11Accumulator_transform {
  int __pyx_n;
  PyObject *name;
  int to_sort;
};
struct __pyx_opt_args_6ultron_6sentry_8Analysis_20SecurityValueHolders_19SecurityValueHolder_transform;

/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":20
 *     cpdef double value_by_name(self, name)
 *     cpdef shift(self, int n)
 *     cpdef transform(self, data, str name=*, str category_field=*, bint dropna=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_6ultron_6sentry_8Analysis_20SecurityValueHolders_19SecurityValueHolder_transform {
  int __pyx_n;
  PyObject *name;
  PyObject *category_field;
  int dropna;
};

/* "ultron/sentry/Analysis/SeriesValues.pxd":5
 * cimport numpy as np
 * 
 * cdef class SeriesValues(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef public dict name_mapping
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_vtab;
  PyObject *name_mapping;
  PyArrayObject *values;
  PyArrayObject *name_array;
};


/* "ultron/sentry/Math/Distributions/NormalDistribution.pxd":3
 * # -*- coding: utf-8 -*-
 * 
 * cdef class InverseCumulativeNormal(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _average
 */
struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *__pyx_vtab;
  double _average;
  double _sigma;
  int _fullAcc;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":6
 * 
 * 
 * cdef class IAccumulator(object):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator {
  PyObject_HEAD
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":10
 * 
 * 
 * cdef class Accumulator(IAccumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public bint _isFull
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator __pyx_base;
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *__pyx_vtab;
  int _isFull;
  PyObject *_dependency;
  size_t _window;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":25
 * 
 * 
 * cdef class Negative(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _inner
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":33
 * 
 * 
 * cdef class CombinedValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _left
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_left;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_right;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":41
 * 
 * 
 * cdef class AddedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":46
 * 
 * 
 * cdef class MinusedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":51
 * 
 * 
 * cdef class MultipliedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":56
 * 
 * 
 * cdef class DividedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":61
 * 
 * 
 * cdef class LtOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":66
 * 
 * 
 * cdef class LeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":71
 * 
 * 
 * cdef class GtOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":76
 * 
 * 
 * cdef class GeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":81
 * 
 * 
 * cdef class EqOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":86
 * 
 * 
 * cdef class NeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":91
 * 
 * 
 * cdef class Identity(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _value
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _value;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":99
 * 
 * 
 * cdef class Current(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _current
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _current;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":107
 * 
 * 
 * cdef class Latest(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _latest
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _latest;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":115
 * 
 * 
 * cdef class CompoundedValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _left
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_left;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_right;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":124
 * 
 * 
 * cdef class IIF(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _cond
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_cond;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_left;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_right;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":134
 * 
 * 
 * cdef class BasicFunction(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _origValue
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
  double _origValue;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_inner;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":142
 * 
 * 
 * cdef class Exp(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":147
 * 
 * 
 * cdef class Log(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":152
 * 
 * 
 * cdef class Sqrt(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":157
 * 
 * 
 * cdef class Pow(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _n
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
  double _n;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":164
 * 
 * 
 * cdef class Abs(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":169
 * 
 * 
 * cdef class Sign(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":174
 * 
 * 
 * cdef class Acos(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":179
 * 
 * 
 * cdef class Acosh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":184
 * 
 * 
 * cdef class Asin(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":189
 * 
 * 
 * cdef class Asinh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":194
 * 
 * 
 * cdef class NormInv(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cdef InvNormImpl _inv
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
  struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *_inv;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":201
 * 
 * 
 * cdef class Ceil(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":206
 * 
 * 
 * cdef class Floor(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":211
 * 
 * 
 * cdef class Round(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":215
 *     cpdef double result(self)
 * 
 * cdef class Sigmoid(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":219
 *     cpdef double result(self)
 * 
 * cdef class Tanh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */
struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh {
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":7
 * 
 * 
 * cdef class SecurityValueHolder(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef public list _dependency
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *__pyx_vtab;
  PyObject *_dependency;
  int _window;
  struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *_holderTemplate;
  int updated;
  PyObject *_innerHolders;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *cached;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":23
 * 
 * 
 * cdef class SecuritySingleValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _compHolder
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_compHolder;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":29
 * 
 * 
 * cdef class SecurityBinaryValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 *     cdef public SecurityValueHolder _compHolder1
 *     cdef public SecurityValueHolder _compHolder2
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_compHolder1;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_compHolder2;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":35
 * 
 * 
 * cdef class SecurityStatelessSingleValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _compHolder
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_compHolder;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":42
 * 
 * 
 * cdef class FilteredSecurityValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _filter
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_filter;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_computer;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":52
 * 
 * 
 * cdef class IdentitySecurityValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public object _value
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *_value;
  PyObject *_symbols;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":63
 * 
 * 
 * cdef class SecurityConstArrayValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 *     cdef SeriesValues _values
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *_values;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":72
 * 
 * 
 * cdef class SecurityUnitoryValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _right
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_right;
  PyObject *_op;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":83
 * 
 * 
 * cdef class SecurityNegValueHolder(SecurityUnitoryValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":87
 * 
 * 
 * cdef class SecurityInvertValueHolder(SecurityUnitoryValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":91
 * 
 * 
 * cdef class SecurityCurrentValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 *     cdef dict _symbol_values
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *_symbol_values;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":100
 * 
 * 
 * cdef class SecurityLatestValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 *     cdef dict _symbol_values
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *_symbol_values;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":112
 * 
 * 
 * cdef class SecurityCombinedValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _left
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_left;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_right;
  PyObject *_op;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":123
 *     cpdef push(self, dict data)
 * 
 * cdef class SecurityXorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name)
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":128
 * 
 * 
 * cdef class SecurityAddedValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":132
 * 
 * 
 * cdef class SecuritySubbedValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":136
 * 
 * 
 * cdef class SecurityMultipliedValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":140
 * 
 * 
 * cdef class SecurityDividedValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":144
 * 
 * 
 * cdef class SecurityLtOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":148
 * 
 * 
 * cdef class SecurityLeOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":152
 * 
 * 
 * cdef class SecurityGtOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":156
 * 
 * 
 * cdef class SecurityGeOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":160
 * 
 * 
 * cdef class SecurityEqOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":164
 * 
 * 
 * cdef class SecurityNeOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":168
 * 
 * 
 * cdef class SecurityAndOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":172
 * 
 * 
 * cdef class SecurityOrOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":176
 * 
 * 
 * cdef class SecurityShiftedValueHolder(SecuritySingleValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":179
 *     pass
 * 
 * cdef class SecurityDeltaValueHolder(SecuritySingleValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":183
 * 
 * 
 * cdef class SecurityIIFValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _flag
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_flag;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_left;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_right;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":15
 * 
 * 
 * cdef class CrossSectionValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _inner
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_inner;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_group;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":44
 * 
 * 
 * cdef class CSTopNSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef int _n;
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  int _n;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":90
 * 
 * 
 * cdef class CSTopNPercentileSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _n;
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  double _n;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":136
 * 
 * 
 * cdef class CSBottomNSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef int _n;
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  int _n;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":182
 * 
 * 
 * cdef class CSBottomNPercentileSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _n;
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  double _n;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":228
 * 
 * 
 * cdef class CSRankedSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, innerValue, groups=None):
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":271
 * 
 * 
 * cdef class CSAverageSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, innerValue, groups=None):
 *         super(CSAverageSecurityValueHolder, self).__init__(innerValue, groups)
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":313
 * 
 * 
 * cdef class CSPercentileSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, innerValue, groups=None):
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":356
 * 
 * 
 * cdef class CSAverageAdjustedSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, innerValue, groups=None):
 *         super(CSAverageAdjustedSecurityValueHolder, self).__init__(innerValue, groups)
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":405
 * 
 * 
 * cdef class CSZScoreSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, innerValue, groups=None):
 *         super(CSZScoreSecurityValueHolder, self).__init__(innerValue, groups)
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":447
 * 
 * 
 * cdef class CSFillNASecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, innerValue, groups=None):
 *         super(CSFillNASecurityValueHolder, self).__init__(innerValue, groups)
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
};


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":489
 * 
 * 
 * cdef class CSResidueSecurityValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _left
 */
struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder {
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_left;
  struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *_right;
};



/* "ultron/sentry/Analysis/SeriesValues.pxd":5
 * cimport numpy as np
 * 
 * cdef class SeriesValues(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef public dict name_mapping
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*mask)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, PyArrayObject *, int __pyx_skip_dispatch);
  PyObject *(*index)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*rank)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_rank *__pyx_optional_args);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*top_n)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n *__pyx_optional_args);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*bottom_n)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n *__pyx_optional_args);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*top_n_percentile)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n_percentile *__pyx_optional_args);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*bottom_n_percentile)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, double, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n_percentile *__pyx_optional_args);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*zscore)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_zscore *__pyx_optional_args);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*unit)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*mean)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_mean *__pyx_optional_args);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*fillna)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_fillna *__pyx_optional_args);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*percentile)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_percentile *__pyx_optional_args);
  double (*dot)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*res)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch);
  PyObject *(*to_dict)(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_vtabptr_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues;


/* "ultron/sentry/Math/Distributions/NormalDistribution.pxd":3
 * # -*- coding: utf-8 -*-
 * 
 * cdef class InverseCumulativeNormal(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _average
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal {
  double (*_standard_value)(struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *, double);
  double (*inv)(struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *, double);
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal *__pyx_vtabptr_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":10
 * 
 * 
 * cdef class Accumulator(IAccumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public bint _isFull
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator {
  int (*isFull)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch);
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch);
  double (*result)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, int __pyx_skip_dispatch);
  PyObject *(*transform)(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_11Accumulator_transform *__pyx_optional_args);
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":25
 * 
 * 
 * cdef class Negative(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _inner
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":33
 * 
 * 
 * cdef class CombinedValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _left
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":41
 * 
 * 
 * cdef class AddedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":46
 * 
 * 
 * cdef class MinusedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":51
 * 
 * 
 * cdef class MultipliedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":56
 * 
 * 
 * cdef class DividedValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":61
 * 
 * 
 * cdef class LtOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":66
 * 
 * 
 * cdef class LeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":71
 * 
 * 
 * cdef class GtOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":76
 * 
 * 
 * cdef class GeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":81
 * 
 * 
 * cdef class EqOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":86
 * 
 * 
 * cdef class NeOperatorValueHolder(CombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":91
 * 
 * 
 * cdef class Identity(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _value
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":99
 * 
 * 
 * cdef class Current(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _current
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":107
 * 
 * 
 * cdef class Latest(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _latest
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":115
 * 
 * 
 * cdef class CompoundedValueHolder(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _left
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":124
 * 
 * 
 * cdef class IIF(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef Accumulator _cond
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":134
 * 
 * 
 * cdef class BasicFunction(Accumulator):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _origValue
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":142
 * 
 * 
 * cdef class Exp(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":147
 * 
 * 
 * cdef class Log(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":152
 * 
 * 
 * cdef class Sqrt(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":157
 * 
 * 
 * cdef class Pow(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cdef public double _n
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":164
 * 
 * 
 * cdef class Abs(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":169
 * 
 * 
 * cdef class Sign(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":174
 * 
 * 
 * cdef class Acos(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":179
 * 
 * 
 * cdef class Acosh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":184
 * 
 * 
 * cdef class Asin(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":189
 * 
 * 
 * cdef class Asinh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":194
 * 
 * 
 * cdef class NormInv(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cdef InvNormImpl _inv
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":201
 * 
 * 
 * cdef class Ceil(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":206
 * 
 * 
 * cdef class Floor(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":211
 * 
 * 
 * cdef class Round(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":215
 *     cpdef double result(self)
 * 
 * cdef class Sigmoid(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid;


/* "ultron/sentry/Math/Accumulators/IAccumulators.pxd":219
 *     cpdef double result(self)
 * 
 * cdef class Tanh(BasicFunction):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double result(self)
 */

struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh {
  struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh *__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":7
 * 
 * 
 * cdef class SecurityValueHolder(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef public list _dependency
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder {
  PyObject *(*value_all)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*value_by_names)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch);
  double (*value_by_name)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*shift)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int, int __pyx_skip_dispatch);
  PyObject *(*transform)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_6ultron_6sentry_8Analysis_20SecurityValueHolders_19SecurityValueHolder_transform *__pyx_optional_args);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":23
 * 
 * 
 * cdef class SecuritySingleValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _compHolder
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":29
 * 
 * 
 * cdef class SecurityBinaryValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 *     cdef public SecurityValueHolder _compHolder1
 *     cdef public SecurityValueHolder _compHolder2
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":35
 * 
 * 
 * cdef class SecurityStatelessSingleValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _compHolder
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":42
 * 
 * 
 * cdef class FilteredSecurityValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _filter
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":52
 * 
 * 
 * cdef class IdentitySecurityValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public object _value
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*value_all)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":63
 * 
 * 
 * cdef class SecurityConstArrayValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 *     cdef SeriesValues _values
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*value_all)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":72
 * 
 * 
 * cdef class SecurityUnitoryValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _right
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*value_all)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder *, int __pyx_skip_dispatch);
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":83
 * 
 * 
 * cdef class SecurityNegValueHolder(SecurityUnitoryValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":87
 * 
 * 
 * cdef class SecurityInvertValueHolder(SecurityUnitoryValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":91
 * 
 * 
 * cdef class SecurityCurrentValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 *     cdef dict _symbol_values
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*value_all)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":100
 * 
 * 
 * cdef class SecurityLatestValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 *     cdef dict _symbol_values
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*value_all)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":112
 * 
 * 
 * cdef class SecurityCombinedValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _left
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*value_all)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder *, int __pyx_skip_dispatch);
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":123
 *     cpdef push(self, dict data)
 * 
 * cdef class SecurityXorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name)
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":128
 * 
 * 
 * cdef class SecurityAddedValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":132
 * 
 * 
 * cdef class SecuritySubbedValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":136
 * 
 * 
 * cdef class SecurityMultipliedValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":140
 * 
 * 
 * cdef class SecurityDividedValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":144
 * 
 * 
 * cdef class SecurityLtOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":148
 * 
 * 
 * cdef class SecurityLeOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":152
 * 
 * 
 * cdef class SecurityGtOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":156
 * 
 * 
 * cdef class SecurityGeOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":160
 * 
 * 
 * cdef class SecurityEqOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":164
 * 
 * 
 * cdef class SecurityNeOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":168
 * 
 * 
 * cdef class SecurityAndOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":172
 * 
 * 
 * cdef class SecurityOrOperatorValueHolder(SecurityCombinedValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":176
 * 
 * 
 * cdef class SecurityShiftedValueHolder(SecuritySingleValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":179
 *     pass
 * 
 * cdef class SecurityDeltaValueHolder(SecuritySingleValueHolder):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder;


/* "ultron/sentry/Analysis/SecurityValueHolders.pxd":183
 * 
 * 
 * cdef class SecurityIIFValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _flag
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*value_all)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder *, int __pyx_skip_dispatch);
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":15
 * 
 * 
 * cdef class CrossSectionValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _inner
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":44
 * 
 * 
 * cdef class CSTopNSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef int _n;
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  PyObject *(*_cal_impl)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":90
 * 
 * 
 * cdef class CSTopNPercentileSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _n;
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  PyObject *(*_cal_impl)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":136
 * 
 * 
 * cdef class CSBottomNSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef int _n;
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  PyObject *(*_cal_impl)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":182
 * 
 * 
 * cdef class CSBottomNPercentileSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef double _n;
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  PyObject *(*_cal_impl)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":228
 * 
 * 
 * cdef class CSRankedSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, innerValue, groups=None):
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  PyObject *(*_cal_impl)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":271
 * 
 * 
 * cdef class CSAverageSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, innerValue, groups=None):
 *         super(CSAverageSecurityValueHolder, self).__init__(innerValue, groups)
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  PyObject *(*_cal_impl)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":313
 * 
 * 
 * cdef class CSPercentileSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, innerValue, groups=None):
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  PyObject *(*_cal_impl)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":356
 * 
 * 
 * cdef class CSAverageAdjustedSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, innerValue, groups=None):
 *         super(CSAverageAdjustedSecurityValueHolder, self).__init__(innerValue, groups)
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":405
 * 
 * 
 * cdef class CSZScoreSecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, innerValue, groups=None):
 *         super(CSZScoreSecurityValueHolder, self).__init__(innerValue, groups)
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  PyObject *(*_cal_impl)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":447
 * 
 * 
 * cdef class CSFillNASecurityValueHolder(CrossSectionValueHolder):             # <<<<<<<<<<<<<<
 *     def __init__(self, innerValue, groups=None):
 *         super(CSFillNASecurityValueHolder, self).__init__(innerValue, groups)
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_base;
  PyObject *(*_cal_impl)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder;


/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":489
 * 
 * 
 * cdef class CSResidueSecurityValueHolder(SecurityValueHolder):             # <<<<<<<<<<<<<<
 * 
 *     cdef public SecurityValueHolder _left
 */

struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder {
  struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder __pyx_base;
  PyObject *(*push)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_push(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_push(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;

/* Module declarations from 'cython' */

/* Module declarations from 'ultron.sentry.Analysis.SeriesValues' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues = 0;

/* Module declarations from 'ultron.sentry.Math.Distributions.NormalDistribution' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal = 0;

/* Module declarations from 'ultron.sentry.Math.Accumulators.IAccumulators' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh = 0;

/* Module declarations from 'ultron.sentry.Analysis.SecurityValueHolders' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder = 0;

/* Module declarations from 'ultron.sentry.Math.MathConstants' */
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_DBL_MIN = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_DBL_MIN (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_DBL_MIN)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_QL_EPSILON = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_QL_EPSILON (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_QL_EPSILON)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT2 = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_SQRT2 (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT2)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT_2 = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_SQRT_2 (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT_2)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRTPI = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_SQRTPI (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRTPI)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_1_SQRTPI = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_1_SQRTPI (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_1_SQRTPI)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_PI = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_M_PI (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_PI)
static double *__pyx_vp_6ultron_6sentry_4Math_13MathConstants_NAN = 0;
#define __pyx_v_6ultron_6sentry_4Math_13MathConstants_NAN (*__pyx_vp_6ultron_6sentry_4Math_13MathConstants_NAN)

/* Module declarations from 'ultron.sentry.Analysis.CrossSectionValueHolders' */
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder = 0;
static PyTypeObject *__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder = 0;
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CrossSectionValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSTopNSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSBottomNSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSRankedSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSAverageSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSPercentileSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSZScoreSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSFillNASecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *, PyObject *); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSResidueSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "ultron.sentry.Analysis.CrossSectionValueHolders"
extern int __pyx_module_is_main_ultron__sentry__Analysis__CrossSectionValueHolders;
int __pyx_module_is_main_ultron__sentry__Analysis__CrossSectionValueHolders = 0;

/* Implementation of 'ultron.sentry.Analysis.CrossSectionValueHolders' */
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_n[] = "n";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_six[] = "six";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_left[] = "left";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_push[] = "push";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_right[] = "right";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_fields[] = "fields";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_groups[] = "groups";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_window[] = "window";
static const char __pyx_k_deepcopy[] = "deepcopy";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_value_all[] = "value_all";
static const char __pyx_k_innerValue[] = "innerValue";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_symbolList[] = "symbolList";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_build_holder[] = "build_holder";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_value_by_name[] = "value_by_name";
static const char __pyx_k_value_by_names[] = "value_by_names";
static const char __pyx_k_mathrm_CSMean_0[] = "\\mathrm{{CSMean}}({0})";
static const char __pyx_k_mathrm_CSRank_0[] = "\\mathrm{{CSRank}}({0})";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_mathrm_CSRes_0_1[] = "\\mathrm{{CSRes}}({0}, {1})";
static const char __pyx_k_mathrm_CSFillNA_0[] = "\\mathrm{{CSFillNA}}({0})";
static const char __pyx_k_mathrm_CSTopN_0_1[] = "\\mathrm{{CSTopN}}({0}, {1})";
static const char __pyx_k_mathrm_CSZscore_0[] = "\\mathrm{{CSZscore}}({0})";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_mathrm_CSBottomN_0_1[] = "\\mathrm{{CSBottomN}}({0}, {1})";
static const char __pyx_k_mathrm_CSPercentile_0[] = "\\mathrm{{CSPercentile}}({0})";
static const char __pyx_k_CrossSectionValueHolder[] = "CrossSectionValueHolder";
static const char __pyx_k_mathrm_CSMeanAdjusted_0[] = "\\mathrm{{CSMeanAdjusted}}({0})";
static const char __pyx_k_mathrm_CSMean_0_groups_1[] = "\\mathrm{{CSMean}}({0}, groups={1})";
static const char __pyx_k_mathrm_CSRank_0_groups_1[] = "\\mathrm{{CSRank}}({0}, groups={1})";
static const char __pyx_k_CSTopNSecurityValueHolder[] = "CSTopNSecurityValueHolder";
static const char __pyx_k_mathrm_CSFillNA_0_groups_1[] = "\\mathrm{{CSFillNA}}({0}, groups={1})";
static const char __pyx_k_mathrm_CSTopN_0_1_groups_2[] = "\\mathrm{{CSTopN}}({0}, {1}, groups={2})";
static const char __pyx_k_mathrm_CSZscore_0_groups_1[] = "\\mathrm{{CSZscore}}({0}, groups={1})";
static const char __pyx_k_CSFillNASecurityValueHolder[] = "CSFillNASecurityValueHolder";
static const char __pyx_k_CSRankedSecurityValueHolder[] = "CSRankedSecurityValueHolder";
static const char __pyx_k_CSZScoreSecurityValueHolder[] = "CSZScoreSecurityValueHolder";
static const char __pyx_k_mathrm_CSTopNPercentile_0_1[] = "\\mathrm{{CSTopNPercentile}}({0}, {1})";
static const char __pyx_k_CSAverageSecurityValueHolder[] = "CSAverageSecurityValueHolder";
static const char __pyx_k_CSBottomNSecurityValueHolder[] = "CSBottomNSecurityValueHolder";
static const char __pyx_k_CSResidueSecurityValueHolder[] = "CSResidueSecurityValueHolder";
static const char __pyx_k_mathrm_CSBottomN_0_1_groups_2[] = "\\mathrm{{CSBottomN}}({0}, {1}, groups={2})";
static const char __pyx_k_mathrm_CSBottomNPercentile_0_1[] = "\\mathrm{{CSBottomNPercentile}}({0}, {1}, groups={2})";
static const char __pyx_k_mathrm_CSMeanAdjusted_0_groups[] = "\\mathrm{{CSMeanAdjusted}}({0}, groups={1})";
static const char __pyx_k_mathrm_CSPercentile_0_groups_1[] = "\\mathrm{{CSPercentile}}({0}, groups={1})";
static const char __pyx_k_pyx_unpickle_CSAverageAdjusted[] = "__pyx_unpickle_CSAverageAdjustedSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSAverageSecurity[] = "__pyx_unpickle_CSAverageSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSBottomNPercenti[] = "__pyx_unpickle_CSBottomNPercentileSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSBottomNSecurity[] = "__pyx_unpickle_CSBottomNSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSFillNASecurityV[] = "__pyx_unpickle_CSFillNASecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSPercentileSecur[] = "__pyx_unpickle_CSPercentileSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSRankedSecurityV[] = "__pyx_unpickle_CSRankedSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSResidueSecurity[] = "__pyx_unpickle_CSResidueSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSTopNPercentileS[] = "__pyx_unpickle_CSTopNPercentileSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSTopNSecurityVal[] = "__pyx_unpickle_CSTopNSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CSZScoreSecurityV[] = "__pyx_unpickle_CSZScoreSecurityValueHolder";
static const char __pyx_k_pyx_unpickle_CrossSectionValue[] = "__pyx_unpickle_CrossSectionValueHolder";
static const char __pyx_k_CSPercentileSecurityValueHolder[] = "CSPercentileSecurityValueHolder";
static const char __pyx_k_mathrm_CSTopNPercentile_0_1_gro[] = "\\mathrm{{CSTopNPercentile}}({0}, {1}, groups={2})";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_CSAverageAdjustedSecurityValueHo[] = "CSAverageAdjustedSecurityValueHolder";
static const char __pyx_k_CSBottomNPercentileSecurityValue[] = "CSBottomNPercentileSecurityValueHolder";
static const char __pyx_k_CSTopNPercentileSecurityValueHol[] = "CSTopNPercentileSecurityValueHolder";
static const char __pyx_k_Incompatible_checksums_s_vs_0x24[] = "Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x7b[] = "Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xe2[] = "Incompatible checksums (%s vs 0xe27c424 = (_dependency, _holderTemplate, _innerHolders, _left, _right, _window, cached, updated))";
static const char __pyx_k_mathrm_CSBottomNPercentile_0_1_2[] = "\\mathrm{{CSBottomNPercentile}}({0}, {1})";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_ultron_sentry_Analysis_CrossSect[] = "ultron.sentry.Analysis.CrossSectionValueHolders";
static const char __pyx_k_ultron_sentry_Analysis_SecurityV[] = "ultron.sentry.Analysis.SecurityValueHolders";
static PyObject *__pyx_n_s_CSAverageAdjustedSecurityValueHo;
static PyObject *__pyx_n_s_CSAverageSecurityValueHolder;
static PyObject *__pyx_n_s_CSBottomNPercentileSecurityValue;
static PyObject *__pyx_n_s_CSBottomNSecurityValueHolder;
static PyObject *__pyx_n_s_CSFillNASecurityValueHolder;
static PyObject *__pyx_n_s_CSPercentileSecurityValueHolder;
static PyObject *__pyx_n_s_CSRankedSecurityValueHolder;
static PyObject *__pyx_n_s_CSResidueSecurityValueHolder;
static PyObject *__pyx_n_s_CSTopNPercentileSecurityValueHol;
static PyObject *__pyx_n_s_CSTopNSecurityValueHolder;
static PyObject *__pyx_n_s_CSZScoreSecurityValueHolder;
static PyObject *__pyx_n_s_CrossSectionValueHolder;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x24;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x7b;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xe2;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_build_holder;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_deepcopy;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_fields;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_groups;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_innerValue;
static PyObject *__pyx_n_s_left;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_kp_s_mathrm_CSBottomNPercentile_0_1;
static PyObject *__pyx_kp_s_mathrm_CSBottomNPercentile_0_1_2;
static PyObject *__pyx_kp_s_mathrm_CSBottomN_0_1;
static PyObject *__pyx_kp_s_mathrm_CSBottomN_0_1_groups_2;
static PyObject *__pyx_kp_s_mathrm_CSFillNA_0;
static PyObject *__pyx_kp_s_mathrm_CSFillNA_0_groups_1;
static PyObject *__pyx_kp_s_mathrm_CSMeanAdjusted_0;
static PyObject *__pyx_kp_s_mathrm_CSMeanAdjusted_0_groups;
static PyObject *__pyx_kp_s_mathrm_CSMean_0;
static PyObject *__pyx_kp_s_mathrm_CSMean_0_groups_1;
static PyObject *__pyx_kp_s_mathrm_CSPercentile_0;
static PyObject *__pyx_kp_s_mathrm_CSPercentile_0_groups_1;
static PyObject *__pyx_kp_s_mathrm_CSRank_0;
static PyObject *__pyx_kp_s_mathrm_CSRank_0_groups_1;
static PyObject *__pyx_kp_s_mathrm_CSRes_0_1;
static PyObject *__pyx_kp_s_mathrm_CSTopNPercentile_0_1;
static PyObject *__pyx_kp_s_mathrm_CSTopNPercentile_0_1_gro;
static PyObject *__pyx_kp_s_mathrm_CSTopN_0_1;
static PyObject *__pyx_kp_s_mathrm_CSTopN_0_1_groups_2;
static PyObject *__pyx_kp_s_mathrm_CSZscore_0;
static PyObject *__pyx_kp_s_mathrm_CSZscore_0_groups_1;
static PyObject *__pyx_n_s_n;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_s_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_s_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_push;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_CSAverageAdjusted;
static PyObject *__pyx_n_s_pyx_unpickle_CSAverageSecurity;
static PyObject *__pyx_n_s_pyx_unpickle_CSBottomNPercenti;
static PyObject *__pyx_n_s_pyx_unpickle_CSBottomNSecurity;
static PyObject *__pyx_n_s_pyx_unpickle_CSFillNASecurityV;
static PyObject *__pyx_n_s_pyx_unpickle_CSPercentileSecur;
static PyObject *__pyx_n_s_pyx_unpickle_CSRankedSecurityV;
static PyObject *__pyx_n_s_pyx_unpickle_CSResidueSecurity;
static PyObject *__pyx_n_s_pyx_unpickle_CSTopNPercentileS;
static PyObject *__pyx_n_s_pyx_unpickle_CSTopNSecurityVal;
static PyObject *__pyx_n_s_pyx_unpickle_CSZScoreSecurityV;
static PyObject *__pyx_n_s_pyx_unpickle_CrossSectionValue;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_right;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_six;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_symbolList;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_ultron_sentry_Analysis_CrossSect;
static PyObject *__pyx_n_s_ultron_sentry_Analysis_SecurityV;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_value_all;
static PyObject *__pyx_n_s_value_by_name;
static PyObject *__pyx_n_s_value_by_names;
static PyObject *__pyx_n_s_window;
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_10symbolList___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_2push(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_2__set__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_4__del__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_2__set__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_4__del__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_4__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_n, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_n, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_n, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_n, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_left, PyObject *__pyx_v_right); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_10symbolList___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_2push(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_4value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_8value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_10__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_2__set__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_4__del__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_2__set__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_4__del__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_12__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_14__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CrossSectionValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_2__pyx_unpickle_CSTopNSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_4__pyx_unpickle_CSTopNPercentileSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_6__pyx_unpickle_CSBottomNSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_8__pyx_unpickle_CSBottomNPercentileSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_10__pyx_unpickle_CSRankedSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_12__pyx_unpickle_CSAverageSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_14__pyx_unpickle_CSPercentileSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_16__pyx_unpickle_CSAverageAdjustedSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_18__pyx_unpickle_CSZScoreSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_20__pyx_unpickle_CSFillNASecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_22__pyx_unpickle_CSResidueSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_37815087;
static PyObject *__pyx_int_129248972;
static PyObject *__pyx_int_237487140;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_codeobj__4;
static PyObject *__pyx_codeobj__6;
static PyObject *__pyx_codeobj__8;
static PyObject *__pyx_codeobj__10;
static PyObject *__pyx_codeobj__12;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__24;
static PyObject *__pyx_codeobj__26;
/* Late includes */

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":20
 *     cdef public SecurityValueHolder _group
 * 
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CrossSectionValueHolder, self).__init__()
 *         self._inner = build_holder(innerValue)
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_groups,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 20, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_groups = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 20, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":21
 * 
 *     def __init__(self, innerValue, groups=None):
 *         super(CrossSectionValueHolder, self).__init__()             # <<<<<<<<<<<<<<
 *         self._inner = build_holder(innerValue)
 *         self._group = build_holder(groups) if groups else None
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":22
 *     def __init__(self, innerValue, groups=None):
 *         super(CrossSectionValueHolder, self).__init__()
 *         self._inner = build_holder(innerValue)             # <<<<<<<<<<<<<<
 *         self._group = build_holder(groups) if groups else None
 *         if self._group:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_build_holder); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_innerValue) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_innerValue);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_inner);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_inner));
  __pyx_v_self->_inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":23
 *         super(CrossSectionValueHolder, self).__init__()
 *         self._inner = build_holder(innerValue)
 *         self._group = build_holder(groups) if groups else None             # <<<<<<<<<<<<<<
 *         if self._group:
 *             self._window = max(self._inner.window, self._group.window)
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_groups); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 23, __pyx_L1_error)
  if (__pyx_t_4) {
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_build_holder); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_v_groups) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_groups);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(0, 23, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  } else {
    __Pyx_INCREF(Py_None);
    __pyx_t_1 = Py_None;
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_group);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_group));
  __pyx_v_self->_group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":24
 *         self._inner = build_holder(innerValue)
 *         self._group = build_holder(groups) if groups else None
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self._window = max(self._inner.window, self._group.window)
 *             self._dependency = list(set(self._inner.fields + self._group.fields))
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->_group)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 24, __pyx_L1_error)
  if (__pyx_t_4) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":25
 *         self._group = build_holder(groups) if groups else None
 *         if self._group:
 *             self._window = max(self._inner.window, self._group.window)             # <<<<<<<<<<<<<<
 *             self._dependency = list(set(self._inner.fields + self._group.fields))
 *         else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_group), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_inner), __pyx_n_s_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 25, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 25, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_4) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1;
    } else {
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = __pyx_t_2;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 25, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->__pyx_base._window = __pyx_t_6;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":26
 *         if self._group:
 *             self._window = max(self._inner.window, self._group.window)
 *             self._dependency = list(set(self._inner.fields + self._group.fields))             # <<<<<<<<<<<<<<
 *         else:
 *             self._window = self._inner.window
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_inner), __pyx_n_s_fields); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 26, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_group), __pyx_n_s_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_Add(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PySet_New(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PySequence_List(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_2);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base._dependency);
    __Pyx_DECREF(__pyx_v_self->__pyx_base._dependency);
    __pyx_v_self->__pyx_base._dependency = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":24
 *         self._inner = build_holder(innerValue)
 *         self._group = build_holder(groups) if groups else None
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self._window = max(self._inner.window, self._group.window)
 *             self._dependency = list(set(self._inner.fields + self._group.fields))
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":28
 *             self._dependency = list(set(self._inner.fields + self._group.fields))
 *         else:
 *             self._window = self._inner.window             # <<<<<<<<<<<<<<
 *             self._dependency = copy.deepcopy(self._inner.fields)
 *         self.updated = 0
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_inner), __pyx_n_s_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 28, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->__pyx_base._window = __pyx_t_6;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":29
 *         else:
 *             self._window = self._inner.window
 *             self._dependency = copy.deepcopy(self._inner.fields)             # <<<<<<<<<<<<<<
 *         self.updated = 0
 *         self.cached = None
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_copy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 29, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_inner), __pyx_n_s_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 29, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 29, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_2);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base._dependency);
    __Pyx_DECREF(__pyx_v_self->__pyx_base._dependency);
    __pyx_v_self->__pyx_base._dependency = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":30
 *             self._window = self._inner.window
 *             self._dependency = copy.deepcopy(self._inner.fields)
 *         self.updated = 0             # <<<<<<<<<<<<<<
 *         self.cached = None
 * 
 */
  __pyx_v_self->__pyx_base.updated = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":31
 *             self._dependency = copy.deepcopy(self._inner.fields)
 *         self.updated = 0
 *         self.cached = None             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
  __pyx_v_self->__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)Py_None);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":20
 *     cdef public SecurityValueHolder _group
 * 
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CrossSectionValueHolder, self).__init__()
 *         self._inner = build_holder(innerValue)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":34
 * 
 *     @property
 *     def symbolList(self):             # <<<<<<<<<<<<<<
 *         return self._inner.symbolList
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_10symbolList_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_10symbolList_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_10symbolList___get__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_10symbolList___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":35
 *     @property
 *     def symbolList(self):
 *         return self._inner.symbolList             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_inner), __pyx_n_s_symbolList); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":34
 * 
 *     @property
 *     def symbolList(self):             # <<<<<<<<<<<<<<
 *         return self._inner.symbolList
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder.symbolList.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":37
 *         return self._inner.symbolList
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._inner.push(data)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_push(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 37, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":38
 * 
 *     cpdef push(self, dict data):
 *         self._inner.push(data)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             self._group.push(data)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_inner), __pyx_n_s_push); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_data);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":39
 *     cpdef push(self, dict data):
 *         self._inner.push(data)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self._group.push(data)
 *         self.updated = 0
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->_group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 39, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":40
 *         self._inner.push(data)
 *         if self._group:
 *             self._group.push(data)             # <<<<<<<<<<<<<<
 *         self.updated = 0
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_group), __pyx_n_s_push); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_data);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":39
 *     cpdef push(self, dict data):
 *         self._inner.push(data)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self._group.push(data)
 *         self.updated = 0
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":41
 *         if self._group:
 *             self._group.push(data)
 *         self.updated = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->__pyx_base.updated = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":37
 *         return self._inner.symbolList
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._inner.push(data)
 *         if self._group:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_2push[] = "CrossSectionValueHolder.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 37, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_2push(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_2push(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":17
 * cdef class CrossSectionValueHolder(SecurityValueHolder):
 * 
 *     cdef public SecurityValueHolder _inner             # <<<<<<<<<<<<<<
 *     cdef public SecurityValueHolder _group
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner___get__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_inner));
  __pyx_r = ((PyObject *)__pyx_v_self->_inner);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_2__set__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_2__set__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(0, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_inner);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_inner));
  __pyx_v_self->_inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder._inner.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_4__del__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_4__del__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_inner);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_inner));
  __pyx_v_self->_inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":18
 * 
 *     cdef public SecurityValueHolder _inner
 *     cdef public SecurityValueHolder _group             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, innerValue, groups=None):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group___get__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_group));
  __pyx_r = ((PyObject *)__pyx_v_self->_group);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_2__set__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_2__set__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(0, 18, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_group);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_group));
  __pyx_v_self->_group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder._group.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_4__del__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_4__del__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_group);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_group));
  __pyx_v_self->_group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_4__reduce_cython__[] = "CrossSectionValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_4__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_4__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.updated); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_group));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->_group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_inner));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->_inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_v_self->__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_group) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->__pyx_base._holderTemplate) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_inner) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.cached) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CrossSectionValue); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, None), state
 *     else:
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CrossSectionValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_CrossSectionValue); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CrossSectionValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6__setstate_cython__[] = "CrossSectionValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CrossSectionValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CrossSectionValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CrossSectionValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CrossSectionValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":48
 *     cdef int _n;
 * 
 *     def __init__(self, innerValue, n, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSTopNSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_n,&__pyx_n_s_groups,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(0, 48, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 48, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_n = values[1];
    __pyx_v_groups = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 48, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_n, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_n, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":49
 * 
 *     def __init__(self, innerValue, n, groups=None):
 *         super(CSTopNSecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 *         self._n = n
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 49, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":50
 *     def __init__(self, innerValue, n, groups=None):
 *         super(CSTopNSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n             # <<<<<<<<<<<<<<
 * 
 *     cdef _cal_impl(self):
 */
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_n); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 50, __pyx_L1_error)
  __pyx_v_self->_n = __pyx_t_4;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":48
 *     cdef int _n;
 * 
 *     def __init__(self, innerValue, n, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSTopNSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":52
 *         self._n = n
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cal_impl", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":53
 * 
 *     cdef _cal_impl(self):
 *         cdef SeriesValues raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 * 
 *         if self._group:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 53, __pyx_L1_error)
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":55
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.top_n(self._n, self._group.value_all())
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 55, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":56
 * 
 *         if self._group:
 *             self.cached = raw_values.top_n(self._n, self._group.value_all())             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values.top_n(self._n)
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 56, __pyx_L1_error)
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->top_n(__pyx_v_raw_values, __pyx_v_self->_n, 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 56, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":55
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.top_n(self._n, self._group.value_all())
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":58
 *             self.cached = raw_values.top_n(self._n, self._group.value_all())
 *         else:
 *             self.cached = raw_values.top_n(self._n)             # <<<<<<<<<<<<<<
 *         self.updated = 1
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->top_n(__pyx_v_raw_values, __pyx_v_self->_n, 0, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 58, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":59
 *         else:
 *             self.cached = raw_values.top_n(self._n)
 *         self.updated = 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":52
 *         self._n = n
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder._cal_impl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":61
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 61, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":62
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":63
 *     cpdef value_all(self):
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":62
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":65
 *             return self.cached
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":66
 *         else:
 *             self._cal_impl()
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":61
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_2value_all[] = "CSTopNSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":68
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 68, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":69
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":70
 *     cpdef double value_by_name(self, name):
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":69
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":72
 *             return self.cached[name]
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 72, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":73
 *         else:
 *             self._cal_impl()
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":68
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_4value_by_name[] = "CSTopNSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":75
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 75, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":76
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             raw_values = raw_values.top_n(self._n, self._group.value_by_names(names))
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":77
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.top_n(self._n, self._group.value_by_names(names))
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 77, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":78
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             raw_values = raw_values.top_n(self._n, self._group.value_by_names(names))             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = raw_values.top_n(self._n)
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._group, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->top_n(__pyx_v_raw_values, __pyx_v_self->_n, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":77
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.top_n(self._n, self._group.value_by_names(names))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":80
 *             raw_values = raw_values.top_n(self._n, self._group.value_by_names(names))
 *         else:
 *             raw_values = raw_values.top_n(self._n)             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->top_n(__pyx_v_raw_values, __pyx_v_self->_n, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 80, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":81
 *         else:
 *             raw_values = raw_values.top_n(self._n)
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":75
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_6value_by_names[] = "CSTopNSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 75, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":83
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSTopN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":84
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSTopN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 84, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":85
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSTopN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSTopN}}({0}, {1})".format(str(self._inner), self._n)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSTopN_0_1_groups_2, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->_n); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 85, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 85, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 85, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_6);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 85, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":84
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSTopN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":87
 *             return "\mathrm{{CSTopN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 *             return "\mathrm{{CSTopN}}({0}, {1})".format(str(self._inner), self._n)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSTopN_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_self->_n); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_9, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 87, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_9, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 87, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 87, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_8, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_8, __pyx_t_6);
      __pyx_t_9 = 0;
      __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 87, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":83
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSTopN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_10__reduce_cython__[] = "CSTopNSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(9); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_4, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 6, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_4, 7, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 8, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_CSTopNSecurityVal); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_129248972);
    __Pyx_GIVEREF(__pyx_int_129248972);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_129248972);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSTopNSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSTopNSecurityVal); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_129248972);
    __Pyx_GIVEREF(__pyx_int_129248972);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_129248972);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSTopNSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_12__setstate_cython__[] = "CSTopNSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSTopNSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSTopNSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSTopNSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSTopNSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":94
 *     cdef double _n;
 * 
 *     def __init__(self, innerValue, n, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSTopNPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_n,&__pyx_n_s_groups,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(0, 94, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 94, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_n = values[1];
    __pyx_v_groups = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 94, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_n, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_n, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  double __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":95
 * 
 *     def __init__(self, innerValue, n, groups=None):
 *         super(CSTopNPercentileSecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 *         self._n = n
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 95, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 95, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 95, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 95, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":96
 *     def __init__(self, innerValue, n, groups=None):
 *         super(CSTopNPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n             # <<<<<<<<<<<<<<
 * 
 *     cdef _cal_impl(self):
 */
  __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_v_n); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 96, __pyx_L1_error)
  __pyx_v_self->_n = __pyx_t_6;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":94
 *     cdef double _n;
 * 
 *     def __init__(self, innerValue, n, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSTopNPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":98
 *         self._n = n
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n_percentile __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cal_impl", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":99
 * 
 *     cdef _cal_impl(self):
 *         cdef SeriesValues raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 * 
 *         if self._group:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 99, __pyx_L1_error)
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":101
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.top_n_percentile(self._n, self._group.value_all())
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 101, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":102
 * 
 *         if self._group:
 *             self.cached = raw_values.top_n_percentile(self._n, self._group.value_all())             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values.top_n_percentile(self._n)
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 102, __pyx_L1_error)
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->top_n_percentile(__pyx_v_raw_values, __pyx_v_self->_n, 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":101
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.top_n_percentile(self._n, self._group.value_all())
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":104
 *             self.cached = raw_values.top_n_percentile(self._n, self._group.value_all())
 *         else:
 *             self.cached = raw_values.top_n_percentile(self._n)             # <<<<<<<<<<<<<<
 *         self.updated = 1
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->top_n_percentile(__pyx_v_raw_values, __pyx_v_self->_n, 0, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":105
 *         else:
 *             self.cached = raw_values.top_n_percentile(self._n)
 *         self.updated = 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":98
 *         self._n = n
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder._cal_impl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":107
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 107, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 107, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":108
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":109
 *     cpdef value_all(self):
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":108
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":111
 *             return self.cached
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":112
 *         else:
 *             self._cal_impl()
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":107
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_2value_all[] = "CSTopNPercentileSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":114
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 114, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 114, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":115
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":116
 *     cpdef double value_by_name(self, name):
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 116, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":115
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":118
 *             return self.cached[name]
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":119
 *         else:
 *             self._cal_impl()
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":114
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_4value_by_name[] = "CSTopNPercentileSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":121
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_top_n_percentile __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 121, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":122
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             raw_values = raw_values.top_n_percentile(self._n, self._group.value_by_names(names))
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":123
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.top_n_percentile(self._n, self._group.value_by_names(names))
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 123, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":124
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             raw_values = raw_values.top_n_percentile(self._n, self._group.value_by_names(names))             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = raw_values.top_n_percentile(self._n)
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._group, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->top_n_percentile(__pyx_v_raw_values, __pyx_v_self->_n, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":123
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.top_n_percentile(self._n, self._group.value_by_names(names))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":126
 *             raw_values = raw_values.top_n_percentile(self._n, self._group.value_by_names(names))
 *         else:
 *             raw_values = raw_values.top_n_percentile(self._n)             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->top_n_percentile(__pyx_v_raw_values, __pyx_v_self->_n, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":127
 *         else:
 *             raw_values = raw_values.top_n_percentile(self._n)
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":121
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_6value_by_names[] = "CSTopNPercentileSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 121, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":129
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSTopNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":130
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSTopNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 130, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":131
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSTopNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSTopNPercentile}}({0}, {1})".format(str(self._inner), self._n)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSTopNPercentile_0_1_gro, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 131, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 131, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->_n); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 131, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 131, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 131, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 131, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_6);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":130
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSTopNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":133
 *             return "\mathrm{{CSTopNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 *             return "\mathrm{{CSTopNPercentile}}({0}, {1})".format(str(self._inner), self._n)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSTopNPercentile_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->_n); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_9, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 133, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_9, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 133, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 133, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_8, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_8, __pyx_t_6);
      __pyx_t_9 = 0;
      __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 133, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":129
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSTopNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_10__reduce_cython__[] = "CSTopNPercentileSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(9); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_4, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 6, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_4, 7, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 8, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_CSTopNPercentileS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_129248972);
    __Pyx_GIVEREF(__pyx_int_129248972);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_129248972);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSTopNPercentileS); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_129248972);
    __Pyx_GIVEREF(__pyx_int_129248972);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_129248972);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_12__setstate_cython__[] = "CSTopNPercentileSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSTopNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":140
 *     cdef int _n;
 * 
 *     def __init__(self, innerValue, n, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSBottomNSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_n,&__pyx_n_s_groups,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(0, 140, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 140, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_n = values[1];
    __pyx_v_groups = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 140, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_n, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_n, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":141
 * 
 *     def __init__(self, innerValue, n, groups=None):
 *         super(CSBottomNSecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 *         self._n = n
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":142
 *     def __init__(self, innerValue, n, groups=None):
 *         super(CSBottomNSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n             # <<<<<<<<<<<<<<
 * 
 *     cdef _cal_impl(self):
 */
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_n); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 142, __pyx_L1_error)
  __pyx_v_self->_n = __pyx_t_4;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":140
 *     cdef int _n;
 * 
 *     def __init__(self, innerValue, n, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSBottomNSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":144
 *         self._n = n
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cal_impl", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":145
 * 
 *     cdef _cal_impl(self):
 *         cdef SeriesValues raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 * 
 *         if self._group:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 145, __pyx_L1_error)
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":147
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.bottom_n(self._n, self._group.value_all())
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 147, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":148
 * 
 *         if self._group:
 *             self.cached = raw_values.bottom_n(self._n, self._group.value_all())             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values.bottom_n(self._n)
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 148, __pyx_L1_error)
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->bottom_n(__pyx_v_raw_values, __pyx_v_self->_n, 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":147
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.bottom_n(self._n, self._group.value_all())
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":150
 *             self.cached = raw_values.bottom_n(self._n, self._group.value_all())
 *         else:
 *             self.cached = raw_values.bottom_n(self._n)             # <<<<<<<<<<<<<<
 *         self.updated = 1
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->bottom_n(__pyx_v_raw_values, __pyx_v_self->_n, 0, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":151
 *         else:
 *             self.cached = raw_values.bottom_n(self._n)
 *         self.updated = 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":144
 *         self._n = n
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder._cal_impl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":153
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 153, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":154
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":155
 *     cpdef value_all(self):
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":154
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":157
 *             return self.cached
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 157, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":158
 *         else:
 *             self._cal_impl()
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":153
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_2value_all[] = "CSBottomNSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":160
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 160, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":161
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":162
 *     cpdef double value_by_name(self, name):
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 162, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":161
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":164
 *             return self.cached[name]
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":165
 *         else:
 *             self._cal_impl()
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":160
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_4value_by_name[] = "CSBottomNSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":167
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 167, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 167, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":168
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             raw_values = raw_values.bottom_n(self._n, self._group.value_by_names(names))
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":169
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.bottom_n(self._n, self._group.value_by_names(names))
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 169, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":170
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             raw_values = raw_values.bottom_n(self._n, self._group.value_by_names(names))             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = raw_values.bottom_n(self._n)
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._group, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->bottom_n(__pyx_v_raw_values, __pyx_v_self->_n, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":169
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.bottom_n(self._n, self._group.value_by_names(names))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":172
 *             raw_values = raw_values.bottom_n(self._n, self._group.value_by_names(names))
 *         else:
 *             raw_values = raw_values.bottom_n(self._n)             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->bottom_n(__pyx_v_raw_values, __pyx_v_self->_n, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":173
 *         else:
 *             raw_values = raw_values.bottom_n(self._n)
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":167
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_6value_by_names[] = "CSBottomNSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 167, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":175
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSBottomN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":176
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSBottomN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 176, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":177
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSBottomN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSBottomN}}({0}, {1})".format(str(self._inner), self._n)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSBottomN_0_1_groups_2, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->_n); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 177, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 177, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 177, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_6);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 177, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":176
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSBottomN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":179
 *             return "\mathrm{{CSBottomN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 *             return "\mathrm{{CSBottomN}}({0}, {1})".format(str(self._inner), self._n)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSBottomN_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_self->_n); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_9, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_9, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_8, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_8, __pyx_t_6);
      __pyx_t_9 = 0;
      __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":175
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSBottomN}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_10__reduce_cython__[] = "CSBottomNSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(9); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_4, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 6, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_4, 7, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 8, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_CSBottomNSecurity); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_129248972);
    __Pyx_GIVEREF(__pyx_int_129248972);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_129248972);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSBottomNSecurity); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_129248972);
    __Pyx_GIVEREF(__pyx_int_129248972);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_129248972);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_12__setstate_cython__[] = "CSBottomNSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSBottomNSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSBottomNSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":186
 *     cdef double _n;
 * 
 *     def __init__(self, innerValue, n, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSBottomNPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_n,&__pyx_n_s_groups,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(0, 186, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 186, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_n = values[1];
    __pyx_v_groups = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 186, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_n, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_n, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  double __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":187
 * 
 *     def __init__(self, innerValue, n, groups=None):
 *         super(CSBottomNPercentileSecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 *         self._n = n
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":188
 *     def __init__(self, innerValue, n, groups=None):
 *         super(CSBottomNPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n             # <<<<<<<<<<<<<<
 * 
 *     cdef _cal_impl(self):
 */
  __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_v_n); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 188, __pyx_L1_error)
  __pyx_v_self->_n = __pyx_t_6;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":186
 *     cdef double _n;
 * 
 *     def __init__(self, innerValue, n, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSBottomNPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 *         self._n = n
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":190
 *         self._n = n
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n_percentile __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cal_impl", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":191
 * 
 *     cdef _cal_impl(self):
 *         cdef SeriesValues raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 * 
 *         if self._group:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 191, __pyx_L1_error)
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":193
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.bottom_n_percentile(self._n, self._group.value_all())
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 193, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":194
 * 
 *         if self._group:
 *             self.cached = raw_values.bottom_n_percentile(self._n, self._group.value_all())             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values.bottom_n_percentile(self._n)
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 194, __pyx_L1_error)
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->bottom_n_percentile(__pyx_v_raw_values, __pyx_v_self->_n, 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":193
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.bottom_n_percentile(self._n, self._group.value_all())
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":196
 *             self.cached = raw_values.bottom_n_percentile(self._n, self._group.value_all())
 *         else:
 *             self.cached = raw_values.bottom_n_percentile(self._n)             # <<<<<<<<<<<<<<
 *         self.updated = 1
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->bottom_n_percentile(__pyx_v_raw_values, __pyx_v_self->_n, 0, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 196, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":197
 *         else:
 *             self.cached = raw_values.bottom_n_percentile(self._n)
 *         self.updated = 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":190
 *         self._n = n
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder._cal_impl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":199
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 199, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 199, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":200
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":201
 *     cpdef value_all(self):
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":200
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":203
 *             return self.cached
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":204
 *         else:
 *             self._cal_impl()
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":199
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_2value_all[] = "CSBottomNPercentileSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":206
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 206, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 206, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":207
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":208
 *     cpdef double value_by_name(self, name):
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":207
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":210
 *             return self.cached[name]
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":211
 *         else:
 *             self._cal_impl()
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":206
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_4value_by_name[] = "CSBottomNPercentileSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":213
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_bottom_n_percentile __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 213, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 213, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":214
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             raw_values = raw_values.bottom_n_percentile(self._n, self._group.value_by_names(names))
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":215
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.bottom_n_percentile(self._n, self._group.value_by_names(names))
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 215, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":216
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             raw_values = raw_values.bottom_n_percentile(self._n, self._group.value_by_names(names))             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = raw_values.bottom_n_percentile(self._n)
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._group, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->bottom_n_percentile(__pyx_v_raw_values, __pyx_v_self->_n, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":215
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.bottom_n_percentile(self._n, self._group.value_by_names(names))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":218
 *             raw_values = raw_values.bottom_n_percentile(self._n, self._group.value_by_names(names))
 *         else:
 *             raw_values = raw_values.bottom_n_percentile(self._n)             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->bottom_n_percentile(__pyx_v_raw_values, __pyx_v_self->_n, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":219
 *         else:
 *             raw_values = raw_values.bottom_n_percentile(self._n)
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":213
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_6value_by_names[] = "CSBottomNPercentileSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 213, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":221
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSBottomNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":222
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSBottomNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 222, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":223
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSBottomNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSBottomNPercentile}}({0}, {1})".format(str(self._inner), self._n)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSBottomNPercentile_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 223, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 223, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->_n); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 223, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 223, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 223, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_4, __pyx_t_5, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 223, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 223, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_6);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 223, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":222
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSBottomNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":225
 *             return "\mathrm{{CSBottomNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 *         else:
 *             return "\mathrm{{CSBottomNPercentile}}({0}, {1})".format(str(self._inner), self._n)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSBottomNPercentile_0_1_2, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->_n); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_9, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 225, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_9, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 225, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 225, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_8, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_8, __pyx_t_6);
      __pyx_t_9 = 0;
      __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 225, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":221
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSBottomNPercentile}}({0}, {1}, groups={2})".format(str(self._inner), self._n, str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_10__reduce_cython__[] = "CSBottomNPercentileSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(9); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_4, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 6, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_4, 7, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 8, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._n, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_CSBottomNPercenti); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_129248972);
    __Pyx_GIVEREF(__pyx_int_129248972);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_129248972);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, None), state
 *     else:
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSBottomNPercenti); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_129248972);
    __Pyx_GIVEREF(__pyx_int_129248972);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_129248972);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_12__setstate_cython__[] = "CSBottomNPercentileSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSBottomNPercentileSecurityValueHolder, (type(self), 0x7b42ecc, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":230
 * cdef class CSRankedSecurityValueHolder(CrossSectionValueHolder):
 * 
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSRankedSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_groups,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 230, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_groups = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 230, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":231
 * 
 *     def __init__(self, innerValue, groups=None):
 *         super(CSRankedSecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef _cal_impl(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":230
 * cdef class CSRankedSecurityValueHolder(CrossSectionValueHolder):
 * 
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSRankedSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":233
 *         super(CSRankedSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_rank __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cal_impl", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":234
 * 
 *     cdef _cal_impl(self):
 *         cdef SeriesValues raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 * 
 *         if self._group:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 234, __pyx_L1_error)
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":236
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.rank(self._group.value_all())
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 236, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":237
 * 
 *         if self._group:
 *             self.cached = raw_values.rank(self._group.value_all())             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values.rank()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 237, __pyx_L1_error)
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->rank(__pyx_v_raw_values, 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":236
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.rank(self._group.value_all())
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":239
 *             self.cached = raw_values.rank(self._group.value_all())
 *         else:
 *             self.cached = raw_values.rank()             # <<<<<<<<<<<<<<
 *         self.updated = 1
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->rank(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":240
 *         else:
 *             self.cached = raw_values.rank()
 *         self.updated = 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":233
 *         super(CSRankedSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder._cal_impl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":242
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 242, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 242, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":243
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":244
 *     cpdef value_all(self):
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":243
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":246
 *             return self.cached
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":247
 *         else:
 *             self._cal_impl()
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":242
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_2value_all[] = "CSRankedSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":249
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 249, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":250
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":251
 *     cpdef double value_by_name(self, name):
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 251, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":250
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":253
 *             return self.cached[name]
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 253, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":254
 *         else:
 *             self._cal_impl()
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 254, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":249
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_4value_by_name[] = "CSRankedSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":256
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_rank __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 256, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":257
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             raw_values = raw_values.rank(self._group.value_by_names(names))
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":258
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.rank(self._group.value_by_names(names))
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 258, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":259
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             raw_values = raw_values.rank(self._group.value_by_names(names))             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = raw_values.rank()
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._group, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->rank(__pyx_v_raw_values, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":258
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.rank(self._group.value_by_names(names))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":261
 *             raw_values = raw_values.rank(self._group.value_by_names(names))
 *         else:
 *             raw_values = raw_values.rank()             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->rank(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":262
 *         else:
 *             raw_values = raw_values.rank()
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":256
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_6value_by_names[] = "CSRankedSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 256, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":264
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSRank}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":265
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSRank}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 265, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":266
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSRank}}({0}, groups={1})".format(str(self._inner), str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSRank}}({0})".format(str(self._inner))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSRank_0_groups_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 266, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 266, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 266, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":265
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSRank}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":268
 *             return "\mathrm{{CSRank}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 *             return "\mathrm{{CSRank}}({0})".format(str(self._inner))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSRank_0, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":264
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSRank}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_10__reduce_cython__[] = "CSRankedSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSRankedSecurityV); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSRankedSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_CSRankedSecurityV); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSRankedSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_12__setstate_cython__[] = "CSRankedSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSRankedSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSRankedSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSRankedSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSRankedSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":272
 * 
 * cdef class CSAverageSecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSAverageSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_groups,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 272, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_groups = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 272, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":273
 * cdef class CSAverageSecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):
 *         super(CSAverageSecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef _cal_impl(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":272
 * 
 * cdef class CSAverageSecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSAverageSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":275
 *         super(CSAverageSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_mean __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cal_impl", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":276
 * 
 *     cdef _cal_impl(self):
 *         cdef SeriesValues raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 * 
 *         if self._group:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 276, __pyx_L1_error)
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":278
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.mean(self._group.value_all())
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 278, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":279
 * 
 *         if self._group:
 *             self.cached = raw_values.mean(self._group.value_all())             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values.mean()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 279, __pyx_L1_error)
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":278
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.mean(self._group.value_all())
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":281
 *             self.cached = raw_values.mean(self._group.value_all())
 *         else:
 *             self.cached = raw_values.mean()             # <<<<<<<<<<<<<<
 *         self.updated = 1
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 281, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":282
 *         else:
 *             self.cached = raw_values.mean()
 *         self.updated = 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":275
 *         super(CSAverageSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder._cal_impl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":284
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 284, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":285
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":286
 *     cpdef value_all(self):
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":285
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":288
 *             return self.cached
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 288, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":289
 *         else:
 *             self._cal_impl()
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":284
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_2value_all[] = "CSAverageSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":291
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 291, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 291, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 291, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":292
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":293
 *     cpdef double value_by_name(self, name):
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":292
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":295
 *             return self.cached[name]
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":296
 *         else:
 *             self._cal_impl()
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":291
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_4value_by_name[] = "CSAverageSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":298
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_mean __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 298, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 298, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":299
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             raw_values = raw_values.mean(self._group.value_by_names(names))
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":300
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.mean(self._group.value_by_names(names))
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 300, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":301
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             raw_values = raw_values.mean(self._group.value_by_names(names))             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = raw_values.mean()
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._group, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":300
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.mean(self._group.value_by_names(names))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":303
 *             raw_values = raw_values.mean(self._group.value_by_names(names))
 *         else:
 *             raw_values = raw_values.mean()             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 303, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":304
 *         else:
 *             raw_values = raw_values.mean()
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":298
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_6value_by_names[] = "CSAverageSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 298, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":306
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSMean}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":307
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSMean}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 307, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":308
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSMean}}({0}, groups={1})".format(str(self._inner), str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSMean}}({0})".format(str(self._inner))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSMean_0_groups_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":307
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSMean}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":310
 *             return "\mathrm{{CSMean}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 *             return "\mathrm{{CSMean}}({0})".format(str(self._inner))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSMean_0, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 310, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 310, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 310, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":306
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSMean}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_10__reduce_cython__[] = "CSAverageSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSAverageSecurity); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSAverageSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_CSAverageSecurity); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSAverageSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_12__setstate_cython__[] = "CSAverageSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSAverageSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSAverageSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSAverageSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSAverageSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":315
 * cdef class CSPercentileSecurityValueHolder(CrossSectionValueHolder):
 * 
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_groups,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 315, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_groups = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 315, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":316
 * 
 *     def __init__(self, innerValue, groups=None):
 *         super(CSPercentileSecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef _cal_impl(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":315
 * cdef class CSPercentileSecurityValueHolder(CrossSectionValueHolder):
 * 
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":318
 *         super(CSPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_percentile __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cal_impl", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":319
 * 
 *     cdef _cal_impl(self):
 *         cdef SeriesValues raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 * 
 *         if self._group:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 319, __pyx_L1_error)
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":321
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.percentile(self._group.value_all())
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 321, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":322
 * 
 *         if self._group:
 *             self.cached = raw_values.percentile(self._group.value_all())             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values.percentile()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 322, __pyx_L1_error)
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->percentile(__pyx_v_raw_values, 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":321
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.percentile(self._group.value_all())
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":324
 *             self.cached = raw_values.percentile(self._group.value_all())
 *         else:
 *             self.cached = raw_values.percentile()             # <<<<<<<<<<<<<<
 *         self.updated = 1
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->percentile(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":325
 *         else:
 *             self.cached = raw_values.percentile()
 *         self.updated = 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":318
 *         super(CSPercentileSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder._cal_impl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":327
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 327, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":328
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":329
 *     cpdef value_all(self):
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":328
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":331
 *             return self.cached
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":332
 *         else:
 *             self._cal_impl()
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":327
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_2value_all[] = "CSPercentileSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":334
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 334, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 334, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":335
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":336
 *     cpdef double value_by_name(self, name):
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":335
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":338
 *             return self.cached[name]
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":339
 *         else:
 *             self._cal_impl()
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 339, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":334
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_4value_by_name[] = "CSPercentileSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":341
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_percentile __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 341, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 341, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 341, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":342
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             raw_values = raw_values.percentile(self._group.value_by_names(names))
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":343
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.percentile(self._group.value_by_names(names))
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 343, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":344
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             raw_values = raw_values.percentile(self._group.value_by_names(names))             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = raw_values.percentile()
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._group, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 344, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->percentile(__pyx_v_raw_values, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 344, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":343
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.percentile(self._group.value_by_names(names))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":346
 *             raw_values = raw_values.percentile(self._group.value_by_names(names))
 *         else:
 *             raw_values = raw_values.percentile()             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->percentile(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 346, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":347
 *         else:
 *             raw_values = raw_values.percentile()
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":341
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_6value_by_names[] = "CSPercentileSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 341, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":349
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSPercentile}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":350
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSPercentile}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 350, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":351
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSPercentile}}({0}, groups={1})".format(str(self._inner), str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSPercentile}}({0})".format(str(self._inner))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSPercentile_0_groups_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":350
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSPercentile}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":353
 *             return "\mathrm{{CSPercentile}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 *             return "\mathrm{{CSPercentile}}({0})".format(str(self._inner))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSPercentile_0, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 353, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 353, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 353, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":349
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSPercentile}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_10__reduce_cython__[] = "CSPercentileSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSPercentileSecur); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_CSPercentileSecur); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_12__setstate_cython__[] = "CSPercentileSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSPercentileSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSPercentileSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":357
 * 
 * cdef class CSAverageAdjustedSecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSAverageAdjustedSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_groups,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 357, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_groups = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 357, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":358
 * cdef class CSAverageAdjustedSecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):
 *         super(CSAverageAdjustedSecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":357
 * 
 * cdef class CSAverageAdjustedSecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSAverageAdjustedSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":360
 *         super(CSAverageAdjustedSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 * 
 *         cdef SeriesValues raw_values
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_mean __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":364
 *         cdef SeriesValues raw_values
 * 
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":365
 * 
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = self._inner.value_all()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":364
 *         cdef SeriesValues raw_values
 * 
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":367
 *             return self.cached
 *         else:
 *             raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 *             if self._group:
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 367, __pyx_L1_error)
    __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":368
 *         else:
 *             raw_values = self._inner.value_all()
 *             if self._group:             # <<<<<<<<<<<<<<
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())
 *             else:
 */
    __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 368, __pyx_L1_error)
    if (__pyx_t_5) {

      /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":369
 *             raw_values = self._inner.value_all()
 *             if self._group:
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())             # <<<<<<<<<<<<<<
 *             else:
 *                 self.cached = raw_values - raw_values.mean()
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 369, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 369, __pyx_L1_error)
      __pyx_t_6.__pyx_n = 1;
      __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
      __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 369, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyNumber_Subtract(((PyObject *)__pyx_v_raw_values), __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 369, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 369, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
      __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":368
 *         else:
 *             raw_values = self._inner.value_all()
 *             if self._group:             # <<<<<<<<<<<<<<
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())
 *             else:
 */
      goto __pyx_L4;
    }

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":371
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())
 *             else:
 *                 self.cached = raw_values - raw_values.mean()             # <<<<<<<<<<<<<<
 *             self.updated = 1
 *             return self.cached
 */
    /*else*/ {
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = PyNumber_Subtract(((PyObject *)__pyx_v_raw_values), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
      __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L4:;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":372
 *             else:
 *                 self.cached = raw_values - raw_values.mean()
 *             self.updated = 1             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
    __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":373
 *                 self.cached = raw_values - raw_values.mean()
 *             self.updated = 1
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":360
 *         super(CSAverageAdjustedSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 * 
 *         cdef SeriesValues raw_values
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_2value_all[] = "CSAverageAdjustedSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":375
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 * 
 *         cdef SeriesValues raw_values
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_mean __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 375, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 375, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":379
 *         cdef SeriesValues raw_values
 * 
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":380
 * 
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = self._inner.value_all()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":379
 *         cdef SeriesValues raw_values
 * 
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":382
 *             return self.cached[name]
 *         else:
 *             raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 *             if self._group:
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 382, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 382, __pyx_L1_error)
    __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":383
 *         else:
 *             raw_values = self._inner.value_all()
 *             if self._group:             # <<<<<<<<<<<<<<
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())
 *             else:
 */
    __pyx_t_6 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 383, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":384
 *             raw_values = self._inner.value_all()
 *             if self._group:
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())             # <<<<<<<<<<<<<<
 *             else:
 *                 self.cached = raw_values - raw_values.mean()
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 384, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 384, __pyx_L1_error)
      __pyx_t_7.__pyx_n = 1;
      __pyx_t_7.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
      __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, &__pyx_t_7)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 384, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyNumber_Subtract(((PyObject *)__pyx_v_raw_values), __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 384, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 384, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
      __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":383
 *         else:
 *             raw_values = self._inner.value_all()
 *             if self._group:             # <<<<<<<<<<<<<<
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())
 *             else:
 */
      goto __pyx_L4;
    }

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":386
 *                 self.cached = raw_values - raw_values.mean(self._group.value_all())
 *             else:
 *                 self.cached = raw_values - raw_values.mean()             # <<<<<<<<<<<<<<
 *             self.updated = 1
 *             return self.cached[name]
 */
    /*else*/ {
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = PyNumber_Subtract(((PyObject *)__pyx_v_raw_values), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 386, __pyx_L1_error)
      __Pyx_GIVEREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
      __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L4:;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":387
 *             else:
 *                 self.cached = raw_values - raw_values.mean()
 *             self.updated = 1             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
    __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":388
 *                 self.cached = raw_values - raw_values.mean()
 *             self.updated = 1
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 388, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":375
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 * 
 *         cdef SeriesValues raw_values
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_4value_by_name[] = "CSAverageAdjustedSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":390
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = NULL;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_mean __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 390, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 390, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":391
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             self.cached = raw_values - raw_values.mean(self._group.value)
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":392
 *     cpdef SeriesValues value_by_names(self, list names):
 *         raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values - raw_values.mean(self._group.value)
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 392, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":393
 *         raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             self.cached = raw_values - raw_values.mean(self._group.value)             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values - raw_values.mean()
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->__pyx_base._group), __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 393, __pyx_L1_error)
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Subtract(((PyObject *)__pyx_v_raw_values), __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":392
 *     cpdef SeriesValues value_by_names(self, list names):
 *         raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values - raw_values.mean(self._group.value)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":395
 *             self.cached = raw_values - raw_values.mean(self._group.value)
 *         else:
 *             self.cached = raw_values - raw_values.mean()             # <<<<<<<<<<<<<<
 *         return raw_values[names]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->mean(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_Subtract(((PyObject *)__pyx_v_raw_values), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 395, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_2);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":396
 *         else:
 *             self.cached = raw_values - raw_values.mean()
 *         return raw_values[names]             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_raw_values), __pyx_v_names); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 396, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":390
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_6value_by_names[] = "CSAverageAdjustedSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 390, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":398
 *         return raw_values[names]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSMeanAdjusted}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":399
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSMeanAdjusted}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 399, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":400
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSMeanAdjusted}}({0}, groups={1})".format(str(self._inner), str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSMeanAdjusted}}({0})".format(str(self._inner))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSMeanAdjusted_0_groups, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 400, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":399
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSMeanAdjusted}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":402
 *             return "\mathrm{{CSMeanAdjusted}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 *             return "\mathrm{{CSMeanAdjusted}}({0})".format(str(self._inner))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSMeanAdjusted_0, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":398
 *         return raw_values[names]
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSMeanAdjusted}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_10__reduce_cython__[] = "CSAverageAdjustedSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSAverageAdjusted); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_CSAverageAdjusted); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_12__setstate_cython__[] = "CSAverageAdjustedSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSAverageAdjustedSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":406
 * 
 * cdef class CSZScoreSecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSZScoreSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_groups,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 406, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_groups = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 406, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":407
 * cdef class CSZScoreSecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):
 *         super(CSZScoreSecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef _cal_impl(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 407, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":406
 * 
 * cdef class CSZScoreSecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSZScoreSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":409
 *         super(CSZScoreSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_zscore __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cal_impl", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":410
 * 
 *     cdef _cal_impl(self):
 *         cdef SeriesValues raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 * 
 *         if self._group:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 410, __pyx_L1_error)
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":412
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.zscore(self._group.value_all())
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 412, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":413
 * 
 *         if self._group:
 *             self.cached = raw_values.zscore(self._group.value_all())             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values.zscore()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 413, __pyx_L1_error)
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->zscore(__pyx_v_raw_values, 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":412
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.zscore(self._group.value_all())
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":415
 *             self.cached = raw_values.zscore(self._group.value_all())
 *         else:
 *             self.cached = raw_values.zscore()             # <<<<<<<<<<<<<<
 *         self.updated = 1
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->zscore(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 415, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":416
 *         else:
 *             self.cached = raw_values.zscore()
 *         self.updated = 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":409
 *         super(CSZScoreSecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder._cal_impl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":418
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 418, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 418, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":419
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":420
 *     cpdef value_all(self):
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":419
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":422
 *             return self.cached
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":423
 *         else:
 *             self._cal_impl()
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":418
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_2value_all[] = "CSZScoreSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":425
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 425, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":426
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":427
 *     cpdef double value_by_name(self, name):
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 427, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 427, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":426
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":429
 *             return self.cached[name]
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":430
 *         else:
 *             self._cal_impl()
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":425
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_4value_by_name[] = "CSZScoreSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":432
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_zscore __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 432, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 432, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":433
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             raw_values = raw_values.zscore(self._group.value_by_names(names))
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":434
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.zscore(self._group.value_by_names(names))
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 434, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":435
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             raw_values = raw_values.zscore(self._group.value_by_names(names))             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = raw_values.zscore()
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._group, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->zscore(__pyx_v_raw_values, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":434
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.zscore(self._group.value_by_names(names))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":437
 *             raw_values = raw_values.zscore(self._group.value_by_names(names))
 *         else:
 *             raw_values = raw_values.zscore()             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->zscore(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":438
 *         else:
 *             raw_values = raw_values.zscore()
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":432
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_6value_by_names[] = "CSZScoreSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 432, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":440
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSZscore}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":441
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSZscore}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 441, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":442
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSZscore}}({0}, groups={1})".format(str(self._inner), str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSZscore}}({0})".format(str(self._inner))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSZscore_0_groups_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 442, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":441
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSZscore}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":444
 *             return "\mathrm{{CSZscore}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 *             return "\mathrm{{CSZscore}}({0})".format(str(self._inner))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSZscore_0, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 444, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 444, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":440
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSZscore}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_10__reduce_cython__[] = "CSZScoreSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSZScoreSecurityV); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_CSZScoreSecurityV); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_12__setstate_cython__[] = "CSZScoreSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSZScoreSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSZScoreSecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":448
 * 
 * cdef class CSFillNASecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSFillNASecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_innerValue = 0;
  PyObject *__pyx_v_groups = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_innerValue,&__pyx_n_s_groups,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_innerValue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 448, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_innerValue = values[0];
    __pyx_v_groups = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 448, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v_self), __pyx_v_innerValue, __pyx_v_groups);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_innerValue, PyObject *__pyx_v_groups) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":449
 * cdef class CSFillNASecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):
 *         super(CSFillNASecurityValueHolder, self).__init__(innerValue, groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef _cal_impl(self):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_innerValue, __pyx_v_groups};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_innerValue);
    __Pyx_GIVEREF(__pyx_v_innerValue);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_innerValue);
    __Pyx_INCREF(__pyx_v_groups);
    __Pyx_GIVEREF(__pyx_v_groups);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_groups);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":448
 * 
 * cdef class CSFillNASecurityValueHolder(CrossSectionValueHolder):
 *     def __init__(self, innerValue, groups=None):             # <<<<<<<<<<<<<<
 *         super(CSFillNASecurityValueHolder, self).__init__(innerValue, groups)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":451
 *         super(CSFillNASecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder__cal_impl(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_fillna __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cal_impl", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":452
 * 
 *     cdef _cal_impl(self):
 *         cdef SeriesValues raw_values = self._inner.value_all()             # <<<<<<<<<<<<<<
 * 
 *         if self._group:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._inner, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 452, __pyx_L1_error)
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":454
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.fillna(self._group.value_all())
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 454, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":455
 * 
 *         if self._group:
 *             self.cached = raw_values.fillna(self._group.value_all())             # <<<<<<<<<<<<<<
 *         else:
 *             self.cached = raw_values.fillna()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_all(__pyx_v_self->__pyx_base._group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 455, __pyx_L1_error)
    __pyx_t_4.__pyx_n = 1;
    __pyx_t_4.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->fillna(__pyx_v_raw_values, 0, &__pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":454
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 *         if self._group:             # <<<<<<<<<<<<<<
 *             self.cached = raw_values.fillna(self._group.value_all())
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":457
 *             self.cached = raw_values.fillna(self._group.value_all())
 *         else:
 *             self.cached = raw_values.fillna()             # <<<<<<<<<<<<<<
 *         self.updated = 1
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->fillna(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 457, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_v_self->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":458
 *         else:
 *             self.cached = raw_values.fillna()
 *         self.updated = 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.__pyx_base.updated = 1;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":451
 *         super(CSFillNASecurityValueHolder, self).__init__(innerValue, groups)
 * 
 *     cdef _cal_impl(self):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_all()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder._cal_impl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":460
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 460, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_3value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 460, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":461
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":462
 *     cpdef value_all(self):
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":461
 * 
 *     cpdef value_all(self):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":464
 *             return self.cached
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":465
 *         else:
 *             self._cal_impl()
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":460
 *         self.updated = 1
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_2value_all[] = "CSFillNASecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_3value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_2value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_2value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":467
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 467, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_5value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 467, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 467, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":468
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":469
 *     cpdef double value_by_name(self, name):
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             self._cal_impl()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 469, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 469, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":468
 * 
 *     cpdef double value_by_name(self, name):
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":471
 *             return self.cached[name]
 *         else:
 *             self._cal_impl()             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_cal_impl(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 471, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":472
 *         else:
 *             self._cal_impl()
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":467
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 *         if self.updated:
 *             return self.cached[name]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_4value_by_name[] = "CSFillNASecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_5value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_4value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_4value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":474
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  struct __pyx_opt_args_6ultron_6sentry_8Analysis_12SeriesValues_12SeriesValues_fillna __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 474, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_7value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 474, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 474, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":475
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)             # <<<<<<<<<<<<<<
 *         if self._group:
 *             raw_values = raw_values.fillna(self._group.value_by_names(names))
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._inner->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._inner, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":476
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.fillna(self._group.value_by_names(names))
 *         else:
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 476, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":477
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 *             raw_values = raw_values.fillna(self._group.value_by_names(names))             # <<<<<<<<<<<<<<
 *         else:
 *             raw_values = raw_values.fillna()
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->__pyx_base._group->__pyx_vtab)->value_by_names(__pyx_v_self->__pyx_base._group, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 477, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.groups = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->fillna(__pyx_v_raw_values, 0, &__pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 477, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":476
 *     cpdef SeriesValues value_by_names(self, list names):
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:             # <<<<<<<<<<<<<<
 *             raw_values = raw_values.fillna(self._group.value_by_names(names))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":479
 *             raw_values = raw_values.fillna(self._group.value_by_names(names))
 *         else:
 *             raw_values = raw_values.fillna()             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_raw_values->__pyx_vtab)->fillna(__pyx_v_raw_values, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_raw_values, ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2));
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":480
 *         else:
 *             raw_values = raw_values.fillna()
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":474
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues raw_values = self._inner.value_by_names(names)
 *         if self._group:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_6value_by_names[] = "CSFillNASecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_7value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 474, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_6value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_6value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":482
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSFillNA}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_9__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_9__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_8__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_8__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":483
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSFillNA}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 483, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":484
 *     def __str__(self):
 *         if self._group:
 *             return "\mathrm{{CSFillNA}}({0}, groups={1})".format(str(self._inner), str(self._group))             # <<<<<<<<<<<<<<
 *         else:
 *             return "\mathrm{{CSFillNA}}({0})".format(str(self._inner))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSFillNA_0_groups_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 484, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 484, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._group)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 484, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_7 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 484, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 484, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 484, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_5);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 484, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":483
 * 
 *     def __str__(self):
 *         if self._group:             # <<<<<<<<<<<<<<
 *             return "\mathrm{{CSFillNA}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":486
 *             return "\mathrm{{CSFillNA}}({0}, groups={1})".format(str(self._inner), str(self._group))
 *         else:
 *             return "\mathrm{{CSFillNA}}({0})".format(str(self._inner))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSFillNA_0, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->__pyx_base._inner)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_5, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":482
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         if self._group:
 *             return "\mathrm{{CSFillNA}}({0}, groups={1})".format(str(self._inner), str(self._group))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_10__reduce_cython__[] = "CSFillNASecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_10__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_10__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.__pyx_base.updated); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base.__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._group));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._group));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._inner));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->__pyx_base._inner));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_v_self->__pyx_base.__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._group, self._holderTemplate, self._inner, self._innerHolders, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._group) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base._holderTemplate) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._inner) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->__pyx_base.__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.cached) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSFillNASecurityV); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._group is not None or self._holderTemplate is not None or self._inner is not None or self._innerHolders is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, None), state
 *     else:
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSFillNASecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_CSFillNASecurityV); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_37815087);
    __Pyx_GIVEREF(__pyx_int_37815087);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_37815087);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSFillNASecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_12__setstate_cython__[] = "CSFillNASecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_12__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_12__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSFillNASecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSFillNASecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSFillNASecurityValueHolder, (type(self), 0x241032f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSFillNASecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":494
 *     cdef public SecurityValueHolder _right
 * 
 *     def __init__(self, left, right):             # <<<<<<<<<<<<<<
 *         super(CSResidueSecurityValueHolder, self).__init__()
 *         self._left = build_holder(left)
 */

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_left = 0;
  PyObject *__pyx_v_right = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_left,&__pyx_n_s_right,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_left)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_right)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 494, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 494, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_left = values[0];
    __pyx_v_right = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 494, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder___init__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self), __pyx_v_left, __pyx_v_right);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder___init__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_left, PyObject *__pyx_v_right) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":495
 * 
 *     def __init__(self, left, right):
 *         super(CSResidueSecurityValueHolder, self).__init__()             # <<<<<<<<<<<<<<
 *         self._left = build_holder(left)
 *         self._right = build_holder(right)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":496
 *     def __init__(self, left, right):
 *         super(CSResidueSecurityValueHolder, self).__init__()
 *         self._left = build_holder(left)             # <<<<<<<<<<<<<<
 *         self._right = build_holder(right)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_build_holder); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_left) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_left);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_left);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_left));
  __pyx_v_self->_left = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":497
 *         super(CSResidueSecurityValueHolder, self).__init__()
 *         self._left = build_holder(left)
 *         self._right = build_holder(right)             # <<<<<<<<<<<<<<
 * 
 *         self._window = max(self._left.window, self._right.window)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_build_holder); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_right) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_right);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_right);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_right));
  __pyx_v_self->_right = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":499
 *         self._right = build_holder(right)
 * 
 *         self._window = max(self._left.window, self._right.window)             # <<<<<<<<<<<<<<
 *         self._dependency = list(set(self._left.fields + self._right.fields))
 *         self.updated = 0
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_right), __pyx_n_s_window); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_left), __pyx_n_s_window); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 499, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_5) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = __pyx_t_1;
  } else {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = __pyx_t_2;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_self->__pyx_base._window = __pyx_t_6;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":500
 * 
 *         self._window = max(self._left.window, self._right.window)
 *         self._dependency = list(set(self._left.fields + self._right.fields))             # <<<<<<<<<<<<<<
 *         self.updated = 0
 *         self.cached = None
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_left), __pyx_n_s_fields); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_right), __pyx_n_s_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PySet_New(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PySequence_List(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v_self->__pyx_base._dependency);
  __pyx_v_self->__pyx_base._dependency = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":501
 *         self._window = max(self._left.window, self._right.window)
 *         self._dependency = list(set(self._left.fields + self._right.fields))
 *         self.updated = 0             # <<<<<<<<<<<<<<
 *         self.cached = None
 * 
 */
  __pyx_v_self->__pyx_base.updated = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":502
 *         self._dependency = list(set(self._left.fields + self._right.fields))
 *         self.updated = 0
 *         self.cached = None             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
  __pyx_v_self->__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)Py_None);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":494
 *     cdef public SecurityValueHolder _right
 * 
 *     def __init__(self, left, right):             # <<<<<<<<<<<<<<
 *         super(CSResidueSecurityValueHolder, self).__init__()
 *         self._left = build_holder(left)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":505
 * 
 *     @property
 *     def symbolList(self):             # <<<<<<<<<<<<<<
 *         return self._left.symbolList
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_10symbolList_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_10symbolList_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_10symbolList___get__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_10symbolList___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":506
 *     @property
 *     def symbolList(self):
 *         return self._left.symbolList             # <<<<<<<<<<<<<<
 * 
 *     cpdef push(self, dict data):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_left), __pyx_n_s_symbolList); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":505
 * 
 *     @property
 *     def symbolList(self):             # <<<<<<<<<<<<<<
 *         return self._left.symbolList
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.symbolList.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":508
 *         return self._left.symbolList
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._left.push(data)
 *         self._right.push(data)
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_push(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_push); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_3push)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 508, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":509
 * 
 *     cpdef push(self, dict data):
 *         self._left.push(data)             # <<<<<<<<<<<<<<
 *         self._right.push(data)
 *         self.updated = 0
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_left), __pyx_n_s_push); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_data);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":510
 *     cpdef push(self, dict data):
 *         self._left.push(data)
 *         self._right.push(data)             # <<<<<<<<<<<<<<
 *         self.updated = 0
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_right), __pyx_n_s_push); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_data);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":511
 *         self._left.push(data)
 *         self._right.push(data)
 *         self.updated = 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef value_all(self):
 */
  __pyx_v_self->__pyx_base.updated = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":508
 *         return self._left.symbolList
 * 
 *     cpdef push(self, dict data):             # <<<<<<<<<<<<<<
 *         self._left.push(data)
 *         self._right.push(data)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_2push[] = "CSResidueSecurityValueHolder.push(self, dict data)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_3push(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyDict_Type), 1, "data", 1))) __PYX_ERR(0, 508, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_2push(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_2push(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_push(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":513
 *         self.updated = 0
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 * 
 *         cdef SeriesValues left_raw_values
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_left_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_right_raw_values = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 513, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5value_all)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 513, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":518
 *         cdef SeriesValues right_raw_values
 * 
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.updated != 0);
  if (__pyx_t_5) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":519
 * 
 *         if self.updated:
 *             return self.cached             # <<<<<<<<<<<<<<
 *         else:
 *             left_raw_values = self._left.value_all()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.cached);
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":518
 *         cdef SeriesValues right_raw_values
 * 
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":521
 *             return self.cached
 *         else:
 *             left_raw_values = self._left.value_all()             # <<<<<<<<<<<<<<
 *             right_raw_values = self._right.value_all()
 *             self.cached = left_raw_values.res(right_raw_values)
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->_left->__pyx_vtab)->value_all(__pyx_v_self->_left, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 521, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 521, __pyx_L1_error)
    __pyx_v_left_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":522
 *         else:
 *             left_raw_values = self._left.value_all()
 *             right_raw_values = self._right.value_all()             # <<<<<<<<<<<<<<
 *             self.cached = left_raw_values.res(right_raw_values)
 *             self.updated = 1
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->_right->__pyx_vtab)->value_all(__pyx_v_self->_right, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 522, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 522, __pyx_L1_error)
    __pyx_v_right_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":523
 *             left_raw_values = self._left.value_all()
 *             right_raw_values = self._right.value_all()
 *             self.cached = left_raw_values.res(right_raw_values)             # <<<<<<<<<<<<<<
 *             self.updated = 1
 *             return self.cached
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_left_raw_values->__pyx_vtab)->res(__pyx_v_left_raw_values, __pyx_v_right_raw_values, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
    __pyx_v_self->__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":524
 *             right_raw_values = self._right.value_all()
 *             self.cached = left_raw_values.res(right_raw_values)
 *             self.updated = 1             # <<<<<<<<<<<<<<
 *             return self.cached
 * 
 */
    __pyx_v_self->__pyx_base.updated = 1;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":525
 *             self.cached = left_raw_values.res(right_raw_values)
 *             self.updated = 1
 *             return self.cached             # <<<<<<<<<<<<<<
 * 
 *     cpdef double value_by_name(self, name):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
    __pyx_r = ((PyObject *)__pyx_v_self->__pyx_base.cached);
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":513
 *         self.updated = 0
 * 
 *     cpdef value_all(self):             # <<<<<<<<<<<<<<
 * 
 *         cdef SeriesValues left_raw_values
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_left_raw_values);
  __Pyx_XDECREF((PyObject *)__pyx_v_right_raw_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_4value_all[] = "CSResidueSecurityValueHolder.value_all(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5value_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_all (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_4value_all(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_4value_all(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_all", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_all(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.value_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":527
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 * 
 *         cdef SeriesValues left_raw_values
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_7value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static double __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_left_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_right_raw_values = 0;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 527, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_7value_by_name)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 527, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 527, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":532
 *         cdef SeriesValues right_raw_values
 * 
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  __pyx_t_6 = (__pyx_v_self->__pyx_base.updated != 0);
  if (__pyx_t_6) {

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":533
 * 
 *         if self.updated:
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 *         else:
 *             left_raw_values = self._left.value_all()
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":532
 *         cdef SeriesValues right_raw_values
 * 
 *         if self.updated:             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 *         else:
 */
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":535
 *             return self.cached[name]
 *         else:
 *             left_raw_values = self._left.value_all()             # <<<<<<<<<<<<<<
 *             right_raw_values = self._right.value_all()
 *             self.cached = left_raw_values.res(right_raw_values)
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->_left->__pyx_vtab)->value_all(__pyx_v_self->_left, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 535, __pyx_L1_error)
    __pyx_v_left_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":536
 *         else:
 *             left_raw_values = self._left.value_all()
 *             right_raw_values = self._right.value_all()             # <<<<<<<<<<<<<<
 *             self.cached = left_raw_values.res(right_raw_values)
 *             self.updated = 1
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->_right->__pyx_vtab)->value_all(__pyx_v_self->_right, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 536, __pyx_L1_error)
    __pyx_v_right_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":537
 *             left_raw_values = self._left.value_all()
 *             right_raw_values = self._right.value_all()
 *             self.cached = left_raw_values.res(right_raw_values)             # <<<<<<<<<<<<<<
 *             self.updated = 1
 *             return self.cached[name]
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_left_raw_values->__pyx_vtab)->res(__pyx_v_left_raw_values, __pyx_v_right_raw_values, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 537, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base.cached);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
    __pyx_v_self->__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":538
 *             right_raw_values = self._right.value_all()
 *             self.cached = left_raw_values.res(right_raw_values)
 *             self.updated = 1             # <<<<<<<<<<<<<<
 *             return self.cached[name]
 * 
 */
    __pyx_v_self->__pyx_base.updated = 1;

    /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":539
 *             self.cached = left_raw_values.res(right_raw_values)
 *             self.updated = 1
 *             return self.cached[name]             # <<<<<<<<<<<<<<
 * 
 *     cpdef SeriesValues value_by_names(self, list names):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->__pyx_base.cached), __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":527
 *             return self.cached
 * 
 *     cpdef double value_by_name(self, name):             # <<<<<<<<<<<<<<
 * 
 *         cdef SeriesValues left_raw_values
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_left_raw_values);
  __Pyx_XDECREF((PyObject *)__pyx_v_right_raw_values);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_7value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6value_by_name[] = "CSResidueSecurityValueHolder.value_by_name(self, name) -> double";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_7value_by_name(PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_name (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6value_by_name(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6value_by_name(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_by_name(__pyx_v_self, __pyx_v_name, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.value_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":541
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues left_raw_values
 *         cdef SeriesValues right_raw_values
 */

static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_9value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names, int __pyx_skip_dispatch) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_left_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_right_raw_values = 0;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_v_raw_values = NULL;
  struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_value_by_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_9value_by_names)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_names) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_names);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(0, 541, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":544
 *         cdef SeriesValues left_raw_values
 *         cdef SeriesValues right_raw_values
 *         left_raw_values = self._left.value_by_names(names)             # <<<<<<<<<<<<<<
 *         right_raw_values = self._right.value_by_names(names)
 *         raw_values = left_raw_values.res(right_raw_values)
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->_left->__pyx_vtab)->value_by_names(__pyx_v_self->_left, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 544, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_left_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":545
 *         cdef SeriesValues right_raw_values
 *         left_raw_values = self._left.value_by_names(names)
 *         right_raw_values = self._right.value_by_names(names)             # <<<<<<<<<<<<<<
 *         raw_values = left_raw_values.res(right_raw_values)
 *         return raw_values
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_v_self->_right->__pyx_vtab)->value_by_names(__pyx_v_self->_right, __pyx_v_names, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_right_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":546
 *         left_raw_values = self._left.value_by_names(names)
 *         right_raw_values = self._right.value_by_names(names)
 *         raw_values = left_raw_values.res(right_raw_values)             # <<<<<<<<<<<<<<
 *         return raw_values
 * 
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_v_left_raw_values->__pyx_vtab)->res(__pyx_v_left_raw_values, __pyx_v_right_raw_values, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_raw_values = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":547
 *         right_raw_values = self._right.value_by_names(names)
 *         raw_values = left_raw_values.res(right_raw_values)
 *         return raw_values             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_raw_values));
  __pyx_r = __pyx_v_raw_values;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":541
 *             return self.cached[name]
 * 
 *     cpdef SeriesValues value_by_names(self, list names):             # <<<<<<<<<<<<<<
 *         cdef SeriesValues left_raw_values
 *         cdef SeriesValues right_raw_values
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_left_raw_values);
  __Pyx_XDECREF((PyObject *)__pyx_v_right_raw_values);
  __Pyx_XDECREF((PyObject *)__pyx_v_raw_values);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_9value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_8value_by_names[] = "CSResidueSecurityValueHolder.value_by_names(self, list names) -> SeriesValues";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_9value_by_names(PyObject *__pyx_v_self, PyObject *__pyx_v_names) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value_by_names (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_names), (&PyList_Type), 1, "names", 1))) __PYX_ERR(0, 541, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_8value_by_names(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self), ((PyObject*)__pyx_v_names));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_8value_by_names(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_names) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value_by_names", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_by_names(__pyx_v_self, __pyx_v_names, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.value_by_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":549
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\mathrm{{CSRes}}({0}, {1})".format(str(self._left), str(self._right))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_11__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_11__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_10__str__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_10__str__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":550
 * 
 *     def __str__(self):
 *         return "\mathrm{{CSRes}}({0}, {1})".format(str(self._left), str(self._right))             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_mathrm_CSRes_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->_left)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self->_right)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 550, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 550, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 550, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 550, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":549
 *         return raw_values
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "\mathrm{{CSRes}}({0}, {1})".format(str(self._left), str(self._right))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":491
 * cdef class CSResidueSecurityValueHolder(SecurityValueHolder):
 * 
 *     cdef public SecurityValueHolder _left             # <<<<<<<<<<<<<<
 *     cdef public SecurityValueHolder _right
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left___get__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_left));
  __pyx_r = ((PyObject *)__pyx_v_self->_left);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_2__set__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_2__set__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(0, 491, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_left);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_left));
  __pyx_v_self->_left = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder._left.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_4__del__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_4__del__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_left);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_left));
  __pyx_v_self->_left = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":492
 * 
 *     cdef public SecurityValueHolder _left
 *     cdef public SecurityValueHolder _right             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, left, right):
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right___get__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right___get__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_right));
  __pyx_r = ((PyObject *)__pyx_v_self->_right);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_2__set__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_2__set__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(0, 492, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_right);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_right));
  __pyx_v_self->_right = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder._right.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_4__del__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_4__del__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_right);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_right));
  __pyx_v_self->_right = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_12__reduce_cython__[] = "CSResidueSecurityValueHolder.__reduce_cython__(self)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_12__reduce_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_12__reduce_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._dependency, self._holderTemplate, self._innerHolders, self._left, self._right, self._window, self.cached, self.updated)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base._window); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.updated); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._dependency);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._dependency);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->__pyx_base._dependency);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._holderTemplate));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._holderTemplate));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->__pyx_base._holderTemplate));
  __Pyx_INCREF(__pyx_v_self->__pyx_base._innerHolders);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._innerHolders);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_self->__pyx_base._innerHolders);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_left));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_left));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->_left));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_right));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_right));
  PyTuple_SET_ITEM(__pyx_t_3, 4, ((PyObject *)__pyx_v_self->_right));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.cached));
  PyTuple_SET_ITEM(__pyx_t_3, 6, ((PyObject *)__pyx_v_self->__pyx_base.cached));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._dependency, self._holderTemplate, self._innerHolders, self._left, self._right, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._dependency, self._holderTemplate, self._innerHolders, self._left, self._right, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._dependency is not None or self._holderTemplate is not None or self._innerHolders is not None or self._left is not None or self._right is not None or self.cached is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._dependency, self._holderTemplate, self._innerHolders, self._left, self._right, self._window, self.cached, self.updated)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._dependency is not None or self._holderTemplate is not None or self._innerHolders is not None or self._left is not None or self._right is not None or self.cached is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->__pyx_base._dependency != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base._holderTemplate) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->__pyx_base._innerHolders != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->_left) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->_right) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.cached) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._holderTemplate is not None or self._innerHolders is not None or self._left is not None or self._right is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._dependency is not None or self._holderTemplate is not None or self._innerHolders is not None or self._left is not None or self._right is not None or self.cached is not None
 *     if use_setstate:
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_CSResidueSecurity); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_237487140);
    __Pyx_GIVEREF(__pyx_int_237487140);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_237487140);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._dependency is not None or self._holderTemplate is not None or self._innerHolders is not None or self._left is not None or self._right is not None or self.cached is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, None), state
 *     else:
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSResidueSecurityValueHolder__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_CSResidueSecurity); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_237487140);
    __Pyx_GIVEREF(__pyx_int_237487140);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_237487140);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSResidueSecurityValueHolder__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_14__setstate_cython__[] = "CSResidueSecurityValueHolder.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_14__setstate_cython__(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_14__setstate_cython__(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CSResidueSecurityValueHolder__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSResidueSecurityValueHolder__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_CSResidueSecurityValueHolder, (type(self), 0xe27c424, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CSResidueSecurityValueHolder__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CrossSectionValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_1__pyx_unpickle_CrossSectionValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CrossSectionValueHolder[] = "__pyx_unpickle_CrossSectionValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_1__pyx_unpickle_CrossSectionValueHolder = {"__pyx_unpickle_CrossSectionValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_1__pyx_unpickle_CrossSectionValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CrossSectionValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_1__pyx_unpickle_CrossSectionValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CrossSectionValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CrossSectionValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CrossSectionValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CrossSectionValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CrossSectionValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CrossSectionValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CrossSectionValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CrossSectionValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CrossSectionValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x241032f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CrossSectionValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CrossSectionValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x24, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CrossSectionValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CrossSectionValueHolder__set_state(<CrossSectionValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CrossSectionValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CrossSectionValueHolder__set_state(<CrossSectionValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CrossSectionValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CrossSectionValueHolder__set_state(<CrossSectionValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CrossSectionValueHolder__set_state(CrossSectionValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CrossSectionValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CrossSectionValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CrossSectionValueHolder__set_state(<CrossSectionValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CrossSectionValueHolder__set_state(<CrossSectionValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CrossSectionValueHolder__set_state(CrossSectionValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CrossSectionValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CrossSectionValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CrossSectionValueHolder__set_state(<CrossSectionValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CrossSectionValueHolder__set_state(CrossSectionValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CrossSectionValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CrossSectionValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CrossSectionValueHolder__set_state(CrossSectionValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_group));
  __pyx_v___pyx_result->_group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_inner));
  __pyx_v___pyx_result->_inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CrossSectionValueHolder__set_state(CrossSectionValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 8) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CrossSectionValueHolder__set_state(CrossSectionValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CrossSectionValueHolder__set_state(<CrossSectionValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CrossSectionValueHolder__set_state(CrossSectionValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CrossSectionValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSTopNSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_3__pyx_unpickle_CSTopNSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_2__pyx_unpickle_CSTopNSecurityValueHolder[] = "__pyx_unpickle_CSTopNSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_3__pyx_unpickle_CSTopNSecurityValueHolder = {"__pyx_unpickle_CSTopNSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_3__pyx_unpickle_CSTopNSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_2__pyx_unpickle_CSTopNSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_3__pyx_unpickle_CSTopNSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSTopNSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSTopNSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSTopNSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSTopNSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSTopNSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSTopNSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_2__pyx_unpickle_CSTopNSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_2__pyx_unpickle_CSTopNSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSTopNSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7b42ecc) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSTopNSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7b42ecc:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSTopNSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7b, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSTopNSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSTopNSecurityValueHolder__set_state(<CSTopNSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSTopNSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSTopNSecurityValueHolder__set_state(<CSTopNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSTopNSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSTopNSecurityValueHolder__set_state(<CSTopNSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSTopNSecurityValueHolder__set_state(CSTopNSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSTopNSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSTopNSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSTopNSecurityValueHolder__set_state(<CSTopNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSTopNSecurityValueHolder__set_state(<CSTopNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSTopNSecurityValueHolder__set_state(CSTopNSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSTopNSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSTopNSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSTopNSecurityValueHolder__set_state(<CSTopNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSTopNSecurityValueHolder__set_state(CSTopNSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSTopNSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSTopNSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSTopNSecurityValueHolder__set_state(CSTopNSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_n = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSTopNSecurityValueHolder__set_state(CSTopNSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 9) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSTopNSecurityValueHolder__set_state(CSTopNSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSTopNSecurityValueHolder__set_state(<CSTopNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSTopNSecurityValueHolder__set_state(CSTopNSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSTopNSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSTopNPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_5__pyx_unpickle_CSTopNPercentileSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_4__pyx_unpickle_CSTopNPercentileSecurityValueHolder[] = "__pyx_unpickle_CSTopNPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_5__pyx_unpickle_CSTopNPercentileSecurityValueHolder = {"__pyx_unpickle_CSTopNPercentileSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_5__pyx_unpickle_CSTopNPercentileSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_4__pyx_unpickle_CSTopNPercentileSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_5__pyx_unpickle_CSTopNPercentileSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSTopNPercentileSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSTopNPercentileSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSTopNPercentileSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSTopNPercentileSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSTopNPercentileSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSTopNPercentileSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_4__pyx_unpickle_CSTopNPercentileSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_4__pyx_unpickle_CSTopNPercentileSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSTopNPercentileSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7b42ecc) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSTopNPercentileSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7b42ecc:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSTopNPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7b, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSTopNPercentileSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(<CSTopNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSTopNPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(<CSTopNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSTopNPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(<CSTopNPercentileSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(CSTopNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSTopNPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(<CSTopNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(<CSTopNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(CSTopNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSTopNPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSTopNPercentileSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(<CSTopNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(CSTopNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(CSTopNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_n = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_3;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(CSTopNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 9) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(CSTopNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(<CSTopNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(CSTopNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSBottomNSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_7__pyx_unpickle_CSBottomNSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_6__pyx_unpickle_CSBottomNSecurityValueHolder[] = "__pyx_unpickle_CSBottomNSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_7__pyx_unpickle_CSBottomNSecurityValueHolder = {"__pyx_unpickle_CSBottomNSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_7__pyx_unpickle_CSBottomNSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_6__pyx_unpickle_CSBottomNSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_7__pyx_unpickle_CSBottomNSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSBottomNSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSBottomNSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSBottomNSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSBottomNSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSBottomNSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSBottomNSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_6__pyx_unpickle_CSBottomNSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_6__pyx_unpickle_CSBottomNSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSBottomNSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7b42ecc) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSBottomNSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7b42ecc:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSBottomNSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7b, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSBottomNSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(<CSBottomNSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSBottomNSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(<CSBottomNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSBottomNSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(<CSBottomNSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(CSBottomNSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSBottomNSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSBottomNSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(<CSBottomNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(<CSBottomNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(CSBottomNSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSBottomNSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSBottomNSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(<CSBottomNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(CSBottomNSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSBottomNSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSBottomNSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(CSBottomNSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_n = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(CSBottomNSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 9) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(CSBottomNSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(<CSBottomNSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSBottomNSecurityValueHolder__set_state(CSBottomNSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSBottomNSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSBottomNPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_9__pyx_unpickle_CSBottomNPercentileSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_8__pyx_unpickle_CSBottomNPercentileSecurityValueHolder[] = "__pyx_unpickle_CSBottomNPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_9__pyx_unpickle_CSBottomNPercentileSecurityValueHolder = {"__pyx_unpickle_CSBottomNPercentileSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_9__pyx_unpickle_CSBottomNPercentileSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_8__pyx_unpickle_CSBottomNPercentileSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_9__pyx_unpickle_CSBottomNPercentileSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSBottomNPercentileSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSBottomNPercentileSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSBottomNPercentileSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSBottomNPercentileSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSBottomNPercentileSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSBottomNPercentileSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_8__pyx_unpickle_CSBottomNPercentileSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_8__pyx_unpickle_CSBottomNPercentileSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSBottomNPercentileSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7b42ecc) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSBottomNPercentileSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x7b42ecc:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSBottomNPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7b, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x7b42ecc:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSBottomNPercentileSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(<CSBottomNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSBottomNPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(<CSBottomNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSBottomNPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(<CSBottomNPercentileSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(CSBottomNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7b42ecc = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _n, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSBottomNPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(<CSBottomNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(<CSBottomNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(CSBottomNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSBottomNPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSBottomNPercentileSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(<CSBottomNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(CSBottomNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(CSBottomNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_n = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_3;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(CSBottomNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 9) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(CSBottomNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(<CSBottomNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(CSBottomNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSRankedSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_11__pyx_unpickle_CSRankedSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_10__pyx_unpickle_CSRankedSecurityValueHolder[] = "__pyx_unpickle_CSRankedSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_11__pyx_unpickle_CSRankedSecurityValueHolder = {"__pyx_unpickle_CSRankedSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_11__pyx_unpickle_CSRankedSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_10__pyx_unpickle_CSRankedSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_11__pyx_unpickle_CSRankedSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSRankedSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSRankedSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSRankedSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSRankedSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSRankedSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSRankedSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_10__pyx_unpickle_CSRankedSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_10__pyx_unpickle_CSRankedSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSRankedSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x241032f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSRankedSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSRankedSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x24, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSRankedSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSRankedSecurityValueHolder__set_state(<CSRankedSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSRankedSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSRankedSecurityValueHolder__set_state(<CSRankedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSRankedSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSRankedSecurityValueHolder__set_state(<CSRankedSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSRankedSecurityValueHolder__set_state(CSRankedSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSRankedSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSRankedSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSRankedSecurityValueHolder__set_state(<CSRankedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSRankedSecurityValueHolder__set_state(<CSRankedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSRankedSecurityValueHolder__set_state(CSRankedSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSRankedSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSRankedSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSRankedSecurityValueHolder__set_state(<CSRankedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSRankedSecurityValueHolder__set_state(CSRankedSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSRankedSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSRankedSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSRankedSecurityValueHolder__set_state(CSRankedSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSRankedSecurityValueHolder__set_state(CSRankedSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 8) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSRankedSecurityValueHolder__set_state(CSRankedSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSRankedSecurityValueHolder__set_state(<CSRankedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSRankedSecurityValueHolder__set_state(CSRankedSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSRankedSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSAverageSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_13__pyx_unpickle_CSAverageSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_12__pyx_unpickle_CSAverageSecurityValueHolder[] = "__pyx_unpickle_CSAverageSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_13__pyx_unpickle_CSAverageSecurityValueHolder = {"__pyx_unpickle_CSAverageSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_13__pyx_unpickle_CSAverageSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_12__pyx_unpickle_CSAverageSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_13__pyx_unpickle_CSAverageSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSAverageSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSAverageSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSAverageSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSAverageSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSAverageSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSAverageSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_12__pyx_unpickle_CSAverageSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_12__pyx_unpickle_CSAverageSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSAverageSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x241032f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSAverageSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSAverageSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x24, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSAverageSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSAverageSecurityValueHolder__set_state(<CSAverageSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSAverageSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSAverageSecurityValueHolder__set_state(<CSAverageSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSAverageSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSAverageSecurityValueHolder__set_state(<CSAverageSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageSecurityValueHolder__set_state(CSAverageSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSAverageSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSAverageSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSAverageSecurityValueHolder__set_state(<CSAverageSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSAverageSecurityValueHolder__set_state(<CSAverageSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSAverageSecurityValueHolder__set_state(CSAverageSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSAverageSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSAverageSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSAverageSecurityValueHolder__set_state(<CSAverageSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageSecurityValueHolder__set_state(CSAverageSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSAverageSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSAverageSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageSecurityValueHolder__set_state(CSAverageSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSAverageSecurityValueHolder__set_state(CSAverageSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 8) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSAverageSecurityValueHolder__set_state(CSAverageSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSAverageSecurityValueHolder__set_state(<CSAverageSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageSecurityValueHolder__set_state(CSAverageSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSAverageSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_15__pyx_unpickle_CSPercentileSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_14__pyx_unpickle_CSPercentileSecurityValueHolder[] = "__pyx_unpickle_CSPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_15__pyx_unpickle_CSPercentileSecurityValueHolder = {"__pyx_unpickle_CSPercentileSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_15__pyx_unpickle_CSPercentileSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_14__pyx_unpickle_CSPercentileSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_15__pyx_unpickle_CSPercentileSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSPercentileSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSPercentileSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSPercentileSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSPercentileSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSPercentileSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSPercentileSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_14__pyx_unpickle_CSPercentileSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_14__pyx_unpickle_CSPercentileSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSPercentileSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x241032f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSPercentileSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x24, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSPercentileSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(<CSPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(<CSPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(<CSPercentileSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(CSPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSPercentileSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSPercentileSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(<CSPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(<CSPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(CSPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSPercentileSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(<CSPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(CSPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSPercentileSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSPercentileSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(CSPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(CSPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 8) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(CSPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(<CSPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSPercentileSecurityValueHolder__set_state(CSPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSPercentileSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSAverageAdjustedSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_17__pyx_unpickle_CSAverageAdjustedSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_16__pyx_unpickle_CSAverageAdjustedSecurityValueHolder[] = "__pyx_unpickle_CSAverageAdjustedSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_17__pyx_unpickle_CSAverageAdjustedSecurityValueHolder = {"__pyx_unpickle_CSAverageAdjustedSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_17__pyx_unpickle_CSAverageAdjustedSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_16__pyx_unpickle_CSAverageAdjustedSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_17__pyx_unpickle_CSAverageAdjustedSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSAverageAdjustedSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSAverageAdjustedSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSAverageAdjustedSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSAverageAdjustedSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSAverageAdjustedSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSAverageAdjustedSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_16__pyx_unpickle_CSAverageAdjustedSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_16__pyx_unpickle_CSAverageAdjustedSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSAverageAdjustedSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x241032f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSAverageAdjustedSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSAverageAdjustedSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x24, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSAverageAdjustedSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(<CSAverageAdjustedSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSAverageAdjustedSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(<CSAverageAdjustedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSAverageAdjustedSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(<CSAverageAdjustedSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(CSAverageAdjustedSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSAverageAdjustedSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(<CSAverageAdjustedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(<CSAverageAdjustedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(CSAverageAdjustedSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSAverageAdjustedSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSAverageAdjustedSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(<CSAverageAdjustedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(CSAverageAdjustedSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(CSAverageAdjustedSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(CSAverageAdjustedSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 8) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(CSAverageAdjustedSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(<CSAverageAdjustedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(CSAverageAdjustedSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSZScoreSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_19__pyx_unpickle_CSZScoreSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_18__pyx_unpickle_CSZScoreSecurityValueHolder[] = "__pyx_unpickle_CSZScoreSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_19__pyx_unpickle_CSZScoreSecurityValueHolder = {"__pyx_unpickle_CSZScoreSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_19__pyx_unpickle_CSZScoreSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_18__pyx_unpickle_CSZScoreSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_19__pyx_unpickle_CSZScoreSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSZScoreSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSZScoreSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSZScoreSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSZScoreSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSZScoreSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSZScoreSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_18__pyx_unpickle_CSZScoreSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_18__pyx_unpickle_CSZScoreSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSZScoreSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x241032f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSZScoreSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSZScoreSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x24, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSZScoreSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(<CSZScoreSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSZScoreSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(<CSZScoreSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSZScoreSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(<CSZScoreSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(CSZScoreSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSZScoreSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSZScoreSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(<CSZScoreSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(<CSZScoreSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(CSZScoreSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSZScoreSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSZScoreSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(<CSZScoreSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(CSZScoreSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSZScoreSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSZScoreSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(CSZScoreSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(CSZScoreSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 8) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(CSZScoreSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(<CSZScoreSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSZScoreSecurityValueHolder__set_state(CSZScoreSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSZScoreSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSFillNASecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_21__pyx_unpickle_CSFillNASecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_20__pyx_unpickle_CSFillNASecurityValueHolder[] = "__pyx_unpickle_CSFillNASecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_21__pyx_unpickle_CSFillNASecurityValueHolder = {"__pyx_unpickle_CSFillNASecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_21__pyx_unpickle_CSFillNASecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_20__pyx_unpickle_CSFillNASecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_21__pyx_unpickle_CSFillNASecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSFillNASecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSFillNASecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSFillNASecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSFillNASecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSFillNASecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSFillNASecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_20__pyx_unpickle_CSFillNASecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_20__pyx_unpickle_CSFillNASecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSFillNASecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x241032f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSFillNASecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x241032f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSFillNASecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x24, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x241032f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSFillNASecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSFillNASecurityValueHolder__set_state(<CSFillNASecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSFillNASecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSFillNASecurityValueHolder__set_state(<CSFillNASecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSFillNASecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSFillNASecurityValueHolder__set_state(<CSFillNASecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSFillNASecurityValueHolder__set_state(CSFillNASecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSFillNASecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x241032f = (_dependency, _group, _holderTemplate, _inner, _innerHolders, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSFillNASecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSFillNASecurityValueHolder__set_state(<CSFillNASecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSFillNASecurityValueHolder__set_state(<CSFillNASecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSFillNASecurityValueHolder__set_state(CSFillNASecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSFillNASecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSFillNASecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSFillNASecurityValueHolder__set_state(<CSFillNASecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSFillNASecurityValueHolder__set_state(CSFillNASecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSFillNASecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSFillNASecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSFillNASecurityValueHolder__set_state(CSFillNASecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._group);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._group));
  __pyx_v___pyx_result->__pyx_base._group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base.__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._inner);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._inner));
  __pyx_v___pyx_result->__pyx_base._inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base.__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSFillNASecurityValueHolder__set_state(CSFillNASecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 8) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSFillNASecurityValueHolder__set_state(CSFillNASecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSFillNASecurityValueHolder__set_state(<CSFillNASecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSFillNASecurityValueHolder__set_state(CSFillNASecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSFillNASecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CSResidueSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23__pyx_unpickle_CSResidueSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_22__pyx_unpickle_CSResidueSecurityValueHolder[] = "__pyx_unpickle_CSResidueSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23__pyx_unpickle_CSResidueSecurityValueHolder = {"__pyx_unpickle_CSResidueSecurityValueHolder", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23__pyx_unpickle_CSResidueSecurityValueHolder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_22__pyx_unpickle_CSResidueSecurityValueHolder};
static PyObject *__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23__pyx_unpickle_CSResidueSecurityValueHolder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSResidueSecurityValueHolder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSResidueSecurityValueHolder", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSResidueSecurityValueHolder", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CSResidueSecurityValueHolder") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CSResidueSecurityValueHolder", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSResidueSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_22__pyx_unpickle_CSResidueSecurityValueHolder(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_22__pyx_unpickle_CSResidueSecurityValueHolder(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSResidueSecurityValueHolder", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xe27c424:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe27c424 = (_dependency, _holderTemplate, _innerHolders, _left, _right, _window, cached, updated))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xe27c424) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xe27c424:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe27c424 = (_dependency, _holderTemplate, _innerHolders, _left, _right, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSResidueSecurityValueHolder.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xe27c424:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe27c424 = (_dependency, _holderTemplate, _innerHolders, _left, _right, _window, cached, updated))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CSResidueSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xe2, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xe27c424:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe27c424 = (_dependency, _holderTemplate, _innerHolders, _left, _right, _window, cached, updated))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe27c424 = (_dependency, _holderTemplate, _innerHolders, _left, _right, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSResidueSecurityValueHolder.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSResidueSecurityValueHolder__set_state(<CSResidueSecurityValueHolder> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe27c424 = (_dependency, _holderTemplate, _innerHolders, _left, _right, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSResidueSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSResidueSecurityValueHolder__set_state(<CSResidueSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = CSResidueSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSResidueSecurityValueHolder__set_state(<CSResidueSecurityValueHolder> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CSResidueSecurityValueHolder__set_state(CSResidueSecurityValueHolder __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSResidueSecurityValueHolder__set_state(((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xe27c424 = (_dependency, _holderTemplate, _innerHolders, _left, _right, _window, cached, updated))" % __pyx_checksum)
 *     __pyx_result = CSResidueSecurityValueHolder.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CSResidueSecurityValueHolder__set_state(<CSResidueSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CSResidueSecurityValueHolder__set_state(<CSResidueSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CSResidueSecurityValueHolder__set_state(CSResidueSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._holderTemplate = __pyx_state[1]; __pyx_result._innerHolders = __pyx_state[2]; __pyx_result._left = __pyx_state[3]; __pyx_result._right = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSResidueSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSResidueSecurityValueHolder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_CSResidueSecurityValueHolder__set_state(<CSResidueSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSResidueSecurityValueHolder__set_state(CSResidueSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._holderTemplate = __pyx_state[1]; __pyx_result._innerHolders = __pyx_state[2]; __pyx_result._left = __pyx_state[3]; __pyx_result._right = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders___pyx_unpickle_CSResidueSecurityValueHolder__set_state(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CSResidueSecurityValueHolder__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_CSResidueSecurityValueHolder__set_state(CSResidueSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._holderTemplate = __pyx_state[1]; __pyx_result._innerHolders = __pyx_state[2]; __pyx_result._left = __pyx_state[3]; __pyx_result._right = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._dependency);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._dependency);
  __pyx_v___pyx_result->__pyx_base._dependency = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._holderTemplate);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._holderTemplate));
  __pyx_v___pyx_result->__pyx_base._holderTemplate = ((struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._innerHolders);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._innerHolders);
  __pyx_v___pyx_result->__pyx_base._innerHolders = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_left);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_left));
  __pyx_v___pyx_result->_left = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_right);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_right));
  __pyx_v___pyx_result->_right = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._window = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.cached);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.cached));
  __pyx_v___pyx_result->__pyx_base.cached = ((struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.updated = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSResidueSecurityValueHolder__set_state(CSResidueSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._holderTemplate = __pyx_state[1]; __pyx_result._innerHolders = __pyx_state[2]; __pyx_result._left = __pyx_state[3]; __pyx_result._right = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 8) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._holderTemplate = __pyx_state[1]; __pyx_result._innerHolders = __pyx_state[2]; __pyx_result._left = __pyx_state[3]; __pyx_result._right = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_CSResidueSecurityValueHolder__set_state(CSResidueSecurityValueHolder __pyx_result, tuple __pyx_state):
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._holderTemplate = __pyx_state[1]; __pyx_result._innerHolders = __pyx_state[2]; __pyx_result._left = __pyx_state[3]; __pyx_result._right = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSResidueSecurityValueHolder__set_state(<CSResidueSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSResidueSecurityValueHolder__set_state(CSResidueSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._holderTemplate = __pyx_state[1]; __pyx_result._innerHolders = __pyx_state[2]; __pyx_result._left = __pyx_state[3]; __pyx_result._right = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ultron.sentry.Analysis.CrossSectionValueHolders.__pyx_unpickle_CSResidueSecurityValueHolder__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":735
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":734
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":738
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 738, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":737
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":741
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":740
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":744
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":743
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":747
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":746
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":751
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":750
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":753
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":749
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":869
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":870
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":868
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":873
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":875
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":874
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":876
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":872
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":882
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 882, __pyx_L3_error)

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":883
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 883, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 884, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 884, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":881
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":880
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":888
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 888, __pyx_L3_error)

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":889
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 889, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 890, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 890, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":887
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":886
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":894
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 894, __pyx_L3_error)

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":895
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 895, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":896
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef extern from *:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 896, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 896, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":893
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *p;
  PyObject *o = __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  p->_inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None); Py_INCREF(Py_None);
  p->_group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *p = (struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_inner);
  Py_CLEAR(p->_group);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder)) __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder);
}

static int __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *p = (struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)o;
  e = ((likely(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder)) ? ((__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_traverse) ? __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder)); if (e) return e;
  if (p->_inner) {
    e = (*v)(((PyObject *)p->_inner), a); if (e) return e;
  }
  if (p->_group) {
    e = (*v)(((PyObject *)p->_group), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *p = (struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *)o;
  if (likely(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder)) { if (__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_clear) __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder);
  tmp = ((PyObject*)p->_inner);
  p->_inner = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_group);
  p->_group = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_symbolList(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_10symbolList_1__get__(o);
}

static PyObject *__pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder__inner(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder__inner(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_inner_5__del__(o);
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder__group(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder__group(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6_group_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_3push, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_2push},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_5__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_7__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_6__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder[] = {
  {(char *)"symbolList", __pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_symbolList, 0, (char *)0, 0},
  {(char *)"_inner", __pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder__inner, __pyx_setprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder__inner, (char *)"_inner: ultron.sentry.Analysis.SecurityValueHolders.SecurityValueHolder", 0},
  {(char *)"_group", __pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder__group, __pyx_setprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder__group, (char *)"_group: ultron.sentry.Analysis.SecurityValueHolders.SecurityValueHolder", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CrossSectionValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CrossSectionValueHolder(innerValue, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSTopNSecurityValueHolder(innerValue, n, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSTopNPercentileSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSTopNPercentileSecurityValueHolder(innerValue, n, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSBottomNSecurityValueHolder(innerValue, n, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSBottomNPercentileSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSBottomNPercentileSecurityValueHolder(innerValue, n, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSRankedSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSRankedSecurityValueHolder(innerValue, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSAverageSecurityValueHolder(innerValue, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSPercentileSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSPercentileSecurityValueHolder(innerValue, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSAverageAdjustedSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSAverageAdjustedSecurityValueHolder(innerValue, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSZScoreSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSZScoreSecurityValueHolder(innerValue, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *p;
  PyObject *o = __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder;
  return o;
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder[] = {
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_3value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_2value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_5value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_4value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_7value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_6value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_11__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_13__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_12__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSFillNASecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_9__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSFillNASecurityValueHolder(innerValue, groups=None)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder;

static PyObject *__pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *p;
  PyObject *o = __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder;
  p->_left = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None); Py_INCREF(Py_None);
  p->_right = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder(PyObject *o) {
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *p = (struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_left);
  Py_CLEAR(p->_right);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder)) __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder);
}

static int __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *p = (struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)o;
  e = ((likely(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder)) ? ((__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_traverse) ? __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder)); if (e) return e;
  if (p->_left) {
    e = (*v)(((PyObject *)p->_left), a); if (e) return e;
  }
  if (p->_right) {
    e = (*v)(((PyObject *)p->_right), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *p = (struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *)o;
  if (likely(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder)) { if (__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_clear) __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder);
  tmp = ((PyObject*)p->_left);
  p->_left = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_right);
  p->_right = ((struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_symbolList(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_10symbolList_1__get__(o);
}

static PyObject *__pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder__left(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder__left(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5_left_5__del__(o);
  }
}

static PyObject *__pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder__right(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_1__get__(o);
}

static int __pyx_setprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder__right(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_3__set__(o, v);
  }
  else {
    return __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6_right_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder[] = {
  {"push", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_3push, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_2push},
  {"value_all", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_5value_all, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_4value_all},
  {"value_by_name", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_7value_by_name, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_6value_by_name},
  {"value_by_names", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_9value_by_names, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_8value_by_names},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_13__reduce_cython__, METH_NOARGS, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_12__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_15__setstate_cython__, METH_O, __pyx_doc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_14__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder[] = {
  {(char *)"symbolList", __pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_symbolList, 0, (char *)0, 0},
  {(char *)"_left", __pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder__left, __pyx_setprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder__left, (char *)"_left: ultron.sentry.Analysis.SecurityValueHolders.SecurityValueHolder", 0},
  {(char *)"_right", __pyx_getprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder__right, __pyx_setprop_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder__right, (char *)"_right: ultron.sentry.Analysis.SecurityValueHolders.SecurityValueHolder", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ultron.sentry.Analysis.CrossSectionValueHolders.CSResidueSecurityValueHolder", /*tp_name*/
  sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_11__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "CSResidueSecurityValueHolder(left, right)", /*tp_doc*/
  __pyx_tp_traverse_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder, /*tp_traverse*/
  __pyx_tp_clear_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_CrossSectionValueHolders(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_CrossSectionValueHolders},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "CrossSectionValueHolders",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_CSAverageAdjustedSecurityValueHo, __pyx_k_CSAverageAdjustedSecurityValueHo, sizeof(__pyx_k_CSAverageAdjustedSecurityValueHo), 0, 0, 1, 1},
  {&__pyx_n_s_CSAverageSecurityValueHolder, __pyx_k_CSAverageSecurityValueHolder, sizeof(__pyx_k_CSAverageSecurityValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_CSBottomNPercentileSecurityValue, __pyx_k_CSBottomNPercentileSecurityValue, sizeof(__pyx_k_CSBottomNPercentileSecurityValue), 0, 0, 1, 1},
  {&__pyx_n_s_CSBottomNSecurityValueHolder, __pyx_k_CSBottomNSecurityValueHolder, sizeof(__pyx_k_CSBottomNSecurityValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_CSFillNASecurityValueHolder, __pyx_k_CSFillNASecurityValueHolder, sizeof(__pyx_k_CSFillNASecurityValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_CSPercentileSecurityValueHolder, __pyx_k_CSPercentileSecurityValueHolder, sizeof(__pyx_k_CSPercentileSecurityValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_CSRankedSecurityValueHolder, __pyx_k_CSRankedSecurityValueHolder, sizeof(__pyx_k_CSRankedSecurityValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_CSResidueSecurityValueHolder, __pyx_k_CSResidueSecurityValueHolder, sizeof(__pyx_k_CSResidueSecurityValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_CSTopNPercentileSecurityValueHol, __pyx_k_CSTopNPercentileSecurityValueHol, sizeof(__pyx_k_CSTopNPercentileSecurityValueHol), 0, 0, 1, 1},
  {&__pyx_n_s_CSTopNSecurityValueHolder, __pyx_k_CSTopNSecurityValueHolder, sizeof(__pyx_k_CSTopNSecurityValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_CSZScoreSecurityValueHolder, __pyx_k_CSZScoreSecurityValueHolder, sizeof(__pyx_k_CSZScoreSecurityValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_CrossSectionValueHolder, __pyx_k_CrossSectionValueHolder, sizeof(__pyx_k_CrossSectionValueHolder), 0, 0, 1, 1},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x24, __pyx_k_Incompatible_checksums_s_vs_0x24, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x24), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x7b, __pyx_k_Incompatible_checksums_s_vs_0x7b, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x7b), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xe2, __pyx_k_Incompatible_checksums_s_vs_0xe2, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xe2), 0, 0, 1, 0},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_build_holder, __pyx_k_build_holder, sizeof(__pyx_k_build_holder), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_deepcopy, __pyx_k_deepcopy, sizeof(__pyx_k_deepcopy), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_fields, __pyx_k_fields, sizeof(__pyx_k_fields), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_groups, __pyx_k_groups, sizeof(__pyx_k_groups), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_innerValue, __pyx_k_innerValue, sizeof(__pyx_k_innerValue), 0, 0, 1, 1},
  {&__pyx_n_s_left, __pyx_k_left, sizeof(__pyx_k_left), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_kp_s_mathrm_CSBottomNPercentile_0_1, __pyx_k_mathrm_CSBottomNPercentile_0_1, sizeof(__pyx_k_mathrm_CSBottomNPercentile_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSBottomNPercentile_0_1_2, __pyx_k_mathrm_CSBottomNPercentile_0_1_2, sizeof(__pyx_k_mathrm_CSBottomNPercentile_0_1_2), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSBottomN_0_1, __pyx_k_mathrm_CSBottomN_0_1, sizeof(__pyx_k_mathrm_CSBottomN_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSBottomN_0_1_groups_2, __pyx_k_mathrm_CSBottomN_0_1_groups_2, sizeof(__pyx_k_mathrm_CSBottomN_0_1_groups_2), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSFillNA_0, __pyx_k_mathrm_CSFillNA_0, sizeof(__pyx_k_mathrm_CSFillNA_0), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSFillNA_0_groups_1, __pyx_k_mathrm_CSFillNA_0_groups_1, sizeof(__pyx_k_mathrm_CSFillNA_0_groups_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSMeanAdjusted_0, __pyx_k_mathrm_CSMeanAdjusted_0, sizeof(__pyx_k_mathrm_CSMeanAdjusted_0), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSMeanAdjusted_0_groups, __pyx_k_mathrm_CSMeanAdjusted_0_groups, sizeof(__pyx_k_mathrm_CSMeanAdjusted_0_groups), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSMean_0, __pyx_k_mathrm_CSMean_0, sizeof(__pyx_k_mathrm_CSMean_0), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSMean_0_groups_1, __pyx_k_mathrm_CSMean_0_groups_1, sizeof(__pyx_k_mathrm_CSMean_0_groups_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSPercentile_0, __pyx_k_mathrm_CSPercentile_0, sizeof(__pyx_k_mathrm_CSPercentile_0), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSPercentile_0_groups_1, __pyx_k_mathrm_CSPercentile_0_groups_1, sizeof(__pyx_k_mathrm_CSPercentile_0_groups_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSRank_0, __pyx_k_mathrm_CSRank_0, sizeof(__pyx_k_mathrm_CSRank_0), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSRank_0_groups_1, __pyx_k_mathrm_CSRank_0_groups_1, sizeof(__pyx_k_mathrm_CSRank_0_groups_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSRes_0_1, __pyx_k_mathrm_CSRes_0_1, sizeof(__pyx_k_mathrm_CSRes_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSTopNPercentile_0_1, __pyx_k_mathrm_CSTopNPercentile_0_1, sizeof(__pyx_k_mathrm_CSTopNPercentile_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSTopNPercentile_0_1_gro, __pyx_k_mathrm_CSTopNPercentile_0_1_gro, sizeof(__pyx_k_mathrm_CSTopNPercentile_0_1_gro), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSTopN_0_1, __pyx_k_mathrm_CSTopN_0_1, sizeof(__pyx_k_mathrm_CSTopN_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSTopN_0_1_groups_2, __pyx_k_mathrm_CSTopN_0_1_groups_2, sizeof(__pyx_k_mathrm_CSTopN_0_1_groups_2), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSZscore_0, __pyx_k_mathrm_CSZscore_0, sizeof(__pyx_k_mathrm_CSZscore_0), 0, 0, 1, 0},
  {&__pyx_kp_s_mathrm_CSZscore_0_groups_1, __pyx_k_mathrm_CSZscore_0_groups_1, sizeof(__pyx_k_mathrm_CSZscore_0_groups_1), 0, 0, 1, 0},
  {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_s_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 0, 1, 0},
  {&__pyx_kp_s_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 0, 1, 0},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_push, __pyx_k_push, sizeof(__pyx_k_push), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSAverageAdjusted, __pyx_k_pyx_unpickle_CSAverageAdjusted, sizeof(__pyx_k_pyx_unpickle_CSAverageAdjusted), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSAverageSecurity, __pyx_k_pyx_unpickle_CSAverageSecurity, sizeof(__pyx_k_pyx_unpickle_CSAverageSecurity), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSBottomNPercenti, __pyx_k_pyx_unpickle_CSBottomNPercenti, sizeof(__pyx_k_pyx_unpickle_CSBottomNPercenti), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSBottomNSecurity, __pyx_k_pyx_unpickle_CSBottomNSecurity, sizeof(__pyx_k_pyx_unpickle_CSBottomNSecurity), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSFillNASecurityV, __pyx_k_pyx_unpickle_CSFillNASecurityV, sizeof(__pyx_k_pyx_unpickle_CSFillNASecurityV), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSPercentileSecur, __pyx_k_pyx_unpickle_CSPercentileSecur, sizeof(__pyx_k_pyx_unpickle_CSPercentileSecur), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSRankedSecurityV, __pyx_k_pyx_unpickle_CSRankedSecurityV, sizeof(__pyx_k_pyx_unpickle_CSRankedSecurityV), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSResidueSecurity, __pyx_k_pyx_unpickle_CSResidueSecurity, sizeof(__pyx_k_pyx_unpickle_CSResidueSecurity), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSTopNPercentileS, __pyx_k_pyx_unpickle_CSTopNPercentileS, sizeof(__pyx_k_pyx_unpickle_CSTopNPercentileS), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSTopNSecurityVal, __pyx_k_pyx_unpickle_CSTopNSecurityVal, sizeof(__pyx_k_pyx_unpickle_CSTopNSecurityVal), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CSZScoreSecurityV, __pyx_k_pyx_unpickle_CSZScoreSecurityV, sizeof(__pyx_k_pyx_unpickle_CSZScoreSecurityV), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CrossSectionValue, __pyx_k_pyx_unpickle_CrossSectionValue, sizeof(__pyx_k_pyx_unpickle_CrossSectionValue), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_right, __pyx_k_right, sizeof(__pyx_k_right), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_six, __pyx_k_six, sizeof(__pyx_k_six), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_symbolList, __pyx_k_symbolList, sizeof(__pyx_k_symbolList), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_ultron_sentry_Analysis_CrossSect, __pyx_k_ultron_sentry_Analysis_CrossSect, sizeof(__pyx_k_ultron_sentry_Analysis_CrossSect), 0, 0, 1, 1},
  {&__pyx_n_s_ultron_sentry_Analysis_SecurityV, __pyx_k_ultron_sentry_Analysis_SecurityV, sizeof(__pyx_k_ultron_sentry_Analysis_SecurityV), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_value_all, __pyx_k_value_all, sizeof(__pyx_k_value_all), 0, 0, 1, 1},
  {&__pyx_n_s_value_by_name, __pyx_k_value_by_name, sizeof(__pyx_k_value_by_name), 0, 0, 1, 1},
  {&__pyx_n_s_value_by_names, __pyx_k_value_by_names, sizeof(__pyx_k_value_by_names), 0, 0, 1, 1},
  {&__pyx_n_s_window, __pyx_k_window, sizeof(__pyx_k_window), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 21, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 884, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":884
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple_)) __PYX_ERR(2, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":890
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(2, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":1
 * def __pyx_unpickle_CrossSectionValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__3 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);
  __pyx_codeobj__4 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__3, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CrossSectionValue, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__5 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);
  __pyx_codeobj__6 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__5, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSTopNSecurityVal, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__6)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__7 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __pyx_codeobj__8 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__7, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSTopNPercentileS, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__8)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__9 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSBottomNSecurity, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__11 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSBottomNPercenti, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__13 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSRankedSecurityV, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__15 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSAverageSecurity, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__17 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSPercentileSecur, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__19 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSAverageAdjusted, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__21 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSZScoreSecurityV, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__23 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__24 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSFillNASecurityV, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__24)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__25 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CSResidueSecurity, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_37815087 = PyInt_FromLong(37815087L); if (unlikely(!__pyx_int_37815087)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_129248972 = PyInt_FromLong(129248972L); if (unlikely(!__pyx_int_129248972)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_237487140 = PyInt_FromLong(237487140L); if (unlikely(!__pyx_int_237487140)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Analysis.SecurityValueHolders"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23CrossSectionValueHolder_push;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CrossSectionValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder._cal_impl = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder *))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_25CSTopNSecurityValueHolder__cal_impl;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSTopNSecurityValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNSecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder._cal_impl = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder *))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_35CSTopNPercentileSecurityValueHolder__cal_impl;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSTopNPercentileSecurityValueHol, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 90, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSTopNPercentileSecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder._cal_impl = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder *))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSBottomNSecurityValueHolder__cal_impl;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSBottomNSecurityValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNSecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder._cal_impl = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder *))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_38CSBottomNPercentileSecurityValueHolder__cal_impl;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSBottomNPercentileSecurityValue, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 182, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSBottomNPercentileSecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder._cal_impl = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder *))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSRankedSecurityValueHolder__cal_impl;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder) < 0) __PYX_ERR(0, 228, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder) < 0) __PYX_ERR(0, 228, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSRankedSecurityValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder) < 0) __PYX_ERR(0, 228, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder) < 0) __PYX_ERR(0, 228, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSRankedSecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder._cal_impl = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder *))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSAverageSecurityValueHolder__cal_impl;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder) < 0) __PYX_ERR(0, 271, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder) < 0) __PYX_ERR(0, 271, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSAverageSecurityValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder) < 0) __PYX_ERR(0, 271, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder) < 0) __PYX_ERR(0, 271, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageSecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder._cal_impl = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder *))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_31CSPercentileSecurityValueHolder__cal_impl;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 313, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 313, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSPercentileSecurityValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 313, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder) < 0) __PYX_ERR(0, 313, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSPercentileSecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_36CSAverageAdjustedSecurityValueHolder_value_by_name;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder) < 0) __PYX_ERR(0, 356, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder) < 0) __PYX_ERR(0, 356, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSAverageAdjustedSecurityValueHo, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder) < 0) __PYX_ERR(0, 356, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder) < 0) __PYX_ERR(0, 356, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSAverageAdjustedSecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder._cal_impl = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder *))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSZScoreSecurityValueHolder__cal_impl;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder) < 0) __PYX_ERR(0, 405, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder) < 0) __PYX_ERR(0, 405, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSZScoreSecurityValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder) < 0) __PYX_ERR(0, 405, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder) < 0) __PYX_ERR(0, 405, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSZScoreSecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.__pyx_base.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.__pyx_base.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.__pyx_base.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder._cal_impl = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder *))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_27CSFillNASecurityValueHolder__cal_impl;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CrossSectionValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder) < 0) __PYX_ERR(0, 447, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder) < 0) __PYX_ERR(0, 447, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSFillNASecurityValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder) < 0) __PYX_ERR(0, 447, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder) < 0) __PYX_ERR(0, 447, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSFillNASecurityValueHolder;
  __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder = &__pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.__pyx_base = *__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.__pyx_base.value_all = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_all;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.__pyx_base.value_by_names = (struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_by_names;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.__pyx_base.value_by_name = (double (*)(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_value_by_name;
  __pyx_vtable_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.push = (PyObject *(*)(struct __pyx_obj_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_28CSResidueSecurityValueHolder_push;
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.tp_base = __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityValueHolder;
  if (PyType_Ready(&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder) < 0) __PYX_ERR(0, 489, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.tp_dictoffset && __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder.tp_dict, __pyx_vtabptr_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder) < 0) __PYX_ERR(0, 489, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CSResidueSecurityValueHolder, (PyObject *)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder) < 0) __PYX_ERR(0, 489, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder) < 0) __PYX_ERR(0, 489, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder = &__pyx_type_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_CSResidueSecurityValueHolder;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 199, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 222, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 226, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 238, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 764, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Analysis.SeriesValues"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SeriesValues", "SeriesValues", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues) __PYX_ERR(4, 5, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_12SeriesValues_SeriesValues)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.Distributions.NormalDistribution"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Distributions.NormalDistribution", "InverseCumulativeNormal", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal) __PYX_ERR(5, 3, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_13Distributions_18NormalDistribution_InverseCumulativeNormal)) __PYX_ERR(5, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.Accumulators.IAccumulators"); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "IAccumulator", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IAccumulator) __PYX_ERR(6, 6, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Accumulator", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator) __PYX_ERR(6, 10, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Accumulator)) __PYX_ERR(6, 10, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Negative", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative) __PYX_ERR(6, 25, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Negative)) __PYX_ERR(6, 25, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "CombinedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder) __PYX_ERR(6, 33, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CombinedValueHolder)) __PYX_ERR(6, 33, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "AddedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder) __PYX_ERR(6, 41, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_AddedValueHolder)) __PYX_ERR(6, 41, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "MinusedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder) __PYX_ERR(6, 46, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MinusedValueHolder)) __PYX_ERR(6, 46, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "MultipliedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder) __PYX_ERR(6, 51, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_MultipliedValueHolder)) __PYX_ERR(6, 51, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "DividedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder) __PYX_ERR(6, 56, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_DividedValueHolder)) __PYX_ERR(6, 56, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "LtOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder) __PYX_ERR(6, 61, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LtOperatorValueHolder)) __PYX_ERR(6, 61, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "LeOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder) __PYX_ERR(6, 66, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_LeOperatorValueHolder)) __PYX_ERR(6, 66, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "GtOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder) __PYX_ERR(6, 71, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GtOperatorValueHolder)) __PYX_ERR(6, 71, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "GeOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder) __PYX_ERR(6, 76, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_GeOperatorValueHolder)) __PYX_ERR(6, 76, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "EqOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder) __PYX_ERR(6, 81, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_EqOperatorValueHolder)) __PYX_ERR(6, 81, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "NeOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder) __PYX_ERR(6, 86, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NeOperatorValueHolder)) __PYX_ERR(6, 86, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Identity", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity) __PYX_ERR(6, 91, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Identity)) __PYX_ERR(6, 91, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Current", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current) __PYX_ERR(6, 99, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Current)) __PYX_ERR(6, 99, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Latest", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest) __PYX_ERR(6, 107, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Latest)) __PYX_ERR(6, 107, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "CompoundedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder) __PYX_ERR(6, 115, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_CompoundedValueHolder)) __PYX_ERR(6, 115, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "IIF", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF) __PYX_ERR(6, 124, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_IIF)) __PYX_ERR(6, 124, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "BasicFunction", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction) __PYX_ERR(6, 134, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_BasicFunction)) __PYX_ERR(6, 134, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Exp", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp) __PYX_ERR(6, 142, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Exp)) __PYX_ERR(6, 142, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Log", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log) __PYX_ERR(6, 147, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Log)) __PYX_ERR(6, 147, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Sqrt", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt) __PYX_ERR(6, 152, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sqrt)) __PYX_ERR(6, 152, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Pow", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow) __PYX_ERR(6, 157, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Pow)) __PYX_ERR(6, 157, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Abs", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs) __PYX_ERR(6, 164, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Abs)) __PYX_ERR(6, 164, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Sign", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign) __PYX_ERR(6, 169, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sign)) __PYX_ERR(6, 169, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Acos", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos) __PYX_ERR(6, 174, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acos)) __PYX_ERR(6, 174, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Acosh", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh) __PYX_ERR(6, 179, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Acosh)) __PYX_ERR(6, 179, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Asin", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin) __PYX_ERR(6, 184, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asin)) __PYX_ERR(6, 184, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Asinh", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh) __PYX_ERR(6, 189, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Asinh)) __PYX_ERR(6, 189, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "NormInv", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv) __PYX_ERR(6, 194, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_NormInv)) __PYX_ERR(6, 194, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Ceil", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil) __PYX_ERR(6, 201, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Ceil)) __PYX_ERR(6, 201, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Floor", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor) __PYX_ERR(6, 206, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Floor)) __PYX_ERR(6, 206, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Round", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round) __PYX_ERR(6, 211, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Round)) __PYX_ERR(6, 211, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Sigmoid", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid) __PYX_ERR(6, 215, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Sigmoid)) __PYX_ERR(6, 215, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Math.Accumulators.IAccumulators", "Tanh", sizeof(struct __pyx_obj_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh) __PYX_ERR(6, 219, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh = (struct __pyx_vtabstruct_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_4Math_12Accumulators_13IAccumulators_Tanh)) __PYX_ERR(6, 219, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Analysis.SecurityValueHolders"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecuritySingleValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder) __PYX_ERR(7, 23, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySingleValueHolder)) __PYX_ERR(7, 23, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityBinaryValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder) __PYX_ERR(7, 29, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityBinaryValueHolder)) __PYX_ERR(7, 29, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityStatelessSingleValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder) __PYX_ERR(7, 35, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityStatelessSingleValueHolder)) __PYX_ERR(7, 35, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "FilteredSecurityValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder) __PYX_ERR(7, 42, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_FilteredSecurityValueHolder)) __PYX_ERR(7, 42, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "IdentitySecurityValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder) __PYX_ERR(7, 52, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_IdentitySecurityValueHolder)) __PYX_ERR(7, 52, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityConstArrayValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder) __PYX_ERR(7, 63, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityConstArrayValueHolder)) __PYX_ERR(7, 63, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityUnitoryValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder) __PYX_ERR(7, 72, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityUnitoryValueHolder)) __PYX_ERR(7, 72, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityNegValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder) __PYX_ERR(7, 83, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNegValueHolder)) __PYX_ERR(7, 83, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityInvertValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder) __PYX_ERR(7, 87, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityInvertValueHolder)) __PYX_ERR(7, 87, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityCurrentValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder) __PYX_ERR(7, 91, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCurrentValueHolder)) __PYX_ERR(7, 91, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityLatestValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder) __PYX_ERR(7, 100, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLatestValueHolder)) __PYX_ERR(7, 100, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityCombinedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder) __PYX_ERR(7, 112, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityCombinedValueHolder)) __PYX_ERR(7, 112, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityXorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder) __PYX_ERR(7, 123, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityXorValueHolder)) __PYX_ERR(7, 123, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityAddedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder) __PYX_ERR(7, 128, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAddedValueHolder)) __PYX_ERR(7, 128, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecuritySubbedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder) __PYX_ERR(7, 132, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecuritySubbedValueHolder)) __PYX_ERR(7, 132, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityMultipliedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder) __PYX_ERR(7, 136, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityMultipliedValueHolder)) __PYX_ERR(7, 136, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityDividedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder) __PYX_ERR(7, 140, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDividedValueHolder)) __PYX_ERR(7, 140, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityLtOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder) __PYX_ERR(7, 144, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLtOperatorValueHolder)) __PYX_ERR(7, 144, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityLeOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder) __PYX_ERR(7, 148, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityLeOperatorValueHolder)) __PYX_ERR(7, 148, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityGtOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder) __PYX_ERR(7, 152, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGtOperatorValueHolder)) __PYX_ERR(7, 152, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityGeOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder) __PYX_ERR(7, 156, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityGeOperatorValueHolder)) __PYX_ERR(7, 156, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityEqOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder) __PYX_ERR(7, 160, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityEqOperatorValueHolder)) __PYX_ERR(7, 160, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityNeOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder) __PYX_ERR(7, 164, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityNeOperatorValueHolder)) __PYX_ERR(7, 164, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityAndOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder) __PYX_ERR(7, 168, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityAndOperatorValueHolder)) __PYX_ERR(7, 168, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityOrOperatorValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder) __PYX_ERR(7, 172, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityOrOperatorValueHolder)) __PYX_ERR(7, 172, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityShiftedValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder) __PYX_ERR(7, 176, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityShiftedValueHolder)) __PYX_ERR(7, 176, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityDeltaValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder) __PYX_ERR(7, 179, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityDeltaValueHolder)) __PYX_ERR(7, 179, __pyx_L1_error)
  __pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder = __Pyx_ImportType(__pyx_t_1, "ultron.sentry.Analysis.SecurityValueHolders", "SecurityIIFValueHolder", sizeof(struct __pyx_obj_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder) __PYX_ERR(7, 183, __pyx_L1_error)
  __pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder = (struct __pyx_vtabstruct_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder*)__Pyx_GetVtable(__pyx_ptype_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder->tp_dict); if (unlikely(!__pyx_vtabptr_6ultron_6sentry_8Analysis_20SecurityValueHolders_SecurityIIFValueHolder)) __PYX_ERR(7, 183, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("ultron.sentry.Math.MathConstants"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "DBL_MIN", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_DBL_MIN, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "QL_EPSILON", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_QL_EPSILON, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_SQRT2", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT2, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_SQRT_2", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRT_2, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_SQRTPI", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_SQRTPI, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_1_SQRTPI", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_1_SQRTPI, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "M_PI", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_M_PI, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "NAN", (void **)&__pyx_vp_6ultron_6sentry_4Math_13MathConstants_NAN, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initCrossSectionValueHolders(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initCrossSectionValueHolders(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_CrossSectionValueHolders(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_CrossSectionValueHolders(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_CrossSectionValueHolders(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'CrossSectionValueHolders' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_CrossSectionValueHolders(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("CrossSectionValueHolders", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ultron__sentry__Analysis__CrossSectionValueHolders) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ultron.sentry.Analysis.CrossSectionValueHolders")) {
      if (unlikely(PyDict_SetItemString(modules, "ultron.sentry.Analysis.CrossSectionValueHolders", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":4
 * 
 * 
 * import copy             # <<<<<<<<<<<<<<
 * import six
 * import numpy as np
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_copy, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_copy, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":5
 * 
 * import copy
 * import six             # <<<<<<<<<<<<<<
 * import numpy as np
 * cimport numpy as np
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_six, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_six, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":6
 * import copy
 * import six
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * cimport cython
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":11
 * from ultron.sentry.Analysis.SeriesValues cimport SeriesValues
 * from ultron.sentry.Analysis.SecurityValueHolders cimport SecurityValueHolder
 * from ultron.sentry.Analysis.SecurityValueHolders import build_holder             # <<<<<<<<<<<<<<
 * from ultron.sentry.Math.MathConstants cimport NAN
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_build_holder);
  __Pyx_GIVEREF(__pyx_n_s_build_holder);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_build_holder);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_ultron_sentry_Analysis_SecurityV, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_build_holder); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_build_holder, __pyx_t_1) < 0) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CrossSectionValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_1__pyx_unpickle_CrossSectionValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CrossSectionValue, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_CrossSectionValueHolder__set_state(<CrossSectionValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CrossSectionValueHolder__set_state(CrossSectionValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_3__pyx_unpickle_CSTopNSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSTopNSecurityVal, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSTopNPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_5__pyx_unpickle_CSTopNPercentileSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSTopNPercentileS, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(<CSTopNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSTopNPercentileSecurityValueHolder__set_state(CSTopNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_7__pyx_unpickle_CSBottomNSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSBottomNSecurity, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSBottomNPercentileSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_9__pyx_unpickle_CSBottomNPercentileSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSBottomNPercenti, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(<CSBottomNPercentileSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSBottomNPercentileSecurityValueHolder__set_state(CSBottomNPercentileSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._n = __pyx_state[5]; __pyx_result._window = __pyx_state[6]; __pyx_result.cached = __pyx_state[7]; __pyx_result.updated = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_11__pyx_unpickle_CSRankedSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSRankedSecurityV, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSAverageSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_13__pyx_unpickle_CSAverageSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSAverageSecurity, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSAverageSecurityValueHolder__set_state(<CSAverageSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageSecurityValueHolder__set_state(CSAverageSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_15__pyx_unpickle_CSPercentileSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSPercentileSecur, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSAverageAdjustedSecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_17__pyx_unpickle_CSAverageAdjustedSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSAverageAdjusted, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(<CSAverageAdjustedSecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSAverageAdjustedSecurityValueHolder__set_state(CSAverageAdjustedSecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_19__pyx_unpickle_CSZScoreSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSZScoreSecurityV, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CSFillNASecurityValueHolder(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_21__pyx_unpickle_CSFillNASecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSFillNASecurityV, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_CSFillNASecurityValueHolder__set_state(<CSFillNASecurityValueHolder> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CSFillNASecurityValueHolder__set_state(CSFillNASecurityValueHolder __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._dependency = __pyx_state[0]; __pyx_result._group = __pyx_state[1]; __pyx_result._holderTemplate = __pyx_state[2]; __pyx_result._inner = __pyx_state[3]; __pyx_result._innerHolders = __pyx_state[4]; __pyx_result._window = __pyx_state[5]; __pyx_result.cached = __pyx_state[6]; __pyx_result.updated = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_6ultron_6sentry_8Analysis_24CrossSectionValueHolders_23__pyx_unpickle_CSResidueSecurityValueHolder, NULL, __pyx_n_s_ultron_sentry_Analysis_CrossSect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CSResidueSecurity, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ultron/sentry/Analysis/CrossSectionValueHolders.pyx":1
 * # -*- coding: utf-8 -*-             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "../../workenv/seleya/lib/python3.6/site-packages/numpy/__init__.pxd":892
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init ultron.sentry.Analysis.CrossSectionValueHolders", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ultron.sentry.Analysis.CrossSectionValueHolders");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* CallNextTpDealloc */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
